/* advautomesh_tri2d_mesh_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



static AutoMesh_Node2D *Node;
static AutoMesh_VolumeElement2D *Element;

static int Filled = 0;
static int EdgeIsShown = 0;
static int NodeIdIsShown = 0;
static int ElementIdIsShown = 0;
static double Ratio = 0.9;

static void SetView (void)
{
  double minX, minY;
  double maxX, maxY;
  double centerX, centerY;
  double size;
  
  AutoMesh_Node2D_getRange 
    (Node, 
     &minX, &minY,
     &maxX, &maxY);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  size = AutoMesh_Distance2D (minX, minY,
			      maxX, maxY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 

  fprintf (stderr, " minX %f minY %f \n",
	   minX, minY);
  fprintf (stderr, " maxX %f maxY %f \n",
	   maxX, maxY);
}

static void DrawMesh (void) 
{
  int nNodes = AutoMesh_Node2D_nNodes (Node);
  int nElements = AutoMesh_VolumeElement2D_nElements (Element);
  int iNode;
  int iElement;
  char buf[256];

  if (Filled) {
    AutoGL_SetColor (0, 1, 1);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 2);

      double x0, y0;
      double x1, y1;
      double x2, y2;
      double cx, cy;
      double vx0, vy0;
      double vx1, vy1;
      double vx2, vy2;
      
      AutoMesh_Node2D_getCoordAt (Node, nodeId0,
				  &x0, &y0);
      AutoMesh_Node2D_getCoordAt (Node, nodeId1,
				  &x1, &y1);
      AutoMesh_Node2D_getCoordAt (Node, nodeId2,
				  &x2, &y2);
      
      cx = (x0 + x1 + x2) / 3;
      cy = (y0 + y1 + y2) / 3;
      
      vx0 = cx + (x0 - cx) * Ratio;
      vy0 = cy + (y0 - cy) * Ratio;

      vx1 = cx + (x1 - cx) * Ratio;
      vy1 = cy + (y1 - cy) * Ratio;

      vx2 = cx + (x2 - cx) * Ratio;
      vy2 = cy + (y2 - cy) * Ratio;
      
      AutoGL_DrawTriangle (vx0, vy0, 0.0,
			   vx1, vy1, 0.0,
			   vx2, vy2, 0.0);
    }
  }

  if (EdgeIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 2);

      double x0, y0;
      double x1, y1;
      double x2, y2;
      double cx, cy;
      double vx0, vy0;
      double vx1, vy1;
      double vx2, vy2;
      
      AutoMesh_Node2D_getCoordAt (Node, nodeId0,
				  &x0, &y0);
      AutoMesh_Node2D_getCoordAt (Node, nodeId1,
				  &x1, &y1);
      AutoMesh_Node2D_getCoordAt (Node, nodeId2,
				  &x2, &y2);
      
      cx = (x0 + x1 + x2) / 3;
      cy = (y0 + y1 + y2) / 3;
      
      vx0 = cx + (x0 - cx) * Ratio;
      vy0 = cy + (y0 - cy) * Ratio;

      vx1 = cx + (x1 - cx) * Ratio;
      vy1 = cy + (y1 - cy) * Ratio;

      vx2 = cx + (x2 - cx) * Ratio;
      vy2 = cy + (y2 - cy) * Ratio;
      
      AutoGL_DrawLine (vx0, vy0, 0.0,
		       vx1, vy1, 0.0);
      AutoGL_DrawLine (vx1, vy1, 0.0,
		       vx2, vy2, 0.0);
      AutoGL_DrawLine (vx2, vy2, 0.0,
		       vx0, vy0, 0.0);
    }
  }

  if (NodeIdIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iNode = 0; iNode < nNodes; iNode++) {
      double x, y;
      
      AutoMesh_Node2D_getCoordAt (Node, iNode,
				  &x, &y);
      sprintf(buf, "%d", iNode);
      AutoGL_DrawString (x, y, 0.0,
			 buf);
    }
  }

  if (ElementIdIsShown) {
    AutoGL_SetColor (0, 0, 1);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_VolumeElement2D_nodeIdAt (Element, iElement, 2);

      double x0, y0;
      double x1, y1;
      double x2, y2;
      double cx, cy;
      
      AutoMesh_Node2D_getCoordAt (Node, nodeId0,
				  &x0, &y0);
      AutoMesh_Node2D_getCoordAt (Node, nodeId1,
				  &x1, &y1);
      AutoMesh_Node2D_getCoordAt (Node, nodeId2,
				  &x2, &y2);
      
      cx = (x0 + x1 + x2) / 3;
      cy = (y0 + y1 + y2) / 3;

      sprintf(buf, "%d", iElement);
      AutoGL_DrawString (cx, cy, 0.0,
			 buf);
    }
  }
}

static void RedrawView (void)
{
  AutoGL_DrawDisplayList ();
}

static void UpdateModel (void) 
{
  AutoGL_OpenDisplayList ();
  DrawMesh ();
  AutoGL_CloseDisplayList ();

  AutoGL_DrawView ();
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri2d_mesh_view  node_file  element_file \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node2D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node2D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement2D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Filled, "Filled");
  AutoGL_AddBoolean (&EdgeIsShown, "EdgeIsShown");
  AutoGL_AddBoolean (&NodeIdIsShown, "NodeIdIsShown");
  AutoGL_AddBoolean (&ElementIdIsShown, "ElementIdIsShown");
  AutoGL_AddReal (&Ratio, "Ratio");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
