/* advautomesh_tri3d_contour_view2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



#ifdef TIME_MEASURE
#include "time_measure.c"
#endif



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_NodeScalar3D *Scalar;

static AutoRes_Table *Triangles;
static AutoRes_Real1DColumn *Triangles_data;

static int EdgeIsShown = 0;

static double MinRange = 0.0;
static double MaxRange = 1.0;

static void SetContourRange (void)
{
  AutoMesh_NodeScalar3D_getRange (Scalar, 
				  &MinRange, &MaxRange);
}

static void MakeTable (void)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;
  int flag;

  Triangles = AutoRes_Table_New ("Triangles");

  Triangles_data = AutoRes_Real1DColumn_New 
    (Triangles, "data", 21);
  flag = AutoRes_Table_addReal1DColumn 
    (Triangles, Triangles_data);
  assert(flag);

  flag = AutoRes_Table_allocateRecords (Triangles, nElements);
  assert(flag);

  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
    int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
    int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;
    double nx, ny, nz;
    
    AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				&x0, &y0, &z0);
    AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				&x1, &y1, &z1);
    AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				&x2, &y2, &z2);
    AutoMesh_SurfaceElement3D_getNormalAt (Element, iElement,
					   &nx, &ny, &nz);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 0, x0);
    AUTORES_R1D_SET(Triangles_data, iElement, 1, y0);
    AUTORES_R1D_SET(Triangles_data, iElement, 2, z0);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 3, x1);
    AUTORES_R1D_SET(Triangles_data, iElement, 4, y1);
    AUTORES_R1D_SET(Triangles_data, iElement, 5, z1);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 6, x2);
    AUTORES_R1D_SET(Triangles_data, iElement, 7, y2);
    AUTORES_R1D_SET(Triangles_data, iElement, 8, z2);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 9, nx);
    AUTORES_R1D_SET(Triangles_data, iElement, 10, ny);
    AUTORES_R1D_SET(Triangles_data, iElement, 11, nz);

    AUTORES_R1D_SET(Triangles_data, iElement, 12, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 13, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 14, 0.0);

    AUTORES_R1D_SET(Triangles_data, iElement, 15, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 16, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 17, 0.0);

    AUTORES_R1D_SET(Triangles_data, iElement, 18, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 19, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 20, 0.0);
  }
}

static void UpdateTable (void)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
    int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
    int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

    double s0, s1, s2;
    double grade0, grade1, grade2;
    double r0, b0, g0;
    double r1, b1, g1;
    double r2, b2, g2;
    
    s0 = AutoMesh_NodeScalar3D_valueAt (Scalar, nodeId0);
    s1 = AutoMesh_NodeScalar3D_valueAt (Scalar, nodeId1);
    s2 = AutoMesh_NodeScalar3D_valueAt (Scalar, nodeId2);

    grade0 = (s0 - MinRange) / (MaxRange - MinRange);
    grade1 = (s1 - MinRange) / (MaxRange - MinRange);
    grade2 = (s2 - MinRange) / (MaxRange - MinRange);

    AutoGL_GetContourColor (&r0, &g0, &b0,
			    grade0);
    AutoGL_GetContourColor (&r1, &g1, &b1,
			    grade1);
    AutoGL_GetContourColor (&r2, &g2, &b2,
			    grade2);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 12, r0);
    AUTORES_R1D_SET(Triangles_data, iElement, 13, g0);
    AUTORES_R1D_SET(Triangles_data, iElement, 14, b0);

    AUTORES_R1D_SET(Triangles_data, iElement, 15, r1);
    AUTORES_R1D_SET(Triangles_data, iElement, 16, g1);
    AUTORES_R1D_SET(Triangles_data, iElement, 17, b1);

    AUTORES_R1D_SET(Triangles_data, iElement, 18, r2);
    AUTORES_R1D_SET(Triangles_data, iElement, 19, g2);
    AUTORES_R1D_SET(Triangles_data, iElement, 20, b2);
  }
}

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawContour (void) 
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;
  double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

  AutoGL_SetTwoSideFaceFlag (0);

  for (iElement = 0; iElement < nElements; iElement++) {
    float x0 = *dataArea++;
    float y0 = *dataArea++;
    float z0 = *dataArea++;
    
    float x1 = *dataArea++;
    float y1 = *dataArea++;
    float z1 = *dataArea++;
    
    float x2 = *dataArea++;
    float y2 = *dataArea++;
    float z2 = *dataArea++;
    
    float nx = *dataArea++;
    float ny = *dataArea++;
    float nz = *dataArea++;

    float r0 = *dataArea++;
    float g0 = *dataArea++;
    float b0 = *dataArea++;
    
    float r1 = *dataArea++;
    float g1 = *dataArea++;
    float b1 = *dataArea++;
    
    float r2 = *dataArea++;
    float g2 = *dataArea++;
    float b2 = *dataArea++;
    
    AutoGL_DrawColorInterpolatedTriangle2 
      (x0, y0, z0, r0, g0, b0,
       x1, y1, z1, r1, g1, b1,
       x2, y2, z2, r2, g2, b2,
       nx, ny, nz);
  }

  AutoGL_SetTwoSideFaceFlag (1);



  if (EdgeIsShown) {
    double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

    AutoGL_SetColor2 (1, 1, 1);

    for (iElement = 0; iElement < nElements; iElement++) {
      float x0 = *dataArea++;
      float y0 = *dataArea++;
      float z0 = *dataArea++;
      
      float x1 = *dataArea++;
      float y1 = *dataArea++;
      float z1 = *dataArea++;
      
      float x2 = *dataArea++;
      float y2 = *dataArea++;
      float z2 = *dataArea++;
      
      dataArea += 12; 
      /* nx, ny, nz */
      /* r0-2, g0-2, b0-2 */
      
      AutoGL_DrawLine2 (x0, y0, z0,
			x1, y1, z1);
      AutoGL_DrawLine2 (x1, y1, z1,
			x2, y2, z2);
      AutoGL_DrawLine2 (x2, y2, z2,
			x0, y0, z0);
    }
  }
}

static void RedrawView (void)
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  AutoGL_DrawContourMap (80, MinRange, MaxRange);

  DrawContour ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif
}

static void UpdateModel (void)
{
  UpdateTable ();
  AutoGL_DrawView ();
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_contour_view2  node_file  element_file \n"
       "        node_scalar_file  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Scalar = AutoMesh_NodeScalar3D_New (Node);
  
  flag = AutoMesh_NodeScalar3D_makeValueColumn (Scalar);
  assert(flag);
  
  AutoRes_TableInOut_AssociateRealColumn (Scalar->nodes_value);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);



  SetContourRange ();
  MakeTable ();
  UpdateTable ();
  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&EdgeIsShown, "EdgeIsShown");
  AutoGL_AddReal (&MinRange, "MinRange");
  AutoGL_AddReal (&MaxRange, "MaxRange");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
