/* advautomesh_tri3d_deform_view2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



#ifdef TIME_MEASURE
#include "time_measure.c"
#endif



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_NodeVector3D *Vector;

static AutoRes_Table *Triangles;
static AutoRes_Real1DColumn *Triangles_data;

static int EdgeIsShown = 0;

static double Scale;

static void SetDeformationRange (void)
{
  double maxLength;
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double size;
  
  maxLength = AutoMesh_NodeVector3D_maxLength (Vector);
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  size = AutoMesh_Distance3D (minRangeX, minRangeY, minRangeZ,
			      maxRangeX, maxRangeY, maxRangeZ) / 2;
  
  if (maxLength == 0.0) {
    Scale = 1.0;
  } else {
    Scale = size * 0.1 / maxLength;
  }
}

static void MakeTable (void)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;
  int flag;

  Triangles = AutoRes_Table_New ("Triangles");

  Triangles_data = AutoRes_Real1DColumn_New 
    (Triangles, "data", 12);
  flag = AutoRes_Table_addReal1DColumn 
    (Triangles, Triangles_data);
  assert(flag);

  flag = AutoRes_Table_allocateRecords (Triangles, nElements);
  assert(flag);

  for (iElement = 0; iElement < nElements; iElement++) {
    AUTORES_R1D_SET(Triangles_data, iElement, 0, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 1, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 2, 0.0);

    AUTORES_R1D_SET(Triangles_data, iElement, 3, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 4, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 5, 0.0);

    AUTORES_R1D_SET(Triangles_data, iElement, 6, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 7, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 8, 0.0);

    AUTORES_R1D_SET(Triangles_data, iElement, 9, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 10, 0.0);
    AUTORES_R1D_SET(Triangles_data, iElement, 11, 0.0);
  }
}

static void UpdateTable (void)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
    int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
    int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;
    double u0, v0, w0;
    double u1, v1, w1;
    double u2, v2, w2;
    double vx0, vy0, vz0;
    double vx1, vy1, vz1;
    double vx2, vy2, vz2;
    double nx, ny, nz;
    
    AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				&x0, &y0, &z0);
    AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				&x1, &y1, &z1);
    AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				&x2, &y2, &z2);
    AutoMesh_NodeVector3D_getValueAt (Vector, nodeId0,
				      &u0, &v0, &w0);
    AutoMesh_NodeVector3D_getValueAt (Vector, nodeId1,
				      &u1, &v1, &w1);
    AutoMesh_NodeVector3D_getValueAt (Vector, nodeId2,
				      &u2, &v2, &w2);

    vx0 = x0 + u0 * Scale;
    vy0 = y0 + v0 * Scale;
    vz0 = z0 + w0 * Scale;

    vx1 = x1 + u1 * Scale;
    vy1 = y1 + v1 * Scale;
    vz1 = z1 + w1 * Scale;

    vx2 = x2 + u2 * Scale;
    vy2 = y2 + v2 * Scale;
    vz2 = z2 + w2 * Scale;
    
    AUTORES_R1D_SET(Triangles_data, iElement, 0, vx0);
    AUTORES_R1D_SET(Triangles_data, iElement, 1, vy0);
    AUTORES_R1D_SET(Triangles_data, iElement, 2, vz0);

    AUTORES_R1D_SET(Triangles_data, iElement, 3, vx1);
    AUTORES_R1D_SET(Triangles_data, iElement, 4, vy1);
    AUTORES_R1D_SET(Triangles_data, iElement, 5, vz1);

    AUTORES_R1D_SET(Triangles_data, iElement, 6, vx2);
    AUTORES_R1D_SET(Triangles_data, iElement, 7, vy2);
    AUTORES_R1D_SET(Triangles_data, iElement, 8, vz2);

    AutoMesh_GetVectorProduct3D (vx0 - vx1, vy0 - vy1, vz0 - vz1,
				 vx0 - vx2, vy0 - vy2, vz0 - vz2,
				 &nx, &ny, &nz);
    AutoMesh_GetNormalizedVector3D (nx, ny, nz,
				    &nx, &ny, &nz);

    AUTORES_R1D_SET(Triangles_data, iElement, 9, nx);
    AUTORES_R1D_SET(Triangles_data, iElement, 10, ny);
    AUTORES_R1D_SET(Triangles_data, iElement, 11, nz);
  }
}

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawDeformation (void) 
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;
  double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

  AutoGL_SetTwoSideFaceFlag (0);

  AutoGL_SetColor2 (1, 0, 1);
  for (iElement = 0; iElement < nElements; iElement++) {
    float vx0 = *dataArea++;
    float vy0 = *dataArea++;
    float vz0 = *dataArea++;
    
    float vx1 = *dataArea++;
    float vy1 = *dataArea++;
    float vz1 = *dataArea++;
    
    float vx2 = *dataArea++;
    float vy2 = *dataArea++;
    float vz2 = *dataArea++;
    
    float nx = *dataArea++;
    float ny = *dataArea++;
    float nz = *dataArea++;

    AutoGL_DrawTriangle2 (vx0, vy0, vz0,
			  vx1, vy1, vz1,
			  vx2, vy2, vz2,
			  nx, ny, nz);
  }

  AutoGL_SetTwoSideFaceFlag (1);



  if (EdgeIsShown) {
    double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

    AutoGL_SetColor2 (0, 1, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      float vx0 = *dataArea++;
      float vy0 = *dataArea++;
      float vz0 = *dataArea++;
      
      float vx1 = *dataArea++;
      float vy1 = *dataArea++;
      float vz1 = *dataArea++;
      
      float vx2 = *dataArea++;
      float vy2 = *dataArea++;
      float vz2 = *dataArea++;
      
      dataArea += 3; /* nx, ny, nz */
    
      AutoGL_DrawLine2 (vx0, vy0, vz0,
			vx1, vy1, vz1);
      AutoGL_DrawLine2 (vx1, vy1, vz1,
			vx2, vy2, vz2);
      AutoGL_DrawLine2 (vx2, vy2, vz2,
			vx0, vy0, vz0);
    }
  }
}

static void RedrawView (void)
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  DrawDeformation ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif
}

static void UpdateModel (void) 
{
  UpdateTable ();
  AutoGL_DrawView ();
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_deform_view2  node_file  element_file \n"
       "        node_vector_file  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Vector = AutoMesh_NodeVector3D_New (Node);
  
  flag = AutoMesh_NodeVector3D_makeValueColumn (Vector);
  assert(flag);
  
  AutoRes_TableInOut_AssociateReal1DColumn (Vector->nodes_value);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);



  SetDeformationRange ();
  MakeTable ();
  UpdateTable ();
  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&EdgeIsShown, "EdgeIsShown");
  AutoGL_AddReal (&Scale, "Scale");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
