/* advautomesh_tri3d_group_frame_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_SurfaceEdge3D *Edge;
static AutoMesh_SurfaceGrouper3D *Grouper;

static int NFrameGroups;
static int NFrameVertexs;

static int FrameIsShown = 0;
static int FrameGroupIsShown = 0;
static int FrameGroupIdIsShown = 0;
static int FrameVertexIsShown = 0;
static int FrameVertexIdIsShown = 0;

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawGroup (void) 
{
  int nEdges = AutoMesh_SurfaceEdge3D_nEdges (Edge);
  int iEdge;
  int nNodes = AutoMesh_Node3D_nNodes (Node);
  int iNode;
  char buf[256];

  if (FrameIsShown) {
    AutoGL_SetColor (1, 1, 1);

    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int frameGroupId 
	= AutoMesh_SurfaceGrouper3D_frameGroupIdAtEdge (Grouper, iEdge);

      if (0 <= frameGroupId) {
	int nodeId0, nodeId1;
	double x0, y0, z0;
	double x1, y1, z1;
	
	AutoMesh_SurfaceEdge3D_getVertexNodeIdsAt 
	  (Edge, iEdge,
	   &nodeId0, &nodeId1);
	
	AutoMesh_Node3D_getCoordAt
	  (Node, nodeId0,
	   &x0, &y0, &z0);
	AutoMesh_Node3D_getCoordAt
	  (Node, nodeId1,
	   &x1, &y1, &z1);

	AutoGL_DrawLine (x0, y0, z0,
			 x1, y1, z1);
      }
    }
  }

  if (FrameGroupIsShown) {
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int frameGroupId 
	= AutoMesh_SurfaceGrouper3D_frameGroupIdAtEdge (Grouper, iEdge);

      if (0 <= frameGroupId) {
	int nodeId0, nodeId1;
	double x0, y0, z0;
	double x1, y1, z1;
	double grade;
	double r, g, b;
	
	AutoMesh_SurfaceEdge3D_getVertexNodeIdsAt 
	  (Edge, iEdge,
	   &nodeId0, &nodeId1);
	
	AutoMesh_Node3D_getCoordAt
	  (Node, nodeId0,
	   &x0, &y0, &z0);
	AutoMesh_Node3D_getCoordAt
	  (Node, nodeId1,
	   &x1, &y1, &z1);

	grade = (frameGroupId % 11) * 0.1;
	AutoGL_GetContourColor (&r, &g, &b, 
				grade);
	AutoGL_SetColor (r, g, b);
	AutoGL_DrawLine (x0, y0, z0,
			 x1, y1, z1);
      }
    }
  }

  if (FrameGroupIdIsShown) {
    AutoGL_SetColor (0, 0, 1);
    
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int frameGroupId 
	= AutoMesh_SurfaceGrouper3D_frameGroupIdAtEdge (Grouper, iEdge);

      if (0 <= frameGroupId) {
	double cx, cy, cz;
	
	AutoMesh_SurfaceEdge3D_getMidPointAt 
	  (Edge, iEdge,
	   &cx, &cy, &cz);
	
	sprintf(buf, "%d", frameGroupId);
	AutoGL_DrawString (cx, cy, cz, buf);
      }
    }
  }

  if (FrameVertexIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iNode = 0; iNode < nNodes; iNode++) {
      int frameVertexId 
	= AutoMesh_SurfaceGrouper3D_frameVertexIdAtNode (Grouper, iNode);

      if (0 <= frameVertexId) {
	double x, y, z;
	
	AutoMesh_Node3D_getCoordAt 
	  (Node, iNode,
	   &x, &y, &z);
	
	AutoGL_DrawDiamondMark3D (x, y, z,
				  3, 1);
      }
    }
  }

  if (FrameVertexIdIsShown) {
    AutoGL_SetColor (0, 1, 0);
    
    for (iNode = 0; iNode < nNodes; iNode++) {
      int frameVertexId 
	= AutoMesh_SurfaceGrouper3D_frameVertexIdAtNode (Grouper, iNode);

      if (0 <= frameVertexId) {
	double x, y, z;
	
	AutoMesh_Node3D_getCoordAt 
	  (Node, iNode,
	   &x, &y, &z);
	
	sprintf(buf, "%d", frameVertexId);
	AutoGL_DrawString (x, y, z, buf);
      }
    }
  }

}

static void RedrawView (void)
{
  AutoGL_DrawDisplayList ();
}

static void UpdateModel (void) 
{
  AutoGL_OpenDisplayList ();
  DrawGroup ();
  AutoGL_CloseDisplayList ();

  AutoGL_DrawView ();
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_group_frame_view  node_file  element_file \n"
       "        edge_vertex_file  \n"
       "        edge_frame_group_file  \n"
       "        node_frame_vertex_file  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Edge = AutoMesh_SurfaceEdge3D_New (Element);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceEdge3D_makeVertexNodeIdsColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_vertexNodeIds);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);
  


  Grouper = AutoMesh_SurfaceGrouper3D_New (Edge);
  assert(Grouper != NULL);

  flag = AutoMesh_SurfaceGrouper3D_makeFrameGroupIdAtEdgeColumn (Grouper);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->edges_frameGroupId);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);

  AutoMesh_SurfaceGrouper3D_setNFrameGroups (Grouper);
  NFrameGroups = AutoMesh_SurfaceGrouper3D_nFrameGroups (Grouper);
  
  fprintf (stderr, " NFrameGroups %d \n",
	   NFrameGroups);

  flag = AutoMesh_SurfaceGrouper3D_makeFrameVertexIdAtNodeColumn (Grouper);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->nodes_frameVertexId);
  flag = AutoRes_TableInOut_Read (argv[5]);
  assert(flag);

  AutoMesh_SurfaceGrouper3D_setNFrameVertexs (Grouper);
  NFrameVertexs = AutoMesh_SurfaceGrouper3D_nFrameVertexs (Grouper);

  fprintf (stderr, " NFrameVertexs %d \n",
	   NFrameVertexs);



  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&FrameIsShown, "FrameIsShown");
  AutoGL_AddBoolean (&FrameGroupIsShown, "FrameGroupIsShown");
  AutoGL_AddBoolean (&FrameGroupIdIsShown, "FrameGroupIsShown");
  AutoGL_AddBoolean (&FrameVertexIsShown, "FrameVertexIsShown");
  AutoGL_AddBoolean (&FrameVertexIdIsShown, "FrameVertexIdIsShown");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
