/* advautomesh_tri3d_group_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_SurfaceGrouper3D *Grouper;
static AutoMesh_Frame3D *Frame;

static int NGroups;

static int Filled = 0;
static int FrameIsShown = 0;
static int GroupIsShown = 0;
static int GroupIdIsShown = 0;

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawGroup (void) 
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int nFrames = AutoMesh_Frame3D_nFrames (Frame);
  int iElement;
  int iFrame;
  char buf[256];

  if (Filled) {
    AutoGL_SetColor (1, 1, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      AutoGL_DrawTriangle (x0, y0, z0,
			   x1, y1, z1,
			   x2, y2, z2);
    }
  }

  if (FrameIsShown) {
    AutoGL_SetColor (0, 0, 1);

    for (iFrame = 0; iFrame < nFrames; iFrame++) {
      double x0, y0, z0;
      double x1, y1, z1;
	
      AutoMesh_Frame3D_getEndPointsAt 
	(Frame, iFrame,
	 &x0, &y0, &z0,
	 &x1, &y1, &z1);
      AutoGL_DrawLine (x0, y0, z0,
		       x1, y1, z1);
    }
  }

  if (GroupIsShown) {
    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);
      int groupId = AutoMesh_SurfaceGrouper3D_groupIdAtElement
	(Grouper, iElement);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      double grade;
      double r, g, b;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      grade = (groupId % 11) * 0.1;
      AutoGL_GetContourColor (&r, &g, &b, 
			      grade);
      AutoGL_SetColor (r, g, b);
      AutoGL_DrawTriangle (x0, y0, z0,
			   x1, y1, z1,
			   x2, y2, z2);
    }
  }

  if (GroupIdIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);
      int groupId = AutoMesh_SurfaceGrouper3D_groupIdAtElement
	(Grouper, iElement);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      double cx, cy, cz;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      cx = (x0 + x1 + x2) / 3;
      cy = (y0 + y1 + y2) / 3;
      cz = (z0 + z1 + z2) / 3;
      
      sprintf(buf, "%d", groupId);
      AutoGL_DrawString (cx, cy, cz, buf);
    }
  }

}

static void RedrawView (void)
{
  AutoGL_DrawDisplayList ();
}

static void UpdateModel (void) 
{
  AutoGL_OpenDisplayList ();
  DrawGroup ();
  AutoGL_CloseDisplayList ();

  AutoGL_DrawView ();
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_group_view  node_file  element_file \n"
       "        element_group_file  frame_file  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Grouper = AutoMesh_SurfaceGrouper3D_NewWithoutEdge (Element);
  assert(Grouper != NULL);

  flag = AutoMesh_SurfaceGrouper3D_makeGroupIdAtElementColumn (Grouper);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->elements_groupId);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);

  AutoMesh_SurfaceGrouper3D_setNGroups (Grouper);
  NGroups = AutoMesh_SurfaceGrouper3D_nGroups (Grouper);

  fprintf (stderr, " NGroups %d \n",
	   NGroups);



  Frame = AutoMesh_Frame3D_New ();
  assert(Frame != NULL);

  flag = AutoMesh_Frame3D_makeEndPointsColumn (Frame);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Frame->frames_endPoint0);
  AutoRes_TableInOut_AssociateReal1DColumn (Frame->frames_endPoint1);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);



  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Filled, "Filled");
  AutoGL_AddBoolean (&FrameIsShown, "FrameIsShown");
  AutoGL_AddBoolean (&GroupIsShown, "GroupIsShown");
  AutoGL_AddBoolean (&GroupIdIsShown, "GroupIdIsShown");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
