/* advautomesh_tri3d_mesh_select.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;

static int Filled = 0;
static int EdgeIsShown = 0;
static int NodeIdIsShown = 0;
static int ElementIdIsShown = 0;

#define SELECTION_NODE 0
#define SELECTION_ELEMENT 1
static int SelectionType = 0;
static int SelectedNodeId = -1;
static int SelectedElementId = -1;

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawMesh (void) 
{
  int nNodes = AutoMesh_Node3D_nNodes (Node);
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iNode;
  int iElement;
  char buf[256];

  if (Filled) {
    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      if (SelectionType == SELECTION_ELEMENT
	  && SelectedElementId == iElement) {
	AutoGL_SetColor (1, 0, 0);
      } else {
	AutoGL_SetColor (0, 1, 1);
      }

      AutoGL_DrawTriangle (x0, y0, z0,
			   x1, y1, z1,
			   x2, y2, z2);
    }
  }

  if (EdgeIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      AutoGL_DrawLine (x0, y0, z0,
		       x1, y1, z1);
      AutoGL_DrawLine (x1, y1, z1,
		       x2, y2, z2);
      AutoGL_DrawLine (x2, y2, z2,
		       x0, y0, z0);
    }
  }

  if (SelectionType == SELECTION_NODE
      && 0 <= SelectedNodeId) {
    double x, y, z;
    
    AutoMesh_Node3D_getCoordAt (Node, SelectedNodeId,
				&x, &y, &z);
    AutoGL_SetColor (1, 0, 0);
    AutoGL_DrawDiamondMark3D 
      (x, y, z,
       10, 1);
  }

  if (NodeIdIsShown) {
    AutoGL_SetColor (1, 0, 0);

    for (iNode = 0; iNode < nNodes; iNode++) {
      double x, y, z;
      
      AutoMesh_Node3D_getCoordAt (Node, iNode,
				  &x, &y, &z);
      sprintf(buf, " %d", iNode);
      
      AutoGL_DrawString (x, y, z,
			 buf);
    }
  }

  if (ElementIdIsShown) {
    AutoGL_SetColor (0, 0, 1);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      double cx, cy, cz;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      cx = (x0 + x1 + x2) / 3;
      cy = (y0 + y1 + y2) / 3;
      cz = (z0 + z1 + z2) / 3;

      sprintf(buf, "%d", iElement);

      AutoGL_DrawString (cx, cy, cz,
			 buf);
    }
  }
}

static void SelectMesh (void) 
{
  int dcX, dcY, dcZ;

  AutoGL_GetPointingDevicePositionDc 
    (&dcX, &dcY, &dcZ);
  
  if (SelectionType == SELECTION_NODE) {
    int toleranceDc = 5;
    int nNodes = AutoMesh_Node3D_nNodes (Node);
    int iNode;
    
    AutoGL_StartSelection (dcX, dcY, toleranceDc);

    for (iNode = 0; iNode < nNodes; iNode++) {
      double x, y, z;
      
      AutoMesh_Node3D_getCoordAt (Node, iNode,
				  &x, &y, &z);

      AutoGL_SetSelectionId (iNode);
      AutoGL_DrawLine (x, y, z,
		       x, y, z);
    }

    AutoGL_EndSelection ();
    SelectedNodeId = AutoGL_GetSelectedId ();
    
    printf (" nodeID %d \n", SelectedNodeId);

  } else if (SelectionType == SELECTION_ELEMENT) {
    int toleranceDc = 1;
    int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
    int iElement;

    AutoGL_StartSelection (dcX, dcY, toleranceDc);

    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
      int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
      int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

      double x0, y0, z0;
      double x1, y1, z1;
      double x2, y2, z2;
      
      AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				  &x0, &y0, &z0);
      AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				  &x1, &y1, &z1);
      AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				  &x2, &y2, &z2);
      
      AutoGL_SetSelectionId (iElement);
      AutoGL_DrawTriangle (x0, y0, z0,
			   x1, y1, z1,
			   x2, y2, z2);
    }

    AutoGL_EndSelection ();
    SelectedElementId = AutoGL_GetSelectedId ();
    
    printf (" elementID %d \n", SelectedElementId);

  } else {
    assert(0);
  }

}

static void RedrawView (void)
{
  DrawMesh ();
}

static void HandleEvent (void)
{
  int event = AutoGL_GetViewEvent ();

  if (event == AUTOGL_EVENT_POINTING_DEVICE_RELEASE) {
    SelectMesh ();
    AutoGL_DrawView ();
  } else if (event == AUTOGL_EVENT_KEY) {
    AutoGL_HandleDefaultKeyEventInMode3D ();
  }
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_mesh_select  node_file  element_file \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (HandleEvent);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Filled, "Filled");
  AutoGL_AddBoolean (&EdgeIsShown, "EdgeIsShown");
  AutoGL_AddBoolean (&NodeIdIsShown, "NodeIdIsShown");
  AutoGL_AddBoolean (&ElementIdIsShown, "ElementIdIsShown");

  AutoGL_AddInteger (&SelectionType, "SelectionType"); 
  AutoGL_SetLabel ("select");
  AutoGL_AddIntegerItem ("node"); 
  AutoGL_AddIntegerItem ("element");
}
