/* advautomesh_tri3d_mesh_select2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <autogl.h>
#include <automesh.h>



#ifdef TIME_MEASURE
#include "time_measure.c"
#endif



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;

static AutoRes_Table *Triangles;
static AutoRes_Real1DColumn *Triangles_data;

static int Filled = 0;
static int EdgeIsShown = 0;

#define SELECTION_NODE 0
#define SELECTION_ELEMENT 1
static int SelectionType = 0;
static int SelectedNodeId = -1;
static int SelectedElementId = -1;

static void StoreTable (void)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;
  int flag;

  Triangles = AutoRes_Table_New ("Triangles");

  Triangles_data = AutoRes_Real1DColumn_New 
    (Triangles, "data", 12);
  flag = AutoRes_Table_addReal1DColumn 
    (Triangles, Triangles_data);
  assert(flag);

  flag = AutoRes_Table_allocateRecords (Triangles, nElements);
  assert(flag);

  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 0);
    int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 1);
    int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (Element, iElement, 2);

    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;
    double nx, ny, nz;
    
    AutoMesh_Node3D_getCoordAt (Node, nodeId0,
				&x0, &y0, &z0);
    AutoMesh_Node3D_getCoordAt (Node, nodeId1,
				&x1, &y1, &z1);
    AutoMesh_Node3D_getCoordAt (Node, nodeId2,
				&x2, &y2, &z2);
    AutoMesh_SurfaceElement3D_getNormalAt (Element, iElement,
					   &nx, &ny, &nz);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 0, x0);
    AUTORES_R1D_SET(Triangles_data, iElement, 1, y0);
    AUTORES_R1D_SET(Triangles_data, iElement, 2, z0);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 3, x1);
    AUTORES_R1D_SET(Triangles_data, iElement, 4, y1);
    AUTORES_R1D_SET(Triangles_data, iElement, 5, z1);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 6, x2);
    AUTORES_R1D_SET(Triangles_data, iElement, 7, y2);
    AUTORES_R1D_SET(Triangles_data, iElement, 8, z2);
    
    AUTORES_R1D_SET(Triangles_data, iElement, 9, nx);
    AUTORES_R1D_SET(Triangles_data, iElement, 10, ny);
    AUTORES_R1D_SET(Triangles_data, iElement, 11, nz);
  }
}

static void SetView (void)
{
  double minX, minY, minZ;
  double maxX, maxY, maxZ;
  double centerX, centerY, centerZ;
  double size;
  
  AutoMesh_Node3D_getRange 
    (Node, 
     &minX, &minY, &minZ,
     &maxX, &maxY, &maxZ);
  centerX = (minX + maxX) / 2;
  centerY = (minY + maxY) / 2;
  centerZ = (minZ + maxZ) / 2;
  size = AutoMesh_Distance3D (minX, minY, minZ,
			      maxX, maxY, maxZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 

  fprintf (stderr, " minX %f minY %f minZ %f \n",
	   minX, minY, minZ);
  fprintf (stderr, " maxX %f maxY %f maxZ %f \n",
	   maxX, maxY, maxZ);
}

static void DrawMesh (void) 
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  int iElement;

  if (Filled) {
    double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

    AutoGL_SetTwoSideFaceFlag (0);

    AutoGL_SetColor2 (0, 1, 1);

    for (iElement = 0; iElement < nElements; iElement++) {
      float x0 = *dataArea++;
      float y0 = *dataArea++;
      float z0 = *dataArea++;
      
      float x1 = *dataArea++;
      float y1 = *dataArea++;
      float z1 = *dataArea++;

      float x2 = *dataArea++;
      float y2 = *dataArea++;
      float z2 = *dataArea++;
      
      float nx = *dataArea++;
      float ny = *dataArea++;
      float nz = *dataArea++;

      if (SelectionType == SELECTION_ELEMENT
	  && SelectedElementId == iElement) {
	AutoGL_SetColor2 (1, 0, 0);
      }

      AutoGL_DrawTriangle2 (x0, y0, z0,
			    x1, y1, z1,
			    x2, y2, z2,
			    nx, ny, nz);

      if (SelectionType == SELECTION_ELEMENT
	  && SelectedElementId == iElement) {
	AutoGL_SetColor2 (0, 1, 1);
      }
    }

    AutoGL_SetTwoSideFaceFlag (1);
  }



  if (EdgeIsShown) {
    double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

    AutoGL_SetColor2 (1, 0, 0);

    for (iElement = 0; iElement < nElements; iElement++) {
      float x0 = *dataArea++;
      float y0 = *dataArea++;
      float z0 = *dataArea++;
      
      float x1 = *dataArea++;
      float y1 = *dataArea++;
      float z1 = *dataArea++;
      
      float x2 = *dataArea++;
      float y2 = *dataArea++;
      float z2 = *dataArea++;
      
      dataArea += 3;  /* nx, ny, nz */

      AutoGL_DrawLine2 (x0, y0, z0,
			x1, y1, z1);
      AutoGL_DrawLine2 (x1, y1, z1,
			x2, y2, z2);
      AutoGL_DrawLine2 (x2, y2, z2,
			x0, y0, z0);
    }
  }



  if (SelectionType == SELECTION_NODE
      && 0 <= SelectedNodeId) {
    double x, y, z;
    
    AutoMesh_Node3D_getCoordAt (Node, SelectedNodeId,
				&x, &y, &z);
    AutoGL_SetColor (1, 0, 0);
    AutoGL_DrawDiamondMark3D 
      (x, y, z,
       10, 1);
  }
}

static void SelectMesh (void) 
{
  int dcX, dcY, dcZ;

  AutoGL_GetPointingDevicePositionDc 
    (&dcX, &dcY, &dcZ);
  
  if (SelectionType == SELECTION_NODE) {
    int toleranceDc = 5;
    int nNodes = AutoMesh_Node3D_nNodes (Node);
    int iNode;
    
    AutoGL_StartSelection (dcX, dcY, toleranceDc);

    for (iNode = 0; iNode < nNodes; iNode++) {
      double x, y, z;
      
      AutoMesh_Node3D_getCoordAt (Node, iNode,
				  &x, &y, &z);

      AutoGL_SetSelectionId (iNode);
      AutoGL_DrawLine (x, y, z,
		       x, y, z);
    }

    AutoGL_EndSelection ();
    SelectedNodeId = AutoGL_GetSelectedId ();
    
    printf (" nodeID %d \n", SelectedNodeId);

  } else if (SelectionType == SELECTION_ELEMENT) {
    int toleranceDc = 1;
    int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
    int iElement;
    double *dataArea = AutoRes_Real1DColumn_dataArea (Triangles_data);

    AutoGL_StartSelection (dcX, dcY, toleranceDc);

    AutoGL_SetTwoSideFaceFlag (0);

    for (iElement = 0; iElement < nElements; iElement++) {
      float x0 = *dataArea++;
      float y0 = *dataArea++;
      float z0 = *dataArea++;
      
      float x1 = *dataArea++;
      float y1 = *dataArea++;
      float z1 = *dataArea++;

      float x2 = *dataArea++;
      float y2 = *dataArea++;
      float z2 = *dataArea++;
      
      float nx = *dataArea++;
      float ny = *dataArea++;
      float nz = *dataArea++;

      AutoGL_SetSelectionId (iElement);
      AutoGL_DrawTriangle2 (x0, y0, z0,
			    x1, y1, z1,
			    x2, y2, z2,
			    nx, ny, nz);
    }

    AutoGL_SetTwoSideFaceFlag (1);

    AutoGL_EndSelection ();
    SelectedElementId = AutoGL_GetSelectedId ();
    
    printf (" elementID %d \n", SelectedElementId);

  } else {
    assert(0);
  }

}

static void RedrawView (void)
{
  DrawMesh ();
}

static void HandleEvent (void)
{
  int event = AutoGL_GetViewEvent ();

  if (event == AUTOGL_EVENT_POINTING_DEVICE_RELEASE) {
#ifdef TIME_MEASURE
    StartMeasureTime ();
#endif

    SelectMesh ();
    AutoGL_DrawView ();

#ifdef TIME_MEASURE
    EndMeasureTime ();
#endif
  } else if (event == AUTOGL_EVENT_KEY) {
    AutoGL_HandleDefaultKeyEventInMode3D ();
  }
}

static void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_mesh_select2  node_file  element_file \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  StoreTable ();
  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (HandleEvent);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Filled, "Filled");
  AutoGL_AddBoolean (&EdgeIsShown, "EdgeIsShown");

  AutoGL_AddInteger (&SelectionType, "SelectionType"); 
  AutoGL_SetLabel ("select");
  AutoGL_AddIntegerItem ("node"); 
  AutoGL_AddIntegerItem ("element");
}
