/* advautomesh_extract_selection.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoRes_Table *DestObjects;
static AutoRes_IntegerColumn *DestObjects_srcObjectId;

static AutoRes_Selection *SrcSelection;
static AutoRes_Selection *DestSelection;

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int nEntrys;
  int iEntry;
  int flag;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_extract_selection \n"
       "        dest_selection_file  src_record_file \n"
       "        src_selection_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  DestSelection = AutoRes_Selection_New ();
  AutoRes_TableInOut_AssociateIntegerColumn (DestSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);



  DestObjects = AutoRes_Table_New ("DestObjects");

  DestObjects_srcObjectId = AutoRes_IntegerColumn_New 
    (DestObjects, "srcObjectId");
  flag = AutoRes_Table_addIntegerColumn 
    (DestObjects, DestObjects_srcObjectId);
  assert(flag);



  SrcSelection = AutoRes_Selection_New ();
  assert(SrcSelection != NULL);

  nEntrys = AutoRes_Selection_nEntrys (DestSelection);

  flag = AutoRes_Selection_reserve (SrcSelection, nEntrys);
  assert(flag);

  for (iEntry = 0; iEntry < nEntrys; iEntry++) {
    int destRecordId = AutoRes_Selection_objectId (DestSelection, iEntry);
    int srcRecordId = AUTORES_I(DestObjects_srcObjectId, destRecordId);

    AutoRes_Selection_addObject (SrcSelection, srcRecordId);
  }



  AutoRes_TableInOut_AssociateIntegerColumn (SrcSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);



  AutoRes_Selection_delete (SrcSelection);
  AutoRes_Selection_delete (DestSelection);

  AutoRes_Table_delete (DestObjects);
  DestObjects = NULL;
  DestObjects_srcObjectId = NULL;

  return 0;
}

