/* advautomesh_extract_value.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoRes_Table *SrcObjects;
static AutoRes_Real1DColumn *SrcObjects_values;

static AutoRes_Table *DestObjects;
static AutoRes_IntegerColumn *DestObjects_srcObjectId;
static AutoRes_Real1DColumn *DestObjects_values;

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int nComponents;
  int nDestObjects;
  int iDestObject;
  int iComponent;
  int flag;

  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_extract_value  nComponentsPerRecord \n"
       "        src_value_file  src_record_file \n"
       "        dest_value_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);

  nComponents = atoi (argv[1]);



  SrcObjects = AutoRes_Table_New ("SrcObjects");

  SrcObjects_values = AutoRes_Real1DColumn_New 
    (SrcObjects, "values", nComponents);
  flag = AutoRes_Table_addReal1DColumn 
    (SrcObjects, SrcObjects_values);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (SrcObjects_values);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);



  DestObjects = AutoRes_Table_New ("DestObjects");

  DestObjects_srcObjectId = AutoRes_IntegerColumn_New 
    (DestObjects, "srcObjectId");
  flag = AutoRes_Table_addIntegerColumn 
    (DestObjects, DestObjects_srcObjectId);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (DestObjects_srcObjectId);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);

  DestObjects_values = AutoRes_Real1DColumn_New 
    (DestObjects, "values", nComponents);
  flag = AutoRes_Table_addReal1DColumn 
    (DestObjects, DestObjects_values);
  assert(flag);



  nDestObjects = AutoRes_Table_nRecords (DestObjects);

  for (iDestObject = 0; iDestObject < nDestObjects; iDestObject++) {
    int srcObjectId = AUTORES_I(DestObjects_srcObjectId, iDestObject);

    for (iComponent = 0; iComponent < nComponents; iComponent++) {
      double value = AUTORES_R1D(SrcObjects_values, srcObjectId, iComponent);
    
      AUTORES_R1D_SET(DestObjects_values, iDestObject, iComponent, value);
    }
  }

  AutoRes_TableInOut_AssociateReal1DColumn (DestObjects_values);
  flag = AutoRes_TableInOut_Write (argv[4]);
  assert(flag);



  AutoRes_Table_delete (SrcObjects);
  SrcObjects = NULL;
  SrcObjects_values = NULL;

  AutoRes_Table_delete (DestObjects);
  DestObjects = NULL;
  DestObjects_srcObjectId = NULL;
  DestObjects_values = NULL;

  return 0;
}

