/* advautomesh_mid_value.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoRes_Table *SrcObjects;
static AutoRes_Real1DColumn *SrcObjects_values;

static AutoRes_Table *DestObjects;
static AutoRes_Integer1DColumn *DestObjects_srcObjectIds;
static AutoRes_Real1DColumn *DestObjects_values;

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int nComponents;
  int nDestObjects;
  int iDestObject;
  int iComponent;
  int flag;

  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_mid_value  nComponentsPerRecord \n"
       "        src_value_file  mid_src_record_file \n"
       "        dest_value_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);

  nComponents = atoi (argv[1]);



  SrcObjects = AutoRes_Table_New ("SrcObjects");

  SrcObjects_values = AutoRes_Real1DColumn_New 
    (SrcObjects, "values", nComponents);
  flag = AutoRes_Table_addReal1DColumn 
    (SrcObjects, SrcObjects_values);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (SrcObjects_values);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);



  DestObjects = AutoRes_Table_New ("DestObjects");

  DestObjects_srcObjectIds = AutoRes_Integer1DColumn_New 
    (DestObjects, "srcObjectIds", 2);
  flag = AutoRes_Table_addInteger1DColumn 
    (DestObjects, DestObjects_srcObjectIds);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (DestObjects_srcObjectIds);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);

  DestObjects_values = AutoRes_Real1DColumn_New 
    (DestObjects, "values", nComponents);
  flag = AutoRes_Table_addReal1DColumn 
    (DestObjects, DestObjects_values);
  assert(flag);



  nDestObjects = AutoRes_Table_nRecords (DestObjects);

  for (iDestObject = 0; iDestObject < nDestObjects; iDestObject++) {
    int srcObjectId0 = AUTORES_I1D
      (DestObjects_srcObjectIds, iDestObject, 0);
    int srcObjectId1 = AUTORES_I1D
      (DestObjects_srcObjectIds, iDestObject, 1);

    for (iComponent = 0; iComponent < nComponents; iComponent++) {
      if (srcObjectId1 == -1) {
	double value = AUTORES_R1D
	  (SrcObjects_values, srcObjectId0, iComponent);
	
	AUTORES_R1D_SET(DestObjects_values, iDestObject, iComponent, 
			value);
      } else {
	double value0 = AUTORES_R1D
	  (SrcObjects_values, srcObjectId0, iComponent);
	double value1 = AUTORES_R1D
	  (SrcObjects_values, srcObjectId1, iComponent);
	double value = (value0 + value1) / 2;
	
	AUTORES_R1D_SET(DestObjects_values, iDestObject, iComponent, 
			value);
      }
    }
  }

  AutoRes_TableInOut_AssociateReal1DColumn (DestObjects_values);
  flag = AutoRes_TableInOut_Write (argv[4]);
  assert(flag);



  AutoRes_Table_delete (SrcObjects);
  SrcObjects = NULL;
  SrcObjects_values = NULL;

  AutoRes_Table_delete (DestObjects);
  DestObjects = NULL;
  DestObjects_srcObjectIds = NULL;
  DestObjects_values = NULL;

  return 0;
}

