/* advautomesh_tetra3d_boundary.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_VolumeElement3D *Element;
static AutoMesh_VolumeFace3D *Face;

static AutoMesh_VolumeBoundaryMaker3D *Bm;
static AutoMesh_Node3D *DestNode;
static AutoMesh_SurfaceElement3D *DestElement;

static void PerformCommand (void)
{
  int nodeFlag;
  int elementFlag;
  int volumeNodeIdFlag;
  int volumeElementIdFlag;
  int flag;
  
  Bm = AutoMesh_VolumeBoundaryMaker3D_New (Face);
  assert(Bm != NULL);
  
  nodeFlag = 1;
  elementFlag = 1;
  volumeNodeIdFlag = 1;
  volumeElementIdFlag = 1;
  flag = AutoMesh_VolumeBoundaryMaker3D_do 
    (Bm, nodeFlag, elementFlag, volumeNodeIdFlag, volumeElementIdFlag);
  assert(flag);

  DestNode = Bm->surfaceNode;
  DestElement = Bm->surfaceElement;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 8) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_boundary  node_file  element_file \n"
       "        face_file  \n"
       "        node_file_OUT  element_file_OUT  \n"
       "        src_node_file_OUT  \n"
       "        boundary_src_element_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement3D_New 
    (Node, AUTOMESH_POLYHEDRON_TETRA, 4);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Face = AutoMesh_VolumeFace3D_New (Element);
  assert(Element != NULL);

  flag = AutoMesh_VolumeFace3D_makeElementIdsColumn (Face);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Face->faces_elementIds);
  AutoRes_TableInOut_AssociateInteger1DColumn (Face->faces_elementFaceNums);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);



  PerformCommand ();

  AutoRes_TableInOut_AssociateReal1DColumn (DestNode->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[4]);
  assert(flag);
  
  AutoRes_TableInOut_AssociateInteger1DColumn (DestElement->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Bm->surfaceNodes_volumeNodeId);
  flag = AutoRes_TableInOut_Write (argv[6]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn 
    (Bm->surfaceElements_volumeElementId);
  AutoRes_TableInOut_AssociateIntegerColumn 
    (Bm->surfaceElements_volumeElementFaceId);
  flag = AutoRes_TableInOut_Write (argv[7]);
  assert(flag);



  AutoMesh_VolumeBoundaryMaker3D_delete (Bm);
  AutoMesh_SurfaceElement3D_delete (DestElement);
  AutoMesh_Node3D_delete (DestNode);

  AutoMesh_VolumeFace3D_delete (Face);
  AutoMesh_VolumeElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

