/* advautomesh_tetra3d_generate_mid_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_VolumeElement3D *Element;
static AutoMesh_VolumeEdge3D *Edge;

static AutoMesh_VolumeMidNodeGenerator3D *Mng;
static AutoMesh_Node3D *DestNode;
static AutoMesh_VolumeElement3D *DestElement;

static void PerformCommand (void)
{
  int nodeFlag;
  int elementFlag;
  int srcNodeIdsFlag;
  int destNodeIdFlag;
  int flag;
  
  Mng = AutoMesh_VolumeMidNodeGenerator3D_New (Edge);
  assert(Mng != NULL);
  
  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdsFlag = 1;
  destNodeIdFlag = 0;
  flag = AutoMesh_VolumeMidNodeGenerator3D_do 
    (Mng, 
     nodeFlag, elementFlag,
     srcNodeIdsFlag, destNodeIdFlag);
  assert(flag);

  DestNode = Mng->destNode;
  DestElement = Mng->destElement;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 8) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_generate_mid_node  node_file  element_file \n"
       "        edge_vertex_file  element_edge_file  \n"
       "        node_file_OUT  element10_file_OUT  \n"
       "        mid_node_generator_src_node_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement3D_New 
    (Node, AUTOMESH_POLYHEDRON_TETRA, 4);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Edge = AutoMesh_VolumeEdge3D_New (Element);
  assert(Element != NULL);

  flag = AutoMesh_VolumeEdge3D_makeVertexNodeIdsColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_vertexNodeIds);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);
  
  flag = AutoMesh_VolumeEdge3D_makeEdgeIdsAtElementColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->elements_edgeIds);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);



  PerformCommand ();

  AutoRes_TableInOut_AssociateReal1DColumn (DestNode->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);
  
  AutoRes_TableInOut_AssociateInteger1DColumn (DestElement->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[6]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Mng->destNodes_srcNodeIds);
  flag = AutoRes_TableInOut_Write (argv[7]);
  assert(flag);



  AutoMesh_VolumeMidNodeGenerator3D_delete (Mng);
  AutoMesh_VolumeElement3D_delete (DestElement);
  AutoMesh_Node3D_delete (DestNode);

  AutoMesh_VolumeEdge3D_delete (Edge);
  AutoMesh_VolumeElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

