/* advautomesh_tetra3d_remove_mid_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_VolumeElement3D *Element;

static AutoMesh_VolumeMidNodeRemover3D *Mnr;
static AutoMesh_Node3D *DestNode;
static AutoMesh_VolumeElement3D *DestElement;

static void PerformCommand (void)
{
  int nodeFlag;
  int elementFlag;
  int srcNodeIdsFlag;
  int flag;
  
  Mnr = AutoMesh_VolumeMidNodeRemover3D_New (Element);
  assert(Mnr != NULL);
  
  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdsFlag = 1;
  flag = AutoMesh_VolumeMidNodeRemover3D_do 
    (Mnr, 
     nodeFlag, elementFlag,
     srcNodeIdsFlag);
  assert(flag);

  DestNode = Mnr->destNode;
  DestElement = Mnr->destElement;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_remove_mid_node  node_file  element10_file \n"
       "        node_file_OUT  element_file_OUT  \n"
       "        src_node_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement3D_New 
    (Node, AUTOMESH_POLYHEDRON_TETRA, 10);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  PerformCommand ();

  AutoRes_TableInOut_AssociateReal1DColumn (DestNode->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);
  
  AutoRes_TableInOut_AssociateInteger1DColumn (DestElement->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[4]);

  AutoRes_TableInOut_AssociateIntegerColumn (Mnr->destNodes_srcNodeId);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);



  AutoMesh_VolumeMidNodeRemover3D_delete (Mnr);
  AutoMesh_VolumeElement3D_delete (DestElement);
  AutoMesh_Node3D_delete (DestNode);

  AutoMesh_VolumeElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

