/* advautomesh_tetra3d_select_boundary.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoRes_Table *Faces;  
static AutoRes_IntegerColumn *Faces_elementId;  
static AutoRes_IntegerColumn *Faces_elementFaceNum;  

static AutoRes_Selection *FaceSelection;

static AutoRes_Table *SelectedFaces;  
static AutoRes_IntegerColumn *SelectedFaces_elementId;  
static AutoRes_IntegerColumn *SelectedFaces_elementFaceNum;  



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int nEntrys;
  int iEntry;
  int count;
  int flag;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_select_boundary \n"
       "        boundary_face_file  \n"
       "        face_selection_file  \n"
       "        element_selection_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Faces = AutoRes_Table_New ("Faces");
  assert(Faces != NULL);
  
  Faces_elementId = AutoRes_IntegerColumn_New 
    (Faces, "elementId");
  assert(Faces_elementId != NULL);
  flag = AutoRes_Table_addIntegerColumn 
    (Faces, Faces_elementId);
  assert(flag);
  
  Faces_elementFaceNum = AutoRes_IntegerColumn_New 
    (Faces, "elementFaceNum");
  assert(Faces_elementFaceNum != NULL);
  flag = AutoRes_Table_addIntegerColumn 
    (Faces, Faces_elementFaceNum);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Faces_elementId);
  AutoRes_TableInOut_AssociateIntegerColumn (Faces_elementFaceNum);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);



  FaceSelection = AutoRes_Selection_New ();
  AutoRes_TableInOut_AssociateIntegerColumn 
    (FaceSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);

  nEntrys = AutoRes_Selection_nEntrys (FaceSelection);



  SelectedFaces = AutoRes_Table_New ("SelectedFaces");
  assert(SelectedFaces != NULL);
  
  SelectedFaces_elementId = AutoRes_IntegerColumn_New 
    (SelectedFaces, "elementId");
  assert(SelectedFaces_elementId != NULL);
  flag = AutoRes_Table_addIntegerColumn 
    (SelectedFaces, SelectedFaces_elementId);
  assert(flag);
  
  SelectedFaces_elementFaceNum = AutoRes_IntegerColumn_New 
    (SelectedFaces, "elementFaceNum");
  assert(SelectedFaces_elementFaceNum != NULL);
  flag = AutoRes_Table_addIntegerColumn 
    (SelectedFaces, SelectedFaces_elementFaceNum);
  assert(flag);

  flag = AutoRes_Table_allocateRecords (SelectedFaces, nEntrys);
  assert(flag);



  count = 0;
  for (iEntry = 0; iEntry < nEntrys; iEntry++) {
    int faceId = AutoRes_Selection_objectId 
      (FaceSelection, iEntry);

    int elementId = AUTORES_I(Faces_elementId, faceId);
    int elementFaceNum = AUTORES_I(Faces_elementFaceNum, faceId);

    AUTORES_I_SET(SelectedFaces_elementId, count, elementId);
    AUTORES_I_SET(SelectedFaces_elementFaceNum, count, elementFaceNum);
    count++;
  }
  


  AutoRes_TableInOut_AssociateIntegerColumn (SelectedFaces_elementId);
  AutoRes_TableInOut_AssociateIntegerColumn (SelectedFaces_elementFaceNum);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);



  AutoRes_Table_delete (Faces);
  AutoRes_Selection_delete (FaceSelection);
  AutoRes_Table_delete (SelectedFaces);

  return 0;
}

