/* advautomesh_tri2d_edge.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



#ifdef TIME_MEASURE
#include "time_measure.c"
#endif



/************************
	 Command
*************************/

static AutoMesh_Node2D *Node;
static AutoMesh_VolumeElement2D *Element;

static AutoMesh_VolumeEdge2D *Edge;

static void PerformCommand (void)
{
  int elementIdsFlag;
  int flag;

  Edge = AutoMesh_VolumeEdge2D_New (Element);
  assert(Edge != NULL);

  elementIdsFlag = 1;
  flag = AutoMesh_VolumeEdge2D_extract (Edge, elementIdsFlag);
  assert(flag);

  flag = AutoMesh_VolumeEdge2D_collectNeighbor (Edge);
  assert(flag);
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri2d_edge  node_file  element_file \n"
       "        edge_file_OUT  \n"
       "        element_edge_file_OUT  \n"
       "        element_neighbor_file_OUT  \n"
       "        edge_vertex_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node2D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node2D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement2D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);

  

#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  PerformCommand ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_elementIds);
  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_elementEdgeNums);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->elements_edgeIds);
  flag = AutoRes_TableInOut_Write (argv[4]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_neighborIds);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_vertexNodeIds);
  flag = AutoRes_TableInOut_Write (argv[6]);
  assert(flag);
  


  AutoMesh_VolumeEdge2D_delete (Edge);

  AutoMesh_VolumeElement2D_delete (Element);
  AutoMesh_Node2D_delete (Node);

  return 0;
}

