/* advautomesh_tri2d_extract.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node2D *Node;
static AutoMesh_VolumeElement2D *Element;
static AutoRes_Selection *ElementSelection;

static AutoMesh_VolumeExtractor2D *Extractor;
static AutoMesh_Node2D *DestNode;
static AutoMesh_VolumeElement2D *DestElement;

static void PerformCommand (void)
{
  int nodeFlag;
  int elementFlag;
  int srcElementIdFlag;
  int destElementIdFlag;
  int srcNodeIdFlag;
  int flag;
  
  Extractor = AutoMesh_VolumeExtractor2D_New (Element, ElementSelection);
  assert(Extractor != NULL);
  
  nodeFlag = 1;
  elementFlag = 1;
  srcElementIdFlag = 0;
  destElementIdFlag = 0;
  srcNodeIdFlag = 1;
  flag = AutoMesh_VolumeExtractor2D_do 
    (Extractor, 
     nodeFlag, elementFlag,
     srcElementIdFlag, destElementIdFlag,
     srcNodeIdFlag);
  assert(flag);

  DestNode = Extractor->destNode;
  DestElement = Extractor->destElement;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri2d_extract  node_file  element_file \n"
       "        element_selection_file  \n"
       "        node_file_OUT  element_file_OUT  \n"
       "        src_node_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node2D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node2D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement2D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  ElementSelection = AutoRes_Selection_New ();
  AutoRes_TableInOut_AssociateIntegerColumn (ElementSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);



  PerformCommand ();

  AutoRes_TableInOut_AssociateReal1DColumn (DestNode->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[4]);
  assert(flag);
  
  AutoRes_TableInOut_AssociateInteger1DColumn (DestElement->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Extractor->destNodes_srcNodeId);
  flag = AutoRes_TableInOut_Write (argv[6]);
  assert(flag);



  AutoMesh_VolumeExtractor2D_delete (Extractor);
  AutoMesh_VolumeElement2D_delete (DestElement);
  AutoMesh_Node2D_delete (DestNode);

  AutoRes_Selection_delete (ElementSelection);
  AutoMesh_VolumeElement2D_delete (Element);
  AutoMesh_Node2D_delete (Node);

  return 0;
}

