/* advautomesh_tri3d_divide.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_SurfaceEdge3D *Edge;

static AutoMesh_SurfaceDivider3D *Divider;
static AutoMesh_Node3D *DestNode;
static AutoMesh_SurfaceElement3D *DestElement;

static void PerformCommand (void)
{
  int srcNodeIdsFlag;
  int srcElementIdFlag;
  int destNodeIdFlag;
  int destElementIdsFlag;
  int flag;
  
  Divider = AutoMesh_SurfaceDivider3D_New (Edge);
  assert(Divider != NULL);
  
  srcNodeIdsFlag = 1;
  srcElementIdFlag = 1;
  destNodeIdFlag = 0;
  destElementIdsFlag = 0;
  flag = AutoMesh_SurfaceDivider3D_do 
    (Divider, 
     srcNodeIdsFlag, srcElementIdFlag,
     destNodeIdFlag, destElementIdsFlag);
  assert(flag);

  DestNode = Divider->destNode;
  DestElement = Divider->destElement;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 9) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_divide  node_file  element_file \n"
       "        edge_vertex_file  element_edge_file  \n"
       "        node_file_OUT  element_file_OUT  \n"
       "        divider_src_node_file_OUT  \n"
       "        divider_src_element_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Edge = AutoMesh_SurfaceEdge3D_New (Element);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceEdge3D_makeVertexNodeIdsColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_vertexNodeIds);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);
  
  flag = AutoMesh_SurfaceEdge3D_makeEdgeIdsAtElementColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->elements_edgeIds);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);



  PerformCommand ();

  AutoRes_TableInOut_AssociateReal1DColumn (DestNode->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);
  
  AutoRes_TableInOut_AssociateInteger1DColumn (DestElement->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[6]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Divider->destNodes_srcNodeIds);
  flag = AutoRes_TableInOut_Write (argv[7]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn 
    (Divider->destElements_srcElementId);
  AutoRes_TableInOut_AssociateIntegerColumn 
    (Divider->destElements_srcElementPartId);
  flag = AutoRes_TableInOut_Write (argv[8]);
  assert(flag);



  AutoMesh_SurfaceDivider3D_delete (Divider);
  AutoMesh_SurfaceElement3D_delete (DestElement);
  AutoMesh_Node3D_delete (DestNode);

  AutoMesh_SurfaceEdge3D_delete (Edge);
  AutoMesh_SurfaceElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

