/* advautomesh_tri3d_group.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



#ifdef TIME_MEASURE
#include "time_measure.c"
#endif



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_SurfaceEdge3D *Edge;

static AutoMesh_SurfaceGrouper3D *Grouper;
static AutoMesh_Frame3D *Frame;

/* degree 0-90 */
static double Angle;

static void PerformCommand (void)
{
  int groupIdFlag;
  int frameFlag;
  int frameGroupIdFlag;
  int frameVertexIdFlag;
  int flag;
  
  Grouper = AutoMesh_SurfaceGrouper3D_New (Edge);
  assert(Grouper != NULL);
  
  groupIdFlag = 1;
  frameFlag = 1;
  frameGroupIdFlag = 1;
  frameVertexIdFlag = 1;
  flag = AutoMesh_SurfaceGrouper3D_do 
    (Grouper, 
     Angle,
     groupIdFlag, frameFlag, frameGroupIdFlag, frameVertexIdFlag);
  assert(flag);

  Frame = Grouper->frame;
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 13) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_group  node_file  element_file \n"
       "        edge_file  element_edge_file  \n"
       "        element_neighbor_file  edge_vertex_file  \n"
       "        angle  \n"
       "        element_group_file_OUT  \n"
       "        edge_frame_file_OUT  \n"
       "        frame_file_OUT  \n"
       "        edge_frame_group_file_OUT  \n"
       "        node_frame_vertex_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Edge = AutoMesh_SurfaceEdge3D_New (Element);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceEdge3D_makeElementIdsColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_elementIds);
  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_elementEdgeNums);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);
  
  flag = AutoMesh_SurfaceEdge3D_makeEdgeIdsAtElementColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->elements_edgeIds);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);
  
  flag = AutoMesh_SurfaceElement3D_makeNeighborIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_neighborIds);
  flag = AutoRes_TableInOut_Read (argv[5]);
  assert(flag);

  flag = AutoMesh_SurfaceEdge3D_makeVertexNodeIdsColumn (Edge);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Edge->edges_vertexNodeIds);
  flag = AutoRes_TableInOut_Read (argv[6]);
  assert(flag);

  Angle = atof (argv[7]);



#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  PerformCommand ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->elements_groupId);
  flag = AutoRes_TableInOut_Write (argv[8]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->edges_frameFlag);
  flag = AutoRes_TableInOut_Write (argv[9]);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Frame->frames_endPoint0);
  AutoRes_TableInOut_AssociateReal1DColumn (Frame->frames_endPoint1);
  flag = AutoRes_TableInOut_Write (argv[10]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->edges_frameGroupId);
  flag = AutoRes_TableInOut_Write (argv[11]);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->nodes_frameVertexId);
  flag = AutoRes_TableInOut_Write (argv[12]);
  assert(flag);



  AutoMesh_SurfaceGrouper3D_delete (Grouper);
  AutoMesh_Frame3D_delete (Frame);

  AutoMesh_SurfaceEdge3D_delete (Edge);
  AutoMesh_SurfaceElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

