/* advautomesh_tri3d_group_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



/************************
	 Command
*************************/

static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;
static AutoMesh_SurfaceGrouper3D *Grouper;
static AutoRes_Selection *GroupSelection;
static AutoRes_Selection *NodeSelection;

static void PerformCommand (void)
{
  int flag;
  
  flag = AutoMesh_SurfaceGrouper3D_makeMarkFlagAtNodeColumn (Grouper);
  assert(flag);

  NodeSelection = AutoMesh_SurfaceGrouper3D_getGroupNode 
    (Grouper, GroupSelection);
  assert(NodeSelection != NULL);
}



int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;

  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_group_node  node_file  element_file \n"
       "        element_group_file  group_selection_file  \n"
       "        node_selection_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Grouper = AutoMesh_SurfaceGrouper3D_NewWithoutEdge (Element);
  assert(Grouper != NULL);

  flag = AutoMesh_SurfaceGrouper3D_makeGroupIdAtElementColumn (Grouper);
  assert(flag);

  AutoRes_TableInOut_AssociateIntegerColumn (Grouper->elements_groupId);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);



  GroupSelection = AutoRes_Selection_New ();
  AutoRes_TableInOut_AssociateIntegerColumn (GroupSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Read (argv[4]);
  assert(flag);



  PerformCommand ();

  AutoRes_TableInOut_AssociateIntegerColumn (NodeSelection->entrys_objectId);
  flag = AutoRes_TableInOut_Write (argv[5]);
  assert(flag);



  AutoRes_Selection_delete (NodeSelection);
  AutoRes_Selection_delete (GroupSelection);
  AutoMesh_SurfaceGrouper3D_delete (Grouper);
  AutoMesh_SurfaceElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}

