/* advautomesh_tetra3d_face_stageB.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 12000000

const int FaceNodeMap[N_ELEMENT_FACES][3] = {
  { 1, 2, 3 },
  { 0, 3, 2 },
  { 0, 1, 3 },
  { 0, 2, 1 }
};

int NElements;
int Element_faceIds[MAX_ELEMENTS][N_ELEMENT_FACES];



/* Face */

#define N_FACE_NODES 3
#define MAX_FACES 30000000

int NFaces;
int Face_elementIds[MAX_FACES][2];
int Face_elementFaceIds[MAX_FACES][2];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  /* skip connectivity */
  
  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  
  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int nodeId0, nodeId1, nodeId2;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;

    fscanf (fp, "%d %d %d %d %d %d %d %d", 
	    &faceId,
	    &nodeId0, &nodeId1, &nodeId2,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);
    
    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    Face_elementIds[iFace][0] = elementId0;

    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);
    Face_elementFaceIds[iFace][0] = elementFaceId0;

    assert(0 <= elementId1 || elementId1 == -1);
    assert(elementId1 < NElements);
    Face_elementIds[iFace][1] = elementId1;

    assert(0 <= elementFaceId1 || elementFaceId1 == -1);
    assert(elementFaceId1 < N_ELEMENT_FACES);
    Face_elementFaceIds[iFace][1] = elementFaceId1;
  }

  fclose (fp);
}

void WriteElementFaceFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementFace;
  
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement);

    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      fprintf (fp, "%d ", 
	       Element_faceIds[iElement][iElementFace]);
    }

    fprintf (fp, "\n");
  }

  fclose (fp);
}

void WriteElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementFace;
  
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement);

    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceId = Element_faceIds[iElement][iElementFace];
      int neighborId;
      
      if (Face_elementIds[faceId][0] == iElement) {
	neighborId = Face_elementIds[faceId][1];
      } else {
	assert(Face_elementIds[faceId][1] == iElement);
	neighborId = Face_elementIds[faceId][0];
      }

      fprintf (fp, "%d ", 
	       neighborId);
    }

    fprintf (fp, "\n");
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iFace;
  int iElement;
  int iElementFace;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      /* dummy  parameter -2 */
      Element_faceIds[iElement][iElementFace] = -2;
    }
  }
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int elementId0 = Face_elementIds[iFace][0];
    int elementFaceId0 = Face_elementFaceIds[iFace][0];
    int elementId1 = Face_elementIds[iFace][1];
    int elementFaceId1 = Face_elementFaceIds[iFace][1];
    
    Element_faceIds[elementId0][elementFaceId0] = iFace;
    if (elementId1 == -1) {
      Element_faceIds[elementId1][elementFaceId1] = -1;
    } else {
      Element_faceIds[elementId1][elementFaceId1] = iFace;
    }
  }

  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceId = Element_faceIds[iElement][iElementFace];

      assert(0 <= faceId || faceId == -1);
      assert(faceId < NFaces);
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_face_stageB  node_file  element_file \n"
       "        face_file \n"
       "        element_face_file_OUT  element_neighbor_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadFaceFile (argv[3]);

  PerformCommand ();

  WriteElementFaceFile (argv[4]);
  WriteElementNeighborFile (argv[5]);

  return 0;
}
