/* advautomesh_tetra3d_section_contour_view4.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>

#include <GL/gl.h>
#include <GL/glu.h>



#if 1
#include "../../util/time_measure.c"
#define TIME_MEASURE
#endif



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}

void GetNormalizedVector 
(double x, double y, double z,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  double length = sqrt (x * x + y * y + z * z);

  *nx_OUT = x / length;
  *ny_OUT = y / length;
  *nz_OUT = z / length;
}

void GetVectorProduct 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  *x_OUT = y0 * z1 - z0 * y1;
  *y_OUT = z0 * x1 - x0 * z1;
  *z_OUT = x0 * y1 - y0 * x1;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];
double Node_sectionValue[MAX_NODES];
double Node_scalar[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_EDGES 6
#define MAX_ELEMENTS 12000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 0, 1 }, 
  { 0, 2 }, 
  { 0, 3 }, 
  { 1, 2 }, 
  { 2, 3 }, 
  { 3, 1 }  
};

const int NSectionEdges[8] = {
  0, 3, 3, 4, 3, 4, 4, 3
};

const int SectionEdgeIds[8][4] = {
  { -1, -1, -1, -1 },
  { 0, 1, 2, -1 },
  { 0, 3, 5, -1 },
  { 3, 1, 2, 5 },
  { 3, 1, 4, -1 },
  { 0, 3, 4, 2 },
  { 0, 5, 4, 1 },
  { 2, 5, 4, -1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Element_getEdgePoint 
(int elementId, 
 int elementEdgeId, double ratio,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  int nodeId0, nodeId1;
  double x0, y0, z0;
  double x1, y1, z1;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);

  nodeId0 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][0]];
  nodeId1 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][1]];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];
  z0 = Node_coord[nodeId0][2];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];
  z1 = Node_coord[nodeId1][2];
  
  *x_OUT = x0 + (x1 - x0) * ratio;
  *y_OUT = y0 + (y1 - y0) * ratio;
  *z_OUT = z0 + (z1 - z0) * ratio;
}

double Element_edgeScalar 
(int elementId, 
 int elementEdgeId, double ratio)
{
  int nodeId0, nodeId1;
  double scalar0, scalar1;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);

  nodeId0 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][0]];
  nodeId1 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][1]];

  scalar0 = Node_scalar[nodeId0];
  scalar1 = Node_scalar[nodeId1];
  
  return scalar0 + (scalar1 - scalar0) * ratio;
}

void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeScalarFile (const char *baseFileName, int stepId)
{
  char fileName[256];
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  sprintf (fileName, "%s.%06d", 
	   baseFileName,
	   (stepId + 1) * 20);

  fprintf (stderr, " read %s \n",
	   fileName);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double scalar;

    fscanf (fp, "%d %lf", 
	    &nodeId,
	    &scalar);
    assert(nodeId == iNode);
    Node_scalar[iNode] = scalar;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Section */

#define MAX_SECTIONS 1000000

int NSections;
int Section_elementId[MAX_SECTIONS];
int Section_elementEdgeIds[MAX_SECTIONS][3];
double Section_ratios[MAX_SECTIONS][3];

double PlaneA, PlaneB, PlaneC = 1.0, PlaneD;

float Section_vertexs[MAX_SECTIONS][3][3];
float Section_normal[MAX_SECTIONS][3];
float Section_colors[MAX_SECTIONS][3][3];



/* Wireframe */

#define MAX_WIREFRAMES 1000000

int NWireframes;
double Wireframe_coord0[MAX_WIREFRAMES][3];
double Wireframe_coord1[MAX_WIREFRAMES][3];

float Wireframe_vertexs[MAX_WIREFRAMES][2][3];



void SectionGroup_clear (void)
{
  NSections = 0;
}

void SectionGroup_addSection 
(int elementId, 
 int elementEdgeId0, double ratio0,
 int elementEdgeId1, double ratio1,
 int elementEdgeId2, double ratio2)
{
  assert(NSections < MAX_SECTIONS);
  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId0);
  assert(elementEdgeId0 < N_ELEMENT_EDGES);
  assert(0 <= elementEdgeId1);
  assert(elementEdgeId1 < N_ELEMENT_EDGES);
  assert(0 <= elementEdgeId2);
  assert(elementEdgeId2 < N_ELEMENT_EDGES);

  Section_elementId[NSections] = elementId;
  Section_elementEdgeIds[NSections][0] = elementEdgeId0;
  Section_elementEdgeIds[NSections][1] = elementEdgeId1;
  Section_elementEdgeIds[NSections][2] = elementEdgeId2;
  Section_ratios[NSections][0] = ratio0;
  Section_ratios[NSections][1] = ratio1;
  Section_ratios[NSections][2] = ratio2;

  NSections++;
}

void ReadWireframeFile (const char *fileName)
{
  FILE *fp;
  int iWireframe;
  
  assert(NWireframes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NWireframes);
  assert(0 <= NWireframes);
  assert(NWireframes <= MAX_WIREFRAMES);
  
  for (iWireframe = 0; iWireframe < NWireframes; iWireframe++) {
    int wireframeId;
    double x0, y0, z0;
    double x1, y1, z1;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	    &wireframeId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1);
    assert(wireframeId == iWireframe);
    Wireframe_coord0[iWireframe][0] = x0;
    Wireframe_coord0[iWireframe][1] = y0;
    Wireframe_coord0[iWireframe][2] = z0;

    Wireframe_coord1[iWireframe][0] = x1;
    Wireframe_coord1[iWireframe][1] = y1;
    Wireframe_coord1[iWireframe][2] = z1;
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void SetNodeSectionValue (void)
{
  int iNode;
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    Node_sectionValue[iNode] 
      = PlaneA * x + PlaneB * y + PlaneC * z + PlaneD;
  }
}

int SectionMask (int elementId, double offset)
{
  int nodeId0 = Element_nodeIds[elementId][0];
  int nodeId1 = Element_nodeIds[elementId][1];
  int nodeId2 = Element_nodeIds[elementId][2];
  int nodeId3 = Element_nodeIds[elementId][3];
  unsigned int mask = 0;
  
  if (offset < Node_sectionValue[nodeId0]) mask |= 1;
  if (offset < Node_sectionValue[nodeId1]) mask |= 2;
  if (offset < Node_sectionValue[nodeId2]) mask |= 4;
  if (offset < Node_sectionValue[nodeId3]) mask |= 8;
  if ((mask & 8) != 0) mask = (~mask) & 15;

  assert(0 <= mask);
  assert(mask < 8);

  return mask;
}

int GetSection 
(int elementId, double offset,
 int *nEdges_OUT,
 int elementEdgeIds_OUT[4], double ratios_OUT[4])
{
  unsigned int mask = SectionMask (elementId, offset);
  int iEdge;

  *nEdges_OUT = NSectionEdges[mask];
  if (*nEdges_OUT == 0) return 0;
  
  assert(*nEdges_OUT == 3 || *nEdges_OUT == 4);

  for (iEdge = 0; iEdge < *nEdges_OUT; iEdge++) {
    int elementEdgeId = SectionEdgeIds[mask][iEdge];

    int elementNodeId0 = EdgeNodeMap[elementEdgeId][0];
    int elementNodeId1 = EdgeNodeMap[elementEdgeId][1];

    int nodeId0 = Element_nodeIds[elementId][elementNodeId0];
    int nodeId1 = Element_nodeIds[elementId][elementNodeId1];

    double ratio = (offset - Node_sectionValue[nodeId0]) 
      / (Node_sectionValue[nodeId1] - Node_sectionValue[nodeId0]);
    
    elementEdgeIds_OUT[iEdge] = elementEdgeId; 
    ratios_OUT[iEdge] = ratio;
  }    

  return 1;
}

void CheckElementSection (int elementId, double offset)
{
  int nEdges;
  int elementEdgeIds[4];
  double ratios[4];
  
  if (GetSection
      (elementId, offset,
       &nEdges, elementEdgeIds, ratios)) {
    assert(nEdges == 3 || nEdges == 4);
    SectionGroup_addSection 
      (elementId, 
       elementEdgeIds[0], ratios[0],
       elementEdgeIds[1], ratios[1],
       elementEdgeIds[2], ratios[2]);
    if (nEdges == 4) {
      SectionGroup_addSection 
	(elementId, 
	 elementEdgeIds[2], ratios[2],
	 elementEdgeIds[3], ratios[3],
	 elementEdgeIds[0], ratios[0]);
    }
  }
}

void MakeSection (double offset)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    CheckElementSection (iElement, offset);
  }
}



/********************************
	 Visualization
********************************/

double MinRange = 0.0;
double MaxRange = 1.0;

double Offset = 0.0;
double Shift = 1.0;

int Edge = 0;

char NodeScalarBaseFileName[256];
#define MAX_STEP 100
int StepId = 0;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void SetScalarRange (void)
{
  int iNode;

  assert(0 < NNodes);

  MinRange = MaxRange = Node_scalar[0];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double scalar = Node_scalar[iNode];
    
    if (scalar < MinRange) MinRange = scalar;
    if (MaxRange < scalar) MaxRange = scalar;
  }
}

void PrepareSectionVertexAndNormal (void)
{
  int iSection;

  for (iSection = 0; iSection < NSections; iSection++) {
    double ex0, ey0, ez0;
    double ex1, ey1, ez1;
    double ex2, ey2, ez2;
    double nx, ny, nz;

    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][0],
       Section_ratios[iSection][0],
       &ex0, &ey0, &ez0);

    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][1],
       Section_ratios[iSection][1],
       &ex1, &ey1, &ez1);

    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][2],
       Section_ratios[iSection][2],
       &ex2, &ey2, &ez2);

    Section_vertexs[iSection][0][0] = ex0;
    Section_vertexs[iSection][0][1] = ey0;
    Section_vertexs[iSection][0][2] = ez0;
    
    Section_vertexs[iSection][1][0] = ex1;
    Section_vertexs[iSection][1][1] = ey1;
    Section_vertexs[iSection][1][2] = ez1;
    
    Section_vertexs[iSection][2][0] = ex2;
    Section_vertexs[iSection][2][1] = ey2;
    Section_vertexs[iSection][2][2] = ez2;

    GetVectorProduct (ex0 - ex1, ey0 - ey1, ez0 - ez1,
		      ex0 - ex2, ey0 - ey2, ez0 - ez2,
		      &nx, &ny, &nz);
    GetNormalizedVector (nx, ny, nz,
			 &nx, &ny, &nz);
    Section_normal[iSection][0] = nx;
    Section_normal[iSection][1] = ny;
    Section_normal[iSection][2] = nz;
  }
}

void PrepareSectionColor ()
{
  int iSection;
  int iSectionVertex;

  for (iSection = 0; iSection < NSections; iSection++) {
    for (iSectionVertex = 0; 
	 iSectionVertex < 3; 
	 iSectionVertex++) {
      double scalar = Element_edgeScalar 
	(Section_elementId[iSection], 
	 Section_elementEdgeIds[iSection][iSectionVertex],
	 Section_ratios[iSection][iSectionVertex]);
      double grade = (scalar - MinRange) / (MaxRange - MinRange);
      double red, green, blue;
      
      AutoGL_GetContourColor 
	(&red, &green, &blue,
	 grade);

      Section_colors[iSection][iSectionVertex][0] = red;
      Section_colors[iSection][iSectionVertex][1] = green;
      Section_colors[iSection][iSectionVertex][2] = blue;
    }
  }
}

void PrepareWireframe (void)
{
  int iWireframe;

  for (iWireframe = 0; iWireframe < NWireframes; iWireframe++) {
    double nodeX0 = Wireframe_coord0[iWireframe][0];
    double nodeY0 = Wireframe_coord0[iWireframe][1];
    double nodeZ0 = Wireframe_coord0[iWireframe][2];
    
    double nodeX1 = Wireframe_coord1[iWireframe][0];
    double nodeY1 = Wireframe_coord1[iWireframe][1];
    double nodeZ1 = Wireframe_coord1[iWireframe][2];
    
    Wireframe_vertexs[iWireframe][0][0] = nodeX0;
    Wireframe_vertexs[iWireframe][0][1] = nodeY0;
    Wireframe_vertexs[iWireframe][0][2] = nodeZ0;

    Wireframe_vertexs[iWireframe][1][0] = nodeX1;
    Wireframe_vertexs[iWireframe][1][1] = nodeY1;
    Wireframe_vertexs[iWireframe][1][2] = nodeZ1;
  }
}

void DrawWireframe (void)
{
  int iWireframe;

  AutoGL_BeginNativeCall ();

  glDisable (GL_LIGHTING);
  
  glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
  
  glBegin (GL_LINES);
  for (iWireframe = 0; iWireframe < NWireframes; iWireframe++) {
    glVertex3f (Wireframe_vertexs[iWireframe][0][0], 
		Wireframe_vertexs[iWireframe][0][1], 
		Wireframe_vertexs[iWireframe][0][2]);
    glVertex3f (Wireframe_vertexs[iWireframe][1][0], 
		Wireframe_vertexs[iWireframe][1][1], 
		Wireframe_vertexs[iWireframe][1][2]);
  }
  glEnd ();

  AutoGL_EndNativeCall ();
}

void RedrawView (void) 
{
  int iSection;

#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  DrawWireframe ();

  AutoGL_BeginNativeCall ();

  glEnable (GL_LIGHTING);

  glBegin (GL_TRIANGLES);
  for (iSection = 0; iSection < NSections; iSection++) {
    glNormal3f (Section_normal[iSection][0], 
		Section_normal[iSection][1], 
		Section_normal[iSection][2]);

    glColor3f (Section_colors[iSection][0][0], 
	       Section_colors[iSection][0][1], 
	       Section_colors[iSection][0][2]);
    glVertex3f (Section_vertexs[iSection][0][0], 
		Section_vertexs[iSection][0][1], 
		Section_vertexs[iSection][0][2]);
    
    glColor3f (Section_colors[iSection][1][0], 
	       Section_colors[iSection][1][1], 
	       Section_colors[iSection][1][2]);
    glVertex3f (Section_vertexs[iSection][1][0], 
		Section_vertexs[iSection][1][1], 
		Section_vertexs[iSection][1][2]);
    
    glColor3f (Section_colors[iSection][2][0], 
	       Section_colors[iSection][2][1], 
	       Section_colors[iSection][2][2]);
    glVertex3f (Section_vertexs[iSection][2][0], 
		Section_vertexs[iSection][2][1], 
		Section_vertexs[iSection][2][2]);

    glNormal3f (Section_normal[iSection][0] * -1.0f, 
		Section_normal[iSection][1] * -1.0f, 
		Section_normal[iSection][2] * -1.0f);

    glColor3f (Section_colors[iSection][2][0], 
	       Section_colors[iSection][2][1], 
	       Section_colors[iSection][2][2]);
    glVertex3f (Section_vertexs[iSection][2][0], 
		Section_vertexs[iSection][2][1], 
		Section_vertexs[iSection][2][2]);

    glColor3f (Section_colors[iSection][1][0], 
	       Section_colors[iSection][1][1], 
	       Section_colors[iSection][1][2]);
    glVertex3f (Section_vertexs[iSection][1][0], 
		Section_vertexs[iSection][1][1], 
		Section_vertexs[iSection][1][2]);
    
    glColor3f (Section_colors[iSection][0][0], 
	       Section_colors[iSection][0][1], 
	       Section_colors[iSection][0][2]);
    glVertex3f (Section_vertexs[iSection][0][0], 
		Section_vertexs[iSection][0][1], 
		Section_vertexs[iSection][0][2]);
  }
  glEnd ();

  AutoGL_EndNativeCall ();

  if (Edge) {
    AutoGL_BeginNativeCall ();

    glDisable (GL_LIGHTING);
    
    glColor4f (1.0f, 1.0f, 1.0f, 1.0f);
  
    glBegin (GL_LINES);
    for (iSection = 0; iSection < NSections; iSection++) {
      glVertex3f (Section_vertexs[iSection][0][0], 
		  Section_vertexs[iSection][0][1], 
		  Section_vertexs[iSection][0][2]);
      glVertex3f (Section_vertexs[iSection][1][0], 
		  Section_vertexs[iSection][1][1], 
		  Section_vertexs[iSection][1][2]);
      
      glVertex3f (Section_vertexs[iSection][1][0], 
		  Section_vertexs[iSection][1][1], 
		  Section_vertexs[iSection][1][2]);
      glVertex3f (Section_vertexs[iSection][2][0], 
		  Section_vertexs[iSection][2][1], 
		  Section_vertexs[iSection][2][2]);

      glVertex3f (Section_vertexs[iSection][2][0], 
		  Section_vertexs[iSection][2][1], 
		  Section_vertexs[iSection][2][2]);
      glVertex3f (Section_vertexs[iSection][0][0], 
		  Section_vertexs[iSection][0][1], 
		  Section_vertexs[iSection][0][2]);
    }
    glEnd ();

    AutoGL_EndNativeCall ();
  }

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif
}

void MakeSectionGroup (void)
{
  SectionGroup_clear ();
  MakeSection (Offset);

  PrepareSectionVertexAndNormal ();
  PrepareSectionColor ();
}

void SetSectionPlane (void) 
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  SetNodeSectionValue ();

  Offset = 0.0;
  MakeSectionGroup ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoGL_DrawView ();
}

void Increment (void) 
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  Offset += Shift;
  MakeSectionGroup ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoGL_DrawView ();
}

void Decrement (void) 
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  Offset -= Shift;
  MakeSectionGroup ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoGL_DrawView ();
}

void IncrementStep (void) 
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  StepId += 1;
  if (MAX_STEP - 1 < StepId) {
    StepId = MAX_STEP - 1;
  }
  ReadNodeScalarFile (NodeScalarBaseFileName, StepId);
  PrepareSectionColor ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoGL_DrawView ();
}

void DecrementStep (void) 
{
#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  StepId -= 1;
  if (StepId < 0) {
    StepId = 0;
  }
  ReadNodeScalarFile (NodeScalarBaseFileName, StepId);
  PrepareSectionColor ();

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif

  AutoGL_DrawView ();
}

void UpdateModel (void) 
{
  PrepareSectionColor ();

  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tetra3d_section_contour_view4  node_file  element_file \n"
       "     node_scalar_file  wireframe_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);

  strcpy (NodeScalarBaseFileName, argv[3]);
  ReadNodeScalarFile (NodeScalarBaseFileName, StepId);

  ReadWireframeFile (argv[4]);

  SetView ();
  SetScalarRange ();
  PrepareWireframe ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddReal (&PlaneA, "PlaneA");
  AutoGL_AddReal (&PlaneB, "PlaneB");
  AutoGL_AddReal (&PlaneC, "PlaneC");
  AutoGL_AddReal (&PlaneD, "PlaneD");
  AutoGL_AddCallback (SetSectionPlane, "SetSectionPlane");      

  AutoGL_AddReal (&Shift, "Shift");
  AutoGL_AddCallback (Increment, "Increment");      
  AutoGL_AddCallback (Decrement, "Decrement");      

  AutoGL_AddCallback (IncrementStep, "IncrementStep");      
  AutoGL_AddCallback (DecrementStep, "DecrementStep");      

  AutoGL_AddReal (&MinRange, "MinRange");
  AutoGL_AddReal (&MaxRange, "MaxRange");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
