/* advautomesh_tri3d_deform_view2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>

#include <GL/gl.h>
#include <GL/glu.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}

void GetNormalizedVector 
(double x, double y, double z,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  double length = sqrt (x * x + y * y + z * z);

  *nx_OUT = x / length;
  *ny_OUT = y / length;
  *nz_OUT = z / length;
}

void GetVectorProduct 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  *x_OUT = y0 * z1 - z0 * y1;
  *y_OUT = z0 * x1 - x0 * z1;
  *z_OUT = x0 * y1 - y0 * x1;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 3000000

int NNodes;
double Node_coord[MAX_NODES][3];
double Node_vector[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 5000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];

float Element_tips[MAX_ELEMENTS][N_ELEMENT_NODES][3];
float Element_normal[MAX_ELEMENTS][3];



/* Edge */

#define MAX_EDGES 7000000

int NEdges;
int Edge_nodeIds[MAX_EDGES][2];

float Edge_tips[MAX_ELEMENTS][2][3];



void Element_getNormal 
(int elementId,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  int nodeId0 = Element_nodeIds[elementId][0];
  int nodeId1 = Element_nodeIds[elementId][1];
  int nodeId2 = Element_nodeIds[elementId][2];

  double x0 = Node_coord[nodeId0][0];
  double y0 = Node_coord[nodeId0][1];
  double z0 = Node_coord[nodeId0][2];

  double x1 = Node_coord[nodeId1][0];
  double y1 = Node_coord[nodeId1][1];
  double z1 = Node_coord[nodeId1][2];

  double x2 = Node_coord[nodeId2][0];
  double y2 = Node_coord[nodeId2][1];
  double z2 = Node_coord[nodeId2][2];

  GetVectorProduct (x0 - x1, y0 - y1, z0 - z1,
		    x0 - x2, y0 - y2, z0 - z2,
		    nx_OUT, ny_OUT, nz_OUT);
  GetNormalizedVector (*nx_OUT, *ny_OUT, *nz_OUT,
		       nx_OUT, ny_OUT, nz_OUT);
}

void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double vectorX, vectorY, vectorZ;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId,
	    &vectorX, &vectorY, &vectorZ);
    assert(nodeId == iNode);
    Node_vector[iNode][0] = vectorX;
    Node_vector[iNode][1] = vectorY;
    Node_vector[iNode][2] = vectorZ;
  }

  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int nodeId0, nodeId1;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;
    
    fscanf (fp, "%d %d %d %d %d %d %d", 
	    &edgeId,
	    &nodeId0, &nodeId1,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);
    Edge_nodeIds[iEdge][0] = nodeId0;
    Edge_nodeIds[iEdge][1] = nodeId1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

int Fill = 0;
int Edge = 0;
double Scale;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void SetVectorScale (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  double maxLength;
  int iNode;

  assert(0 < NNodes);

  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  maxLength = 0.0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    double vx = Node_vector[iNode][0];
    double vy = Node_vector[iNode][1];
    double vz = Node_vector[iNode][2];
    double length = sqrt (vx * vx + vy * vy + vz * vz);
    
    if (maxLength < length) maxLength = length;
  }

  Scale = size / maxLength / 5;
}

void PrepareElementNormal (void)
{
  int iElement;
  int iElementNode;

  for (iElement = 0; iElement < NElements; iElement++) {
    double nx, ny, nz;

    Element_getNormal (iElement,
		       &nx, &ny, &nz);
    Element_normal[iElement][0] = nx;
    Element_normal[iElement][1] = ny;
    Element_normal[iElement][2] = nz;
  }
}

void PrepareElementTip ()
{
  int iElement;
  int iElementNode;

  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementNode = 0;
	 iElementNode < N_ELEMENT_NODES;
	 iElementNode++) {
      int nodeId = Element_nodeIds[iElement][iElementNode];

      double nodeX = Node_coord[nodeId][0];
      double nodeY = Node_coord[nodeId][1];
      double nodeZ = Node_coord[nodeId][2];
      
      double vectorX = Node_vector[nodeId][0];
      double vectorY = Node_vector[nodeId][1];
      double vectorZ = Node_vector[nodeId][2];
      
      double tipX = nodeX + vectorX * Scale;
      double tipY = nodeY + vectorY * Scale;
      double tipZ = nodeZ + vectorZ * Scale;
      
      Element_tips[iElement][iElementNode][0] = tipX;
      Element_tips[iElement][iElementNode][1] = tipY;
      Element_tips[iElement][iElementNode][2] = tipZ;
    }
  }
}

void PrepareEdgeTip (void)
{
  int iEdge;
  int iEdgeNode;

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    for (iEdgeNode = 0; iEdgeNode < 2; iEdgeNode++) {
      int nodeId = Edge_nodeIds[iEdge][iEdgeNode];
      
      double nodeX = Node_coord[nodeId][0];
      double nodeY = Node_coord[nodeId][1];
      double nodeZ = Node_coord[nodeId][2];
      
      double vectorX = Node_vector[nodeId][0];
      double vectorY = Node_vector[nodeId][1];
      double vectorZ = Node_vector[nodeId][2];
      
      double tipX = nodeX + vectorX * Scale;
      double tipY = nodeY + vectorY * Scale;
      double tipZ = nodeZ + vectorZ * Scale;
      
      Edge_tips[iEdge][iEdgeNode][0] = tipX;
      Edge_tips[iEdge][iEdgeNode][1] = tipY;
      Edge_tips[iEdge][iEdgeNode][2] = tipZ;
    }
  }
}

void PrepareRendering (void)
{
  PrepareElementNormal ();
  PrepareElementTip ();
  PrepareEdgeTip ();
}

#if 1
#include "../../util/time_measure.c"
#define TIME_MEASURE
#endif

void RedrawView (void) 
{
  int iElement;
  int iEdge;

#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  if (Fill) {
    AutoGL_BeginNativeCall ();

    glEnable (GL_LIGHTING);

    glColor4f (1.0f, 0.0f, 1.0f, 1.0f);

    glBegin (GL_TRIANGLES);
    for (iElement = 0; iElement < NElements; iElement++) {
      glNormal3f (Element_normal[iElement][0], 
		  Element_normal[iElement][1], 
		  Element_normal[iElement][2]); 
      glVertex3f (Element_tips[iElement][0][0], 
		  Element_tips[iElement][0][1], 
		  Element_tips[iElement][0][2]);
      glVertex3f (Element_tips[iElement][1][0], 
		  Element_tips[iElement][1][1], 
		  Element_tips[iElement][1][2]);
      glVertex3f (Element_tips[iElement][2][0], 
		  Element_tips[iElement][2][1], 
		  Element_tips[iElement][2][2]);
    }
    glEnd ();

    AutoGL_EndNativeCall ();
  }

  if (Edge) {
    AutoGL_BeginNativeCall ();

    glDisable (GL_LIGHTING);
    
    glColor4f (0.0f, 1.0f, 0.0f, 1.0f);

    glBegin (GL_LINES);
    for (iEdge = 0; iEdge < NEdges; iEdge++) {
      glVertex3f (Edge_tips[iEdge][0][0], 
		  Edge_tips[iEdge][0][1], 
		  Edge_tips[iEdge][0][2]);
      glVertex3f (Edge_tips[iEdge][1][0], 
		  Edge_tips[iEdge][1][1], 
		  Edge_tips[iEdge][1][2]);
    }
    glEnd ();

    AutoGL_EndNativeCall ();
  }

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif
}

void UpdateModel (void) 
{
  PrepareElementTip ();
  PrepareEdgeTip ();

  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_deform_view2  node_file  element_file \n"
       "        node_vector_file  edge_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeVectorFile (argv[3]);
  ReadEdgeFile (argv[4]);

  SetView ();
  SetVectorScale ();
  PrepareRendering ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Fill, "Fill");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddReal (&Scale, "Scale");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
