/* advautomesh_tri3d_strip.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 2000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

const int EdgeOppositeNodeMap[N_ELEMENT_EDGES] = { 
  0, 1, 2 
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_EDGES];
int Element_isMarked[MAX_ELEMENTS];



int Element_containsNode 
(int elementId, 
 int nodeId)
{
  int iElementNode;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= nodeId);
  assert(nodeId < NNodes);

  for (iElementNode = 0; 
       iElementNode < N_ELEMENT_NODES; 
       iElementNode++) {
    if (Element_nodeIds[elementId][iElementNode] == nodeId) {
      return 1;
    }
  }

  return 0;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iElement;
  int iElementEdge;
  
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);

    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int neighborId;

      fscanf (fp, "%d", 
	      &neighborId);
      assert((0 <= neighborId && neighborId < NElements) 
	     || neighborId == -1);
      Element_neighborIds[iElement][iElementEdge] = neighborId;
    }
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Strip */

#define MAX_STRIPS 10000000
#define MAX_VERTEXS 10000000

int NStrips;
int Strip_startVertexId[MAX_STRIPS];
int Strip_nVertexs[MAX_STRIPS];
int NVertexs;
int Vertex_nodeId[MAX_VERTEXS];
int Vertex_elementId[MAX_VERTEXS];



void StripGroup_addStrip (void)
{
  assert(NStrips < MAX_STRIPS);

  Strip_startVertexId[NStrips] = NVertexs;
  Strip_nVertexs[NStrips] = 0;
  NStrips++;
}

void StripGroup_addVertex (int nodeId, int elementId)
{
  assert(0 < NStrips);
  assert(NVertexs < MAX_VERTEXS);
  assert(0 <= nodeId);
  assert(nodeId < NNodes);
  assert(0 <= elementId || elementId == -1);
  assert(elementId < NElements);

  Strip_nVertexs[NStrips - 1]++;
  Vertex_nodeId[NVertexs] = nodeId;
  Vertex_elementId[NVertexs] = elementId;
  NVertexs++;
}

void WriteStripFile (const char *fileName)
{
  FILE *fp;
  int iStrip;
  int iVertex;
  
  assert(0 < NStrips);
  assert(0 < NVertexs);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NStrips);
  
  for (iStrip = 0; iStrip < NStrips; iStrip++) {
    fprintf (fp, "%d   %d\n", 
	     iStrip,
	     Strip_nVertexs[iStrip]);

    for (iVertex = Strip_startVertexId[iStrip]; 
	 iVertex < Strip_startVertexId[iStrip] + Strip_nVertexs[iStrip]; 
	 iVertex++) {
      fprintf (fp, "  %d   %d %d\n", 
	       iVertex - Strip_startVertexId[iStrip],
	       Vertex_nodeId[iVertex], 
	       Vertex_elementId[iVertex]);
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

int FindTheThirdNode (int elementId, int nodeId0, int nodeId1)
{
  int iElementNode;
  int flags[N_ELEMENT_NODES];

  for (iElementNode = 0;
       iElementNode < N_ELEMENT_NODES;
       iElementNode++) {
    flags[iElementNode] = 1;
  }

  for (iElementNode = 0;
       iElementNode < N_ELEMENT_NODES;
       iElementNode++) {
    int elementNodeId = Element_nodeIds[elementId][iElementNode];

    if (elementNodeId == nodeId0 
	|| elementNodeId == nodeId1) {
      flags[iElementNode] = 0;
    }
  }

  for (iElementNode = 0;
       iElementNode < N_ELEMENT_NODES;
       iElementNode++) {
    int elementNodeId = Element_nodeIds[elementId][iElementNode];

    if (flags[iElementNode]) {
      return elementNodeId;
    }
  }

  assert(0);
  return -1;
}

void PerformCommand (void)
{
  int currentElementId;
  int currentElementEdgeId;
  int currentNodeId;
  int prevElementId;
  int prevPrevElementId;
  int nextElementId;
  int iElement;
  int iElementEdge;

  for (iElement = 0; iElement < NElements; iElement++) {
    Element_isMarked[iElement] = 0;
  }

  for (;;) {
    /* the beginning part of a new strip */
    currentElementId = -1;
    for (iElement = 0; iElement < NElements; iElement++) {
      if (!Element_isMarked[iElement]) {
	currentElementId = iElement;
	break;
      }
    }
    if (currentElementId == -1) {
      /* no more new strip */
      break;
    }

    StripGroup_addStrip ();

    prevPrevElementId = -1;
    prevElementId = -1;

    /* the succesive part of the strip */
    for (;;) {
      assert(currentElementId != -1);
      nextElementId = -1;
      currentElementEdgeId = -1;
      currentNodeId = -1;

      for (iElementEdge = 0; 
	   iElementEdge < N_ELEMENT_EDGES; 
	   iElementEdge++) {
	int neighborId 
	  = Element_neighborIds[currentElementId][iElementEdge];
	int oppositeNodeId;
	int edgeNodeId0, edgeNodeId1;
	int nextNodeId;
	
	if (neighborId == -1) continue;

	if (Element_isMarked[neighborId]) continue;

	oppositeNodeId = Element_nodeIds
	  [currentElementId]
	  [EdgeOppositeNodeMap[iElementEdge]];

	edgeNodeId0 = Element_nodeIds
	  [currentElementId][EdgeNodeMap[iElementEdge][0]];
	edgeNodeId1 = Element_nodeIds
	  [currentElementId][EdgeNodeMap[iElementEdge][1]];
	nextNodeId = FindTheThirdNode 
	  (neighborId, edgeNodeId0, edgeNodeId1);

	if (prevPrevElementId != -1) {
	  assert(prevPrevElementId != -1);
	  assert(prevElementId != -1);
	  
	  if (Element_containsNode 
	      (prevPrevElementId, oppositeNodeId)) {
	    if (!Element_containsNode 
		(prevElementId, nextNodeId)) {
	      currentElementEdgeId = iElementEdge;
	      currentNodeId = oppositeNodeId;
	      nextElementId = neighborId;
	      break;
	    }
	  }

	} else if (prevElementId != -1) { 
	  assert(prevPrevElementId == -1);
	  assert(prevElementId != -1);

	  if (!Element_containsNode 
	      (prevElementId, nextNodeId)) {
	    currentElementEdgeId = iElementEdge;
	    currentNodeId = oppositeNodeId;
	    nextElementId = neighborId;
	    break;
	  }

	} else {
	  assert(prevPrevElementId == -1);
	  assert(prevElementId == -1);

	  currentElementEdgeId = iElementEdge;
	  currentNodeId = oppositeNodeId;
	  nextElementId = neighborId;
	  break;

	}  /* END IF prevPrevElementId != -1 */

      }

      if (nextElementId == -1) {
	/* the last part of the strip */
	int nodeId0 = Element_nodeIds[currentElementId][0];
	int nodeId1 = Element_nodeIds[currentElementId][1];
	int nodeId2 = Element_nodeIds[currentElementId][2];

	if (prevPrevElementId == -1) {

	  if (prevElementId == -1) {
	    /* a single triangle */
	    StripGroup_addVertex (nodeId0, prevPrevElementId);
	    StripGroup_addVertex (nodeId1, prevElementId);
	    StripGroup_addVertex (nodeId2, currentElementId);

	  } else {   /* prevElementId != -1 */
	    /* two triangles */
	    assert(prevElementId != -1);

	    if (!Element_containsNode (prevElementId, nodeId0)) {
	      assert(Element_containsNode (prevElementId, nodeId1));
	      assert(Element_containsNode (prevElementId, nodeId2));
	      StripGroup_addVertex (nodeId1, prevPrevElementId);
	      StripGroup_addVertex (nodeId2, prevElementId);
	      StripGroup_addVertex (nodeId0, currentElementId);

	    } else if (!Element_containsNode (prevElementId, nodeId1)) {
	      assert(Element_containsNode (prevElementId, nodeId2));
	      assert(Element_containsNode (prevElementId, nodeId0));
	      StripGroup_addVertex (nodeId2, prevPrevElementId);
	      StripGroup_addVertex (nodeId0, prevElementId);
	      StripGroup_addVertex (nodeId1, currentElementId);

	    } else {
	      assert(!Element_containsNode (prevElementId, nodeId2));
	      assert(Element_containsNode (prevElementId, nodeId0));
	      assert(Element_containsNode (prevElementId, nodeId1));
	      StripGroup_addVertex (nodeId0, prevPrevElementId);
	      StripGroup_addVertex (nodeId1, prevElementId);
	      StripGroup_addVertex (nodeId2, currentElementId);

	    }

	  }  /* END IF prevElementId == -1 */

	}  else {  /* prevPrevElementId != -1 */

	  /* three triangles */
	  assert(prevPrevElementId != -1);
	  assert(prevElementId != -1);

	  if (Element_containsNode (prevPrevElementId, nodeId0)) {
	    assert(Element_containsNode (prevElementId, nodeId0));
	    
	    StripGroup_addVertex (nodeId0, prevPrevElementId);

	    if (Element_containsNode (prevElementId, nodeId1)) {
	      assert(!Element_containsNode (prevElementId, nodeId2));
	      StripGroup_addVertex (nodeId1, prevElementId);
	      StripGroup_addVertex (nodeId2, currentElementId);
	    } else {
	      assert(Element_containsNode (prevElementId, nodeId2));
	      assert(!Element_containsNode (prevElementId, nodeId1));
	      StripGroup_addVertex (nodeId2, prevElementId);
	      StripGroup_addVertex (nodeId1, currentElementId);
	    }

	  } else if (Element_containsNode (prevPrevElementId, nodeId1)) {
	    assert(Element_containsNode (prevElementId, nodeId1));

	    StripGroup_addVertex (nodeId1, prevPrevElementId);

	    if (Element_containsNode (prevElementId, nodeId2)) {
	      assert(!Element_containsNode (prevElementId, nodeId0));
	      StripGroup_addVertex (nodeId2, prevElementId);
	      StripGroup_addVertex (nodeId0, currentElementId);
	    } else {
	      assert(Element_containsNode (prevElementId, nodeId0));
	      assert(!Element_containsNode (prevElementId, nodeId2));
	      StripGroup_addVertex (nodeId0, prevElementId);
	      StripGroup_addVertex (nodeId2, currentElementId);
	    }

	  } else {
	    assert(Element_containsNode (prevPrevElementId, nodeId2));
	    assert(Element_containsNode (prevElementId, nodeId2));
	    
	    StripGroup_addVertex (nodeId2, prevPrevElementId);
	    
	    if (Element_containsNode (prevElementId, nodeId0)) {
	      assert(!Element_containsNode (prevElementId, nodeId1));
	      StripGroup_addVertex (nodeId0, prevElementId);
	      StripGroup_addVertex (nodeId1, currentElementId);
	    } else {
	      assert(Element_containsNode (prevElementId, nodeId1));
	      assert(!Element_containsNode (prevElementId, nodeId0));
	      StripGroup_addVertex (nodeId1, prevElementId);
	      StripGroup_addVertex (nodeId0, currentElementId);
	    }

	  }

	} /* END IF prevPrevElementId == -1 */

	Element_isMarked[currentElementId] = 1;

	/* break because here is the last part of the strip */
	break;
      } /* END IF nextElementId == -1 */

      StripGroup_addVertex (currentNodeId, prevPrevElementId);
      Element_isMarked[currentElementId] = 1;
      
      prevPrevElementId = prevElementId;
      prevElementId = currentElementId;
      currentElementId = nextElementId;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_tri3d_strip  node_file  element_file \n"
       "        element_neighbor_file \n"
       "        strip_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadElementNeighborFile (argv[3]);

  PerformCommand ();

  WriteStripFile (argv[4]);

  return 0;
}
