/* advautomesh_h_add_attrib.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



#define MAX_OBJECTS 1000000

int NObjects;
int Object_attributeId[MAX_OBJECTS];
int NAttributes;



void ReadAttributeFile (const char *fileName)
{
  FILE *fp;
  int iObject;
  int maxAttributeId = -1;

  assert(NObjects == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NObjects);
  
  for (iObject = 0; iObject < NObjects; iObject++) {
    int objectId;
    int attributeId;

    fscanf (fp, "%d %d", 
	    &objectId,
	    &attributeId);
    assert(objectId == iObject);
    assert(0 <= attributeId);

    Object_attributeId[iObject] = attributeId;

    if (maxAttributeId < attributeId) {
      maxAttributeId = attributeId;
    }
  }
  
  fclose (fp);
  
  NAttributes = maxAttributeId + 1;
}

void ReadListFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 <= NObjects);
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int objectId;
    
    fscanf (fp, "%d %d", 
	    &itemId,
	    &objectId);
    assert(itemId == iItem);
    assert(0 <= objectId);
    assert(objectId < NObjects);

    Object_attributeId[objectId] = NAttributes;
  }

  NAttributes++;

  fclose (fp);
}

void WriteAttributeFile (const char *fileName)
{
  FILE *fp;
  int iObject;

  assert(0 <= NObjects);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NObjects);
  
  for (iObject = 0; iObject < NObjects; iObject++) {
    fprintf (fp, "%d   %d\n", 
	     iObject,  
	     Object_attributeId[iObject]); 
  }
  
  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_add_attrib  attribute_file \n"
       "        list_file \n"
       "        attribute_file_OUT \n");
    exit (1);
  }

  ReadAttributeFile (argv[1]);
  ReadListFile (argv[2]);

  WriteAttributeFile (argv[3]);

  return 0;
}
