/* advautomesh_h_hexa3d_node_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

int NodeId = 0;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void RedrawView (void) 
{
  int iNode;
  char buf[256];

  AutoGL_SetColor (1, 1, 0);

  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1]; 
    double z = Node_coord[iNode][2]; 

    AutoGL_DrawCrossMark3D (x, y, z,
			    3);

    if (NodeId) {
      sprintf(buf, "%d", iNode);
      AutoGL_DrawString (x, y, z,
			 buf);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_node_view  node_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&NodeId, "NodeId");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
