/* advautomesh_h_hexa3d20n_vertex.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_isOnVertex[MAX_NODES];
int Node_newId[MAX_NODES];
int NVertexNodes;



/* Element */

#define N_ELEMENT_NODES 20
#define N_ELEMENT_VERTEXS 8
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void WriteVertexNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);
  assert(0 < NElements);
  assert(0 < NVertexNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NVertexNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnVertex[iNode]) {
      fprintf (fp, "%d   %22.15e %22.15e %22.15e\n",
	       Node_newId[iNode],
	       Node_coord[iNode][0], 
	       Node_coord[iNode][1], 
	       Node_coord[iNode][2]);
    }
  }

  fclose (fp);
}

void WriteVertexElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;

  assert(0 < NNodes);
  assert(0 < NElements);
  assert(0 < NVertexNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NElements);

  for (iElement = 0; iElement < NElements; iElement++) {
    int nodeId0 = Element_nodeIds[iElement][0];
    int nodeId1 = Element_nodeIds[iElement][1];
    int nodeId2 = Element_nodeIds[iElement][2];
    int nodeId3 = Element_nodeIds[iElement][3];
    int nodeId4 = Element_nodeIds[iElement][4];
    int nodeId5 = Element_nodeIds[iElement][5];
    int nodeId6 = Element_nodeIds[iElement][6];
    int nodeId7 = Element_nodeIds[iElement][7];
    
    fprintf (fp, "%d   %d %d %d %d %d %d %d %d\n",
	     iElement,
	     Node_newId[nodeId0], 
	     Node_newId[nodeId1], 
	     Node_newId[nodeId2], 
	     Node_newId[nodeId3],
	     Node_newId[nodeId4], 
	     Node_newId[nodeId5], 
	     Node_newId[nodeId6], 
	     Node_newId[nodeId7]);
  }

  fclose (fp);
}

void WriteVertexNodeSrcFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);
  assert(0 < NElements);
  assert(0 < NVertexNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NVertexNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnVertex[iNode]) {
      fprintf (fp, "%d   %d\n",
	       Node_newId[iNode],
	       iNode);
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iElement;
  int iElementVertex;
  int count;

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOnVertex[iNode] = 0;
  }
  
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementVertex = 0; 
	 iElementVertex < N_ELEMENT_VERTEXS; 
	 iElementVertex++) {
      int ndoeId = Element_nodeIds[iElement][iElementVertex];
      
      Node_isOnVertex[ndoeId] = 1;
    }
  }

  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnVertex[iNode]) {
      Node_newId[iNode] = count;
      count++;
    } else {
      Node_newId[iNode] = -1;
    }
  }
  NVertexNodes = count;

  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnVertex[iNode]) {
      assert(Node_newId[iNode] == count);
      count++;
    } else {
      assert(Node_newId[iNode] == -1);
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d20n_vertex  node_file  element_file \n"
       "        vertex_node_file_OUT  vertex_element_file_OUT \n"
       "        vertex_node_src_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);

  PerformCommand ();

  WriteVertexNodeFile (argv[3]);
  WriteVertexElementFile (argv[4]);
  WriteVertexNodeSrcFile (argv[5]);

  return 0;
}
