/* advautomesh_h_hexa3d_add_face_dof_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 3



/* Face */

#define MAX_FACES 1000000

int NFaces;
int Face_hasValue[MAX_FACES][N_NODE_DOFS];
double Face_value[MAX_FACES][N_NODE_DOFS];



void ReadFaceFile (const char *fileName)
{
  FILE *fp;

  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  /* skip data */

  fclose (fp);
}

void ClearFaceDofValue (void)
{
  int iFace;
  int iDof;

  for (iFace = 0; iFace < NFaces; iFace++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Face_hasValue[iFace][iDof] = 0;
      Face_value[iFace][iDof] = 0.0;
    }
  }
}

void ReadFaceDofBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NFaces);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int faceId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &faceId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= faceId);
    assert(faceId < NFaces);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Face_hasValue[faceId][dofId] = 1;
    Face_value[faceId][dofId] = value;
  }

  fclose (fp);
}

void WriteFaceDofBcFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int iDof;
  int count;

  assert(0 < NFaces);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Face_hasValue[iFace][iDof]) {
	count++;
      }
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Face_hasValue[iFace][iDof]) {
	fprintf (fp, "%d   %d %d %f\n", 
		 count,
		 iFace, iDof, Face_value[iFace][iDof]);
	count++;
      }
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_add_face_dof_bc  face_file \n"
       "        face_dof_bc_file_A  face_dof_bc_file_B \n"
       "        face_dof_bc_file_OUT \n");
    exit (1);
  }

  ReadFaceFile (argv[1]);

  ClearFaceDofValue ();

  ReadFaceDofBcFile (argv[2]);
  ReadFaceDofBcFile (argv[3]);

  WriteFaceDofBcFile (argv[4]);

  return 0;
}


