/* advautomesh_h_hexa3d_add_face_scalar_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Face */

#define MAX_FACES 1000000

int NFaces;
int Face_hasValue[MAX_FACES];
double Face_scalar[MAX_FACES];



void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;

  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  /* skip data */

  fclose (fp);



  for (iFace = 0; iFace < NFaces; iFace++) {
    Face_hasValue[iFace] = 0;
    Face_scalar[iFace] = 0.0;
  }
}

void ReadFaceScalarBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NFaces);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int faceId;
    double value;
    
    fscanf (fp, "%d %d %lf",
	    &itemId, 
	    &faceId, &value);
    assert(itemId == iItem);
    assert(0 <= faceId);
    assert(faceId < NFaces);

    Face_hasValue[faceId] = 1;
    Face_scalar[faceId] = value;
  }

  fclose (fp);
}

void WriteFaceScalarBcFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int count;

  assert(0 < NFaces);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_hasValue[iFace]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_hasValue[iFace]) {
      fprintf (fp, "%d   %d %f\n", 
	       count,
	       iFace, Face_scalar[iFace]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_add_face_scalar_bc \n"
       "        face_file \n"
       "        face_scalar_bc_file_A  face_scalar_bc_file_B \n"
       "        face_scalar_bc_file_OUT \n");
    exit (1);
  }

  ReadFaceFile (argv[1]);
  ReadFaceScalarBcFile (argv[2]);
  ReadFaceScalarBcFile (argv[3]);

  WriteFaceScalarBcFile (argv[4]);

  return 0;
}


