/* advautomesh_h_hexa3d_add_node_scalar_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
int Node_hasValue[MAX_NODES];
double Node_scalar[MAX_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate value */

  fclose (fp);



  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_hasValue[iNode] = 0;
    Node_scalar[iNode] = 0.0;
  }
}

void ReadNodeScalarBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int nodeId;
    double value;
    
    fscanf (fp, "%d %d %lf",
	    &itemId, 
	    &nodeId, &value);
    assert(itemId == iItem);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);

    Node_hasValue[nodeId] = 1;
    Node_scalar[nodeId] = value;
  }

  fclose (fp);
}

void WriteNodeScalarBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      fprintf (fp, "%d   %d %f\n", 
	       count,
	       iNode, Node_scalar[iNode]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_add_node_scalar_bc \n"
       "        node_file \n"
       "        node_scalar_bc_file_A  node_scalar_bc_file_B \n"
       "        node_scalar_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadNodeScalarBcFile (argv[2]);
  ReadNodeScalarBcFile (argv[3]);

  WriteNodeScalarBcFile (argv[4]);

  return 0;
}


