/* advautomesh_h_hexa3d_boundary_scalar.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_scalar[MAX_NODES];
int Node_isOnBoundary[MAX_NODES];



void ReadNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double scalar;

    fscanf (fp, "%d %lf", 
	    &nodeId,
	    &scalar);
    assert(nodeId == iNode);
    Node_scalar[iNode] = scalar;
  }

  fclose (fp);
}

void ReadBoundaryNodeSrcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iBoundaryNode;
  int nBoundaryNodes;

  assert(0 < NNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOnBoundary[iNode] = 0;
  }
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d",
	  &nBoundaryNodes);
  
  for (iBoundaryNode = 0; 
       iBoundaryNode < nBoundaryNodes; 
       iBoundaryNode++) {
    int boundaryNodeId;
    int nodeId;

    fscanf (fp, "%d %d",
	    &boundaryNodeId,
	    &nodeId);
    assert(boundaryNodeId == iBoundaryNode);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    Node_isOnBoundary[nodeId] = 1;
  }

  fclose (fp);
}

void WriteBoundaryNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int nBoundaryNodes;
  int count;
  
  assert(0 < NNodes);

  nBoundaryNodes = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      nBoundaryNodes++;
    }
  }

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   nBoundaryNodes);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      fprintf (fp, "%d   %22.15e\n", 
	       count, 
	       Node_scalar[iNode]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_boundary_scalar  \n"
       "        node_scalar_file  boundary_node_src_file  \n"
       "        boundary_node_scalar_file_OUT \n");
    exit (1);
  }

  ReadNodeScalarFile (argv[1]);
  ReadBoundaryNodeSrcFile (argv[2]);

  WriteBoundaryNodeScalarFile (argv[3]);

  return 0;
}
