/* advautomesh_h_hexa3d_neighbor.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Element */

#define N_ELEMENT_NODES 8
#define N_ELEMENT_FACES 6
#define MAX_ELEMENTS 1000000

int NElements;
int Element_faceIds[MAX_ELEMENTS][N_ELEMENT_FACES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_FACES];



/* Face */

#define N_FACE_NODES 4
#define MAX_FACES 2000000

int NFaces;
int Face_elementIds[MAX_FACES][2];
int Face_elementFaceIds[MAX_FACES][2];



void ReadElementFile (const char *fileName)
{
  FILE *fp;

  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  /* skip connectivity */
  
  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  
  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &faceId,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);
    
    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    Face_elementIds[iFace][0] = elementId0;

    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);
    Face_elementFaceIds[iFace][0] = elementFaceId0;

    assert(0 <= elementId1 || elementId1 == -1);
    assert(elementId1 < NElements);
    Face_elementIds[iFace][1] = elementId1;

    assert(0 <= elementFaceId1 || elementFaceId1 == -1);
    assert(elementFaceId1 < N_ELEMENT_FACES);
    Face_elementFaceIds[iFace][1] = elementFaceId1;
  }

  fclose (fp);
}

void ReadElementFaceFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementFace;
  int count;
  
  assert(0 < NElements);
  assert(0 < NFaces);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);

    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceId;
      
      fscanf (fp, "%d", 
	      &faceId);
      assert(0 <= faceId);
      assert(faceId < NFaces);
      Element_faceIds[iElement][iElementFace] = faceId;
    }
  }

  fclose (fp);
}

void WriteElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementFace;
  
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement);

    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      fprintf (fp, "%d ", 
	       Element_neighborIds[iElement][iElementFace]);
    }

    fprintf (fp, "\n");
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iElement;
  int iElementFace;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceId = Element_faceIds[iElement][iElementFace];
      int neighborId;
      
      if (Face_elementIds[faceId][0] == iElement) {
	neighborId = Face_elementIds[faceId][1];
      } else {
	assert(Face_elementIds[faceId][1] == iElement);
	neighborId = Face_elementIds[faceId][0];
      }
      Element_neighborIds[iElement][iElementFace] = neighborId;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_neighbor \n"
       "        element_file  face_file  element_face_file \n"
       "        element_neighbor_file_OUT \n");
    exit (1);
  }

  ReadElementFile (argv[1]);
  ReadFaceFile (argv[2]);
  ReadElementFaceFile (argv[3]);

  PerformCommand ();

  WriteElementNeighborFile (argv[4]);

  return 0;
}
