/* advautomesh_h_hexa3d_node_on_plane.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_isOn[MAX_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void WriteNodeListFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOn[iNode]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOn[iNode]) {
      fprintf (fp, "%d   %d\n", 
	       count,
	       iNode);
      count++;
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

double PlaneA, PlaneB, PlaneC, PlaneD;

void PerformCommand (void)
{
  int iNode;

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOn[iNode] = 0;
  }

  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    double value = PlaneA * x + PlaneB * y + PlaneC * z + PlaneD;
    
    if (fabs (value) < Eps) {
      Node_isOn[iNode] = 1;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_node_on_plane  node_file \n"
       "        a  b  c  d \n"
       "        node_list_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  PlaneA = atof (argv[2]);
  PlaneB = atof (argv[3]);
  PlaneC = atof (argv[4]);
  PlaneD = atof (argv[5]);

  PerformCommand ();

  WriteNodeListFile (argv[6]);

  return 0;
}


