/* advautomesh_h_hexa3d_topology.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

typedef enum {
  NODE_TOPOLOGY_INTERNAL,
  NODE_TOPOLOGY_FACE,
  NODE_TOPOLOGY_EDGE,
  NODE_TOPOLOGY_VERTEX
} NodeTopologyType;

#define MAX_NODES 1000000

int NNodes;
NodeTopologyType Node_topologyType[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 8
#define N_ELEMENT_FACES 6
#define MAX_ELEMENTS 1000000

const int FaceNodeMap[N_ELEMENT_FACES][4] = {
  { 0, 3, 2, 1 },
  { 4, 5, 6, 7 },
  { 0, 1, 5, 4 },
  { 1, 2, 6, 5 },
  { 2, 3, 7, 6 },
  { 0, 4, 7, 3 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Face */

#define N_FACE_NODES 4
#define MAX_FACES 2000000

int NFaces;
int Face_nodeIds[MAX_FACES][N_FACE_NODES];
int Face_isOnBoundary[MAX_FACES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  
  assert(NFaces == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &faceId,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);

    Face_nodeIds[iFace][0] 
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][0]];
    Face_nodeIds[iFace][1] 
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][1]];
    Face_nodeIds[iFace][2] 
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][2]];
    Face_nodeIds[iFace][3] 
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][3]];

    if (elementId1 == -1) {
      Face_isOnBoundary[iFace] = 1;
    } else {
      Face_isOnBoundary[iFace] = 0;
    }
  }

  fclose (fp);
}

void WriteNodeTopologyFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %d\n", 
	     iNode, 
	     Node_topologyType[iNode]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iFace;
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_topologyType[iNode] = NODE_TOPOLOGY_INTERNAL;
  }

  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_isOnBoundary[iFace]) {
      int nodeId0 = Face_nodeIds[iFace][0];
      int nodeId1 = Face_nodeIds[iFace][1];
      int nodeId2 = Face_nodeIds[iFace][2];
      int nodeId3 = Face_nodeIds[iFace][3];
      
      Node_topologyType[nodeId0] = NODE_TOPOLOGY_FACE;
      Node_topologyType[nodeId1] = NODE_TOPOLOGY_FACE;
      Node_topologyType[nodeId2] = NODE_TOPOLOGY_FACE;
      Node_topologyType[nodeId3] = NODE_TOPOLOGY_FACE;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_hexa3d_topology  node_file  element_file \n"
       "        face_file \n"
       "        node_topology_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadFaceFile (argv[3]);

  PerformCommand ();

  WriteNodeTopologyFile (argv[4]);

  return 0;
}
