/* advautomesh_h_quad2d_add_edge_dof_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 2



/* Edge */

#define MAX_EDGES 2000000

int NEdges;
int Edge_hasValue[MAX_EDGES][N_NODE_DOFS];
double Edge_value[MAX_EDGES][N_NODE_DOFS];



void ReadEdgeFile (const char *fileName)
{
  FILE *fp;

  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  /* skip data */

  fclose (fp);
}

void ClearEdgeDofValue (void)
{
  int iEdge;
  int iDof;

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Edge_hasValue[iEdge][iDof] = 0;
      Edge_value[iEdge][iDof] = 0.0;
    }
  }
}

void ReadEdgeDofBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int edgeId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &edgeId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= edgeId);
    assert(edgeId < NEdges);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Edge_hasValue[edgeId][dofId] = 1;
    Edge_value[edgeId][dofId] = value;
  }

  fclose (fp);
}

void WriteEdgeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  int iDof;
  int count;

  assert(0 < NEdges);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Edge_hasValue[iEdge][iDof]) {
	count++;
      }
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Edge_hasValue[iEdge][iDof]) {
	fprintf (fp, "%d   %d %d %f\n", 
		 count,
		 iEdge, iDof, Edge_value[iEdge][iDof]);
	count++;
      }
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_quad2d_add_edge_dof_bc  edge_file \n"
       "        edge_dof_bc_file_A  edge_dof_bc_file_B \n"
       "        edge_dof_bc_file_OUT \n");
    exit (1);
  }

  ReadEdgeFile (argv[1]);

  ClearEdgeDofValue ();

  ReadEdgeDofBcFile (argv[2]);
  ReadEdgeDofBcFile (argv[3]);

  WriteEdgeDofBcFile (argv[4]);

  return 0;
}


