/* advautomesh_h_quad2d_add_edge_vector_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Edge */

#define MAX_EDGES 2000000

int NEdges;
int Edge_hasValue[MAX_EDGES];
double Edge_vector[MAX_EDGES][2];



void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;

  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  /* skip data */

  fclose (fp);



  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    Edge_hasValue[iEdge] = 0;
    Edge_vector[iEdge][0] = 0.0;
    Edge_vector[iEdge][1] = 0.0;
  }
}

void ReadEdgeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int edgeId;
    double valueX, valueY;
    
    fscanf (fp, "%d %d %lf %lf",
	    &itemId, 
	    &edgeId, &valueX, &valueY);
    assert(itemId == iItem);
    assert(0 <= edgeId);
    assert(edgeId < NEdges);

    Edge_hasValue[edgeId] = 1;
    Edge_vector[edgeId][0] = valueX;
    Edge_vector[edgeId][1] = valueY;
  }

  fclose (fp);
}

void WriteEdgeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  int count;

  assert(0 < NEdges);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_hasValue[iEdge]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_hasValue[iEdge]) {
      fprintf (fp, "%d   %d %f %f\n", 
	       count,
	       iEdge, 
	       Edge_vector[iEdge][0], 
	       Edge_vector[iEdge][1]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_quad2d_add_edge_vector_bc \n"
       "        edge_file \n"
       "        edge_vector_bc_file_A  edge_vector_bc_file_B \n"
       "        edge_vector_bc_file_OUT \n");
    exit (1);
  }

  ReadEdgeFile (argv[1]);
  ReadEdgeVectorBcFile (argv[2]);
  ReadEdgeVectorBcFile (argv[3]);

  WriteEdgeVectorBcFile (argv[4]);

  return 0;
}


