/* advautomesh_h_quad2d_add_node_dof_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 2
#define MAX_NODES 1000000

int NNodes;
int Node_hasValue[MAX_NODES][N_NODE_DOFS];
double Node_value[MAX_NODES][N_NODE_DOFS];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate value */

  fclose (fp);
}

void ClearNodeDofValue (void)
{
  int iNode;
  int iDof;

  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Node_hasValue[iNode][iDof] = 0;
      Node_value[iNode][iDof] = 0.0;
    }
  }
}

void ReadNodeDofBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int nodeId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &nodeId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Node_hasValue[nodeId][dofId] = 1;
    Node_value[nodeId][dofId] = value;
  }

  fclose (fp);
}

void WriteNodeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iDof;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	count++;
      }
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	fprintf (fp, "%d   %d %d %f\n", 
		 count,
		 iNode, iDof, Node_value[iNode][iDof]);
	count++;
      }
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_quad2d_add_node_dof_bc  node_file \n"
       "        node_dof_bc_file_A  node_dof_bc_file_B \n"
       "        node_dof_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  ClearNodeDofValue ();

  ReadNodeDofBcFile (argv[2]);
  ReadNodeDofBcFile (argv[3]);

  WriteNodeDofBcFile (argv[4]);

  return 0;
}


