/* advautomesh_h_quad2d_divide.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_EDGES 4
#define MAX_ELEMENTS 1000000

int NElements;
int Element_isDeleted[MAX_ELEMENTS];
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_edgeIds[MAX_ELEMENTS][N_ELEMENT_EDGES];
int Element_centerNodeId[MAX_ELEMENTS];



/* Edge */

#define MAX_EDGES 1000000

int NEdges;
int Edge_nodeIds[MAX_EDGES][2];
int Edge_midNodeId[MAX_EDGES];



void Element_getCenter 
(int elementId, 
 double *centerX_OUT, double *centerY_OUT)
{
  assert(0 <= elementId);
  assert(elementId < NElements);

  {
    int nodeId0 = Element_nodeIds[elementId][0];
    int nodeId1 = Element_nodeIds[elementId][1];
    int nodeId2 = Element_nodeIds[elementId][2];
    int nodeId3 = Element_nodeIds[elementId][3];
    
    double x0 = Node_coord[nodeId0][0];
    double y0 = Node_coord[nodeId0][1];
    
    double x1 = Node_coord[nodeId1][0];
    double y1 = Node_coord[nodeId1][1];
    
    double x2 = Node_coord[nodeId2][0];
    double y2 = Node_coord[nodeId2][1];
    
    double x3 = Node_coord[nodeId3][0];
    double y3 = Node_coord[nodeId3][1];
    
    *centerX_OUT = (x0 + x1 + x2 + x3) / 4;
    *centerY_OUT = (y0 + y1 + y2 + y3) / 4;
  }
}

void Edge_getMidPoint 
(int edgeId, 
 double *midX_OUT, double *midY_OUT)
{
  assert(0 <= edgeId);
  assert(edgeId < NEdges);

  {
    int edgeNodeId0 = Edge_nodeIds[edgeId][0];
    int edgeNodeId1 = Edge_nodeIds[edgeId][1];
    
    double x0 = Node_coord[edgeNodeId0][0];
    double y0 = Node_coord[edgeNodeId0][1];
    
    double x1 = Node_coord[edgeNodeId1][0];
    double y1 = Node_coord[edgeNodeId1][1];
    
    *midX_OUT = (x0 + x1) / 2;
    *midY_OUT = (y0 + y1) / 2;
  }
}

int Mesh_addNewNode (double x, double y)
{
  int nodeId = NNodes;

  assert(NNodes < MAX_NODES);
  NNodes++;

  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  
  return nodeId;
}

int Mesh_addNewElement 
(int nodeId0, int nodeId1, int nodeId2, int nodeId3)
{
  int elementId = NElements;

  assert(0 <= nodeId0);
  assert(nodeId0 < NNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NNodes);
  assert(0 <= nodeId2);
  assert(nodeId2 < NNodes);
  assert(0 <= nodeId3);
  assert(nodeId3 < NNodes);

  assert(NElements < MAX_ELEMENTS);
  NElements++;

  Element_isDeleted[elementId] = 0;

  Element_nodeIds[elementId][0] = nodeId0;
  Element_nodeIds[elementId][1] = nodeId1;
  Element_nodeIds[elementId][2] = nodeId2;
  Element_nodeIds[elementId][3] = nodeId3;

  Element_edgeIds[elementId][0] = -1;
  Element_edgeIds[elementId][1] = -1;
  Element_edgeIds[elementId][2] = -1;
  Element_edgeIds[elementId][3] = -1;

  return elementId;
}

void Mesh_deleteElement (int elementId)
{
  assert(0 <= elementId);
  assert(elementId < NElements);

  Element_isDeleted[elementId] = 1;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    Element_isDeleted[iElement] = 0;
  }

  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadEdgeVertexFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NNodes);
  assert(0 < NElements);
  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int nodeId0, nodeId1;
    
    fscanf (fp, "%d %d %d", 
	    &edgeId,
	    &nodeId0, &nodeId1);
    assert(edgeId == iEdge);

    assert(0 <= nodeId0);
    assert(nodeId0 < NNodes);
    Edge_nodeIds[iEdge][0] = nodeId0;

    assert(0 <= nodeId1);
    assert(nodeId1 < NNodes);
    Edge_nodeIds[iEdge][1] = nodeId1;
  }

  fclose (fp);
}

void ReadElementEdgeFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementEdge;
  int count;
  
  assert(0 < NElements);
  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);
    
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeId;

      fscanf (fp, "%d", 
	      &edgeId);
      assert(0 <= edgeId);
      assert(edgeId < NEdges);
      Element_edgeIds[iElement][iElementEdge] = edgeId;
    }
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1]); 
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int nActualElements;
  int count;

  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  nActualElements = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    nActualElements++;
  }

  fprintf (fp, "%d\n", 
	   nActualElements);
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;

    fprintf (fp, "%d   ", 
	     count); 
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fprintf (fp, "%d ", 
	       Element_nodeIds[iElement][iElementNode]); 
    }
    fprintf (fp, "\n");
    count++;
  }
  
  fclose (fp);
}



/************************
	 Command
*************************/

void MakeMidNodeOnEdges (void)
{
  int iEdge;
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    double midX, midY;
    
    Edge_getMidPoint (iEdge,
		      &midX, &midY);
    Edge_midNodeId[iEdge] = Mesh_addNewNode (midX, midY);
  }
}

void MakeCenterNodeOnElements (void)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    double centerX, centerY;
    
    Element_getCenter (iElement,
		       &centerX, &centerY);
    Element_centerNodeId[iElement] = Mesh_addNewNode (centerX, centerY);
  }
}

void DivideElements (void)
{
  int iElement;
  int nOldElements = NElements;

  for (iElement = 0; iElement < nOldElements; iElement++) {
    int nodeId0 = Element_nodeIds[iElement][0];
    int nodeId1 = Element_nodeIds[iElement][1];
    int nodeId2 = Element_nodeIds[iElement][2];
    int nodeId3 = Element_nodeIds[iElement][3];

    int edgeId0 = Element_edgeIds[iElement][0];
    int edgeId1 = Element_edgeIds[iElement][1];
    int edgeId2 = Element_edgeIds[iElement][2];
    int edgeId3 = Element_edgeIds[iElement][3];

    int edgeNodeId0 = Edge_midNodeId[edgeId0];
    int edgeNodeId1 = Edge_midNodeId[edgeId1];
    int edgeNodeId2 = Edge_midNodeId[edgeId2];
    int edgeNodeId3 = Edge_midNodeId[edgeId3];
    
    int centerNodeId = Element_centerNodeId[iElement];

    assert(edgeNodeId0 != -1);
    assert(edgeNodeId1 != -1);
    assert(edgeNodeId2 != -1);
    assert(edgeNodeId3 != -1);

    assert(centerNodeId != -1);

    Mesh_addNewElement (nodeId0, edgeNodeId0, centerNodeId, edgeNodeId3);
    Mesh_addNewElement (edgeNodeId0, nodeId1, edgeNodeId1, centerNodeId);
    Mesh_addNewElement (edgeNodeId3, centerNodeId, edgeNodeId2, nodeId3);
    Mesh_addNewElement (centerNodeId, edgeNodeId1, nodeId2, edgeNodeId2);
    Mesh_deleteElement (iElement);
  }
}

void PerformCommand (void)
{
  MakeMidNodeOnEdges ();
  MakeCenterNodeOnElements ();
  DivideElements ();
}



int main (int argc, char *argv[])
{
  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_quad2d_divide  node_file  element_file \n"
       "        edge_vertex_file  element_edge_file \n"
       "        node_file_OUT  element_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadEdgeVertexFile (argv[3]);
  ReadElementEdgeFile (argv[4]);

  PerformCommand ();

  WriteNodeFile (argv[5]);
  WriteElementFile (argv[6]);

  return 0;
}


