/* advautomesh_h_quad2d_edge2node_vector_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, 
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
int Node_hasValue[MAX_NODES];
double Node_vector[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_EDGES 4
#define MAX_ELEMENTS 1000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 0, 1 },
  { 1, 2 },
  { 2, 3 },
  { 3, 0 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Edge */

#define MAX_EDGES 2000000

int NEdges;
int Edge_elementIds[MAX_NODES][2];
int Edge_elementEdgeIds[MAX_NODES][2];
int Edge_hasValue[MAX_EDGES];
double Edge_vector[MAX_EDGES][2];



double Edge_length 
(int edgeId)
{
  assert(0 <= edgeId);
  assert(edgeId < NEdges);

  {
    int elementId = Edge_elementIds[edgeId][0];
    int elementEdgeId = Edge_elementEdgeIds[edgeId][0];
    
    int nodeId0 
      = Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][0]];
    int nodeId1 
      = Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][1]];
    
    double x0 = Node_coord[nodeId0][0];
    double y0 = Node_coord[nodeId0][1];
    
    double x1 = Node_coord[nodeId1][0];
    double y1 = Node_coord[nodeId1][1];
    
    return Distance (x0, y0,
		     x1, y1);
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &edgeId,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementEdgeId0);
    assert(elementEdgeId0 < N_ELEMENT_EDGES);

    if (0 <= elementId1) {
      assert(elementId1 < NElements);
      assert(0 <= elementEdgeId1);
      assert(elementEdgeId1 < N_ELEMENT_EDGES);
    } else {
      assert(elementId1 == -1);
      assert(elementEdgeId1 == -1);
    }

    Edge_elementIds[iEdge][0] = elementId0;
    Edge_elementIds[iEdge][1] = elementId1;

    Edge_elementEdgeIds[iEdge][0] = elementEdgeId0;
    Edge_elementEdgeIds[iEdge][1] = elementEdgeId1;
  }

  fclose (fp);
}

void ReadEdgeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  int nItems;
  int iItem;

  assert(0 < NEdges);

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    Edge_hasValue[iEdge] = 0;
    Edge_vector[iEdge][0] = 0.0;
    Edge_vector[iEdge][1] = 0.0;
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int edgeId;
    double valueX, valueY;
    
    fscanf (fp, "%d %d %lf %lf",
	    &itemId, 
	    &edgeId, &valueX, &valueY);
    assert(itemId == iItem);
    assert(0 <= edgeId);
    assert(edgeId < NEdges);
    assert(Edge_elementIds[edgeId][1] == -1);

    Edge_hasValue[edgeId] = 1;
    Edge_vector[edgeId][0] = valueX;
    Edge_vector[edgeId][1] = valueY;
  }

  fclose (fp);
}

void WriteNodeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      fprintf (fp, "%d   %d %f %f\n", 
	       count,
	       iNode, 
	       Node_vector[iNode][0], 
	       Node_vector[iNode][1]);
      count++;
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iEdge;

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_hasValue[iNode] = 0;
    Node_vector[iNode][0] = 0.0;
    Node_vector[iNode][1] = 0.0;
  }

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_elementIds[iEdge][1] == -1) {
      int elementId = Edge_elementIds[iEdge][0];
      int elementEdgeId = Edge_elementEdgeIds[iEdge][0];
      
      int nodeId0 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][0]];
      int nodeId1 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][1]];

      if (Edge_hasValue[iEdge]) {
	double valueX = Edge_vector[iEdge][0];
	double valueY = Edge_vector[iEdge][1];
	
	double length = Edge_length (iEdge);
	
	double nodalValueX = valueX * length / 2;
	double nodalValueY = valueY * length / 2;
	
	Node_hasValue[nodeId0] = 1;
	Node_vector[nodeId0][0] += nodalValueX;
	Node_vector[nodeId0][1] += nodalValueY;
	
	Node_hasValue[nodeId1] = 1;
	Node_vector[nodeId1][0] += nodalValueX;
	Node_vector[nodeId1][1] += nodalValueY;
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_quad2d_edge2node_vector_bc  node_file  element_file \n"
       "        edge_file  edge_vector_bc_file \n"
       "        node_vector_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadEdgeFile (argv[3]);
  ReadEdgeVectorBcFile (argv[4]);

  PerformCommand ();

  WriteNodeVectorBcFile (argv[5]);

  return 0;
}


