/* advautomesh_h_tetra3d10n_mesh2adv.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <Adv/AdvDocument.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 10
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void WriteAdvIOFile (const char *fileName)
{
  AdvDocFile *advFile;
  AdvDocument *doc;
  int iElement;
  int iElementNode;
  int iNode;
  adv_off_t offset;

  advFile = adv_dio_file_open (fileName, "c");



  /* elements */
  
  doc = adv_dio_create (advFile, adv_dio_make_documentid ("Element"));
  
  adv_dio_set_property (doc, "content_type", "Element");
  adv_dio_set_property_int32 (doc, "num_items", NElements);
  adv_dio_set_property (doc, "format", "i4i4i4i4i4i4i4i4i4i4");
  adv_dio_set_property_int32 (doc, "num_nodes_per_element", 10);
  adv_dio_set_property (doc, "element_type", "3DQuadraticTetrahedron");
  adv_dio_set_property_int32 (doc, "dimension", 3);
  adv_dio_set_property_int32 (doc, "index_byte", 4);
  
  offset = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId = Element_nodeIds[iElement][iElementNode];
      
      offset += adv_dio_write_int32 (doc, offset, nodeId);
    }
  }
  
  adv_dio_close (doc);



  /* nodes */
  
  doc = adv_dio_create (advFile, adv_dio_make_documentid ("Node"));

  adv_dio_set_property (doc, "content_type", "Node");
  adv_dio_set_property_int32 (doc, "num_items", NNodes);
  adv_dio_set_property (doc, "format", "f8f8f8");
  adv_dio_set_property_int32 (doc, "dimension", 3);

  offset = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];

    offset += adv_dio_write_float64 (doc, offset, x);
    offset += adv_dio_write_float64 (doc, offset, y);
    offset += adv_dio_write_float64 (doc, offset, z);
  }
  
  adv_dio_close (doc);



  adv_dio_file_close (advFile);
}



int main (int argc, char* argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d10n_mesh2adv \n"
       "        node_file  element_file \n"
       "        advio_file_OUT  \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  
  WriteAdvIOFile (argv[3]);

  return 0;
}

