/* advautomesh_h_tetra3d_section_contour_view2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];
double Node_scalar[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_EDGES 6
#define MAX_ELEMENTS 12000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 0, 1 }, 
  { 0, 2 }, 
  { 0, 3 }, 
  { 1, 2 }, 
  { 2, 3 }, 
  { 3, 1 }  
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Element_getEdgePoint 
(int elementId, 
 int elementEdgeId, double ratio,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  int nodeId0, nodeId1;
  double x0, y0, z0;
  double x1, y1, z1;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);

  nodeId0 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][0]];
  nodeId1 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][1]];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];
  z0 = Node_coord[nodeId0][2];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];
  z1 = Node_coord[nodeId1][2];
  
  *x_OUT = x0 + (x1 - x0) * ratio;
  *y_OUT = y0 + (y1 - y0) * ratio;
  *z_OUT = z0 + (z1 - z0) * ratio;
}

double Element_edgeScalar 
(int elementId, 
 int elementEdgeId, double ratio)
{
  int nodeId0, nodeId1;
  double scalar0, scalar1;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);

  nodeId0 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][0]];
  nodeId1 = Element_nodeIds
    [elementId][EdgeNodeMap[elementEdgeId][1]];

  scalar0 = Node_scalar[nodeId0];
  scalar1 = Node_scalar[nodeId1];
  
  return scalar0 + (scalar1 - scalar0) * ratio;
}

void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double scalar;

    fscanf (fp, "%d %lf", 
	    &nodeId,
	    &scalar);
    assert(nodeId == iNode);
    Node_scalar[iNode] = scalar;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Section */

#define MAX_SECTIONS 1000000

int NSections;
int Section_elementId[MAX_SECTIONS];
int Section_elementEdgeIds[MAX_SECTIONS][3];
double Section_ratios[MAX_SECTIONS][3];



/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][3];
double Frame_coord1[MAX_FRAMES][3];



void ReadSectionFile (const char *fileName)
{
  FILE *fp;
  int iSection;
  
  assert(0 < NElements);
  assert(NSections == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NSections);
  assert(0 < NSections);
  assert(NSections <= MAX_SECTIONS);
  
  for (iSection = 0; iSection < NSections; iSection++) {
    int sectionId;
    int elementId;
    int elementEdgeId0, elementEdgeId1, elementEdgeId2;
    double ratio0, ratio1, ratio2;
    
    fscanf (fp, "%d %d %d %lf %d %lf %d %lf", 
	    &sectionId,
	    &elementId,
	    &elementEdgeId0, &ratio0,
	    &elementEdgeId1, &ratio1,
	    &elementEdgeId2, &ratio2);
    assert(sectionId == iSection);
    assert(0 <= elementId);
    assert(elementId < NElements);
    Section_elementId[iSection] = elementId;
    
    assert(0 <= elementEdgeId0);
    assert(elementEdgeId0 < N_ELEMENT_EDGES);
    Section_elementEdgeIds[iSection][0] = elementEdgeId0;
    Section_ratios[iSection][0] = ratio0;
    
    assert(0 <= elementEdgeId1);
    assert(elementEdgeId1 < N_ELEMENT_EDGES);
    Section_elementEdgeIds[iSection][1] = elementEdgeId1;
    Section_ratios[iSection][1] = ratio1;
    
    assert(0 <= elementEdgeId2);
    assert(elementEdgeId2 < N_ELEMENT_EDGES);
    Section_elementEdgeIds[iSection][2] = elementEdgeId2;
    Section_ratios[iSection][2] = ratio2;
  }

  fclose (fp);
}

void ReadFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(NFrames == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFrames);
  assert(0 <= NFrames);
  assert(NFrames <= MAX_FRAMES);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    int frameId;
    double x0, y0, z0;
    double x1, y1, z1;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	    &frameId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1);
    assert(frameId == iFrame);
    Frame_coord0[iFrame][0] = x0;
    Frame_coord0[iFrame][1] = y0;
    Frame_coord0[iFrame][2] = z0;

    Frame_coord1[iFrame][0] = x1;
    Frame_coord1[iFrame][1] = y1;
    Frame_coord1[iFrame][2] = z1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

double MinRange = 0.0;
double MaxRange = 1.0;
int Edge = 0;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void SetScalarRange (void)
{
  int iNode;

  assert(0 < NNodes);

  MinRange = MaxRange = Node_scalar[0];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double scalar = Node_scalar[iNode];
    
    if (scalar < MinRange) MinRange = scalar;
    if (MaxRange < scalar) MaxRange = scalar;
  }
}

void DrawFrame (void)
{
  int iFrame;

  AutoGL_SetColor (1, 1, 1);
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    AutoGL_DrawLine (Frame_coord0[iFrame][0],
		     Frame_coord0[iFrame][1],
		     Frame_coord0[iFrame][2],
		     Frame_coord1[iFrame][0],
		     Frame_coord1[iFrame][1],
		     Frame_coord1[iFrame][2]);
  }
}

#if 1
#include "../../util/time_measure.c"
#define TIME_MEASURE
#endif

void DrawContourTriangle
(double x0, double y0, double z0, double grade0, 
 double x1, double y1, double z1, double grade1,
 double x2, double y2, double z2, double grade2)
{
  const double gradeDifference = 0.1;
  
  if (fabs (grade0 - grade1) < gradeDifference
      && fabs (grade1 - grade2) < gradeDifference
      && fabs (grade2 - grade0) < gradeDifference) {
    AutoGL_DrawContourTriangle (x0, y0, z0, grade0,
				x1, y1, z1, grade1,
				x2, y2, z2, grade2);
  } else {
    double x01 = (x0 + x1) / 2;
    double y01 = (y0 + y1) / 2;
    double z01 = (z0 + z1) / 2;

    double x12 = (x1 + x2) / 2;
    double y12 = (y1 + y2) / 2;
    double z12 = (z1 + z2) / 2;

    double x20 = (x2 + x0) / 2;
    double y20 = (y2 + y0) / 2;
    double z20 = (z2 + z0) / 2;

    double grade01 = (grade0 + grade1) / 2;
    double grade12 = (grade1 + grade2) / 2;
    double grade20 = (grade2 + grade0) / 2;

    DrawContourTriangle (x0, y0, z0, grade0,
			 x01, y01, z01, grade01,
			 x20, y20, z20, grade20);
    DrawContourTriangle (x1, y1, z1, grade1,
			 x12, y12, z12, grade12,
			 x01, y01, z01, grade01);
    DrawContourTriangle (x2, y2, z2, grade2,
			 x20, y20, z20, grade20,
			 x12, y12, z12, grade12);
    DrawContourTriangle (x01, y01, z01, grade01,
			 x12, y12, z12, grade12,
			 x20, y20, z20, grade20);
  }
}

void RedrawView (void) 
{
  int iSection;

#ifdef TIME_MEASURE
  StartMeasureTime ();
#endif

  DrawFrame ();

  for (iSection = 0; iSection < NSections; iSection++) {
    double ex0, ey0, ez0;
    double ex1, ey1, ez1;
    double ex2, ey2, ez2;
    double scalar0, scalar1, scalar2;
    double grade0, grade1, grade2;
    
    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][0],
       Section_ratios[iSection][0],
       &ex0, &ey0, &ez0);
    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][1],
       Section_ratios[iSection][1],
       &ex1, &ey1, &ez1);
    Element_getEdgePoint 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][2],
       Section_ratios[iSection][2],
       &ex2, &ey2, &ez2);
    
    scalar0 = Element_edgeScalar 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][0],
       Section_ratios[iSection][0]);
    scalar1 = Element_edgeScalar 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][1],
       Section_ratios[iSection][1]);
    scalar2 = Element_edgeScalar 
      (Section_elementId[iSection], 
       Section_elementEdgeIds[iSection][2],
       Section_ratios[iSection][2]);

    grade0 = (scalar0 - MinRange) / (MaxRange - MinRange); 
    grade1 = (scalar1 - MinRange) / (MaxRange - MinRange); 
    grade2 = (scalar2 - MinRange) / (MaxRange - MinRange); 

    if (Edge) {
      AutoGL_SetColor (1, 1, 1);
      AutoGL_DrawLine (ex0, ey0, ez0,
		       ex1, ey1, ez1);
      AutoGL_DrawLine (ex1, ey1, ez1,
		       ex2, ey2, ez2);
      AutoGL_DrawLine (ex2, ey2, ez2,
		       ex0, ey0, ez0);
    }

    DrawContourTriangle (ex0, ey0, ez0, grade0,
			 ex1, ey1, ez1, grade1,
			 ex2, ey2, ez2, grade2);
  }

#ifdef TIME_MEASURE
  EndMeasureTime ();
#endif
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_section_contour_view2  node_file  element_file \n"
       "     node_scalar_file  section_file  frame_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeScalarFile (argv[3]);
  ReadSectionFile (argv[4]);
  ReadFrameFile (argv[5]);

  SetView ();
  SetScalarRange ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddReal (&MinRange, "MinRange");
  AutoGL_AddReal (&MaxRange, "MaxRange");
  AutoGL_AddBoolean (&Edge, "Edge");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
