/* advautomesh_h_tetra3d_section_isoline_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
	 Other Data
*************************/

/* Isoline */

#define MAX_LEVELS 10000
#define MAX_ISOLINES 1000000

int NLevels;
int Level_startIsolineId[MAX_LEVELS];
int Level_nIsolines[MAX_LEVELS];
int NIsolines;
double Isoline_coord0[MAX_ISOLINES][3];
double Isoline_coord1[MAX_ISOLINES][3];

double MinLevel, MaxLevel;
int NDivisions;



/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][3];
double Frame_coord1[MAX_FRAMES][3];



void FrameGroup_getRange 
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iFrame;

  assert(0 < NFrames);

  *minRangeX_OUT = *maxRangeX_OUT = Frame_coord0[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Frame_coord0[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Frame_coord0[0][2];
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    double x0 = Frame_coord0[iFrame][0];
    double y0 = Frame_coord0[iFrame][1];
    double z0 = Frame_coord0[iFrame][2];
    
    double x1 = Frame_coord1[iFrame][0];
    double y1 = Frame_coord1[iFrame][1];
    double z1 = Frame_coord1[iFrame][2];

    if (x0 < *minRangeX_OUT) *minRangeX_OUT = x0;
    if (y0 < *minRangeY_OUT) *minRangeY_OUT = y0;
    if (z0 < *minRangeZ_OUT) *minRangeZ_OUT = z0;

    if (*maxRangeX_OUT < x0) *maxRangeX_OUT = x0;
    if (*maxRangeY_OUT < y0) *maxRangeY_OUT = y0;
    if (*maxRangeZ_OUT < z0) *maxRangeZ_OUT = z0;

    if (x1 < *minRangeX_OUT) *minRangeX_OUT = x1;
    if (y1 < *minRangeY_OUT) *minRangeY_OUT = y1;
    if (z1 < *minRangeZ_OUT) *minRangeZ_OUT = z1;

    if (*maxRangeX_OUT < x1) *maxRangeX_OUT = x1;
    if (*maxRangeY_OUT < y1) *maxRangeY_OUT = y1;
    if (*maxRangeZ_OUT < z1) *maxRangeZ_OUT = z1;
  }
}

void ReadSectionIsolineFile (const char *fileName)
{
  FILE *fp;
  int iLevel;
  int iIsoline;
  int count;
  
  assert(NLevels == 0);
  assert(NIsolines == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NLevels);
  assert(0 < NLevels);
  assert(NLevels <= MAX_LEVELS);
  
  count = 0;
  for (iLevel = 0; iLevel < NLevels; iLevel++) {
    int levelId;
    int startId;
    int nIsolines;

    fscanf (fp, "%d %d", 
	    &levelId,
	    &nIsolines);
    assert(levelId == iLevel);
    assert(0 <= nIsolines);
    startId = count;
    count += nIsolines;
    assert(count <= MAX_ISOLINES);
    Level_startIsolineId[iLevel] = startId;
    Level_nIsolines[iLevel] = nIsolines;

    for (iIsoline = startId; 
	 iIsoline < startId + nIsolines; 
	 iIsoline++) {
      int isolineId;
      double x0, y0, z0;
      double x1, y1, z1;
      
      fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	      &isolineId,
	      &x0, &y0, &z0,
	      &x1, &y1, &z1);
      assert(isolineId == iIsoline - startId);
      Isoline_coord0[iIsoline][0] = x0;
      Isoline_coord0[iIsoline][1] = y0;
      Isoline_coord0[iIsoline][2] = z0;
      
      Isoline_coord1[iIsoline][0] = x1;
      Isoline_coord1[iIsoline][1] = y1;
      Isoline_coord1[iIsoline][2] = z1;
    }
  }

  fclose (fp);
}

void ReadFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(NFrames == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFrames);
  assert(0 <= NFrames);
  assert(NFrames <= MAX_FRAMES);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    int frameId;
    double x0, y0, z0;
    double x1, y1, z1;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	    &frameId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1);
    assert(frameId == iFrame);
    Frame_coord0[iFrame][0] = x0;
    Frame_coord0[iFrame][1] = y0;
    Frame_coord0[iFrame][2] = z0;

    Frame_coord1[iFrame][0] = x1;
    Frame_coord1[iFrame][1] = y1;
    Frame_coord1[iFrame][2] = z1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  FrameGroup_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void DrawFrame (void)
{
  int iFrame;

  AutoGL_SetColor (1, 1, 1);
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    AutoGL_DrawLine (Frame_coord0[iFrame][0],
		     Frame_coord0[iFrame][1],
		     Frame_coord0[iFrame][2],
		     Frame_coord1[iFrame][0],
		     Frame_coord1[iFrame][1],
		     Frame_coord1[iFrame][2]);
  }
}

void RedrawView (void) 
{
  int iLevel;
  int iIsoline;

  AutoGL_DrawLineContourMap (80, MinLevel, MaxLevel, NDivisions);

  DrawFrame ();

  for (iLevel = 0; iLevel < NLevels; iLevel++) {
    int startId = Level_startIsolineId[iLevel];
    int nIsolines = Level_nIsolines[iLevel];
    double grade = 1.0 / NDivisions * iLevel;
    double red, green, blue;

    AutoGL_GetContourColor (&red, &green, &blue,
			    grade);
    AutoGL_SetColor (red, green, blue);
    
    for (iIsoline = startId; 
	 iIsoline < startId + nIsolines; 
	 iIsoline++) {
      double ex0 = Isoline_coord0[iIsoline][0];
      double ey0 = Isoline_coord0[iIsoline][1];
      double ez0 = Isoline_coord0[iIsoline][2];
      
      double ex1 = Isoline_coord1[iIsoline][0];
      double ey1 = Isoline_coord1[iIsoline][1];
      double ez1 = Isoline_coord1[iIsoline][2];
      
      AutoGL_DrawLine (ex0, ey0, ez0,
		       ex1, ey1, ez1);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_section_isoline_view \n"
       "     section_isoline_file  frame_file \n"
       "     minLevel  maxLevel  nDivisions \n");
    exit (1);
  }

  ReadSectionIsolineFile (argv[1]);
  ReadFrameFile (argv[2]);

  MinLevel = atof (argv[3]);
  MaxLevel = atof (argv[4]);
  NDivisions = atoi (argv[5]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
