/* advautomesh_h_tetra3d10n_face2node_dof_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 3
#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_hasValue[MAX_NODES][N_NODE_DOFS];
double Node_value[MAX_NODES][N_NODE_DOFS];



/* Element */

#define N_ELEMENT_NODES 10
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 1000000

const int FaceNodeMap[N_ELEMENT_FACES][6] = {
  { 1, 2, 3, 8, 9, 7 }, 
  { 0, 3, 2, 8, 5, 6 }, 
  { 0, 1, 3, 9, 6, 4 }, 
  { 0, 2, 1, 7, 4, 5 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Face */

#define MAX_FACES 1000000

int NFaces;
int Face_elementIds[MAX_NODES][2];
int Face_elementFaceIds[MAX_NODES][2];
int Face_hasValue[MAX_FACES][N_NODE_DOFS];
double Face_value[MAX_FACES][N_NODE_DOFS];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  
  assert(NFaces == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &faceId,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);

    if (0 <= elementId1) {
      assert(elementId1 < NElements);
      assert(0 <= elementFaceId1);
      assert(elementFaceId1 < N_ELEMENT_FACES);
    } else {
      assert(elementId1 == -1);
      assert(elementFaceId1 == -1);
    }

    Face_elementIds[iFace][0] = elementId0;
    Face_elementIds[iFace][1] = elementId1;

    Face_elementFaceIds[iFace][0] = elementFaceId0;
    Face_elementFaceIds[iFace][1] = elementFaceId1;
  }

  fclose (fp);
}

void ReadFaceDofBcFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int iDof;
  int nItems;
  int iItem;

  assert(0 < NFaces);

  for (iFace = 0; iFace < NFaces; iFace++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Face_hasValue[iFace][iDof] = 0;
      Face_value[iFace][iDof] = 0.0;
    }
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int faceId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &faceId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= faceId);
    assert(faceId < NFaces);
    assert(Face_elementIds[faceId][1] == -1);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Face_hasValue[faceId][dofId] = 1;
    Face_value[faceId][dofId] = value;
  }

  fclose (fp);
}

void WriteNodeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iDof;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	count++;
      }
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	fprintf (fp, "%d   %d %d %f\n", 
		 count,
		 iNode, iDof, Node_value[iNode][iDof]);
	count++;
      }
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iDof;
  int iFace;

  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Node_hasValue[iNode][iDof] = 0;
      Node_value[iNode][iDof] = 0.0;
    }
  }

  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_elementIds[iFace][1] == -1) {
      int elementId = Face_elementIds[iFace][0];
      int elementFaceId = Face_elementFaceIds[iFace][0];
    
      int nodeId0 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][0]];
      int nodeId1 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][1]];
      int nodeId2 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][2]];

      int nodeId3 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][3]];
      int nodeId4 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][4]];
      int nodeId5 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][5]];

      for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
	if (Face_hasValue[iFace][iDof]) {
	  double value = Face_value[iFace][iDof];

	  Node_hasValue[nodeId0][iDof] = 1;
	  Node_value[nodeId0][iDof] = value;
	  
	  Node_hasValue[nodeId1][iDof] = 1;
	  Node_value[nodeId1][iDof] = value;
	  
	  Node_hasValue[nodeId2][iDof] = 1;
	  Node_value[nodeId2][iDof] = value;

	  Node_hasValue[nodeId3][iDof] = 1;
	  Node_value[nodeId3][iDof] = value;
	  
	  Node_hasValue[nodeId4][iDof] = 1;
	  Node_value[nodeId4][iDof] = value;
	  
	  Node_hasValue[nodeId5][iDof] = 1;
	  Node_value[nodeId5][iDof] = value;
	}
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d10n_face2node_dof_bc  node_file  element_file \n"
       "        face_file  face_dof_bc_file \n"
       "        node_dof_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadFaceFile (argv[3]);
  ReadFaceDofBcFile (argv[4]);

  PerformCommand ();

  WriteNodeDofBcFile (argv[5]);

  return 0;
}


