/* advautomesh_h_tetra3d_add_face_vector_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Face */

#define MAX_FACES 1000000

int NFaces;
int Face_hasValue[MAX_FACES];
double Face_vector[MAX_FACES][3];



void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;

  assert(NFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  /* skip data */

  fclose (fp);



  for (iFace = 0; iFace < NFaces; iFace++) {
    Face_hasValue[iFace] = 0;
    Face_vector[iFace][0] = 0.0;
    Face_vector[iFace][1] = 0.0;
    Face_vector[iFace][2] = 0.0;
  }
}

void ReadFaceVectorBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NFaces);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int faceId;
    double valueX, valueY, valueZ;
    
    fscanf (fp, "%d %d %lf %lf %lf",
	    &itemId, 
	    &faceId, &valueX, &valueY, &valueZ);
    assert(itemId == iItem);
    assert(0 <= faceId);
    assert(faceId < NFaces);

    Face_hasValue[faceId] = 1;
    Face_vector[faceId][0] = valueX;
    Face_vector[faceId][1] = valueY;
    Face_vector[faceId][2] = valueZ;
  }

  fclose (fp);
}

void WriteFaceVectorBcFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int count;

  assert(0 < NFaces);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_hasValue[iFace]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_hasValue[iFace]) {
      fprintf (fp, "%d   %d %f %f %f\n", 
	       count,
	       iFace, 
	       Face_vector[iFace][0], 
	       Face_vector[iFace][1], 
	       Face_vector[iFace][2]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_add_face_vector_bc \n"
       "        face_file \n"
       "        face_vector_bc_file_A  face_vector_bc_file_B \n"
       "        face_vector_bc_file_OUT \n");
    exit (1);
  }

  ReadFaceFile (argv[1]);
  ReadFaceVectorBcFile (argv[2]);
  ReadFaceVectorBcFile (argv[3]);

  WriteFaceVectorBcFile (argv[4]);

  return 0;
}


