/* advautomesh_h_tetra3d_add_node_vector_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
int Node_hasValue[MAX_NODES];
double Node_vector[MAX_NODES][3];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate value */

  fclose (fp);



  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_hasValue[iNode] = 0;
    Node_vector[iNode][0] = 0.0;
    Node_vector[iNode][1] = 0.0;
    Node_vector[iNode][2] = 0.0;
  }
}

void ReadNodeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int nodeId;
    double valueX, valueY, valueZ;
    
    fscanf (fp, "%d %d %lf %lf %lf",
	    &itemId, 
	    &nodeId, &valueX, &valueY, &valueZ);
    assert(itemId == iItem);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);

    Node_hasValue[nodeId] = 1;
    Node_vector[nodeId][0] = valueX;
    Node_vector[nodeId][1] = valueY;
    Node_vector[nodeId][2] = valueZ;
  }

  fclose (fp);
}

void WriteNodeVectorBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_hasValue[iNode]) {
      fprintf (fp, "%d   %d %f %f %f\n", 
	       count,
	       iNode, 
	       Node_vector[iNode][0], 
	       Node_vector[iNode][1], 
	       Node_vector[iNode][2]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_add_node_vector_bc \n"
       "        node_file \n"
       "        node_vector_bc_file_A  node_vector_bc_file_B \n"
       "        node_vector_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadNodeVectorBcFile (argv[2]);
  ReadNodeVectorBcFile (argv[3]);

  WriteNodeVectorBcFile (argv[4]);

  return 0;
}


