/* advautomesh_h_tetra3d_boundary.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_isOnBoundary[MAX_NODES];
int Node_newId[MAX_NODES];
int NBoundaryNodes;



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 5000000

const int FaceNodeMap[N_ELEMENT_FACES][3] = {
  { 1, 2, 3 },
  { 0, 3, 2 },
  { 0, 1, 3 },
  { 0, 2, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}




/************************
	 Other Data
*************************/

/* Boundary Face */

#define N_FACE_NODES 3
#define MAX_BOUNDARY_FACES 30000000

int NBoundaryFaces;
int BoundaryFace_faceIds[MAX_BOUNDARY_FACES];
int BoundaryFace_nodeIds[MAX_BOUNDARY_FACES][N_FACE_NODES];
int BoundaryFace_elementIds[MAX_BOUNDARY_FACES];
int BoundaryFace_elementFaceIds[MAX_BOUNDARY_FACES];



void ReadFaceFileIntoBoundary (const char *fileName)
{
  FILE *fp;
  int iFace;
  int nFaces;
  
  assert(0 < NNodes);
  assert(0 < NElements);
  assert(NBoundaryFaces == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nFaces);
  assert(0 < nFaces);
  
  NBoundaryFaces = 0;
  for (iFace = 0; iFace < nFaces; iFace++) {
    int faceId;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;
    
    fscanf (fp, "%d %d %d %d %d", 
	    &faceId,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);

    if (elementId1 != -1) continue;

    assert(NBoundaryFaces < MAX_BOUNDARY_FACES);

    BoundaryFace_faceIds[NBoundaryFaces] = iFace;

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);

    BoundaryFace_elementIds[NBoundaryFaces] = elementId0;
    BoundaryFace_elementFaceIds[NBoundaryFaces] = elementFaceId0;

    BoundaryFace_nodeIds[NBoundaryFaces][0]
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][0]];
    BoundaryFace_nodeIds[NBoundaryFaces][1]
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][1]];
    BoundaryFace_nodeIds[NBoundaryFaces][2]
      = Element_nodeIds[elementId0][FaceNodeMap[elementFaceId0][2]];

    NBoundaryFaces++;
  }

  fclose (fp);
}

void WriteBoundaryNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NBoundaryNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      fprintf (fp, "%d   %22.15e %22.15e %22.15e\n",
	       Node_newId[iNode],
	       Node_coord[iNode][0], 
	       Node_coord[iNode][1], 
	       Node_coord[iNode][2]);
    }
  }

  fclose (fp);
}

void WriteBoundaryFaceFile (const char *fileName)
{
  FILE *fp;
  int iBoundaryFace;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NBoundaryFaces);

  for (iBoundaryFace = 0; 
       iBoundaryFace < NBoundaryFaces; 
       iBoundaryFace++) {
    int nodeId0 = BoundaryFace_nodeIds[iBoundaryFace][0];
    int nodeId1 = BoundaryFace_nodeIds[iBoundaryFace][1];
    int nodeId2 = BoundaryFace_nodeIds[iBoundaryFace][2];
    
    fprintf (fp, "%d   %d %d %d\n",
	     iBoundaryFace,
	     Node_newId[nodeId0], 
	     Node_newId[nodeId1], 
	     Node_newId[nodeId2]);
  }

  fclose (fp);
}

void WriteBoundaryNodeSrcFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NBoundaryNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      fprintf (fp, "%d   %d\n",
	       Node_newId[iNode],
	       iNode);
    }
  }

  fclose (fp);
}

void WriteBoundaryFaceSrcFile (const char *fileName)
{
  FILE *fp;
  int iBoundaryFace;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n",
	   NBoundaryFaces);

  for (iBoundaryFace = 0; 
       iBoundaryFace < NBoundaryFaces; 
       iBoundaryFace++) {
    fprintf (fp, "%d   %d %d %d\n",
	     iBoundaryFace,
	     BoundaryFace_faceIds[iBoundaryFace],
	     BoundaryFace_elementIds[iBoundaryFace],
	     BoundaryFace_elementFaceIds[iBoundaryFace]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iBoundaryFace;
  int iBoundaryFaceNode;
  int count;

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOnBoundary[iNode] = 0;
  }

  for (iBoundaryFace = 0; 
       iBoundaryFace < NBoundaryFaces; 
       iBoundaryFace++) {
    for (iBoundaryFaceNode = 0; 
	 iBoundaryFaceNode < N_FACE_NODES; 
	 iBoundaryFaceNode++) {
      int nodeId = BoundaryFace_nodeIds[iBoundaryFace][iBoundaryFaceNode];
      
      Node_isOnBoundary[nodeId] = 1;
    }
  }

  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      Node_newId[iNode] = count;
      count++;
    } else {
      Node_newId[iNode] = -1;
    }
  }
  NBoundaryNodes = count;
}



int main (int argc, char *argv[])
{
  if (argc != 8) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_boundary  node_file  element_file \n"
       "        face_file \n"
       "        boundary_node_file_OUT  boundary_face_file_OUT \n"
       "        boundary_node_src_file_OUT  boundary_face_src_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadFaceFileIntoBoundary (argv[3]);

  PerformCommand ();

  WriteBoundaryNodeFile (argv[4]);
  WriteBoundaryFaceFile (argv[5]);
  WriteBoundaryNodeSrcFile (argv[6]);
  WriteBoundaryFaceSrcFile (argv[7]);

  return 0;
}
