/* advautomesh_h_tetra3d_boundary_vector.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_vector[MAX_NODES][3];
int Node_isOnBoundary[MAX_NODES];



void ReadNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double vectorX, vectorY, vectorZ;
    
    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId,
	    &vectorX, &vectorY, &vectorZ);
    assert(nodeId == iNode);
    Node_vector[iNode][0] = vectorX;
    Node_vector[iNode][1] = vectorY;
    Node_vector[iNode][2] = vectorZ;
  }

  fclose (fp);
}

void ReadBoundaryNodeSrcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iBoundaryNode;
  int nBoundaryNodes;

  assert(0 < NNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOnBoundary[iNode] = 0;
  }
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d",
	  &nBoundaryNodes);
  
  for (iBoundaryNode = 0; 
       iBoundaryNode < nBoundaryNodes; 
       iBoundaryNode++) {
    int boundaryNodeId;
    int nodeId;

    fscanf (fp, "%d %d",
	    &boundaryNodeId,
	    &nodeId);
    assert(boundaryNodeId == iBoundaryNode);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    Node_isOnBoundary[nodeId] = 1;
  }

  fclose (fp);
}

void WriteBoundaryNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int nBoundaryNodes;
  int count;
  
  assert(0 < NNodes);

  nBoundaryNodes = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      nBoundaryNodes++;
    }
  }

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   nBoundaryNodes);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnBoundary[iNode]) {
      fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	       count, 
	       Node_vector[iNode][0],
	       Node_vector[iNode][1],
	       Node_vector[iNode][2]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_boundary_vector  \n"
       "        node_vector_file  boundary_node_src_file  \n"
       "        boundary_node_vector_file_OUT \n");
    exit (1);
  }

  ReadNodeVectorFile (argv[1]);
  ReadBoundaryNodeSrcFile (argv[2]);

  WriteBoundaryNodeVectorFile (argv[3]);

  return 0;
}
