/* advautomesh_h_tetra3d_delaunay3.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



#if 0
#define PRINT
#endif
#if 1
#define PRINT_NODE_ID
#endif

#if 1
#define MORE_DUMMY_NODE
#endif



/************************
	 Basic
*************************/

#if 0
#define DEFAULT_EPS 0.00001
#else
#define DEFAULT_EPS 0.0000001
#endif

double Eps = DEFAULT_EPS;



void SortId3 (int id0, int id1, int id2,
	      int *minId_OUT, int *midId_OUT, int *maxId_OUT)
{
  assert(id0 != id1);
  assert(id0 != id2);
  assert(id1 != id2);

  if (id0 < id1 && id0 < id2) {
    *minId_OUT = id0;
  }
  if (id1 < id0 && id1 < id2) {
    *minId_OUT = id1;
  }
  if (id2 < id0 && id2 < id1) {
    *minId_OUT = id2;
  }

  if ((id1 < id0 && id0 < id2)
      || (id2 < id0 && id0 < id1)) {
    *midId_OUT = id0;
  }
  if ((id0 < id1 && id1 < id2)
      || (id2 < id1 && id1 < id0)) {
    *midId_OUT = id1;
  }
  if ((id0 < id2 && id2 < id1)
      || (id1 < id2 && id2 < id0)) {
    *midId_OUT = id2;
  }

  if (id1 < id0 && id2 < id0) {
    *maxId_OUT = id0;
  }
  if (id0 < id1 && id2 < id1) {
    *maxId_OUT = id1;
  }
  if (id0 < id2 && id1 < id2) {
    *maxId_OUT = id2;
  }
}

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}

void GetNormalizedVector 
(double x, double y, double z,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  double length = sqrt (x * x + y * y + z * z);

  *nx_OUT = x / length;
  *ny_OUT = y / length;
  *nz_OUT = z / length;
}

void GetVectorProduct 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  *x_OUT = y0 * z1 - z0 * y1;
  *y_OUT = z0 * x1 - x0 * z1;
  *z_OUT = x0 * y1 - y0 * x1;
}

void GetPlaneEquation 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double *a_OUT, double *b_OUT, double *c_OUT, double *d_OUT)
{
  double nx, ny, nz;

  GetVectorProduct (x1 - x0, y1 - y0, z1 - z0,
		    x2 - x0, y2 - y0, z2 - z0,
		    &nx, &ny, &nz);
  GetNormalizedVector (nx, ny, nz,
		       &nx, &ny, &nz);
  *a_OUT = nx;
  *b_OUT = ny;
  *c_OUT = nz;
  *d_OUT = -(nx * x0 + ny * y0 + nz * z0);
}

void GetTetraCircumcenter 
(double x0, double y0, double z0, 
 double x1, double y1, double z1, 
 double x2, double y2, double z2,
 double x3, double y3, double z3,
 double *centerX_OUT, double *centerY_OUT, double *centerZ_OUT)
{
  double ax = x0;
  double ay = y0;
  double az = z0;
  double bx = x1;
  double by = y1;
  double bz = z1;
  double cx = x2;
  double cy = y2;
  double cz = z2;
  double dx = x3;
  double dy = y3;
  double dz = z3;
  
  double a1 = (ax - bx) * 2;
  double a2 = (ax - cx) * 2;
  double b1 = (ay - by) * 2;
  double b2 = (ay - cy) * 2;
  double c1 = (az - bz) * 2;
  double c2 = (az - cz) * 2;
  double d1 = ax * ax + ay * ay + az * az 
    - bx * bx - by * by - bz * bz;
  double d2 = ax * ax + ay * ay + az * az 
    - cx * cx - cy * cy - cz * cz;
  
  double a3 = (ax - dx) * 2;
  double b3 = (ay - dy) * 2;
  double c3 = (az - dz) * 2;
  double d3 = ax * ax + ay * ay + az * az
    - dx * dx - dy * dy - dz * dz;

  double detA = a1 * b2 * c3 + a2 * b3 * c1 + a3 * b1 * c2
    - a3 * b2 * c1 - a2 * b1 * c3 - a1 * b3 * c2;
  double detX = d1 * b2 * c3 + d2 * b3 * c1 + d3 * b1 * c2
    - d3 * b2 * c1 - d2 * b1 * c3 - d1 * b3 * c2;
  double detY = a1 * d2 * c3 + a2 * d3 * c1 + a3 * d1 * c2
    - a3 * d2 * c1 - a2 * d1 * c3 - a1 * d3 * c2;
  double detZ = a1 * b2 * d3 + a2 * b3 * d1 + a3 * b1 * d2
    - a3 * b2 * d1 - a2 * b1 * d3 - a1 * b3 * d2;

  double ansX = detX / detA;
  double ansY = detY / detA;
  double ansZ = detZ / detA;

  *centerX_OUT = ansX;
  *centerY_OUT = ansY;
  *centerZ_OUT = ansZ;
}

double SignedTetraVolume
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double x3, double y3, double z3)
{
  double x10 = x1 - x0;
  double y10 = y1 - y0;
  double z10 = z1 - z0;

  double x20 = x2 - x0;
  double y20 = y2 - y0;
  double z20 = z2 - z0;
  
  double x30 = x3 - x0;
  double y30 = y3 - y0;
  double z30 = z3 - z0;
  
  double d23 = y20 * z30 - z20 * y30;
  double d31 = y30 * z10 - z30 * y10;
  double d12 = y10 * z20 - z10 * y20;
  
  return (x10 * d23 + x20 * d31 + x30 * d12) / 6;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
int NAllNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 5000000

const int FaceNodeMap[N_ELEMENT_FACES][3] = {
  { 1, 2, 3 },
  { 0, 3, 2 },
  { 0, 1, 3 },
  { 0, 2, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_FACES];
double Element_circumcenter[MAX_ELEMENTS][3];
double Element_circumradius[MAX_ELEMENTS];

#define MAX_DELETED_ELEMENTS 10000

int NDeletedElements;
int DeletedElement_elementId[MAX_DELETED_ELEMENTS];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

int Mesh_addDummyNode (double x, double y, double z)
{
  int nodeId = NAllNodes;
  
  assert(NAllNodes < MAX_NODES);
  NAllNodes++;
  
  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  Node_coord[nodeId][2] = z;

  return nodeId;
}

int Mesh_nodeIsDummy (int nodeId)
{
  assert(0 <= nodeId);
  assert(nodeId < NAllNodes);
  
  return NNodes <= nodeId;
}

static void CalcElementCircumsphere (int elementId)
{
  int n0 = Element_nodeIds[elementId][0];
  int n1 = Element_nodeIds[elementId][1];
  int n2 = Element_nodeIds[elementId][2];
  int n3 = Element_nodeIds[elementId][3];
  
  double x0 = Node_coord[n0][0];
  double y0 = Node_coord[n0][1];
  double z0 = Node_coord[n0][2];

  double x1 = Node_coord[n1][0];
  double y1 = Node_coord[n1][1];
  double z1 = Node_coord[n1][2];

  double x2 = Node_coord[n2][0];
  double y2 = Node_coord[n2][1];
  double z2 = Node_coord[n2][2];

  double x3 = Node_coord[n3][0];
  double y3 = Node_coord[n3][1];
  double z3 = Node_coord[n3][2];

  double centerX, centerY, centerZ;
  double radius;

  GetTetraCircumcenter 
    (x0, y0, z0, 
     x1, y1, z1,
     x2, y2, z2,
     x3, y3, z3,
     &centerX, &centerY, &centerZ);
  radius = Distance (centerX, centerY, centerZ,
		     x0, y0, z0);

  Element_circumcenter[elementId][0] = centerX;
  Element_circumcenter[elementId][1] = centerY;
  Element_circumcenter[elementId][2] = centerZ;
  Element_circumradius[elementId] = radius;
}

void Mesh_deleteElement (int elementId)
{
  int iNode;
  int iFace;

  assert(0 <= elementId);
  assert(elementId < NElements);

  assert(NDeletedElements < MAX_DELETED_ELEMENTS);
  
  DeletedElement_elementId[NDeletedElements] = elementId;
  NDeletedElements++;

  /* clear memory of <elementId> */

  for (iNode = 0; iNode < N_ELEMENT_NODES; iNode++) {
    Element_nodeIds[elementId][iNode] = -1;
  }
  
  for (iFace = 0; iFace < N_ELEMENT_FACES; iFace++) {
    Element_neighborIds[elementId][iFace] = -1;
  }
}

static int GetElementIdFromDeletedPool (void)
{
  int elementId = -1;

  if (0 < NDeletedElements) {
    elementId = DeletedElement_elementId[NDeletedElements - 1];
    NDeletedElements--;

    assert(0 <= elementId);
    assert(elementId < NElements);
  }
  return elementId;
}

int Mesh_elementIsDeleted (int elementId)
{
  int iDeletedElement;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(NDeletedElements < MAX_DELETED_ELEMENTS);
  
  for (iDeletedElement = 0; 
       iDeletedElement < NDeletedElements; 
       iDeletedElement++) {
    if (DeletedElement_elementId[iDeletedElement] == elementId) {
      return 1;
    }
  }
  return 0;
}

int Mesh_addNewElement 
(int nodeId0, int nodeId1, int nodeId2, int nodeId3)
{
  int elementId;
  int iFace;

  assert(0 <= nodeId0);
  assert(nodeId0 < NAllNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NAllNodes);
  assert(0 <= nodeId2);
  assert(nodeId2 < NAllNodes);
  assert(0 <= nodeId3);
  assert(nodeId3 < NAllNodes);

  elementId = GetElementIdFromDeletedPool ();
  if (elementId == -1) {
    elementId = NElements;
    
    assert(NElements < MAX_ELEMENTS);
    NElements++;
  }

  Element_nodeIds[elementId][0] = nodeId0;
  Element_nodeIds[elementId][1] = nodeId1;
  Element_nodeIds[elementId][2] = nodeId2;
  Element_nodeIds[elementId][3] = nodeId3;

  for (iFace = 0; iFace < N_ELEMENT_FACES; iFace++) {
    Element_neighborIds[elementId][iFace] = -1;
  }

  CalcElementCircumsphere (elementId);

  return elementId;
}

void Mesh_overwriteElement 
(int elementId,
 int nodeId0, int nodeId1, int nodeId2, int nodeId3)
{
  int iFace;

  assert(0 <= elementId);
  assert(elementId < NElements);

  assert(0 <= nodeId0);
  assert(nodeId0 < NAllNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NAllNodes);
  assert(0 <= nodeId2);
  assert(nodeId2 < NAllNodes);
  assert(0 <= nodeId3);
  assert(nodeId3 < NAllNodes);

  Element_nodeIds[elementId][0] = nodeId0;
  Element_nodeIds[elementId][1] = nodeId1;
  Element_nodeIds[elementId][2] = nodeId2;
  Element_nodeIds[elementId][3] = nodeId3;

  for (iFace = 0; iFace < N_ELEMENT_FACES; iFace++) {
    Element_neighborIds[elementId][iFace] = -1;
  }

  CalcElementCircumsphere (elementId);
}

int Mesh_elementIsDummy (int elementId)
{
  int n0, n1, n2, n3;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(!Mesh_elementIsDeleted (elementId));
  
  n0 = Element_nodeIds[elementId][0];
  n1 = Element_nodeIds[elementId][1];
  n2 = Element_nodeIds[elementId][2];
  n3 = Element_nodeIds[elementId][3];

  return Mesh_nodeIsDummy (n0)
    || Mesh_nodeIsDummy (n1)
    || Mesh_nodeIsDummy (n2)
    || Mesh_nodeIsDummy (n3);
}

int Mesh_elementIsWithinCircumsphere 
(int elementId, 
 double x, double y, double z) 
{
  double centerX = Element_circumcenter[elementId][0];
  double centerY = Element_circumcenter[elementId][1];
  double centerZ = Element_circumcenter[elementId][2];
  double radius = Element_circumradius[elementId];
  
  double dx = x - centerX;
  double dy = y - centerY;
  double dz = z - centerZ;
  double d2 = dx * dx + dy * dy + dz * dz;
  
  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(!Mesh_elementIsDeleted (elementId));

  /* WARNING */
  /* meaning of tolerance changed to r^2 */
  return (d2 < radius * radius + Eps);
}

void Mesh_checkOrientation (int elementId)
{
  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(!Mesh_elementIsDeleted (elementId));

  {
    int n0 = Element_nodeIds[elementId][0];
    int n1 = Element_nodeIds[elementId][1];
    int n2 = Element_nodeIds[elementId][2];
    int n3 = Element_nodeIds[elementId][3];

    double x0 = Node_coord[n0][0];
    double y0 = Node_coord[n0][1];
    double z0 = Node_coord[n0][2];
    
    double x1 = Node_coord[n1][0];
    double y1 = Node_coord[n1][1];
    double z1 = Node_coord[n1][2];
    
    double x2 = Node_coord[n2][0];
    double y2 = Node_coord[n2][1];
    double z2 = Node_coord[n2][2];
    
    double x3 = Node_coord[n3][0];
    double y3 = Node_coord[n3][1];
    double z3 = Node_coord[n3][2];
    
    double volume = SignedTetraVolume 
      (x0, y0, z0,
       x1, y1, z1,
       x2, y2, z2,
       x3, y3, z3);
    
    if (volume < 0.0) {
      int n0 = Element_nodeIds[elementId][0];
      int n1 = Element_nodeIds[elementId][1];
      
      Element_nodeIds[elementId][0] = n1;
      Element_nodeIds[elementId][1] = n0;
    }
  }
}

void Mesh_tryToSetNeighbor 
(int elementId, 
 int theOtherElementId)
{
  int iFace;
  int jFace;
  int found;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(!Mesh_elementIsDeleted (elementId));
  
  assert(0 <= theOtherElementId);
  assert(theOtherElementId < NElements);
  assert(!Mesh_elementIsDeleted (theOtherElementId));
  
  if (elementId == theOtherElementId) {
    return;
  }
  assert(elementId != theOtherElementId);

  found = 0;
  for (iFace = 0; iFace < N_ELEMENT_FACES; iFace++) {
    int ifn0, ifn1, ifn2;
    
    {
      int nodeId0 = Element_nodeIds[elementId][FaceNodeMap[iFace][0]];
      int nodeId1 = Element_nodeIds[elementId][FaceNodeMap[iFace][1]];
      int nodeId2 = Element_nodeIds[elementId][FaceNodeMap[iFace][2]];
      
      SortId3 (nodeId0, nodeId1, nodeId2,
	       &ifn0, &ifn1, &ifn2);
    }
    
    for (jFace = 0; jFace < N_ELEMENT_FACES; jFace++) {
      int jfn0, jfn1, jfn2;
      
      {
	int nodeId0 = Element_nodeIds[theOtherElementId][FaceNodeMap[jFace][0]];
	int nodeId1 = Element_nodeIds[theOtherElementId][FaceNodeMap[jFace][1]];
	int nodeId2 = Element_nodeIds[theOtherElementId][FaceNodeMap[jFace][2]];
	
	SortId3 (nodeId0, nodeId1, nodeId2,
		 &jfn0, &jfn1, &jfn2);
      }
      
      if (ifn0 == jfn0 && ifn1 == jfn1 && ifn2 == jfn2) {
	Element_neighborIds[elementId][iFace] = theOtherElementId;
	Element_neighborIds[theOtherElementId][jFace] = elementId;
	
	found = 1;
	break;
      }
    }

    if (found) {
      break;
    }
  }
}

void Mesh_setNeighborForAll (void)
{
  int iElement;
  int jElement;

  for (iElement = 0; iElement < NElements; iElement++) {
    for (jElement = iElement + 1; jElement < NElements; jElement++) {
      Mesh_tryToSetNeighbor (iElement, jElement);
    }
  }
}

void Mesh_printNodes (void)
{
  int iNode;
  
  printf ("Nodes\n");
  printf ("%d\n", NNodes);
  printf (" NAllNodes %d\n", NAllNodes);
  
  for (iNode = 0; iNode < NAllNodes; iNode++) {
    printf ("%d x %f y %f z %f ", 
	    iNode, 
	    Node_coord[iNode][0], 
	    Node_coord[iNode][1], 
	    Node_coord[iNode][2]);
    if (Mesh_nodeIsDummy (iNode)) {
      printf (" isDummy ");
    }
    printf ("\n");
  }
}

void Mesh_printElements (void)
{
  int iElement;

  printf ("Elements\n");
  printf ("%d \n", 
	  NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int n0 = Element_nodeIds[iElement][0];
    int n1 = Element_nodeIds[iElement][1];
    int n2 = Element_nodeIds[iElement][2];
    int n3 = Element_nodeIds[iElement][3];
    
    printf ("%d nodeIds %d %d %d %d ", 
	    iElement,
	    n0, n1, n2, n3); 
    printf ("center %f %f %f radius %f ", 
	    Element_circumcenter[iElement][0], 
	    Element_circumcenter[iElement][1], 
	    Element_circumcenter[iElement][2], 
	    Element_circumradius[iElement]); 

    printf ("neighborIds %d %d %d %d ", 
	    Element_neighborIds[iElement][0],
	    Element_neighborIds[iElement][1],
	    Element_neighborIds[iElement][2],
	    Element_neighborIds[iElement][3]);

    if (Mesh_elementIsDeleted (iElement)) {
      printf (" isDeleted ");
    } else {
      printf (" isAlive ");
    }
    if (Mesh_elementIsDummy (iElement)) {
      printf (" isDummy ");
    } else {
      printf (" notDummy ");
    }

    printf ("\n");
  }
}



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);

  NAllNodes = NNodes;
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int count;

  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (!Mesh_elementIsDeleted (iElement)
	&& !Mesh_elementIsDummy (iElement)) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);

  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (!Mesh_elementIsDeleted (iElement)
	&& !Mesh_elementIsDummy (iElement)) {
      fprintf (fp, "%d   ", 
	       count); 
      for (iElementNode = 0; 
	   iElementNode < N_ELEMENT_NODES; 
	   iElementNode++) {
	fprintf (fp, "%d ", 
		 Element_nodeIds[iElement][iElementNode]); 
      }
      fprintf (fp, "\n");
      
      count++;
    }
  }
  
  fclose (fp);
}



/************************
       Other Data
*************************/

#define MAX_COLLECTED_ELEMENTS 10000

int NCollectedElements;
int CollectedElement_elementId[MAX_COLLECTED_ELEMENTS];

#define MAX_COLLECTED_FACES 10000

int NCollectedFaces;
int CollectedFace_nodeIds[MAX_COLLECTED_FACES][3];
int CollectedFace_insideElementId[MAX_COLLECTED_FACES];
int CollectedFace_insideElementFaceId[MAX_COLLECTED_FACES];
int CollectedFace_outsideElementId[MAX_COLLECTED_FACES];

#define MAX_NEW_ELEMENTS 10000

int NNewElements;
int NewElement_elementId[MAX_NEW_ELEMENTS];

void ClearElementCollection (void)
{
  NCollectedElements = 0;
}

void AddElementIntoCollection (int elementId)
{
  int ceId;

  assert(NCollectedElements < MAX_COLLECTED_ELEMENTS);
  ceId = NCollectedElements;
  NCollectedElements++;
  
  CollectedElement_elementId[ceId] = elementId;
}

int ElementIsInCollection (int elementId)
{
  int iCe;

  for (iCe = 0; iCe < NCollectedElements; iCe++) {
    if (CollectedElement_elementId[iCe] == elementId) {
      return 1;
    }
  }
  return 0;
}

void ClearFaceCollection (void)
{
  NCollectedFaces = 0;
}

void AddFaceIntoCollection (int elementId, int elementFaceId)
{
  int cfId;
  int nodeId0 = Element_nodeIds[elementId][FaceNodeMap[elementFaceId][0]];
  int nodeId1 = Element_nodeIds[elementId][FaceNodeMap[elementFaceId][1]];
  int nodeId2 = Element_nodeIds[elementId][FaceNodeMap[elementFaceId][2]];

  assert(NCollectedFaces < MAX_COLLECTED_FACES);
  cfId = NCollectedFaces;
  NCollectedFaces++;

  CollectedFace_insideElementId[cfId] = elementId;
  CollectedFace_insideElementFaceId[cfId] = elementFaceId;
  CollectedFace_nodeIds[cfId][0] = nodeId0;
  CollectedFace_nodeIds[cfId][1] = nodeId1;
  CollectedFace_nodeIds[cfId][2] = nodeId2;
}

void FindNeighborElement 
(int elementId,
 double x, double y, double z)
{
  int iFace;

  assert(elementId != -1);

  for (iFace = 0; iFace < N_ELEMENT_FACES; iFace++) {
    int neighborId = Element_neighborIds[elementId][iFace];

    if (neighborId == -1) {
      /* reach boundary of element collection */

      AddFaceIntoCollection (elementId, iFace);

      continue;
    }

    assert(!Mesh_elementIsDeleted (neighborId));

    if (ElementIsInCollection (neighborId)) {
      continue;
    }

    if (Mesh_elementIsWithinCircumsphere (neighborId, x, y, z)) {
      /* element <neighborId> is inside collection */
      
      AddElementIntoCollection (neighborId);
      FindNeighborElement (neighborId, x, y, z);
    } else {
      /* element <neighborId> is outside collection */
      /* reach boundary of element collection */
      
      AddFaceIntoCollection (elementId, iFace);
    }
  }
}

void FindFirstElement (double x, double y, double z)
{
  int iElement;
  int elementIdFound;
  
  elementIdFound = -1;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (!Mesh_elementIsDeleted (iElement)) {
      if (Mesh_elementIsWithinCircumsphere (iElement, x, y, z)) {
	elementIdFound = iElement;
	break;
      }
    }
  }
  assert(elementIdFound != -1);
  
  AddElementIntoCollection (elementIdFound);
  FindNeighborElement (elementIdFound, x, y, z);
}

void CollectElements (int nodeId)
{
  double x = Node_coord[nodeId][0];
  double y = Node_coord[nodeId][1];
  double z = Node_coord[nodeId][2];

  ClearElementCollection ();
  ClearFaceCollection ();

  FindFirstElement (x, y, z);
}

int NodeIsOnFace (int nodeId)
{
  double x = Node_coord[nodeId][0];
  double y = Node_coord[nodeId][1];
  double z = Node_coord[nodeId][2];

  int iCf;

  for (iCf = 0; iCf < NCollectedFaces; iCf++) {
    int n0 = CollectedFace_nodeIds[iCf][0];
    int n1 = CollectedFace_nodeIds[iCf][1];
    int n2 = CollectedFace_nodeIds[iCf][2];

    double a, b, c, d;
    double equationValue;

    GetPlaneEquation
      (Node_coord[n0][0], Node_coord[n0][1], Node_coord[n0][2],
       Node_coord[n1][0], Node_coord[n1][1], Node_coord[n1][2],
       Node_coord[n2][0], Node_coord[n2][1], Node_coord[n2][2],
       &a, &b, &c, &d);
    equationValue = a * x + b * y + c * z + d;
    
    if (fabs (equationValue) < Eps) {
      fprintf (stderr, " node is on face %d \n",
	       nodeId);

      return 1;
    }
  }
  return 0;
}

void SetOutsideElementOfCollectedFaces (void)
{
  int iCf;

  for (iCf = 0; iCf < NCollectedFaces; iCf++) {
    int insideElementId = CollectedFace_insideElementId[iCf];
    int insideElementFaceId = CollectedFace_insideElementFaceId[iCf];
    
    int outsideElementId 
      = Element_neighborIds[insideElementId][insideElementFaceId];

    assert(outsideElementId != -1
	   || outsideElementId == -1);
    
    CollectedFace_outsideElementId[iCf] = outsideElementId;
  }
}

void DeleteOldElementsAndAddNewElements (int nodeId)
{
  int iCf;
  int iCe;

  NNewElements = 0;
  for (iCf = 0; iCf < NCollectedFaces; iCf++) {
    int n0 = CollectedFace_nodeIds[iCf][0];
    int n1 = CollectedFace_nodeIds[iCf][1];
    int n2 = CollectedFace_nodeIds[iCf][2];
    
    iCe = iCf;

    if (iCe < NCollectedElements) {
      int elementId = CollectedElement_elementId[iCe];
      
      Mesh_overwriteElement (elementId, 
			     n0, n1, n2, nodeId);

      assert(NNewElements < MAX_NEW_ELEMENTS);
      NewElement_elementId[NNewElements] = elementId;
      NNewElements++;
    } else {
      int elementId = Mesh_addNewElement (n0, n1, n2, nodeId);

      assert(NNewElements < MAX_NEW_ELEMENTS);
      NewElement_elementId[NNewElements] = elementId;
      NNewElements++;
    }
  }

  for (iCe = 0; iCe < NCollectedElements; iCe++) {
    iCf = iCe;

    if (iCf < NCollectedFaces) {
      /* do nothing */
    } else {
      int elementId = CollectedElement_elementId[iCe];

      Mesh_deleteElement (elementId);
    }
  }
}

void ModifyNeighbors (void)
{
  int iNe;
  int jNe;
  int iCf;

  for (iNe = 0; iNe < NNewElements; iNe++) {
    for (jNe = iNe + 1; jNe < NNewElements; jNe++) {
      Mesh_tryToSetNeighbor (NewElement_elementId[iNe], 
			     NewElement_elementId[jNe]);
    }
  }
  
  for (iCf = 0; iCf < NCollectedFaces; iCf++) {
    int outsideElementId = CollectedFace_outsideElementId[iCf];
    
    if (outsideElementId != -1) {
      for (iNe = 0; iNe < NNewElements; iNe++) {
	Mesh_tryToSetNeighbor (outsideElementId, 
			       NewElement_elementId[iNe]);
      }
    }
  }
}

void PrintCollectedElements (void)
{
  int iCe;

  printf ("CollectedElements\n");
  printf ("%d\n", NCollectedElements);
  
  for (iCe = 0; iCe < NCollectedElements; iCe++) {
    int elementId = CollectedElement_elementId[iCe];

    int n0 = Element_nodeIds[elementId][0];
    int n1 = Element_nodeIds[elementId][1];
    int n2 = Element_nodeIds[elementId][2];
    int n3 = Element_nodeIds[elementId][3];

    printf ("%d elementId %d nodeIds %d %d %d %d\n", 
	    iCe,
	    elementId, n0, n1, n2, n3); 
  }
}

void PrintCollectedFaces (void)
{
  int iCf;

  printf ("CollectedFaces\n");
  printf ("%d\n", NCollectedFaces);
  
  for (iCf = 0; iCf < NCollectedFaces; iCf++) {
    int n0 = CollectedFace_nodeIds[iCf][0];
    int n1 = CollectedFace_nodeIds[iCf][1];
    int n2 = CollectedFace_nodeIds[iCf][2];

    printf ("%d nodeIds %d %d %d\n", 
	    iCf,
	    n0, n1, n2); 
  }
}



/************************
	 Command
*************************/

static void MakeInitialMesh (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  double x0, y0, z0;
  double x1, y1, z1;
  int n0, n1, n2, n3, n4, n5, n6, n7;

  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2 * 1.5;
  
  /* corners of extended region */
  x0 = centerX - size;
  y0 = centerY - size;
  z0 = centerZ - size;
  
  x1 = centerX + size;
  y1 = centerY + size;
  z1 = centerZ + size;

  /* 8 dummy nodes on corners of extended region */
  n0 = Mesh_addDummyNode (x0, y0, z0);
  n1 = Mesh_addDummyNode (x1, y0, z0);
  n2 = Mesh_addDummyNode (x1, y1, z0);
  n3 = Mesh_addDummyNode (x0, y1, z0);
  n4 = Mesh_addDummyNode (x0, y0, z1);
  n5 = Mesh_addDummyNode (x1, y0, z1);
  n6 = Mesh_addDummyNode (x1, y1, z1);
  n7 = Mesh_addDummyNode (x0, y1, z1);

  /* 5 initial tetrahedrons */
  Mesh_addNewElement (n1, n3, n4, n0);
  Mesh_addNewElement (n3, n6, n1, n2);
  Mesh_addNewElement (n4, n1, n6, n5);
  Mesh_addNewElement (n3, n4, n6, n7);
  Mesh_addNewElement (n4, n1, n3, n6);

  Mesh_setNeighborForAll ();

#ifdef PRINT
  printf (" MakeInitialMesh \n");
  Mesh_printNodes ();
  Mesh_printElements ();
#endif
}

static int InsertNode (int nodeId)
{
#ifdef PRINT
  printf (" InsertNode %d %f %f %f\n",
	  nodeId,
	  Node_coord[nodeId][0],
	  Node_coord[nodeId][1],
	  Node_coord[nodeId][2]);
#endif

  CollectElements (nodeId);
  if (NodeIsOnFace (nodeId)) {
    return 0;
  }

  SetOutsideElementOfCollectedFaces ();

#ifdef PRINT
  printf (" CollectElements \n");
  PrintCollectedElements ();
  PrintCollectedFaces ();
#endif

  DeleteOldElementsAndAddNewElements (nodeId);
  ModifyNeighbors ();

#ifdef PRINT
  printf (" DeleteOldElementsAndAddNewElements \n");
#if 0
  Mesh_printElements ();
#endif
#endif

  return 1;
}

void CheckOrientation (void)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (!Mesh_elementIsDeleted (iElement)) {
      Mesh_checkOrientation (iElement);
    }
  }
  
#ifdef PRINT
  printf (" CheckOrientation \n");
#if 0
  Mesh_printElements ();
#endif
#endif
}

static void AddMoreDummyNodes (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  int op;
  int i, j;
  int flag;

  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2 * 1.5;
  
  /* dummy points near the 6 faces */
  for (op = 0; op < 6; op++) {
    for (i = -4; i <= 4; i++) {
      for (j = -4; j <= 4; j++) {
	double offsetX, offsetY, offsetZ;
	double px, py, pz;
	int nodeId;

	switch (op) {
	case 0:
	  offsetX = 0.8;
	  offsetY = 0.18 * i;
	  offsetZ = 0.18 * j;
	  break;
	case 1:
	  offsetX = -0.8;
	  offsetY = 0.18 * i;
	  offsetZ = 0.18 * j;
	  break;
	case 2:
	  offsetX = 0.18 * i;
	  offsetY = 0.8;
	  offsetZ = 0.18 * j;
	  break;
	case 3:
	  offsetX = 0.18 * i;
	  offsetY = -0.8;
	  offsetZ = 0.18 * j;
	  break;
	case 4:
	  offsetX = 0.18 * i;
	  offsetY = 0.18 * j;
	  offsetZ = 0.8;
	  break;
	case 5:
	  offsetX = 0.18 * i;
	  offsetY = 0.18 * j;
	  offsetZ = -0.8;
	  break;
	default:
	  assert(0);
	  break;
	}

	px = centerX + offsetX * size;
	py = centerY + offsetY * size;
	pz = centerZ + offsetZ * size;

	nodeId = Mesh_addDummyNode (px, py, pz);
	
	flag = InsertNode (nodeId);
	assert(flag);
      }
    }
  }

#ifdef PRINT
  printf (" AddMoreDummyNodes \n");
  Mesh_printNodes ();
  Mesh_printElements ();
#endif
}

static int Node_isDone[MAX_NODES];

void PerformCommand (void)
{
  int iNode;
  int flag;

  MakeInitialMesh ();

#ifdef MORE_DUMMY_NODE
  AddMoreDummyNodes ();
#endif

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isDone[iNode] = 0;
  }

#ifdef PRINT_NODE_ID
  printf (" the first 100, including vertexs \n");
#endif

  for (iNode = 0; iNode < 100; iNode++) {
    if (iNode < NNodes) {
      if (!Node_isDone[iNode]) {
	flag = InsertNode (iNode);
	
	if (flag) {
	  Node_isDone[iNode] = 1;
	}
      }
    }
  }

#ifdef PRINT_NODE_ID
  printf (" each 1000 \n");
#endif

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (iNode % 1000 != 0) continue;
    
    if (!Node_isDone[iNode]) {
      flag = InsertNode (iNode);
      
      if (flag) {
	Node_isDone[iNode] = 1;
      }
    }
  }

#ifdef PRINT_NODE_ID
  printf (" each 100 \n");
#endif

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (iNode % 100 != 0) continue;

    if (!Node_isDone[iNode]) {
      flag = InsertNode (iNode);
      
      if (flag) {
	Node_isDone[iNode] = 1;
      }
    }
  }

#ifdef PRINT_NODE_ID
  printf (" each 10 \n");
#endif

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (iNode % 10 != 0) continue;

    if (!Node_isDone[iNode]) {
      flag = InsertNode (iNode);
      
      if (flag) {
	Node_isDone[iNode] = 1;
      }
    }
  }

#ifdef PRINT_NODE_ID
  printf (" the rest \n");
#endif

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (!Node_isDone[iNode]) {
      flag = InsertNode (iNode);
      
      if (flag) {
	Node_isDone[iNode] = 1;
      }
    }
    
#ifdef PRINT_NODE_ID
    if (iNode % 1000 == 1) {
      printf (" iNode %d \n",
	      iNode);
    }
#endif    
  }

  for (iNode = 0; iNode < NNodes; iNode++) {
    if (!Node_isDone[iNode]) {
      fprintf (stderr, " fail to insert node %d \n",
	       iNode);
    }
  }

  CheckOrientation ();
}



int main (int argc, char *argv[])
{
  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_delaunay3  node_file \n"
       "        element_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  PerformCommand ();

  WriteElementFile (argv[2]);

  return 0;
}





