/* advautomesh_h_tetra3d_extract.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_isMarked[MAX_NODES];
int Node_newId[MAX_NODES];
int NMarkedNodes;



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 12000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_isExtracted[MAX_ELEMENTS];
int NExtractedElements;



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadExtractedElementListFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int nItems;
  int iItem;

  assert(0 < NElements);

  for (iElement = 0; iElement < NElements; iElement++) {
    Element_isExtracted[iElement] = 0;
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  NExtractedElements = nItems;
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int elementId;
    
    fscanf (fp, "%d %d", 
	    &itemId,
	    &elementId);
    assert(itemId == iItem);
    assert(0 <= elementId);
    assert(elementId < NElements);
    Element_isExtracted[elementId] = 1;
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);
  assert(0 < NMarkedNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NMarkedNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isMarked[iNode]) {
      fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	       Node_newId[iNode],
	       Node_coord[iNode][0],
	       Node_coord[iNode][1],
	       Node_coord[iNode][2]); 
    }
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int count;

  assert(0 < NNodes);
  assert(0 < NElements);
  assert(0 < NExtractedElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NExtractedElements);
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isExtracted[iElement]) {
      fprintf (fp, "%d   ", 
	       count); 
      for (iElementNode = 0; 
	   iElementNode < N_ELEMENT_NODES; 
	   iElementNode++) {
	int nodeId = Element_nodeIds[iElement][iElementNode];
	
	fprintf (fp, "%d ", 
		 Node_newId[nodeId]); 
      }
      fprintf (fp, "\n");
      count++;
    }
  }
  
  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iElement;
  int iElementNode;
  int count;
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isMarked[iNode] = 0;
  }
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isExtracted[iElement]) {
      for (iElementNode = 0; 
	   iElementNode < N_ELEMENT_NODES; 
	   iElementNode++) {
	int nodeId = Element_nodeIds[iElement][iElementNode];
	
	Node_isMarked[nodeId] = 1;
      }
    }
  }

  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_newId[iNode] = -1;
    if (Node_isMarked[iNode]) {
      Node_newId[iNode] = count;
      count++;
    }
  }
  NMarkedNodes = count;
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_extract  node_file  element_file \n"
       "        extracted_element_list_file \n"
       "        node_file_OUT  element_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadExtractedElementListFile (argv[3]);

  PerformCommand ();

  WriteNodeFile (argv[4]);
  WriteElementFile (argv[5]);

  return 0;
}
