/* advautomesh_h_tetra3d_face_on_plane.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
int Node_isOn[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 1000000

const int FaceNodeMap[N_ELEMENT_FACES][3] = {
  { 1, 2, 3 },
  { 0, 3, 2 },
  { 0, 1, 3 },
  { 0, 2, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Face */

#define MAX_FACES 1000000

int NFaces;
int Face_elementIds[MAX_NODES][2];
int Face_elementFaceIds[MAX_NODES][2];
int Face_isOn[MAX_FACES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadFaceFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  
  assert(NFaces == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NFaces);
  assert(0 < NFaces);
  assert(NFaces <= MAX_FACES);
  
  for (iFace = 0; iFace < NFaces; iFace++) {
    int faceId;
    int elementId0, elementId1;
    int elementFaceId0, elementFaceId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &faceId,
	    &elementId0, &elementFaceId0,
	    &elementId1, &elementFaceId1);
    assert(faceId == iFace);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementFaceId0);
    assert(elementFaceId0 < N_ELEMENT_FACES);

    if (0 <= elementId1) {
      assert(elementId1 < NElements);
      assert(0 <= elementFaceId1);
      assert(elementFaceId1 < N_ELEMENT_FACES);
    } else {
      assert(elementId1 == -1);
      assert(elementFaceId1 == -1);
    }

    Face_elementIds[iFace][0] = elementId0;
    Face_elementIds[iFace][1] = elementId1;

    Face_elementFaceIds[iFace][0] = elementFaceId0;
    Face_elementFaceIds[iFace][1] = elementFaceId1;
  }

  fclose (fp);
}

void WriteFaceListFile (const char *fileName)
{
  FILE *fp;
  int iFace;
  int count;

  assert(0 < NFaces);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_isOn[iFace]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_isOn[iFace]) {
      fprintf (fp, "%d   %d\n", 
	       count,
	       iFace);
      count++;
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

double PlaneA, PlaneB, PlaneC, PlaneD;

void PerformCommand (void)
{
  int iNode;
  int iFace;
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOn[iNode] = 0;
  }
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    double value = PlaneA * x + PlaneB * y + PlaneC * z + PlaneD;
    
    if (fabs (value) < Eps) {
      Node_isOn[iNode] = 1;
    }
  }

  for (iFace = 0; iFace < NFaces; iFace++) {
    Face_isOn[iFace] = 0;
  }

  for (iFace = 0; iFace < NFaces; iFace++) {
    if (Face_elementIds[iFace][1] == -1) {
      int elementId = Face_elementIds[iFace][0];
      int elementFaceId = Face_elementFaceIds[iFace][0];
    
      int nodeId0 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][0]];
      int nodeId1 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][1]];
      int nodeId2 
	= Element_nodeIds[elementId][FaceNodeMap[elementFaceId][2]];
    
      if (Node_isOn[nodeId0]
	  && Node_isOn[nodeId1]
	  && Node_isOn[nodeId2]) {
	Face_isOn[iFace] = 1;
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 9) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_face_on_plane  node_file  element_file \n"
       "        face_file  a  b  c  d \n"
       "        face_list_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadFaceFile (argv[3]);

  PlaneA = atof (argv[4]);
  PlaneB = atof (argv[5]);
  PlaneC = atof (argv[6]);
  PlaneD = atof (argv[7]);

  PerformCommand ();

  WriteFaceListFile (argv[8]);

  return 0;
}
