/* advautomesh_h_tetra3d_grid.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Point */

#define MAX_POINTS 1000000

int NPoints;
double Point_coord[MAX_POINTS][3];



void PointGroup_addPoint 
(double x, double y, double z)
{
  assert(NPoints < MAX_POINTS);
  
  Point_coord[NPoints][0] = x;
  Point_coord[NPoints][1] = y;
  Point_coord[NPoints][2] = z;
  
  NPoints++;
}

void WritePointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(0 <= NPoints);
  
  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NPoints);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iPoint,
	     Point_coord[iPoint][0], 
	     Point_coord[iPoint][1], 
	     Point_coord[iPoint][2]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (double interval)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  int nDivisions;
  int ix, iy, iz;

  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;
  nDivisions = size / interval;

  for (ix = -nDivisions; ix <= nDivisions; ix++) {
    double x = centerX + interval * ix;
    
    for (iy = -nDivisions; iy <= nDivisions; iy++) {
      double y = centerY + interval * iy;

      for (iz = -nDivisions; iz <= nDivisions; iz++) {
	double z = centerZ + interval * iz;

	PointGroup_addPoint (x, y, z);
      }
    }
  }
}



int main (int argc, char *argv[])
{
  double interval;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_grid  node_file \n"
       "        interval \n"
       "        point_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  interval = atof (argv[2]);
  PerformCommand (interval);

  WritePointFile (argv[3]);

  return 0;
}
