/* advautomesh_h_tetra3d_import_msh.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 12000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void ReadMshFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iElement;
  int iElementNode;

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;
      
      fscanf (fp, "%d", 
	      &nodeId);
      Element_nodeIds[iElement][iElementNode] = nodeId;
    }
  }

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x, y, z;
    
    fscanf (fp, "%lf %lf %lf", 
	    &x, &y, &z);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y;
    Node_coord[iNode][2] = z;
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1],
	     Node_coord[iNode][2]); 
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement); 
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fprintf (fp, "%d ", 
	       Element_nodeIds[iElement][iElementNode]); 
    }
    fprintf (fp, "\n");
  }
  
  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_import_msh  msh_file \n"
       "     node_file_OUT  element_file_OUT \n");
    exit (1);
  }

  ReadMshFile (argv[1]);

  WriteNodeFile (argv[2]);
  WriteElementFile (argv[3]);

  return 0;
}
