/* advautomesh_h_tetra3d_insert.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



#if 1
#define PRINT
#endif



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001
#define AREA_COORD_EPS 0.00001
#define VOLUME_COORD_EPS 0.00001

double Eps = DEFAULT_EPS;



void SortId3 (int id0, int id1, int id2,
	      int *minId_OUT, int *midId_OUT, int *maxId_OUT)
{
  assert(id0 != id1);
  assert(id0 != id2);
  assert(id1 != id2);

  if (id0 < id1 && id0 < id2) {
    *minId_OUT = id0;
  }
  if (id1 < id0 && id1 < id2) {
    *minId_OUT = id1;
  }
  if (id2 < id0 && id2 < id1) {
    *minId_OUT = id2;
  }

  if ((id1 < id0 && id0 < id2)
      || (id2 < id0 && id0 < id1)) {
    *midId_OUT = id0;
  }
  if ((id0 < id1 && id1 < id2)
      || (id2 < id1 && id1 < id0)) {
    *midId_OUT = id1;
  }
  if ((id0 < id2 && id2 < id1)
      || (id1 < id2 && id2 < id0)) {
    *midId_OUT = id2;
  }

  if (id1 < id0 && id2 < id0) {
    *maxId_OUT = id0;
  }
  if (id0 < id1 && id2 < id1) {
    *maxId_OUT = id1;
  }
  if (id0 < id2 && id1 < id2) {
    *maxId_OUT = id2;
  }
}

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}

double SignedTetraVolume
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double x3, double y3, double z3)
{
  double x10 = x1 - x0;
  double y10 = y1 - y0;
  double z10 = z1 - z0;

  double x20 = x2 - x0;
  double y20 = y2 - y0;
  double z20 = z2 - z0;
  
  double x30 = x3 - x0;
  double y30 = y3 - y0;
  double z30 = z3 - z0;
  
  double d23 = y20 * z30 - z20 * y30;
  double d31 = y30 * z10 - z30 * y10;
  double d12 = y10 * z20 - z10 * y20;
  
  return (x10 * d23 + x20 * d31 + x30 * d12) / 6;
}

void GetNormalizedVector 
(double x, double y, double z,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  double length = sqrt (x * x + y * y + z * z);

  *nx_OUT = x / length;
  *ny_OUT = y / length;
  *nz_OUT = z / length;
}

void GetVectorProduct 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  *x_OUT = y0 * z1 - z0 * y1;
  *y_OUT = z0 * x1 - x0 * z1;
  *z_OUT = x0 * y1 - y0 * x1;
}

void GetPlaneEquation 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double *a_OUT, double *b_OUT, double *c_OUT, double *d_OUT)
{
  double nx, ny, nz;

  GetVectorProduct (x1 - x0, y1 - y0, z1 - z0,
		    x2 - x0, y2 - y0, z2 - z0,
		    &nx, &ny, &nz);
  GetNormalizedVector (nx, ny, nz,
		       &nx, &ny, &nz);
  *a_OUT = nx;
  *b_OUT = ny;
  *c_OUT = nz;
  *d_OUT = -(nx * x0 + ny * y0 + nz * z0);
}

int GetIntersectionBetweenPlaneAndSeg 
(double a, double b, double c, double d, 
 double sx0, double sy0, double sz0, 
 double sx1, double sy1, double sz1, 
 double *intX_OUT, double *intY_OUT, double *intZ_OUT) 
{
  double value0 = a * sx0 + b * sy0 + c * sz0 + d;
  double value1 = a * sx1 + b * sy1 + c * sz1 + d;

  if (fabs (value0 - value1) < Eps) return 0;

  if (fabs (value0) < Eps) {
    *intX_OUT = sx0;
    *intY_OUT = sy0;
    *intZ_OUT = sz0;
    return 1;
  } else if (fabs (value1) < Eps) {
    *intX_OUT = sx1;
    *intY_OUT = sy1;
    *intZ_OUT = sz1;
    return 1;
  } else if (value0 * value1 < Eps * Eps) {
    *intX_OUT = (sx1 * value0 - sx0 * value1) / (value0 - value1);
    *intY_OUT = (sy1 * value0 - sy0 * value1) / (value0 - value1);
    *intZ_OUT = (sz1 * value0 - sz0 * value1) / (value0 - value1);
    return 1;
  }
  return 0;
}

double TriArea
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2)
{
  double vx, vy, vz;
  
  GetVectorProduct (x1 - x0, y1 - y0, z1 - z0,
		    x2 - x0, y2 - y0, z2 - z0,
		    &vx, &vy, &vz);
  return sqrt (vx * vx + vy * vy + vz * vz) / 2;
}

int TriContainsPoint 
(double tx0, double ty0, double tz0, 
 double tx1, double ty1, double tz1, 
 double tx2, double ty2, double tz2, 
 double x, double y, double z)
{
  double area = TriArea (tx0, ty0, tz0, 
			 tx1, ty1, tz1, 
			 tx2, ty2, tz2);
  double area0 = TriArea (tx1, ty1, tz1, 
			  tx2, ty2, tz2, 
			  x, y, z);
  double area1 = TriArea (tx2, ty2, tz2, 
			  tx0, ty0, tz0, 
			  x, y, z);
  double area2 = TriArea (tx0, ty0, tz0, 
			  tx1, ty1, tz1, 
			  x, y, z);
  
  if (fabs ((area0 + area1 + area2) / area - 1.0) < AREA_COORD_EPS) {
    return 1;
  }
  return 0;
}

int GetIntersectionBetweenTriAndSeg 
(double tx0, double ty0, double tz0, 
 double tx1, double ty1, double tz1, 
 double tx2, double ty2, double tz2, 
 double sx0, double sy0, double sz0, 
 double sx1, double sy1, double sz1, 
 double *intX_OUT, double *intY_OUT, double *intZ_OUT) 
{
  double a, b, c, d;
  int flag;
  
  GetPlaneEquation (tx0, ty0, tz0, 
		    tx1, ty1, tz1, 
		    tx2, ty2, tz2,
		    &a, &b, &c, &d);
  flag = GetIntersectionBetweenPlaneAndSeg 
    (a, b, c, d,
     sx0, sy0, sz0, 
     sx1, sy1, sz1, 
     intX_OUT, intY_OUT, intZ_OUT);
  if (flag) {
    if (TriContainsPoint (tx0, ty0, tz0, 
			  tx1, ty1, tz1, 
			  tx2, ty2, tz2,
			  *intX_OUT, *intY_OUT, *intZ_OUT)) {
      return 1;
    }
  }

  return 0;
}



/************************
	 Mesh
*************************/

/* Node */

typedef enum {
  NODE_TOPOLOGY_INTERNAL,
  NODE_TOPOLOGY_FACE,
  NODE_TOPOLOGY_EDGE,
  NODE_TOPOLOGY_VERTEX
} NodeTopologyType;

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
NodeTopologyType Node_topologyType[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_EDGES 6
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 1000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 0, 1 }, 
  { 0, 2 }, 
  { 0, 3 }, 
  { 1, 2 }, 
  { 2, 3 }, 
  { 3, 1 }  
};

const int FaceNodeMap[N_ELEMENT_FACES][3] = {
  { 1, 2, 3 },
  { 0, 3, 2 },
  { 0, 1, 3 },
  { 0, 2, 1 }
};

int NElements;
int Element_isDeleted[MAX_ELEMENTS];
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Node_moveTo 
(int nodeId, 
 double x, double y, double z)
{
  assert(0 <= nodeId);
  assert(nodeId < NNodes);

  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  Node_coord[nodeId][2] = z;
}

void Node_setTopology 
(int nodeId, 
 NodeTopologyType topologyType)
{
  assert(0 <= nodeId);
  assert(nodeId < NNodes);

  switch (topologyType) {
  case NODE_TOPOLOGY_INTERNAL:
    if (Node_topologyType[nodeId] != NODE_TOPOLOGY_VERTEX
	&& Node_topologyType[nodeId] != NODE_TOPOLOGY_EDGE
	&& Node_topologyType[nodeId] != NODE_TOPOLOGY_FACE) {
      Node_topologyType[nodeId] = NODE_TOPOLOGY_INTERNAL;
    }
    break;
  case NODE_TOPOLOGY_FACE:
    if (Node_topologyType[nodeId] != NODE_TOPOLOGY_VERTEX
	&& Node_topologyType[nodeId] != NODE_TOPOLOGY_EDGE) {
      Node_topologyType[nodeId] = NODE_TOPOLOGY_FACE;
    }
    break;
  case NODE_TOPOLOGY_EDGE:
    if (Node_topologyType[nodeId] != NODE_TOPOLOGY_VERTEX) {
      Node_topologyType[nodeId] = NODE_TOPOLOGY_EDGE;
    }
    break;
  case NODE_TOPOLOGY_VERTEX:
    Node_topologyType[nodeId] = NODE_TOPOLOGY_VERTEX;
    break;
  default:
    assert(0);
    break;
  }
}

int Element_containsPoint 
(int elementId, 
 double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;
  double x0, y0, z0;
  double x1, y1, z1;
  double x2, y2, z2;
  double x3, y3, z3;
  double volume;
  double volume0, volume1, volume2, volume3;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];
  z0 = Node_coord[nodeId0][2];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];
  z1 = Node_coord[nodeId1][2];

  x2 = Node_coord[nodeId2][0];
  y2 = Node_coord[nodeId2][1];
  z2 = Node_coord[nodeId2][2];
  
  x3 = Node_coord[nodeId3][0];
  y3 = Node_coord[nodeId3][1];
  z3 = Node_coord[nodeId3][2];
  
  volume = SignedTetraVolume (x0, y0, z0, 
			      x1, y1, z1, 
			      x2, y2, z2, 
			      x3, y3, z3);
  volume0 = SignedTetraVolume (x, y, z, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume1 = SignedTetraVolume (x0, y0, z0, 
			       x, y, z, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume2 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x, y, z, 
			       x3, y3, z3);
  volume3 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x, y, z);

  *L0_OUT = volume0 / volume;
  *L1_OUT = volume1 / volume;
  *L2_OUT = volume2 / volume;
  *L3_OUT = volume3 / volume;

  if (0.0 - VOLUME_COORD_EPS < *L0_OUT 
      && *L0_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L1_OUT 
      && *L1_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L2_OUT 
      && *L2_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L3_OUT 
      && *L3_OUT < 1.0 + VOLUME_COORD_EPS) {
    return 1;
  }
  return 0;
}

int Mesh_findNodeAt (double x, double y, double z)
{
  int iNode;

  for (iNode = 0; iNode < NNodes; iNode++) {
    double nodeX = Node_coord[iNode][0];
    double nodeY = Node_coord[iNode][1];
    double nodeZ = Node_coord[iNode][2];

    if (Distance (x, y, z, 
		  nodeX, nodeY, nodeZ) < Eps) {
      return iNode;
    }
  }

  return -1;
}

int Mesh_addNewNode 
(double x, double y, double z,
 NodeTopologyType topologyType)
{
  int nodeId = NNodes;

  assert(NNodes < MAX_NODES);
  NNodes++;

  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  Node_coord[nodeId][2] = z;

  Node_topologyType[nodeId] = topologyType;
  
  return nodeId;
}

int Mesh_addNewElement (int nodeId0, int nodeId1, int nodeId2, int nodeId3)
{
  int elementId = NElements;

  assert(0 <= nodeId0);
  assert(nodeId0 < NNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NNodes);
  assert(0 <= nodeId2);
  assert(nodeId2 < NNodes);
  assert(0 <= nodeId3);
  assert(nodeId3 < NNodes);

  assert(NElements < MAX_ELEMENTS);
  NElements++;

  Element_isDeleted[elementId] = 0;

  Element_nodeIds[elementId][0] = nodeId0;
  Element_nodeIds[elementId][1] = nodeId1;
  Element_nodeIds[elementId][2] = nodeId2;
  Element_nodeIds[elementId][3] = nodeId3;

  return elementId;
}

void Mesh_deleteElement (int elementId)
{
  assert(0 <= elementId);
  assert(elementId < NElements);

  Element_isDeleted[elementId] = 1;
}

void Mesh_divideElementOnEdge 
(int elementId, int elementEdgeId, int newNodeId)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);
  assert(0 <= newNodeId);
  assert(newNodeId < NNodes);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  switch (elementEdgeId) {
  case 0:
    Mesh_addNewElement (newNodeId, nodeId2, nodeId0, nodeId3);
    Mesh_addNewElement (newNodeId, nodeId1, nodeId2, nodeId3);
    break;
  case 1:
    Mesh_addNewElement (newNodeId, nodeId3, nodeId0, nodeId1);
    Mesh_addNewElement (newNodeId, nodeId2, nodeId3, nodeId1);
    break;
  case 2:
    Mesh_addNewElement (newNodeId, nodeId1, nodeId0, nodeId2);
    Mesh_addNewElement (newNodeId, nodeId3, nodeId1, nodeId2);
    break;
  case 3:
    Mesh_addNewElement (newNodeId, nodeId0, nodeId1, nodeId3);
    Mesh_addNewElement (newNodeId, nodeId2, nodeId0, nodeId3);
    break;
  case 4:
    Mesh_addNewElement (newNodeId, nodeId0, nodeId2, nodeId1);
    Mesh_addNewElement (newNodeId, nodeId3, nodeId0, nodeId1);
    break;
  case 5:
    Mesh_addNewElement (newNodeId, nodeId0, nodeId3, nodeId2);
    Mesh_addNewElement (newNodeId, nodeId1, nodeId0, nodeId2);
    break;
  default:
    assert(0);
    break;
  }
  
  Mesh_deleteElement (elementId);
}

void Mesh_divideElementOnFace 
(int elementId, int elementFaceId, int newNodeId)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementFaceId);
  assert(elementFaceId < N_ELEMENT_FACES);
  assert(0 <= newNodeId);
  assert(newNodeId < NNodes);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  switch (elementFaceId) {
  case 0:
    Mesh_addNewElement (nodeId3, nodeId2, newNodeId, nodeId0);
    Mesh_addNewElement (nodeId2, nodeId1, newNodeId, nodeId0);
    Mesh_addNewElement (nodeId1, nodeId3, newNodeId, nodeId0);
    break;
  case 1:
    Mesh_addNewElement (nodeId2, nodeId3, newNodeId, nodeId1);
    Mesh_addNewElement (nodeId3, nodeId0, newNodeId, nodeId1);
    Mesh_addNewElement (nodeId0, nodeId2, newNodeId, nodeId1);
    break;
  case 2:
    Mesh_addNewElement (nodeId3, nodeId1, newNodeId, nodeId2);
    Mesh_addNewElement (nodeId1, nodeId0, newNodeId, nodeId2);
    Mesh_addNewElement (nodeId0, nodeId3, newNodeId, nodeId2);
    break;
  case 3:
    Mesh_addNewElement (nodeId1, nodeId2, newNodeId, nodeId3);
    Mesh_addNewElement (nodeId2, nodeId0, newNodeId, nodeId3);
    Mesh_addNewElement (nodeId0, nodeId1, newNodeId, nodeId3);
    break;
  default:
    assert(0);
    break;
  }

  Mesh_deleteElement (elementId);
}

void Mesh_divideElementInside 
(int elementId, int newNodeId)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;

  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= newNodeId);
  assert(newNodeId < NNodes);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  Mesh_addNewElement (nodeId3, nodeId2, nodeId1, newNodeId);
  Mesh_addNewElement (nodeId2, nodeId3, nodeId0, newNodeId);
  Mesh_addNewElement (nodeId3, nodeId1, nodeId0, newNodeId);
  Mesh_addNewElement (nodeId1, nodeId2, nodeId0, newNodeId);
  Mesh_deleteElement (elementId);
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    Element_isDeleted[iElement] = 0;
  }

  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeTopologyFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    NodeTopologyType topologyType;
    
    fscanf (fp, "%d %d", 
	    &nodeId,
	    (int *)&topologyType); 
    assert(nodeId == iNode);
    switch (topologyType) {
    case NODE_TOPOLOGY_INTERNAL:
    case NODE_TOPOLOGY_FACE:
    case NODE_TOPOLOGY_EDGE:
    case NODE_TOPOLOGY_VERTEX:
      Node_topologyType[iNode] = topologyType;
      break;
    default:
      assert(0);
      break;
    }
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1],
	     Node_coord[iNode][2]); 
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int nActualElements;
  int count;

  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  nActualElements = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    nActualElements++;
  }

  fprintf (fp, "%d\n", 
	   nActualElements);
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;

    fprintf (fp, "%d   ", 
	     count); 
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fprintf (fp, "%d ", 
	       Element_nodeIds[iElement][iElementNode]); 
    }
    fprintf (fp, "\n");
    count++;
  }
  
  fclose (fp);
}

void WriteNodeTopologyFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %d\n", 
	     iNode, Node_topologyType[iNode]);
  }

  fclose (fp);
}



static void CheckSliver (void)
{
  int iElement;

  fprintf (stderr, " CheckSliver \n");

  for (iElement = 0; iElement < NElements; iElement++) {
    int n0 = Element_nodeIds[iElement][0];
    int n1 = Element_nodeIds[iElement][1];
    int n2 = Element_nodeIds[iElement][2];
    int n3 = Element_nodeIds[iElement][3];
    
    double x0 = Node_coord[n0][0];
    double y0 = Node_coord[n0][1];
    double z0 = Node_coord[n0][2];
    
    double x1 = Node_coord[n1][0];
    double y1 = Node_coord[n1][1];
    double z1 = Node_coord[n1][2];
    
    double x2 = Node_coord[n2][0];
    double y2 = Node_coord[n2][1];
    double z2 = Node_coord[n2][2];
    
    double x3 = Node_coord[n3][0];
    double y3 = Node_coord[n3][1];
    double z3 = Node_coord[n3][2];
    
    double volume = SignedTetraVolume
      (x0, y0, z0, 
       x1, y1, z1,
       x2, y2, z2,
       x3, y3, z3);
    if (fabs (volume) < 0.0001) {
      fprintf (stderr, " sliver found %d \n",
	       iElement);
    }
  }
}



/************************
	 Other Data
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][3];



/* Segment */

#define MAX_SEGS 100000

int NSegs;
double Seg_coord0[MAX_SEGS][3];
double Seg_coord1[MAX_SEGS][3];



/* Triangle */

#define MAX_TRIS 100000

int NTris;
double Tri_coord0[MAX_TRIS][3];
double Tri_coord1[MAX_TRIS][3];
double Tri_coord2[MAX_TRIS][3];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(NPoints == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NPoints);
  assert(0 <= NPoints);
  assert(NPoints <= MAX_POINTS);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    int pointId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &pointId,
	    &x, &y, &z);
    assert(pointId == iPoint);
    Point_coord[iPoint][0] = x;
    Point_coord[iPoint][1] = y; 
    Point_coord[iPoint][2] = z; 
  }

  fclose (fp);
}

void ReadSegFile (const char *fileName)
{
  FILE *fp;
  int iSeg;

  assert(NSegs == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NSegs);
  assert(0 <= NSegs);
  assert(NSegs <= MAX_SEGS);
  
  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    int segId;
    double x0, y0, z0, x1, y1, z1;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf", 
	    &segId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1);
    assert(segId == iSeg);

    Seg_coord0[iSeg][0] = x0;
    Seg_coord0[iSeg][1] = y0; 
    Seg_coord0[iSeg][2] = z0; 

    Seg_coord1[iSeg][0] = x1;
    Seg_coord1[iSeg][1] = y1; 
    Seg_coord1[iSeg][2] = z1; 
  }

  fclose (fp);
}

void ReadTriFile (const char *fileName)
{
  FILE *fp;
  int iTri;

  assert(NTris == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NTris);
  assert(0 <= NTris);
  assert(NTris <= MAX_TRIS);
  
  for (iTri = 0; iTri < NTris; iTri++) {
    int triId;
    double x0, y0, z0, x1, y1, z1, x2, y2, z2;

    fscanf (fp, "%d %lf %lf %lf %lf %lf %lf %lf %lf %lf", 
	    &triId,
	    &x0, &y0, &z0, 
	    &x1, &y1, &z1, 
	    &x2, &y2, &z2);
    assert(triId == iTri);

    Tri_coord0[iTri][0] = x0;
    Tri_coord0[iTri][1] = y0; 
    Tri_coord0[iTri][2] = z0; 

    Tri_coord1[iTri][0] = x1;
    Tri_coord1[iTri][1] = y1; 
    Tri_coord1[iTri][2] = z1; 

    Tri_coord2[iTri][0] = x2;
    Tri_coord2[iTri][1] = y2; 
    Tri_coord2[iTri][2] = z2; 
  }

  fclose (fp);
}



/************************
	 Command
*************************/

#if 0
double Tolerance = 0.1;
#else
double Tolerance = 0.2;
#endif

int CheckMovingVertex 
(int elementId,
 double x, double y, double z,
 double L0, double L1, double L2, double L3,
 NodeTopologyType topologyType)
{
  int elementNodeId;
  
  elementNodeId = -1;

  if (1.0 - Tolerance * 2.1 < L0) {
    elementNodeId = 0;
  } else if (1.0 - Tolerance * 2.1 < L1) {
    elementNodeId = 1;
  } else if (1.0 - Tolerance * 2.1 < L2) {
    elementNodeId = 2;
  } else if (1.0 - Tolerance * 2.1 < L3) {
    elementNodeId = 3;
  }

  if (1.0 - VOLUME_COORD_EPS < L0) {
    elementNodeId = -1;
  } else if (1.0 - VOLUME_COORD_EPS < L1) {
    elementNodeId = -1;
  } else if (1.0 - VOLUME_COORD_EPS < L2) {
    elementNodeId = -1;
  } else if (1.0 - VOLUME_COORD_EPS < L3) {
    elementNodeId = -1;
  }

  if (elementNodeId != -1) {
    int nodeId = Element_nodeIds[elementId][elementNodeId];
    
    Node_moveTo (nodeId, x, y, z);
    Node_setTopology (nodeId, topologyType);
    return 1;
  }
  return 0;
}

void DivideEdge
(int edgeNodeId0, int edgeNodeId1,
 int newNodeId)
{
  int iElement;
  int iElementEdge;

  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int nodeId0 = Element_nodeIds[iElement]
	[EdgeNodeMap[iElementEdge][0]];
      int nodeId1 = Element_nodeIds[iElement]
	[EdgeNodeMap[iElementEdge][1]];

      if ((nodeId0 == edgeNodeId0 && nodeId1 == edgeNodeId1)
	  || (nodeId1 == edgeNodeId0 && nodeId0 == edgeNodeId1)) {
	Mesh_divideElementOnEdge (iElement, iElementEdge, newNodeId);
	break;
      }
    }
  }
}

int CheckDividingEdge 
(int elementId,
 double x, double y, double z,
 double L0, double L1, double L2, double L3,
 NodeTopologyType topologyType)
{
  int elementEdgeId;
  
  elementEdgeId = -1;

  if (L2 < Tolerance 
      && L3 < Tolerance) {
    elementEdgeId = 0;
  } else if (L3 < Tolerance 
	     && L1 < Tolerance) {
    elementEdgeId = 1;
  } else if (L1 < Tolerance
	     && L2 < Tolerance) {
    elementEdgeId = 2;
  } else if (L0 < Tolerance 
	     && L3 < Tolerance) {
    elementEdgeId = 3;
  } else if (L0 < Tolerance 
	     && L1 < Tolerance) {
    elementEdgeId = 4;
  } else if (L0 < Tolerance 
	     && L2 < Tolerance) {
    elementEdgeId = 5;
  }

  if (L2 < VOLUME_COORD_EPS 
      && L3 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  } else if (L3 < VOLUME_COORD_EPS 
	     && L1 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  } else if (L1 < VOLUME_COORD_EPS 
	     && L2 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  } else if (L0 < VOLUME_COORD_EPS 
	     && L3 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  } else if (L0 < VOLUME_COORD_EPS 
	     && L1 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  } else if (L0 < VOLUME_COORD_EPS 
	     && L2 < VOLUME_COORD_EPS) {
    elementEdgeId = -1;
  }

  if (elementEdgeId != -1) {
    int nodeId0 = Element_nodeIds[elementId]
      [EdgeNodeMap[elementEdgeId][0]];
    int nodeId1 = Element_nodeIds[elementId]
      [EdgeNodeMap[elementEdgeId][1]];



    int nodeId = Mesh_findNodeAt (x, y, z);


    int newNodeId = Mesh_addNewNode (x, y, z, topologyType);


    assert(nodeId == -1);


    DivideEdge (nodeId0, nodeId1, 
		newNodeId);
    return 1;
  }
  return 0;
}

void DivideFace
(int faceNodeId0, int faceNodeId1, int faceNodeId2,
 int newNodeId)
{
  int minFaceNodeId, midFaceNodeId, maxFaceNodeId; 
  int iElement;
  int iElementFace;

  SortId3 (faceNodeId0, faceNodeId1, faceNodeId2,
	   &minFaceNodeId, &midFaceNodeId, &maxFaceNodeId);

  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int nodeId0 = Element_nodeIds[iElement]
	[FaceNodeMap[iElementFace][0]];
      int nodeId1 = Element_nodeIds[iElement]
	[FaceNodeMap[iElementFace][1]];
      int nodeId2 = Element_nodeIds[iElement]
	[FaceNodeMap[iElementFace][2]];

      int minNodeId, midNodeId, maxNodeId;

      SortId3 (nodeId0, nodeId1, nodeId2,
	       &minNodeId, &midNodeId, &maxNodeId);

      if (minFaceNodeId == minNodeId
	  && midFaceNodeId == midNodeId
	  && maxFaceNodeId == maxNodeId) {
	Mesh_divideElementOnFace (iElement, iElementFace, newNodeId);
	break;
      }
    }
  }
}

int CheckDividingFace 
(int elementId,
 double x, double y, double z,
 double L0, double L1, double L2, double L3,
 NodeTopologyType topologyType)
{
  int elementFaceId;
  
  elementFaceId = -1;

  if (L0 < Tolerance) {
    elementFaceId = 0;
  } else if (L1 < Tolerance) {
    elementFaceId = 1;
  } else if (L2 < Tolerance) {
    elementFaceId = 2;
  } else if (L3 < Tolerance) {
    elementFaceId = 3;
  }

  if (L0 < VOLUME_COORD_EPS) {
    elementFaceId = -1;
  } else if (L1 < VOLUME_COORD_EPS) {
    elementFaceId = -1;
  } else if (L2 < VOLUME_COORD_EPS) {
    elementFaceId = -1;
  } else if (L3 < VOLUME_COORD_EPS) {
    elementFaceId = -1;
  }

  if (elementFaceId != -1) {
    int nodeId0 = Element_nodeIds[elementId]
      [FaceNodeMap[elementFaceId][0]];
    int nodeId1 = Element_nodeIds[elementId]
      [FaceNodeMap[elementFaceId][1]];
    int nodeId2 = Element_nodeIds[elementId]
      [FaceNodeMap[elementFaceId][2]];



    int nodeId = Mesh_findNodeAt (x, y, z);



    int newNodeId = Mesh_addNewNode (x, y, z, topologyType);


    assert(nodeId == -1);

    
    DivideFace (nodeId0, nodeId1, nodeId2, 
		newNodeId);
    return 1;
  }
  return 0;
}

int CheckIntersectionWithPoint 
(double px, double py, double pz)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    double L0, L1, L2, L3;

    if (Element_isDeleted[iElement]) continue;
    
    if (Element_containsPoint 
	(iElement, px, py, pz,
	 &L0, &L1, &L2, &L3)) {
      if (CheckMovingVertex 
	  (iElement, px, py, pz,
	   L0, L1, L2, L3,
	   NODE_TOPOLOGY_VERTEX)) {
	return 1;
      } else if (CheckDividingEdge 
		 (iElement, px, py, pz,
		  L0, L1, L2, L3,
		  NODE_TOPOLOGY_VERTEX)) {
	return 1;
      } else if (CheckDividingFace 
		 (iElement, px, py, pz,
		  L0, L1, L2, L3,
		  NODE_TOPOLOGY_VERTEX)) {
	return 1;
      }
    }
  }
  return 0;
}

void CheckPoint (double x, double y, double z)
{
  for (;;) {
    int found = CheckIntersectionWithPoint (x, y, z);

    if (found) {
      /* nothing */
    } else {
      break;
    }
  }
}

int CheckIntersectionWithSeg 
(double sx0, double sy0, double sz0, 
 double sx1, double sy1, double sz1)
{
  int iElement;
  int iElementFace;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceNodeId0 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][0]];
      int faceNodeId1 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][1]];
      int faceNodeId2 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][2]];

      double fx0 = Node_coord[faceNodeId0][0];
      double fy0 = Node_coord[faceNodeId0][1];
      double fz0 = Node_coord[faceNodeId0][2];

      double fx1 = Node_coord[faceNodeId1][0];
      double fy1 = Node_coord[faceNodeId1][1];
      double fz1 = Node_coord[faceNodeId1][2];

      double fx2 = Node_coord[faceNodeId2][0];
      double fy2 = Node_coord[faceNodeId2][1];
      double fz2 = Node_coord[faceNodeId2][2];

      double intX, intY, intZ;

      if (GetIntersectionBetweenTriAndSeg 
	  (fx0, fy0, fz0, 
	   fx1, fy1, fz1, 
	   fx2, fy2, fz2, 
	   sx0, sy0, sz0, 
	   sx1, sy1, sz1, 
	   &intX, &intY, &intZ)) {
	int flag;
	double L0, L1, L2, L3;

	flag = Element_containsPoint 
	  (iElement, intX, intY, intZ,
	   &L0, &L1, &L2, &L3);
	assert(flag);
	
	if (CheckMovingVertex 
	    (iElement, intX, intY, intZ,
	     L0, L1, L2, L3,
	     NODE_TOPOLOGY_EDGE)) {
	  return 1;
	} else if (CheckDividingEdge 
		   (iElement, intX, intY, intZ,
		    L0, L1, L2, L3,
		    NODE_TOPOLOGY_EDGE)) {
	  return 1;
	}
      }
    }
  }
  return 0;
}

void CheckSeg (double sx0, double sy0, double sz0, 
	       double sx1, double sy1, double sz1)
{
  for (;;) {
    int found = CheckIntersectionWithSeg 
      (sx0, sy0, sz0,
       sx1, sy1, sz1);

    if (found) {
      /* nothing */
    } else {
      break;
    }
  }
}

int CheckIntersectionWithTri 
(double tx0, double ty0, double tz0, 
 double tx1, double ty1, double tz1, 
 double tx2, double ty2, double tz2)
{
  int iElement;
  int iElementEdge;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeNodeId0 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][0]];
      int edgeNodeId1 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][1]];

      double ex0 = Node_coord[edgeNodeId0][0];
      double ey0 = Node_coord[edgeNodeId0][1];
      double ez0 = Node_coord[edgeNodeId0][2];

      double ex1 = Node_coord[edgeNodeId1][0];
      double ey1 = Node_coord[edgeNodeId1][1];
      double ez1 = Node_coord[edgeNodeId1][2];

      double intX, intY, intZ;

      if (GetIntersectionBetweenTriAndSeg 
	  (tx0, ty0, tz0, 
	   tx1, ty1, tz1, 
	   tx2, ty2, tz2, 
	   ex0, ey0, ez0, 
	   ex1, ey1, ez1, 
	   &intX, &intY, &intZ)) {
	int flag;
	double L0, L1, L2, L3;
	
	flag = Element_containsPoint 
	  (iElement, intX, intY, intZ,
	   &L0, &L1, &L2, &L3);
	assert(flag);
	
	if (CheckMovingVertex 
	    (iElement, intX, intY, intZ,
	     L0, L1, L2, L3,
	     NODE_TOPOLOGY_FACE)) {
	  return 1;
	}
      }
    }
  }
  return 0;
}

void CheckTri (double tx0, double ty0, double tz0, 
	       double tx1, double ty1, double tz1, 
	       double tx2, double ty2, double tz2)
{
  for (;;) {
    int found = CheckIntersectionWithTri 
      (tx0, ty0, tz0,
       tx1, ty1, tz1,
       tx2, ty2, tz2);
    
    if (found) {
      /* nothing */
    } else {
      break;
    }
  }
}

int FindIntersectionWithSeg 
(double sx0, double sy0, double sz0, 
 double sx1, double sy1, double sz1,
 double *intX_OUT, double *intY_OUT, double *intZ_OUT)
{
  int iElement;
  int iElementFace;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int faceNodeId0 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][0]];
      int faceNodeId1 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][1]];
      int faceNodeId2 
	= Element_nodeIds[iElement][FaceNodeMap[iElementFace][2]];

      double fx0 = Node_coord[faceNodeId0][0];
      double fy0 = Node_coord[faceNodeId0][1];
      double fz0 = Node_coord[faceNodeId0][2];

      double fx1 = Node_coord[faceNodeId1][0];
      double fy1 = Node_coord[faceNodeId1][1];
      double fz1 = Node_coord[faceNodeId1][2];

      double fx2 = Node_coord[faceNodeId2][0];
      double fy2 = Node_coord[faceNodeId2][1];
      double fz2 = Node_coord[faceNodeId2][2];

      if (GetIntersectionBetweenTriAndSeg 
	  (fx0, fy0, fz0, 
	   fx1, fy1, fz1, 
	   fx2, fy2, fz2, 
	   sx0, sy0, sz0, 
	   sx1, sy1, sz1, 
	   intX_OUT, intY_OUT, intZ_OUT)) {
	int flag;
	double L0, L1, L2, L3;

	flag = Element_containsPoint 
	  (iElement, *intX_OUT, *intY_OUT, *intZ_OUT,
	   &L0, &L1, &L2, &L3);
	assert(flag);
	
	if (L0 < 1.0 - VOLUME_COORD_EPS
	    && L1 < 1.0 - VOLUME_COORD_EPS
	    && L2 < 1.0 - VOLUME_COORD_EPS
	    && L3 < 1.0 - VOLUME_COORD_EPS) {
	  return 1;
	}
      }
    }
  }
  return 0;
}

int FindIntersectionWithTri 
(double tx0, double ty0, double tz0, 
 double tx1, double ty1, double tz1, 
 double tx2, double ty2, double tz2,
 double *intX_OUT, double *intY_OUT, double *intZ_OUT)
{
  int iElement;
  int iElementEdge;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isDeleted[iElement]) continue;
    
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeNodeId0 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][0]];
      int edgeNodeId1 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][1]];

      double ex0 = Node_coord[edgeNodeId0][0];
      double ey0 = Node_coord[edgeNodeId0][1];
      double ez0 = Node_coord[edgeNodeId0][2];

      double ex1 = Node_coord[edgeNodeId1][0];
      double ey1 = Node_coord[edgeNodeId1][1];
      double ez1 = Node_coord[edgeNodeId1][2];

      if (GetIntersectionBetweenTriAndSeg 
	  (tx0, ty0, tz0, 
	   tx1, ty1, tz1, 
	   tx2, ty2, tz2, 
	   ex0, ey0, ez0, 
	   ex1, ey1, ez1, 
	   intX_OUT, intY_OUT, intZ_OUT)) {
	int flag;
	double L0, L1, L2, L3;
	
	flag = Element_containsPoint 
	  (iElement, *intX_OUT, *intY_OUT, *intZ_OUT,
	   &L0, &L1, &L2, &L3);
	assert(flag);
	
	if (L0 < 1.0 - VOLUME_COORD_EPS
	    && L1 < 1.0 - VOLUME_COORD_EPS
	    && L2 < 1.0 - VOLUME_COORD_EPS
	    && L3 < 1.0 - VOLUME_COORD_EPS) {
	  return 1;
	}
      }
    }
  }
  return 0;
}

void InsertPointIntoElement 
(int elementId, int newNodeId,
 double L0, double L1, double L2, double L3)
{
  if (fabs (L0 - 1.0) < VOLUME_COORD_EPS
      || fabs (L1 - 1.0) < VOLUME_COORD_EPS
      || fabs (L2 - 1.0) < VOLUME_COORD_EPS
      || fabs (L3 - 1.0) < VOLUME_COORD_EPS) return;

  if (fabs (L2) < VOLUME_COORD_EPS 
      && fabs (L3) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 0, newNodeId);
  } else if (fabs (L3) < VOLUME_COORD_EPS 
	     && fabs (L1) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 1, newNodeId);
  } else if (fabs (L1) < VOLUME_COORD_EPS 
	     && fabs (L2) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 2, newNodeId);
  } else if (fabs (L0) < VOLUME_COORD_EPS 
	     && fabs (L3) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 3, newNodeId);
  } else if (fabs (L0) < VOLUME_COORD_EPS 
	     && fabs (L1) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 4, newNodeId);
  } else if (fabs (L0) < VOLUME_COORD_EPS 
	     && fabs (L2) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnEdge (elementId, 5, newNodeId);

  } else if (fabs (L0) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnFace (elementId, 0, newNodeId);
  } else if (fabs (L1) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnFace (elementId, 1, newNodeId);
  } else if (fabs (L2) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnFace (elementId, 2, newNodeId);
  } else if (fabs (L3) < VOLUME_COORD_EPS) {
    Mesh_divideElementOnFace (elementId, 3, newNodeId);

  } else {
    Mesh_divideElementInside (elementId, newNodeId);
  }
}

void Divide 
(double x, double y, double z,
 NodeTopologyType topologyType)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    double L0, L1, L2, L3;

    if (Element_isDeleted[iElement]) continue;
    
    if (Element_containsPoint (iElement, x, y, z,
			       &L0, &L1, &L2, &L3)) {
      if (L0 < 1.0 - VOLUME_COORD_EPS
	  && L1 < 1.0 - VOLUME_COORD_EPS
	  && L2 < 1.0 - VOLUME_COORD_EPS
	  && L3 < 1.0 - VOLUME_COORD_EPS) {
	int nodeId;
	
	nodeId = Mesh_findNodeAt (x, y, z);
	if (nodeId != -1) {
	  Node_setTopology (nodeId, topologyType);
	} else {
	  nodeId = Mesh_addNewNode (x, y, z, topologyType);
	}

	InsertPointIntoElement (iElement, nodeId, 
				L0, L1, L2, L3);
      } else {
	int nodeId = Mesh_findNodeAt (x, y, z);

	assert(nodeId != -1);
	Node_setTopology (nodeId, topologyType);
      }
    }
  }
}

void InsertSeg (double sx0, double sy0, double sz0, 
		double sx1, double sy1, double sz1)
{
  for (;;) {
    double intX, intY, intZ;
    int found = FindIntersectionWithSeg 
      (sx0, sy0, sz0,
       sx1, sy1, sz1,
       &intX, &intY, &intZ);

    if (found) {
#ifdef PRINT
      fprintf (stderr, " insert seg found \n");
#endif

      Divide (intX, intY, intZ, NODE_TOPOLOGY_EDGE);
    } else {
      break;
    }
  }
}

void InsertTri (double tx0, double ty0, double tz0, 
		double tx1, double ty1, double tz1, 
		double tx2, double ty2, double tz2)
{
  for (;;) {
    double intX, intY, intZ;
    int found = FindIntersectionWithTri 
      (tx0, ty0, tz0,
       tx1, ty1, tz1,
       tx2, ty2, tz2,
       &intX, &intY, &intZ);
    
    if (found) {
#ifdef PRINT
      fprintf (stderr, " insert tri found \n");
#endif

      Divide (intX, intY, intZ, NODE_TOPOLOGY_FACE);
    } else {
      break;
    }

#if 0
    CheckSliver ();
#endif
  }
}

void PerformCommand (void)
{
  int iPoint;
  int iSeg;
  int iTri;

#if 0
  CheckSliver ();

  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double px = Point_coord[iPoint][0];
    double py = Point_coord[iPoint][1];
    double pz = Point_coord[iPoint][2];
    
    CheckPoint (px, py, pz);
  }

  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double sx0 = Seg_coord0[iSeg][0];
    double sy0 = Seg_coord0[iSeg][1];
    double sz0 = Seg_coord0[iSeg][2];

    double sx1 = Seg_coord1[iSeg][0];
    double sy1 = Seg_coord1[iSeg][1];
    double sz1 = Seg_coord1[iSeg][2];
    
    CheckSeg (sx0, sy0, sz0, 
	      sx1, sy1, sz1);
  }

  for (iTri = 0; iTri < NTris; iTri++) {
    double tx0 = Tri_coord0[iTri][0];
    double ty0 = Tri_coord0[iTri][1];
    double tz0 = Tri_coord0[iTri][2];

    double tx1 = Tri_coord1[iTri][0];
    double ty1 = Tri_coord1[iTri][1];
    double tz1 = Tri_coord1[iTri][2];
    
    double tx2 = Tri_coord2[iTri][0];
    double ty2 = Tri_coord2[iTri][1];
    double tz2 = Tri_coord2[iTri][2];
    
    CheckTri (tx0, ty0, tz0, 
	      tx1, ty1, tz1, 
	      tx2, ty2, tz2);
  }

  CheckSliver ();
#endif

  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    double px = Point_coord[iPoint][0];
    double py = Point_coord[iPoint][1];
    double pz = Point_coord[iPoint][2];
    
    Divide (px, py, pz, NODE_TOPOLOGY_VERTEX);
  }

  for (iSeg = 0; iSeg < NSegs; iSeg++) {
    double sx0 = Seg_coord0[iSeg][0];
    double sy0 = Seg_coord0[iSeg][1];
    double sz0 = Seg_coord0[iSeg][2];

    double sx1 = Seg_coord1[iSeg][0];
    double sy1 = Seg_coord1[iSeg][1];
    double sz1 = Seg_coord1[iSeg][2];
    
    InsertSeg (sx0, sy0, sz0, 
	       sx1, sy1, sz1);
  }

  for (iTri = 0; iTri < NTris; iTri++) {
    double tx0 = Tri_coord0[iTri][0];
    double ty0 = Tri_coord0[iTri][1];
    double tz0 = Tri_coord0[iTri][2];

    double tx1 = Tri_coord1[iTri][0];
    double ty1 = Tri_coord1[iTri][1];
    double tz1 = Tri_coord1[iTri][2];
    
    double tx2 = Tri_coord2[iTri][0];
    double ty2 = Tri_coord2[iTri][1];
    double tz2 = Tri_coord2[iTri][2];
    
    InsertTri (tx0, ty0, tz0, 
	       tx1, ty1, tz1, 
	       tx2, ty2, tz2);
  }
}



int main (int argc, char *argv[])
{
  if (argc != 10) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_insert  node_file  element_file \n"
       "        node_topology_file \n"
       "        point_file  seg_file  tri_file \n"
       "        node_file_OUT  element_file_OUT \n"
       "        node_topology_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeTopologyFile (argv[3]);
  ReadPointFile (argv[4]);
  ReadSegFile (argv[5]);
  ReadTriFile (argv[6]);

  PerformCommand ();

  WriteNodeFile (argv[7]);
  WriteElementFile (argv[8]);
  WriteNodeTopologyFile (argv[9]);

  return 0;
}
