/* advautomesh_h_tetra3d_particle2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define VOLUME_COORD_EPS 0.00001



double SignedTetraVolume
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double x3, double y3, double z3)
{
  double x10 = x1 - x0;
  double y10 = y1 - y0;
  double z10 = z1 - z0;

  double x20 = x2 - x0;
  double y20 = y2 - y0;
  double z20 = z2 - z0;
  
  double x30 = x3 - x0;
  double y30 = y3 - y0;
  double z30 = z3 - z0;
  
  double d23 = y20 * z30 - z20 * y30;
  double d31 = y30 * z10 - z30 * y10;
  double d12 = y10 * z20 - z10 * y20;
  
  return (x10 * d23 + x20 * d31 + x30 * d12) / 6;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];
double Node_vector[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define N_ELEMENT_FACES 4
#define MAX_ELEMENTS 12000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_FACES];



void Element_getVector
(int elementId,
 double L0, double L1, double L2, double L3,
 double *vectorX_OUT, double *vectorY_OUT, double *vectorZ_OUT)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];
  
  *vectorX_OUT 
    = Node_vector[nodeId0][0] * L0
    + Node_vector[nodeId1][0] * L1
    + Node_vector[nodeId2][0] * L2
    + Node_vector[nodeId3][0] * L3;
  *vectorY_OUT
    = Node_vector[nodeId0][1] * L0
    + Node_vector[nodeId1][1] * L1
    + Node_vector[nodeId2][1] * L2
    + Node_vector[nodeId3][1] * L3;
  *vectorZ_OUT
    = Node_vector[nodeId0][2] * L0
    + Node_vector[nodeId1][2] * L1
    + Node_vector[nodeId2][2] * L2
    + Node_vector[nodeId3][2] * L3;
}

int Element_containsPoint 
(int elementId, 
 double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;
  double x0, y0, z0;
  double x1, y1, z1;
  double x2, y2, z2;
  double x3, y3, z3;
  double volume;
  double volume0, volume1, volume2, volume3;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];
  z0 = Node_coord[nodeId0][2];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];
  z1 = Node_coord[nodeId1][2];

  x2 = Node_coord[nodeId2][0];
  y2 = Node_coord[nodeId2][1];
  z2 = Node_coord[nodeId2][2];
  
  x3 = Node_coord[nodeId3][0];
  y3 = Node_coord[nodeId3][1];
  z3 = Node_coord[nodeId3][2];
  
  volume = SignedTetraVolume (x0, y0, z0, 
			      x1, y1, z1, 
			      x2, y2, z2, 
			      x3, y3, z3);
  volume0 = SignedTetraVolume (x, y, z, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume1 = SignedTetraVolume (x0, y0, z0, 
			       x, y, z, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume2 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x, y, z, 
			       x3, y3, z3);
  volume3 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x, y, z);

  *L0_OUT = volume0 / volume;
  *L1_OUT = volume1 / volume;
  *L2_OUT = volume2 / volume;
  *L3_OUT = volume3 / volume;

  if (0.0 - VOLUME_COORD_EPS < *L0_OUT 
      && *L0_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L1_OUT 
      && *L1_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L2_OUT 
      && *L2_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L3_OUT 
      && *L3_OUT < 1.0 + VOLUME_COORD_EPS) {
    return 1;
  }
  return 0;
}

int Mesh_findElementAt 
(double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_containsPoint (iElement, x, y, z,
			       L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
      return iElement;
    }
  }
  return -1;
}

int Mesh_findElementNear 
(int elementId, 
 double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int iElementFace;
  int jElementFace;
  int kElementFace;
  
  assert(elementId != -1);

  if (Element_containsPoint (elementId, x, y, z,
			     L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
    return elementId;
  }

  for (iElementFace = 0; 
       iElementFace < N_ELEMENT_FACES; 
       iElementFace++) {
    int neighborId = Element_neighborIds[elementId][iElementFace];
    
    if (neighborId == -1) continue;

    if (Element_containsPoint (neighborId, x, y, z,
			       L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
      return neighborId;
    }
  }

  for (iElementFace = 0; 
       iElementFace < N_ELEMENT_FACES; 
       iElementFace++) {
    int neighborIdI = Element_neighborIds[elementId][iElementFace];
    
    if (neighborIdI == -1) continue;

    for (jElementFace = 0; 
	 jElementFace < N_ELEMENT_FACES; 
	 jElementFace++) {
      int neighborIdJ = Element_neighborIds[neighborIdI][jElementFace];
      
      if (neighborIdJ == -1) continue;
      if (neighborIdJ == elementId) continue;

      if (Element_containsPoint (neighborIdJ, x, y, z,
				 L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
	return neighborIdJ;
      }
    }
  }

  for (iElementFace = 0; 
       iElementFace < N_ELEMENT_FACES; 
       iElementFace++) {
    int neighborIdI = Element_neighborIds[elementId][iElementFace];
    
    if (neighborIdI == -1) continue;

    for (jElementFace = 0; 
	 jElementFace < N_ELEMENT_FACES; 
	 jElementFace++) {
      int neighborIdJ = Element_neighborIds[neighborIdI][jElementFace];
      
      if (neighborIdJ == -1) continue;
      if (neighborIdJ == elementId) continue;

      for (kElementFace = 0; 
	   kElementFace < N_ELEMENT_FACES; 
	   kElementFace++) {
	int neighborIdK = Element_neighborIds[neighborIdJ][kElementFace];
	
	if (neighborIdK == -1) continue;
	if (neighborIdK == elementId) continue;
	if (neighborIdK == neighborIdI) continue;
	
	if (Element_containsPoint (neighborIdK, x, y, z,
				   L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
	  return neighborIdK;
	}
      }
    }
  }

  return -1;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iElement;
  int iElementFace;
  
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);

    for (iElementFace = 0; 
	 iElementFace < N_ELEMENT_FACES; 
	 iElementFace++) {
      int neighborId;

      fscanf (fp, "%d", 
	      &neighborId);
      assert((0 <= neighborId && neighborId < NElements) 
	     || neighborId == -1);
      Element_neighborIds[iElement][iElementFace] = neighborId;
    }
  }

  fclose (fp);
}

void ReadNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double vectorX, vectorY, vectorZ;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId,
	    &vectorX, &vectorY, &vectorZ);
    assert(nodeId == iNode);
    Node_vector[iNode][0] = vectorX;
    Node_vector[iNode][1] = vectorY;
    Node_vector[iNode][2] = vectorZ;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Particle */

#define MAX_PARTICLES 1000
#define MAX_STEPS 1000

int NParticles;
int NSteps;
double DeltaT;
double Particle_startCoord[MAX_PARTICLES][3]; 

double Point_coord[MAX_PARTICLES][MAX_STEPS][3];
int Point_elementId[MAX_PARTICLES][MAX_STEPS];
double Point_L0[MAX_PARTICLES][MAX_STEPS];
double Point_L1[MAX_PARTICLES][MAX_STEPS];
double Point_L2[MAX_PARTICLES][MAX_STEPS];
double Point_L3[MAX_PARTICLES][MAX_STEPS];



void ParticleGroup_setParticlePoint 
(int particleId, int stepId,
 double x, double y, double z,
 int elementId, 
 double L0, double L1, double L2, double L3)
{
  assert(0 <= particleId);
  assert(particleId < NParticles);
  assert(0 <= stepId);
  assert(stepId < NSteps);
  assert(0 <= elementId || elementId == -1);
  assert(elementId < NElements);

  Point_coord[particleId][stepId][0] = x;
  Point_coord[particleId][stepId][1] = y;
  Point_coord[particleId][stepId][2] = z;

  Point_elementId[particleId][stepId] = elementId;

  Point_L0[particleId][stepId] = L0;
  Point_L1[particleId][stepId] = L1;
  Point_L2[particleId][stepId] = L2;
  Point_L3[particleId][stepId] = L3;
}

void ReadParticleFile (const char *fileName)
{
  FILE *fp;
  int iParticle;
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d %d %lf", 
	  &NParticles,
	  &NSteps,
	  &DeltaT);
  assert(0 < NParticles);
  assert(NParticles <= MAX_PARTICLES);
  assert(1 < NSteps);
  assert(NSteps <= MAX_STEPS);

  for (iParticle = 0; iParticle < NParticles; iParticle++) {
    int particleId;
    double x, y, z;
    
    fscanf (fp, "%d %lf %lf %lf", 
	    &particleId,
	    &x, &y, &z);
    assert(particleId == iParticle);
    Particle_startCoord[iParticle][0] = x;
    Particle_startCoord[iParticle][1] = y;
    Particle_startCoord[iParticle][2] = z;
  }

  fclose (fp);
}

void WriteParticleTraceFile (const char *fileName)
{
  FILE *fp;
  int iParticle;
  int iStep;
  
  assert(0 < NParticles);
  assert(1 < NSteps);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d %d\n", 
	   NParticles, NSteps);
  
  for (iParticle = 0; iParticle < NParticles; iParticle++) {
    fprintf (fp, "%d\n", 
	     iParticle);
    
    for (iStep = 0; iStep < NSteps; iStep++) {
      fprintf (fp, "%d   %22.15e %22.15e %22.15e   %d   %22.15e %22.15e %22.15e %22.15e\n", 
	       iStep,
	       Point_coord[iParticle][iStep][0],
	       Point_coord[iParticle][iStep][1],
	       Point_coord[iParticle][iStep][2],
	       Point_elementId[iParticle][iStep],
	       Point_L0[iParticle][iStep],
	       Point_L1[iParticle][iStep],
	       Point_L2[iParticle][iStep],
	       Point_L3[iParticle][iStep]);
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iParticle;
  int iStep;

  assert(0 < NParticles); 
  assert(1 < NSteps); 

  iStep = 0;
  for (iParticle = 0; iParticle < NParticles; iParticle++) {
    double x = Particle_startCoord[iParticle][0];
    double y = Particle_startCoord[iParticle][1];
    double z = Particle_startCoord[iParticle][2];
    int elementId;
    double L0, L1, L2, L3;

    elementId = Mesh_findElementAt (x, y, z,
				    &L0, &L1, &L2, &L3);
    if (elementId != -1) {
      ParticleGroup_setParticlePoint
	(iParticle, iStep,
	 x, y, z, 
	 elementId, L0, L1, L2, L3);
    } else {
      ParticleGroup_setParticlePoint
	(iParticle, iStep,
	 x, y, z, 
	 -1, 0.0, 0.0, 0.0, 0.0);
    }
  }

  for (iStep = 1; iStep < NSteps; iStep++) {
    for (iParticle = 0; iParticle < NParticles; iParticle++) {
      double x = Point_coord[iParticle][iStep - 1][0];
      double y = Point_coord[iParticle][iStep - 1][1];
      double z = Point_coord[iParticle][iStep - 1][2];
      int elementId = Point_elementId[iParticle][iStep - 1];
      double L0 = Point_L0[iParticle][iStep - 1];
      double L1 = Point_L1[iParticle][iStep - 1];
      double L2 = Point_L2[iParticle][iStep - 1];
      double L3 = Point_L3[iParticle][iStep - 1];
      
      if (elementId != -1) {
	double vectorX, vectorY, vectorZ;

	Element_getVector (elementId, L0, L1, L2, L3,
			   &vectorX, &vectorY, &vectorZ);
	x += vectorX * DeltaT;
	y += vectorY * DeltaT;
	z += vectorZ * DeltaT;

	elementId = Mesh_findElementNear (elementId, x, y, z, 
					  &L0, &L1, &L2, &L3);
	if (elementId == -1) {
	  elementId = Mesh_findElementAt (x, y, z,
					  &L0, &L1, &L2, &L3);
	}
      }

      if (elementId != -1) {
	ParticleGroup_setParticlePoint
	  (iParticle, iStep,
	   x, y, z, 
	   elementId, L0, L1, L2, L3);
      } else {
	ParticleGroup_setParticlePoint
	  (iParticle, iStep,
	   x, y, z, 
	   -1, 0.0, 0.0, 0.0, 0.0);
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_particle2  node_file  element_file \n"
       "        element_neighbor_file  node_vector_file  particle_file \n"
       "        particle_trace_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadElementNeighborFile (argv[3]);
  ReadNodeVectorFile (argv[4]);
  ReadParticleFile (argv[5]);

  PerformCommand ();

  WriteParticleTraceFile (argv[6]);

  return 0;
}
