/* advautomesh_h_tetra3d_sample.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define VOLUME_COORD_EPS 0.00001



double SignedTetraVolume
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double x2, double y2, double z2,
 double x3, double y3, double z3)
{
  double x10 = x1 - x0;
  double y10 = y1 - y0;
  double z10 = z1 - z0;

  double x20 = x2 - x0;
  double y20 = y2 - y0;
  double z20 = z2 - z0;
  
  double x30 = x3 - x0;
  double y30 = y3 - y0;
  double z30 = z3 - z0;
  
  double d23 = y20 * z30 - z20 * y30;
  double d31 = y30 * z10 - z30 * y10;
  double d12 = y10 * z20 - z10 * y20;
  
  return (x10 * d23 + x20 * d31 + x30 * d12) / 6;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 12000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



int Element_containsPoint 
(int elementId, 
 double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;
  double x0, y0, z0;
  double x1, y1, z1;
  double x2, y2, z2;
  double x3, y3, z3;
  double volume;
  double volume0, volume1, volume2, volume3;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];
  z0 = Node_coord[nodeId0][2];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];
  z1 = Node_coord[nodeId1][2];

  x2 = Node_coord[nodeId2][0];
  y2 = Node_coord[nodeId2][1];
  z2 = Node_coord[nodeId2][2];
  
  x3 = Node_coord[nodeId3][0];
  y3 = Node_coord[nodeId3][1];
  z3 = Node_coord[nodeId3][2];
  
  volume = SignedTetraVolume (x0, y0, z0, 
			      x1, y1, z1, 
			      x2, y2, z2, 
			      x3, y3, z3);
  volume0 = SignedTetraVolume (x, y, z, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume1 = SignedTetraVolume (x0, y0, z0, 
			       x, y, z, 
			       x2, y2, z2, 
			       x3, y3, z3);
  volume2 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x, y, z, 
			       x3, y3, z3);
  volume3 = SignedTetraVolume (x0, y0, z0, 
			       x1, y1, z1, 
			       x2, y2, z2, 
			       x, y, z);

  *L0_OUT = volume0 / volume;
  *L1_OUT = volume1 / volume;
  *L2_OUT = volume2 / volume;
  *L3_OUT = volume3 / volume;

  if (0.0 - VOLUME_COORD_EPS < *L0_OUT 
      && *L0_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L1_OUT 
      && *L1_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L2_OUT 
      && *L2_OUT < 1.0 + VOLUME_COORD_EPS
      && 0.0 - VOLUME_COORD_EPS < *L3_OUT 
      && *L3_OUT < 1.0 + VOLUME_COORD_EPS) {
    return 1;
  }
  return 0;
}

int Mesh_findElementAt 
(double x, double y, double z,
 double *L0_OUT, double *L1_OUT, double *L2_OUT, double *L3_OUT)
{
  int iElement;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_containsPoint (iElement, x, y, z,
			       L0_OUT, L1_OUT, L2_OUT, L3_OUT)) {
      return iElement;
    }
  }
  return -1;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Sample */

#define MAX_SAMPLES 1000000

int NSamples;
double Sample_coord[MAX_SAMPLES][3];
int Sample_elementId[MAX_SAMPLES];
double Sample_L0[MAX_SAMPLES];
double Sample_L1[MAX_SAMPLES];
double Sample_L2[MAX_SAMPLES];
double Sample_L3[MAX_SAMPLES];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NSamples);
  assert(0 < NSamples);
  assert(NSamples <= MAX_SAMPLES);

  for (iSample = 0; iSample < NSamples; iSample++) {
    int sampleId;
    double x, y, z;
    
    fscanf (fp, "%d %lf %lf %lf", 
	    &sampleId,
	    &x, &y, &z);
    assert(sampleId == iSample);
    Sample_coord[iSample][0] = x;
    Sample_coord[iSample][1] = y;
    Sample_coord[iSample][2] = z;
  }

  fclose (fp);
}

void WriteSampleFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(0 < NSamples);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NSamples);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e   %d   %22.15e %22.15e %22.15e %22.15e\n", 
	     iSample,
	     Sample_coord[iSample][0],
	     Sample_coord[iSample][1],
	     Sample_coord[iSample][2],
	     Sample_elementId[iSample],
	     Sample_L0[iSample],
	     Sample_L1[iSample],
	     Sample_L2[iSample],
	     Sample_L3[iSample]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iSample;

  for (iSample = 0; iSample < NSamples; iSample++) {
    double x = Sample_coord[iSample][0];
    double y = Sample_coord[iSample][1];
    double z = Sample_coord[iSample][2];
    int elementId;
    double L0, L1, L2, L3;
    
    elementId = Mesh_findElementAt (x, y, z,
				    &L0, &L1, &L2, &L3);
    if (elementId != -1) {
      Sample_elementId[iSample] = elementId;
      Sample_L0[iSample] = L0;
      Sample_L1[iSample] = L1;
      Sample_L2[iSample] = L2;
      Sample_L3[iSample] = L3;
    } else {
      Sample_elementId[iSample] = -1;
      Sample_L0[iSample] = 0.0;
      Sample_L1[iSample] = 0.0;
      Sample_L2[iSample] = 0.0;
      Sample_L3[iSample] = 0.0;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_sample  node_file  element_file \n"
       "        point_file \n"
       "        sample_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadPointFile (argv[3]);

  PerformCommand ();

  WriteSampleFile (argv[4]);

  return 0;
}
