/* advautomesh_h_tetra3d_sample_vector.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 2500000

int NNodes;
double Node_vector[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 12000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Element_getVector
(int elementId,
 double L0, double L1, double L2, double L3,
 double *vectorX_OUT, double *vectorY_OUT, double *vectorZ_OUT)
{
  int nodeId0, nodeId1, nodeId2, nodeId3;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  nodeId3 = Element_nodeIds[elementId][3];
  
  *vectorX_OUT 
    = Node_vector[nodeId0][0] * L0
    + Node_vector[nodeId1][0] * L1
    + Node_vector[nodeId2][0] * L2
    + Node_vector[nodeId3][0] * L3;
  *vectorY_OUT
    = Node_vector[nodeId0][1] * L0
    + Node_vector[nodeId1][1] * L1
    + Node_vector[nodeId2][1] * L2
    + Node_vector[nodeId3][1] * L3;
  *vectorZ_OUT
    = Node_vector[nodeId0][2] * L0
    + Node_vector[nodeId1][2] * L1
    + Node_vector[nodeId2][2] * L2
    + Node_vector[nodeId3][2] * L3;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double vectorX, vectorY, vectorZ;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId,
	    &vectorX, &vectorY, &vectorZ);
    assert(nodeId == iNode);
    Node_vector[iNode][0] = vectorX;
    Node_vector[iNode][1] = vectorY;
    Node_vector[iNode][2] = vectorZ;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Sample */

#define MAX_SAMPLES 1000000

int NSamples;
double Sample_coord[MAX_SAMPLES][3];
int Sample_elementId[MAX_SAMPLES];
double Sample_L0[MAX_SAMPLES];
double Sample_L1[MAX_SAMPLES];
double Sample_L2[MAX_SAMPLES];
double Sample_L3[MAX_SAMPLES];
double Sample_vector[MAX_SAMPLES][3];



void ReadSampleFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(NSamples == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NSamples);
  assert(0 < NSamples);
  assert(NSamples <= MAX_SAMPLES);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    int sampleId;
    double x, y, z;
    int elementId;
    double L0, L1, L2, L3;

    fscanf (fp, "%d %lf %lf %lf %d %lf %lf %lf %lf", 
	    &sampleId,
	    &x, &y, &z, 
	    &elementId, 
	    &L0, &L1, &L2, &L3);
    assert(sampleId == iSample);

    Sample_coord[iSample][0] = x;
    Sample_coord[iSample][1] = y;
    Sample_coord[iSample][2] = z;
    
    assert(0 <= elementId || elementId == -1);
    assert(elementId < NElements);
    Sample_elementId[iSample] = elementId;

    Sample_L0[iSample] = L0;
    Sample_L1[iSample] = L1;
    Sample_L2[iSample] = L2;
    Sample_L3[iSample] = L3;
  }

  fclose (fp);
}

void WriteSampleVectorFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(0 < NSamples);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NSamples);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iSample,
	     Sample_vector[iSample][0],
	     Sample_vector[iSample][1],
	     Sample_vector[iSample][2]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iSample;

  for (iSample = 0; iSample < NSamples; iSample++) {
    int elementId = Sample_elementId[iSample];
    
    if (elementId != -1) {
      double L0 = Sample_L0[iSample];
      double L1 = Sample_L1[iSample];
      double L2 = Sample_L2[iSample];
      double L3 = Sample_L3[iSample];

      Element_getVector (elementId, L0, L1, L2, L3,
			 &Sample_vector[iSample][0], 
			 &Sample_vector[iSample][1], 
			 &Sample_vector[iSample][2]);
    } else {
      Sample_vector[iSample][0] = 0.0;
      Sample_vector[iSample][1] = 0.0;
      Sample_vector[iSample][2] = 0.0;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_sample_vector  node_file  element_file \n"
       "        node_vector_file  sample_file \n"
       "        sample_vector_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeVectorFile (argv[3]);
  ReadSampleFile (argv[4]);

  PerformCommand ();

  WriteSampleVectorFile (argv[5]);

  return 0;
}
