/* advautomesh_h_tetra3d_sc_insert.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}

void GetNormalizedVector 
(double x, double y, double z,
 double *nx_OUT, double *ny_OUT, double *nz_OUT)
{
  double length = sqrt (x * x + y * y + z * z);

  *nx_OUT = x / length;
  *ny_OUT = y / length;
  *nz_OUT = z / length;
}

void GetVectorProduct 
(double x0, double y0, double z0,
 double x1, double y1, double z1,
 double *x_OUT, double *y_OUT, double *z_OUT)
{
  *x_OUT = y0 * z1 - z0 * y1;
  *y_OUT = z0 * x1 - x0 * z1;
  *z_OUT = x0 * y1 - y0 * x1;
}



/************************
	 Other Data
*************************/

/* Crack */

double Center[3];
double XDir[3], YDir[3], ZDir[3];
double XRadius, YRadius;

#define N_DIVISIONS 100



void ReadSurfaceCrackFile (const char *fileName)
{
  FILE *fp;
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%lf %lf %lf", 
	  &Center[0], &Center[1], &Center[2]);
  fscanf (fp, "%lf %lf %lf", 
	  &ZDir[0], &ZDir[1], &ZDir[2]);
  fscanf (fp, "%lf %lf %lf", 
	  &XDir[0], &XDir[1], &XDir[2]);
  fscanf (fp, "%lf %lf", 
	  &XRadius, &YRadius);

  GetNormalizedVector
    (XDir[0], XDir[1], XDir[2],
     &XDir[0], &XDir[1], &XDir[2]);
  GetNormalizedVector
    (ZDir[0], ZDir[1], ZDir[2],
     &ZDir[0], &ZDir[1], &ZDir[2]);
  GetVectorProduct 
    (ZDir[0], ZDir[1], ZDir[2],
     XDir[0], XDir[1], XDir[2],
     &YDir[0], &YDir[1], &YDir[2]);

  fclose (fp);
}

void WritePointFile (const char *fileName)
{
  FILE *fp;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   0);

  fclose (fp);
}

void WriteSegFile (const char *fileName)
{
  FILE *fp;
  int i;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   N_DIVISIONS * 4);
  
  for (i = 0; i < N_DIVISIONS * 4; i++) {
    double angle0 = M_PI / 2 / N_DIVISIONS * i;
    double s0 = sin (angle0);
    double c0 = cos (angle0);

    double x0 = Center[0] 
      + XDir[0] * XRadius * c0 + YDir[0] * YRadius * s0;
    double y0 = Center[1] 
      + XDir[1] * XRadius * c0 + YDir[1] * YRadius * s0;
    double z0 = Center[2] 
      + XDir[2] * XRadius * c0 + YDir[2] * YRadius * s0;

    double angle1 = M_PI / 2 / N_DIVISIONS * (i + 1);
    double s1 = sin (angle1);
    double c1 = cos (angle1);

    double x1 = Center[0] 
      + XDir[0] * XRadius * c1 + YDir[0] * YRadius * s1;
    double y1 = Center[1] 
      + XDir[1] * XRadius * c1 + YDir[1] * YRadius * s1;
    double z1 = Center[2] 
      + XDir[2] * XRadius * c1 + YDir[2] * YRadius * s1;

    fprintf (fp, "%d   %22.15f %22.15f %22.15f   %22.15f %22.15f %22.15f\n", 
	     i,
	     x0, y0, z0,
	     x1, y1, z1);
  }

  fclose (fp);
}

void WriteTriFile (const char *fileName)
{
  FILE *fp;
  int i;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   N_DIVISIONS * 4);
  
  for (i = 0; i < N_DIVISIONS * 4; i++) {
    double angle0 = M_PI / 2 / N_DIVISIONS * i;
    double s0 = sin (angle0);
    double c0 = cos (angle0);

    double x0 = Center[0] 
      + XDir[0] * XRadius * c0 + YDir[0] * YRadius * s0;
    double y0 = Center[1] 
      + XDir[1] * XRadius * c0 + YDir[1] * YRadius * s0;
    double z0 = Center[2] 
      + XDir[2] * XRadius * c0 + YDir[2] * YRadius * s0;

    double angle1 = M_PI / 2 / N_DIVISIONS * (i + 1);
    double s1 = sin (angle1);
    double c1 = cos (angle1);

    double x1 = Center[0] 
      + XDir[0] * XRadius * c1 + YDir[0] * YRadius * s1;
    double y1 = Center[1] 
      + XDir[1] * XRadius * c1 + YDir[1] * YRadius * s1;
    double z1 = Center[2] 
      + XDir[2] * XRadius * c1 + YDir[2] * YRadius * s1;

    fprintf 
      (fp, 
       "%d   %22.15f %22.15f %22.15f   %22.15f %22.15f %22.15f   %22.15f %22.15f %22.15f\n", 
       i,
       Center[0], Center[1], Center[2], 
       x0, y0, z0,
       x1, y1, z1);
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_sc_insert  surface_crack_file \n"
       "        point_file_OUT  seg_file_OUT  tri_file_OUT \n");
    exit (1);
  }

  ReadSurfaceCrackFile (argv[1]);

  WritePointFile (argv[2]);
  WriteSegFile (argv[3]);
  WriteTriFile (argv[4]);

  return 0;
}




