/* advautomesh_h_tetra3d_corner_inout2list.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 5000000

int NElements;
int Element_inoutType[MAX_ELEMENTS];
int Element_cornerInoutType[MAX_ELEMENTS][N_ELEMENT_NODES];



void ReadCornerInoutFile (const char *fileName)
{
  FILE *fp;
  int nPoints;
  int pointId;
  int inoutType;
  int iElement;
  int iElementNode;

  assert(NElements == 0);
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nPoints);
  assert(0 < nPoints);
  assert(nPoints % 5 == 0);

  NElements = nPoints / 5;
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fscanf (fp, "%d %d", 
	    &pointId,
	    &inoutType); 
    assert(pointId == iElement * 5);
    assert(inoutType == 0
	   || inoutType == 1
	   || inoutType == 2);
    
    Element_inoutType[iElement] = inoutType;

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fscanf (fp, "%d %d", 
	      &pointId,
	      &inoutType); 
      assert(pointId == iElement * 5 + iElementNode + 1);
      assert(inoutType == 0
	     || inoutType == 1
	     || inoutType == 2);
      
      Element_cornerInoutType[iElement][iElementNode] = inoutType;
    }
  }
  
  fclose (fp);
}

void WriteElementListFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int count;

  assert(0 < NElements);

  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_inoutType[iElement] == 1
	&& Element_cornerInoutType[iElement][0] == 1
	&& Element_cornerInoutType[iElement][1] == 1
	&& Element_cornerInoutType[iElement][2] == 1
	&& Element_cornerInoutType[iElement][3] == 1) {
      count++;
    }
  }

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_inoutType[iElement] == 1
	&& Element_cornerInoutType[iElement][0] == 1
	&& Element_cornerInoutType[iElement][1] == 1
	&& Element_cornerInoutType[iElement][2] == 1
	&& Element_cornerInoutType[iElement][3] == 1) {
      fprintf (fp, "%d   %d\n", 
	       count, iElement); 

      count++;
    }
  }
  
  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 3) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_corner_inout2list \n"
       "        corner_inout_file \n"
       "        element_list_file_OUT \n");
    exit (1);
  }

  ReadCornerInoutFile (argv[1]);
  WriteElementListFile (argv[2]);

  return 0;
}
