/* advautomesh_h_tetra3d_element_corner.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 5000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];

#define CORNER_RATIO 0.9



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void WriteElementSamplePointFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int count;
  
  assert(0 < NNodes);
  assert(0 < NElements);
  
  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements * (N_ELEMENT_NODES + 1));
  
  count = 0;
  for (iElement = 0; iElement < NElements; iElement++) {
    int nodeId0 = Element_nodeIds[iElement][0];
    int nodeId1 = Element_nodeIds[iElement][1];
    int nodeId2 = Element_nodeIds[iElement][2];
    int nodeId3 = Element_nodeIds[iElement][3];
    
    double centerX = (Node_coord[nodeId0][0] 
		      + Node_coord[nodeId1][0] 
		      + Node_coord[nodeId2][0] 
		      + Node_coord[nodeId3][0]) / 4;
    double centerY = (Node_coord[nodeId0][1] 
		      + Node_coord[nodeId1][1] 
		      + Node_coord[nodeId2][1] 
		      + Node_coord[nodeId3][1]) / 4;
    double centerZ = (Node_coord[nodeId0][2] 
		      + Node_coord[nodeId1][2] 
		      + Node_coord[nodeId2][2] 
		      + Node_coord[nodeId3][2]) / 4;
    
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n",
	     count,
	     centerX, centerY, centerZ);
    count++;

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId = Element_nodeIds[iElement][iElementNode];
      
      double nodeX = Node_coord[nodeId][0];
      double nodeY = Node_coord[nodeId][1];
      double nodeZ = Node_coord[nodeId][2];
      
      double px = centerX + (nodeX - centerX) * CORNER_RATIO;
      double py = centerY + (nodeY - centerY) * CORNER_RATIO;
      double pz = centerZ + (nodeZ - centerZ) * CORNER_RATIO;

      fprintf (fp, "%d   %22.15e %22.15e %22.15e\n",
	       count,
	       px, py, pz);
      count++;
    }
  }
  
  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3_element_corner  node_file  element_file \n"
       "        point_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  WriteElementSamplePointFile (argv[3]);

  return 0;
}
