/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



/********************************
         Polygon Data
********************************/

/* Polygon */

#define MAX_POLYGONS 1000
#define MAX_POLYGON_LOOPS 100
#define MAX_POLYGON_VERTEXS 100

int NPolygons;
int Polygon_nLoops[MAX_POLYGONS];
int Polygon_nVertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS];
double Polygon_vertexs[MAX_POLYGONS][MAX_POLYGON_LOOPS][MAX_POLYGON_VERTEXS][3];



int AddNewPolygon (void)
{
  int polygonId;
  
  polygonId = NPolygons;
  
  assert(NPolygons < MAX_POLYGONS);
  NPolygons++;

  return polygonId;
}

int AddNewLoopIntoPolygon (int polygonId)
{
  int loopId;

  assert(0 <= polygonId);
  assert(polygonId < NPolygons);

  loopId = Polygon_nLoops[polygonId];
  
  assert(Polygon_nLoops[polygonId] < MAX_POLYGON_LOOPS);
  Polygon_nLoops[polygonId]++;
  
  return loopId;
}

void AddNewVertexIntoPolygon 
(int polygonId,
 int loopId,
 double x, double y, double z)
{
  int vertexId;
  
  assert(0 <= polygonId);
  assert(polygonId < NPolygons);
  assert(0 <= loopId);
  assert(loopId < Polygon_nLoops[polygonId]);

  vertexId = Polygon_nVertexs[polygonId][loopId];

  assert(Polygon_nVertexs[polygonId][loopId] < MAX_POLYGON_VERTEXS);
  Polygon_nVertexs[polygonId][loopId]++;
  
  Polygon_vertexs[polygonId][loopId][vertexId][0] = x;
  Polygon_vertexs[polygonId][loopId][vertexId][1] = y;
  Polygon_vertexs[polygonId][loopId][vertexId][2] = z;
}

void InverseLoop 
(int polygonId,
 int loopId)
{
  int nVertexs;
  int iVertex;
  
  assert(0 <= polygonId);
  assert(polygonId < NPolygons);
  assert(0 <= loopId);
  assert(loopId < Polygon_nLoops[polygonId]);

  nVertexs = Polygon_nVertexs[polygonId][loopId];
  
  for (iVertex = 0; iVertex < nVertexs / 2; iVertex++) {
    double x = Polygon_vertexs[polygonId][loopId][iVertex][0];
    double y = Polygon_vertexs[polygonId][loopId][iVertex][1];
    double z = Polygon_vertexs[polygonId][loopId][iVertex][2];
    int oppositeId = nVertexs - 1 - iVertex;
    
    Polygon_vertexs[polygonId][loopId][iVertex][0]
      = Polygon_vertexs[polygonId][loopId][oppositeId][0];
    Polygon_vertexs[polygonId][loopId][iVertex][1]
      = Polygon_vertexs[polygonId][loopId][oppositeId][1];
    Polygon_vertexs[polygonId][loopId][iVertex][2]
      = Polygon_vertexs[polygonId][loopId][oppositeId][2];
    
    Polygon_vertexs[polygonId][loopId][oppositeId][0] = x;
    Polygon_vertexs[polygonId][loopId][oppositeId][1] = y;
    Polygon_vertexs[polygonId][loopId][oppositeId][2] = z;
  }
}

void WritePolygonFile (const char *fileName)
{
  FILE *fp;
  int iPolygon;
  int iLoop;
  int iVertex;

  assert(0 < NPolygons);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NPolygons);
  
  for (iPolygon = 0; iPolygon < NPolygons; iPolygon++) {
    int nLoops = Polygon_nLoops[iPolygon];
    
    fprintf (fp, "%d   %d\n", 
	     iPolygon,
	     nLoops);
    
    for (iLoop = 0; iLoop < nLoops; iLoop++) {
      int nVertexs = Polygon_nVertexs[iPolygon][iLoop];

      fprintf (fp, "  %d\n", 
	       nVertexs);

      for (iVertex = 0; iVertex < nVertexs; iVertex++) {
	double x = Polygon_vertexs[iPolygon][iLoop][iVertex][0];
	double y = Polygon_vertexs[iPolygon][iLoop][iVertex][1];
	double z = Polygon_vertexs[iPolygon][iLoop][iVertex][2];

	fprintf (fp, "  %21.14e %21.14e %21.14e\n", 
		 x, y, z);
      }
    }
  }

  fclose (fp);
}



/********************************
     Axis-aligned Polygon
********************************/

int MakeXYPolygon 
(double originX, double originY,
 double extentX, double extentY,
 double z,
 int clockWiseFlag)
{
  int polygonId;
  int loopId;

  polygonId = AddNewPolygon ();
  loopId = AddNewLoopIntoPolygon (polygonId);

  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX, 
     originY, 
     z);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX + extentX, 
     originY, 
     z);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX + extentX, 
     originY + extentY, 
     z);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX, 
     originY + extentY, 
     z);

  if (clockWiseFlag) {
    InverseLoop (polygonId, loopId);
  }

  return polygonId;
}

int MakeYZPolygon 
(double originY, double originZ,
 double extentY, double extentZ,
 double x,
 int clockWiseFlag)
{
  int polygonId;
  int loopId;

  polygonId = AddNewPolygon ();
  loopId = AddNewLoopIntoPolygon (polygonId);

  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     x,
     originY, 
     originZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     x,
     originY + extentY, 
     originZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     x, 
     originY + extentY, 
     originZ + extentZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     x,
     originY, 
     originZ + extentZ);

  if (clockWiseFlag) {
    InverseLoop (polygonId, loopId);
  }

  return polygonId;
}

int MakeZXPolygon 
(double originZ, double originX,
 double extentZ, double extentX,
 double y,
 int clockWiseFlag)
{
  int polygonId;
  int loopId;

  polygonId = AddNewPolygon ();
  loopId = AddNewLoopIntoPolygon (polygonId);

  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX, 
     y,
     originZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX, 
     y,
     originZ + extentZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX + extentX, 
     y,
     originZ + extentZ);
  AddNewVertexIntoPolygon 
    (polygonId, loopId,
     originX + extentX, 
     y,
     originZ);

  if (clockWiseFlag) {
    InverseLoop (polygonId, loopId);
  }

  return polygonId;
}



/********************************
     Axis-aligned Primitive
********************************/

void MakeBox 
(double originX, double originY, double originZ,
 double extentX, double extentY, double extentZ)
{
  MakeXYPolygon
    (originX, originY, 
     extentX, extentY,
     originZ,
     1);
  MakeXYPolygon
    (originX, originY, 
     extentX, extentY,
     originZ + extentZ,
     0);

  MakeYZPolygon
    (originY, originZ, 
     extentY, extentZ,
     originX,
     1);
  MakeYZPolygon
    (originY, originZ, 
     extentY, extentZ,
     originX + extentX,
     0);

  MakeZXPolygon
    (originZ, originX, 
     extentZ, extentX,
     originY,
     1);
  MakeZXPolygon
    (originZ, originX, 
     extentZ, extentX,
     originY + extentY,
     0);
}




