/* advautomesh_h_tri2d_arrow_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
double Node_vector[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Element_getVector
(int elementId,
 double L0, double L1, double L2,
 double *vectorX_OUT, double *vectorY_OUT)
{
  int nodeId0, nodeId1, nodeId2;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  
  *vectorX_OUT 
    = Node_vector[nodeId0][0] * L0
    + Node_vector[nodeId1][0] * L1
    + Node_vector[nodeId2][0] * L2;
  *vectorY_OUT
    = Node_vector[nodeId0][1] * L0
    + Node_vector[nodeId1][1] * L1
    + Node_vector[nodeId2][1] * L2;
}

void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeVectorFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double vectorX, vectorY;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId,
	    &vectorX, &vectorY);
    assert(nodeId == iNode);
    Node_vector[iNode][0] = vectorX;
    Node_vector[iNode][1] = vectorY;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Sample */

#define MAX_SAMPLES 1000000

int NSamples;
double Sample_coord[MAX_SAMPLES][2];
int Sample_elementId[MAX_SAMPLES];
double Sample_L0[MAX_SAMPLES];
double Sample_L1[MAX_SAMPLES];
double Sample_L2[MAX_SAMPLES];



/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][2];
double Frame_coord1[MAX_FRAMES][2];



void ReadSampleFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(0 < NElements);
  assert(NSamples == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NSamples);
  assert(0 < NSamples);
  assert(NSamples <= MAX_SAMPLES);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    int sampleId;
    double x, y;
    int elementId;
    double L0, L1, L2;
    
    fscanf (fp, "%d %lf %lf %d %lf %lf %lf", 
	    &sampleId,
	    &x, &y,
	    &elementId,
	    &L0, &L1, &L2);
    assert(sampleId == iSample);

    Sample_coord[iSample][0] = x;
    Sample_coord[iSample][1] = y;

    assert(0 <= elementId || elementId == -1);
    assert(elementId < NElements);
    Sample_elementId[iSample] = elementId;

    Sample_L0[iSample] = L0;
    Sample_L1[iSample] = L1;
    Sample_L2[iSample] = L2;
  }

  fclose (fp);
}

void ReadFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(NFrames == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFrames);
  assert(0 <= NFrames);
  assert(NFrames <= MAX_FRAMES);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    int frameId;
    double x0, y0;
    double x1, y1;

    fscanf (fp, "%d %lf %lf %lf %lf", 
	    &frameId,
	    &x0, &y0, 
	    &x1, &y1);
    assert(frameId == iFrame);
    Frame_coord0[iFrame][0] = x0;
    Frame_coord0[iFrame][1] = y0;

    Frame_coord1[iFrame][0] = x1;
    Frame_coord1[iFrame][1] = y1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

double Scale = 1.0;

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void SetVectorScale (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  double maxLength;
  int iNode;

  assert(0 < NNodes);

  Mesh_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  maxLength = 0.0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    double vx = Node_vector[iNode][0];
    double vy = Node_vector[iNode][1];
    double length = sqrt (vx * vx + vy * vy);
    
    if (maxLength < length) maxLength = length;
  }

  Scale = size / maxLength / 5;
}

void DrawFrame (void)
{
  int iFrame;

  AutoGL_SetColor (1, 1, 1);
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    AutoGL_DrawLine (Frame_coord0[iFrame][0],
		     Frame_coord0[iFrame][1],
		     0.0,
		     Frame_coord1[iFrame][0],
		     Frame_coord1[iFrame][1],
		     0.0);
  }
}

void RedrawView (void) 
{
  int iSample;

  DrawFrame ();

  for (iSample = 0; iSample < NSamples; iSample++) {
    int elementId = Sample_elementId[iSample];

    if (elementId != -1) {
      double x = Sample_coord[iSample][0];
      double y = Sample_coord[iSample][1];

      double L0 = Sample_L0[iSample];
      double L1 = Sample_L1[iSample];
      double L2 = Sample_L2[iSample];
      double vx, vy;

      Element_getVector (elementId, L0, L1, L2,
			 &vx, &vy);
      
      AutoGL_SetColor (0, 0, 1);
      AutoGL_DrawCrossMark2D (x, y, 0.0, 3);
      AutoGL_SetColor (1, 1, 0);
      AutoGL_DrawLine (x, y, 0.0,
		       x + vx * Scale, 
		       y + vy * Scale, 
		       0.0);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_arrow_view  node_file  element_file \n"
       "     node_vector_file  sample_file  frame_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeVectorFile (argv[3]);
  ReadSampleFile (argv[4]);
  ReadFrameFile (argv[5]);

  SetView ();
  SetVectorScale ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddReal (&Scale, "Scale");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
