/* advautomesh_h_tri2d_frame_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Other Data
*************************/

/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][2];
double Frame_coord1[MAX_FRAMES][2];



void FrameGroup_getRange 
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iFrame;

  assert(0 < NFrames);

  *minRangeX_OUT = *maxRangeX_OUT = Frame_coord0[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Frame_coord0[0][1];
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    double x0 = Frame_coord0[iFrame][0];
    double y0 = Frame_coord0[iFrame][1];
    
    double x1 = Frame_coord1[iFrame][0];
    double y1 = Frame_coord1[iFrame][1];

    if (x0 < *minRangeX_OUT) *minRangeX_OUT = x0;
    if (y0 < *minRangeY_OUT) *minRangeY_OUT = y0;

    if (*maxRangeX_OUT < x0) *maxRangeX_OUT = x0;
    if (*maxRangeY_OUT < y0) *maxRangeY_OUT = y0;

    if (x1 < *minRangeX_OUT) *minRangeX_OUT = x1;
    if (y1 < *minRangeY_OUT) *minRangeY_OUT = y1;

    if (*maxRangeX_OUT < x1) *maxRangeX_OUT = x1;
    if (*maxRangeY_OUT < y1) *maxRangeY_OUT = y1;
  }
}

void ReadFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(NFrames == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFrames);
  assert(0 <= NFrames);
  assert(NFrames <= MAX_FRAMES);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    int frameId;
    double x0, y0;
    double x1, y1;

    fscanf (fp, "%d %lf %lf %lf %lf", 
	    &frameId,
	    &x0, &y0, 
	    &x1, &y1);
    assert(frameId == iFrame);
    Frame_coord0[iFrame][0] = x0;
    Frame_coord0[iFrame][1] = y0;

    Frame_coord1[iFrame][0] = x1;
    Frame_coord1[iFrame][1] = y1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  FrameGroup_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void DrawFrame (void)
{
  int iFrame;

  AutoGL_SetColor (1, 1, 1);
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    AutoGL_DrawLine (Frame_coord0[iFrame][0],
		     Frame_coord0[iFrame][1],
		     0.0,
		     Frame_coord1[iFrame][0],
		     Frame_coord1[iFrame][1],
		     0.0);
  }
}

void RedrawView (void) 
{
  DrawFrame ();
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_frame_view  frame_file \n");
    exit (1);
  }

  ReadFrameFile (argv[1]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
