/* advautomesh_h_tri2d_node_dof_bc_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 2
#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
int Node_hasValue[MAX_NODES][N_NODE_DOFS];
double Node_value[MAX_NODES][N_NODE_DOFS];



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 2000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_elementIds[MAX_NODES][2];
int Edge_elementEdgeIds[MAX_NODES][2];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &edgeId,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementEdgeId0);
    assert(elementEdgeId0 < N_ELEMENT_EDGES);

    if (0 <= elementId1) {
      assert(elementId1 < NElements);
      assert(0 <= elementEdgeId1);
      assert(elementEdgeId1 < N_ELEMENT_EDGES);
    } else {
      assert(elementId1 == -1);
      assert(elementEdgeId1 == -1);
    }

    Edge_elementIds[iEdge][0] = elementId0;
    Edge_elementIds[iEdge][1] = elementId1;

    Edge_elementEdgeIds[iEdge][0] = elementEdgeId0;
    Edge_elementEdgeIds[iEdge][1] = elementEdgeId1;
  }

  fclose (fp);
}

void ReadNodeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iDof;
  int nItems;
  int iItem;

  assert(0 < NNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Node_hasValue[iNode][iDof] = 0;
      Node_value[iNode][iDof] = 0.0;
    }
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int nodeId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &nodeId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Node_hasValue[nodeId][dofId] = 1;
    Node_value[nodeId][dofId] = value;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

int DofX = 1;
int DofY = 1;
double ArrowLength;
int ShowValue;
double Scale = 1.0;

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 

  ArrowLength = size / 10;
}

void RedrawView (void) 
{
  int iEdge;
  int iNode;
  int iDof;

  AutoGL_SetColor (1, 1, 1);

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_elementIds[iEdge][1] == -1) {
      int elementId = Edge_elementIds[iEdge][0];
      int elementEdgeId = Edge_elementEdgeIds[iEdge][0];

      int nodeId0 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][0]];
      int nodeId1 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][1]];

      double nodeX0 = Node_coord[nodeId0][0];
      double nodeY0 = Node_coord[nodeId0][1];
      
      double nodeX1 = Node_coord[nodeId1][0];
      double nodeY1 = Node_coord[nodeId1][1];

      AutoGL_DrawLine (nodeX0, nodeY0, 0.0,
		       nodeX1, nodeY1, 0.0);
    }
  }

  AutoGL_SetColor (1, 0, 0);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1]; 

    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	double length;

	AutoGL_DrawSquareMark2D (x, y, 0.5,
				 5, 0);

	if (ShowValue) {
	  length = Node_value[iNode][iDof] * Scale;
	} else {
	  length = ArrowLength;
	}

	switch (iDof) {
	case 0:
	  if (!DofX) continue;
	  AutoGL_DrawArrow2D (x, y,
			      x + length, y,
			      1.0,
			      0.2);
	  break;
	case 1:
	  if (!DofY) continue;
	  AutoGL_DrawArrow2D (x, y,
			      x, y + length, 
			      1.0,
			      0.2);
	  break;
	default:
	  assert(0);
	  break;
	}
      }
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_node_dof_bc_view  node_file  element_file \n"
       "     edge_file  node_dof_bc_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadEdgeFile (argv[3]);
  ReadNodeDofBcFile (argv[4]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&DofX, "DofX");
  AutoGL_AddBoolean (&DofY, "DofY");
  AutoGL_AddReal (&ArrowLength, "ArrowLength");

  AutoGL_AddBoolean (&ShowValue, "ShowValue");
  AutoGL_AddReal (&Scale, "Scale");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
