/* advautomesh_h_tri2d_particle_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Particle */

#define MAX_PARTICLES 1000
#define MAX_STEPS 1000

int NParticles;
int NSteps;

double Point_coord[MAX_PARTICLES][MAX_STEPS][2];
int Point_elementId[MAX_PARTICLES][MAX_STEPS];
double Point_L0[MAX_PARTICLES][MAX_STEPS];
double Point_L1[MAX_PARTICLES][MAX_STEPS];
double Point_L2[MAX_PARTICLES][MAX_STEPS];



/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][2];
double Frame_coord1[MAX_FRAMES][2];



void ReadParticleTraceFile (const char *fileName)
{
  FILE *fp;
  int iParticle;
  int iStep;
  
  assert(0 < NElements);
  assert(NParticles == 0);
  assert(NSteps == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d %d", 
	  &NParticles, &NSteps);
  assert(0 < NParticles);
  assert(NParticles <= MAX_PARTICLES);
  assert(1 < NSteps);
  assert(NSteps <= MAX_STEPS);
  
  for (iParticle = 0; iParticle < NParticles; iParticle++) {
    int particleId;

    fscanf (fp, "%d", 
	    &particleId);
    assert(particleId == iParticle);
    
    for (iStep = 0; iStep < NSteps; iStep++) {
      int stepId;
      double x, y;
      int elementId;
      double L0, L1, L2;
      
      fscanf (fp, "%d %lf %lf %d %lf %lf %lf", 
	      &stepId,
	      &x, &y,
	      &elementId,
	      &L0, &L1, &L2);
      assert(stepId == iStep);
      
      Point_coord[iParticle][iStep][0] = x;
      Point_coord[iParticle][iStep][1] = y;

      assert(0 <= elementId || elementId == -1);
      assert(elementId < NElements);
      Point_elementId[iParticle][iStep] = elementId;
      
      Point_L0[iParticle][iStep] = L0;
      Point_L1[iParticle][iStep] = L1;
      Point_L2[iParticle][iStep] = L2;
    }
  }

  fclose (fp);
}

void ReadFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(NFrames == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NFrames);
  assert(0 <= NFrames);
  assert(NFrames <= MAX_FRAMES);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    int frameId;
    double x0, y0;
    double x1, y1;

    fscanf (fp, "%d %lf %lf %lf %lf", 
	    &frameId,
	    &x0, &y0, 
	    &x1, &y1);
    assert(frameId == iFrame);
    Frame_coord0[iFrame][0] = x0;
    Frame_coord0[iFrame][1] = y0;

    Frame_coord1[iFrame][0] = x1;
    Frame_coord1[iFrame][1] = y1;
  }

  fclose (fp);
}



/********************************
	 Visualization
********************************/

int CurrentStep;

void SetView (void)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, 0.0); 
}

void DrawFrame (void)
{
  int iFrame;

  AutoGL_SetColor (1, 1, 1);
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    AutoGL_DrawLine (Frame_coord0[iFrame][0],
		     Frame_coord0[iFrame][1],
		     0.0,
		     Frame_coord1[iFrame][0],
		     Frame_coord1[iFrame][1],
		     0.0);
  }
}

void RedrawView (void) 
{
  int iStep;
  int iParticle;

  DrawFrame ();

  for (iStep = 0; iStep < NSteps; iStep++) {
    for (iParticle = 0; iParticle < NParticles; iParticle++) {
      double x = Point_coord[iParticle][iStep][0];
      double y = Point_coord[iParticle][iStep][1];
      
      if (Point_elementId[iParticle][iStep] != -1) {
	switch (iParticle % 7) {
	case 0: 
	  AutoGL_SetColor (1, 0, 0);   
	  break;
	case 1:
	  AutoGL_SetColor (0, 1, 0);
	  break;
	case 2:
	  AutoGL_SetColor (0, 0, 1);
	  break;
	case 3:
	  AutoGL_SetColor (1, 1, 0);
	  break;
	case 4:
	  AutoGL_SetColor (0, 1, 1);
	  break;
	case 5:
	  AutoGL_SetColor (1, 0, 1);
	  break;
	case 6:
	  AutoGL_SetColor (1, 1, 1);
	  break;
	}
	AutoGL_DrawCrossMark2D (x, y, 0.0, 3);
      }
    }
  }

  for (iParticle = 0; iParticle < NParticles; iParticle++) {
    double x = Point_coord[iParticle][CurrentStep][0];
    double y = Point_coord[iParticle][CurrentStep][1];
    
    if (Point_elementId[iParticle][CurrentStep] != -1) {
      switch (iParticle % 7) {
      case 0: 
	AutoGL_SetColor (1, 0, 0);   
	break;
      case 1:
	AutoGL_SetColor (0, 1, 0);
	break;
      case 2:
	AutoGL_SetColor (0, 0, 1);
	break;
      case 3:
	AutoGL_SetColor (1, 1, 0);
	break;
      case 4:
	AutoGL_SetColor (0, 1, 1);
	break;
      case 5:
	AutoGL_SetColor (1, 0, 1);
	break;
      case 6:
	AutoGL_SetColor (1, 1, 1);
	break;
      }
      AutoGL_DrawDiamondMark2D (x, y, 0.0, 
				10, 1);
    }
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Prev (void) 
{
  CurrentStep--;
  if (CurrentStep < 0) {
    CurrentStep = 0;
  }
  AutoGL_DrawView ();
}

void Next (void) 
{
  CurrentStep++;
  if (NSteps <= CurrentStep) {
    CurrentStep = NSteps - 1;
  }
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_particle_view  node_file  element_file \n"
       "     particle_trace_file  frame_file \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadParticleTraceFile (argv[3]);
  ReadFrameFile (argv[4]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode2D (NULL);
  AutoGL_EnableDragInMode2D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode2D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
  AutoGL_AddCallback (Prev, "Prev");      
  AutoGL_AddCallback (Next, "Next");      
}


