/* advautomesh_h_tri2d6n_vertex_scalar.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_scalar[MAX_NODES];
int Node_isOnVertex[MAX_NODES];
int NVertexNodes;



void ReadNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double scalar;

    fscanf (fp, "%d %lf", 
	    &nodeId,
	    &scalar);
    assert(nodeId == iNode);
    Node_scalar[iNode] = scalar;
  }

  fclose (fp);
}

void ReadVertexNodeSrcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iVertexNode;

  assert(0 < NNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOnVertex[iNode] = 0;
  }
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d",
	  &NVertexNodes);
  
  for (iVertexNode = 0; 
       iVertexNode < NVertexNodes; 
       iVertexNode++) {
    int vertexNodeId;
    int nodeId;

    fscanf (fp, "%d %d",
	    &vertexNodeId,
	    &nodeId);
    assert(vertexNodeId == iVertexNode);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    Node_isOnVertex[nodeId] = 1;
  }

  fclose (fp);
}

void WriteVertexNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;
  
  assert(0 < NNodes);
  assert(0 < NVertexNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NVertexNodes);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOnVertex[iNode]) {
      fprintf (fp, "%d   %22.15e\n", 
	       count, 
	       Node_scalar[iNode]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d6n_vertex_scalar  \n"
       "        node_scalar_file  vertex_node_src_file  \n"
       "        vertex_node_scalar_file_OUT \n");
    exit (1);
  }

  ReadNodeScalarFile (argv[1]);
  ReadVertexNodeSrcFile (argv[2]);

  WriteVertexNodeScalarFile (argv[3]);

  return 0;
}
