/* advautomesh_h_tri2d_add_edge_scalar_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_hasValue[MAX_EDGES];
double Edge_scalar[MAX_EDGES];



void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;

  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  /* skip data */

  fclose (fp);



  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    Edge_hasValue[iEdge] = 0;
    Edge_scalar[iEdge] = 0.0;
  }
}

void ReadEdgeScalarBcFile (const char *fileName)
{
  FILE *fp;
  int nItems;
  int iItem;

  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 <= nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int edgeId;
    double value;
    
    fscanf (fp, "%d %d %lf",
	    &itemId, 
	    &edgeId, &value);
    assert(itemId == iItem);
    assert(0 <= edgeId);
    assert(edgeId < NEdges);

    Edge_hasValue[edgeId] = 1;
    Edge_scalar[edgeId] = value;
  }

  fclose (fp);
}

void WriteEdgeScalarBcFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  int count;

  assert(0 < NEdges);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_hasValue[iEdge]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_hasValue[iEdge]) {
      fprintf (fp, "%d   %d %f\n", 
	       count,
	       iEdge, Edge_scalar[iEdge]);
      count++;
    }
  }

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_add_edge_scalar_bc \n"
       "        edge_file \n"
       "        edge_scalar_bc_file_A  edge_scalar_bc_file_B \n"
       "        edge_scalar_bc_file_OUT \n");
    exit (1);
  }

  ReadEdgeFile (argv[1]);
  ReadEdgeScalarBcFile (argv[2]);
  ReadEdgeScalarBcFile (argv[3]);

  WriteEdgeScalarBcFile (argv[4]);

  return 0;
}


