/* advautomesh_h_tri2d_check_node_duplication.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int jNode;

  for (iNode = 0; iNode < NNodes; iNode++) {
    double xi = Node_coord[iNode][0];
    double yi = Node_coord[iNode][1];

    for (jNode = iNode + 1; jNode < NNodes; jNode++) {
      double xj = Node_coord[jNode][0];
      double yj = Node_coord[jNode][1];
      
      if (Distance (xi, yi, 
		    xj, yj) < Eps) {
	fprintf 
	  (stderr, " duplication : Node %d and %d at (%lf %lf) \n",
	   iNode, jNode, 
	   xi, yi);
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 2) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_check_node_duplication  node_file \n");
    exit (1);
  }
  
  ReadNodeFile (argv[1]);

  PerformCommand ();

  return 0;
}
