/* advautomesh_h_tri2d_edge2node_dof_bc.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define N_NODE_DOFS 2
#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
int Node_hasValue[MAX_NODES][N_NODE_DOFS];
double Node_value[MAX_NODES][N_NODE_DOFS];



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 2000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_elementIds[MAX_NODES][2];
int Edge_elementEdgeIds[MAX_NODES][2];
int Edge_hasValue[MAX_EDGES][N_NODE_DOFS];
double Edge_value[MAX_EDGES][N_NODE_DOFS];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);
  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &edgeId,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);

    assert(elementId0 != -1);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    assert(0 <= elementEdgeId0);
    assert(elementEdgeId0 < N_ELEMENT_EDGES);

    if (0 <= elementId1) {
      assert(elementId1 < NElements);
      assert(0 <= elementEdgeId1);
      assert(elementEdgeId1 < N_ELEMENT_EDGES);
    } else {
      assert(elementId1 == -1);
      assert(elementEdgeId1 == -1);
    }

    Edge_elementIds[iEdge][0] = elementId0;
    Edge_elementIds[iEdge][1] = elementId1;

    Edge_elementEdgeIds[iEdge][0] = elementEdgeId0;
    Edge_elementEdgeIds[iEdge][1] = elementEdgeId1;
  }

  fclose (fp);
}

void ReadEdgeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  int iDof;
  int nItems;
  int iItem;

  assert(0 < NEdges);

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Edge_hasValue[iEdge][iDof] = 0;
      Edge_value[iEdge][iDof] = 0.0;
    }
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int edgeId;
    int dofId;
    double value;
    
    fscanf (fp, "%d %d %d %lf",
	    &itemId, 
	    &edgeId, &dofId, &value);
    assert(itemId == iItem);
    assert(0 <= edgeId);
    assert(edgeId < NEdges);
    assert(Edge_elementIds[edgeId][1] == -1);
    assert(0 <= dofId);
    assert(dofId < N_NODE_DOFS);

    Edge_hasValue[edgeId][dofId] = 1;
    Edge_value[edgeId][dofId] = value;
  }

  fclose (fp);
}

void WriteNodeDofBcFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int iDof;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	count++;
      }
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      if (Node_hasValue[iNode][iDof]) {
	fprintf (fp, "%d   %d %d %f\n", 
		 count,
		 iNode, iDof, Node_value[iNode][iDof]);
	count++;
      }
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iDof;
  int iEdge;

  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
      Node_hasValue[iNode][iDof] = 0;
      Node_value[iNode][iDof] = 0.0;
    }
  }

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_elementIds[iEdge][1] == -1) {
      int elementId = Edge_elementIds[iEdge][0];
      int elementEdgeId = Edge_elementEdgeIds[iEdge][0];
    
      int nodeId0 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][0]];
      int nodeId1 
	= Element_nodeIds[elementId][EdgeNodeMap[elementEdgeId][1]];

      for (iDof = 0; iDof < N_NODE_DOFS; iDof++) {
	if (Edge_hasValue[iEdge][iDof]) {
	  double value = Edge_value[iEdge][iDof];

	  Node_hasValue[nodeId0][iDof] = 1;
	  Node_value[nodeId0][iDof] = value;
	  
	  Node_hasValue[nodeId1][iDof] = 1;
	  Node_value[nodeId1][iDof] = value;
	}
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_edge2node_dof_bc  node_file  element_file \n"
       "        edge_file  edge_dof_bc_file \n"
       "        node_dof_bc_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadEdgeFile (argv[3]);
  ReadEdgeDofBcFile (argv[4]);

  PerformCommand ();

  WriteNodeDofBcFile (argv[5]);

  return 0;
}


