/* advautomesh_h_tri2d_extract_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
int Node_isExtracted[MAX_NODES];
int NExtractedNodes;



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadExtractedNodeListFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int nItems;
  int iItem;

  assert(0 < NNodes);

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isExtracted[iNode] = 0;
  }
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  NExtractedNodes = nItems;
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int nodeId;
    
    fscanf (fp, "%d %d", 
	    &itemId,
	    &nodeId);
    assert(itemId == iItem);
    assert(0 <= nodeId);
    assert(nodeId < NNodes);
    Node_isExtracted[nodeId] = 1;
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);
  assert(0 < NExtractedNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NExtractedNodes);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isExtracted[iNode]) {
      fprintf (fp, "%d   %22.15e %22.15e\n", 
	       count,
	       Node_coord[iNode][0],
	       Node_coord[iNode][1]); 
      count++;
    }
  }
  assert(count == NExtractedNodes);

  fclose (fp);
}



int main (int argc, char *argv[])
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_extract_node  node_file \n"
       "        extracted_node_list_file \n"
       "        node_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadExtractedNodeListFile (argv[2]);

  WriteNodeFile (argv[3]);

  return 0;
}
