/* advautomesh_h_tri2d_frame.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 1000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_EDGES];
int Element_isMarked[MAX_ELEMENTS];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iElement;
  int iElementEdge;
  
  assert(0 < NElements);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);

    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int neighborId;

      fscanf (fp, "%d", 
	      &neighborId);
      assert((0 <= neighborId && neighborId < NElements) 
	     || neighborId == -1);
      Element_neighborIds[iElement][iElementEdge] = neighborId;
    }
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Frame */

#define MAX_FRAMES 1000000

int NFrames;
double Frame_coord0[MAX_FRAMES][2];
double Frame_coord1[MAX_FRAMES][2];



void FrameGroup_addFrame 
(double x0, double y0, 
 double x1, double y1)
{
  assert(NFrames < MAX_FRAMES);

  Frame_coord0[NFrames][0] = x0;
  Frame_coord0[NFrames][1] = y0;

  Frame_coord1[NFrames][0] = x1;
  Frame_coord1[NFrames][1] = y1;

  NFrames++;
}

void WriteFrameFile (const char *fileName)
{
  FILE *fp;
  int iFrame;
  
  assert(0 < NFrames);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NFrames);
  
  for (iFrame = 0; iFrame < NFrames; iFrame++) {
    fprintf (fp, "%d   %22.15e %22.15e   %22.15e %22.15e\n", 
	     iFrame,
	     Frame_coord0[iFrame][0],
	     Frame_coord0[iFrame][1],
	     Frame_coord1[iFrame][0],
	     Frame_coord1[iFrame][1]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iElement;
  int iElementEdge;

  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int neighborId 
	= Element_neighborIds[iElement][iElementEdge];

      if (neighborId == -1) {
	int nodeId0 = Element_nodeIds
	  [iElement][EdgeNodeMap[iElementEdge][0]];
	int nodeId1 = Element_nodeIds
	  [iElement][EdgeNodeMap[iElementEdge][1]];

	FrameGroup_addFrame 
	  (Node_coord[nodeId0][0], 
	   Node_coord[nodeId0][1],
	   Node_coord[nodeId1][0],
	   Node_coord[nodeId1][1]);
      }
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_frame  node_file  element_file \n"
       "        element_neighbor_file \n"
       "        frame_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadElementNeighborFile (argv[3]);

  PerformCommand ();

  WriteFrameFile (argv[4]);

  return 0;
}
