/* advautomesh_h_tri2d_grid.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0,
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Point */

#define MAX_POINTS 100000

int NPoints;
double Point_coord[MAX_POINTS][2];



void PointGroup_addPoint 
(double x, double y)
{
  assert(NPoints < MAX_POINTS);
  
  Point_coord[NPoints][0] = x;
  Point_coord[NPoints][1] = y;
  
  NPoints++;
}

void WritePointFile (const char *fileName)
{
  FILE *fp;
  int iPoint;

  assert(0 <= NPoints);
  
  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NPoints);
  
  for (iPoint = 0; iPoint < NPoints; iPoint++) {
    fprintf (fp, "%d   %22.15e %22.15e\n", 
	     iPoint,
	     Point_coord[iPoint][0], 
	     Point_coord[iPoint][1]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (double interval)
{
  double minRangeX, minRangeY;
  double maxRangeX, maxRangeY;
  double centerX, centerY;
  double size;
  int nDivisions;
  int ix, iy;

  Mesh_getRange 
    (&minRangeX, &minRangeY,
     &maxRangeX, &maxRangeY);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  size = Distance (minRangeX, minRangeY,
		   maxRangeX, maxRangeY) / 2;
  nDivisions = size / interval;

  for (ix = -nDivisions; ix <= nDivisions; ix++) {
    double x = centerX + interval * ix;
    
    for (iy = -nDivisions; iy <= nDivisions; iy++) {
      double y = centerY + interval * iy;

      PointGroup_addPoint (x, y);
    }
  }
}



int main (int argc, char *argv[])
{
  double interval;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_grid  node_file \n"
       "        interval \n"
       "        point_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  interval = atof (argv[2]);
  PerformCommand (interval);

  WritePointFile (argv[3]);

  return 0;
}
