/* advautomesh_h_tri2d_mid_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_edgeIds[MAX_ELEMENTS][N_ELEMENT_EDGES];



/* Edge */

#define MAX_EDGES 1000000

int NEdges;
int Edge_nodeIds[MAX_EDGES][2];
int Edge_midNodeId[MAX_EDGES];



void Edge_getMidPoint 
(int edgeId, 
 double *midX_OUT, double *midY_OUT)
{
  int edgeNodeId0, edgeNodeId1;
  double x0, y0;
  double x1, y1;

  assert(0 <= edgeId);
  assert(edgeId < NEdges);

  edgeNodeId0 = Edge_nodeIds[edgeId][0];
  edgeNodeId1 = Edge_nodeIds[edgeId][1];
  
  x0 = Node_coord[edgeNodeId0][0];
  y0 = Node_coord[edgeNodeId0][1];
  
  x1 = Node_coord[edgeNodeId1][0];
  y1 = Node_coord[edgeNodeId1][1];

  *midX_OUT = (x0 + x1) / 2;
  *midY_OUT = (y0 + y1) / 2;
}

int Mesh_addNewNode (double x, double y)
{
  int nodeId = NNodes;

  assert(NNodes < MAX_NODES);
  NNodes++;

  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  
  return nodeId;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadEdgeVertexFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NNodes);
  assert(0 < NElements);
  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int nodeId0, nodeId1;
    
    fscanf (fp, "%d %d %d", 
	    &edgeId,
	    &nodeId0, &nodeId1);
    assert(edgeId == iEdge);

    assert(0 <= nodeId0);
    assert(nodeId0 < NNodes);
    Edge_nodeIds[iEdge][0] = nodeId0;

    assert(0 <= nodeId1);
    assert(nodeId1 < NNodes);
    Edge_nodeIds[iEdge][1] = nodeId1;
  }

  fclose (fp);
}

void ReadElementEdgeFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementEdge;
  int count;
  
  assert(0 < NElements);
  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);
    
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeId;

      fscanf (fp, "%d", 
	      &edgeId);
      assert(0 <= edgeId);
      assert(edgeId < NEdges);
      Element_edgeIds[iElement][iElementEdge] = edgeId;
    }
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1]); 
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;
  int iElementEdge;

  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement); 
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fprintf (fp, "%d ", 
	       Element_nodeIds[iElement][iElementNode]); 
    }
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeId = Element_edgeIds[iElement][iElementEdge];

      fprintf (fp, "%d ", 
	       Edge_midNodeId[edgeId]); 
    }
    fprintf (fp, "\n");
  }
  
  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iEdge;
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    double midX, midY;
    
    Edge_getMidPoint (iEdge,
		      &midX, &midY);
    Edge_midNodeId[iEdge] = Mesh_addNewNode (midX, midY);
  }
}



int main (int argc, char *argv[])
{
  if (argc != 7) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_mid_node  node_file  element_file \n"
       "        edge_vertex_file  element_edge_file \n"
       "        node_file_OUT  element_6node_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadEdgeVertexFile (argv[3]);
  ReadElementEdgeFile (argv[4]);

  PerformCommand ();

  WriteNodeFile (argv[5]);
  WriteElementFile (argv[6]);

  return 0;
}


