/* advautomesh_h_tri2d_neighbor.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 2000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

int NElements;
int Element_edgeIds[MAX_ELEMENTS][N_ELEMENT_EDGES];
int Element_neighborIds[MAX_ELEMENTS][N_ELEMENT_EDGES];



/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_elementIds[MAX_EDGES][2];
int Edge_elementEdgeIds[MAX_EDGES][2];



void ReadElementFile (const char *fileName)
{
  FILE *fp;

  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  /* skip connectivity */
  
  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &edgeId,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);

    assert(0 <= elementId0);
    assert(elementId0 < NElements);
    Edge_elementIds[iEdge][0] = elementId0;

    assert(0 <= elementEdgeId0);
    assert(elementEdgeId0 < N_ELEMENT_EDGES);
    Edge_elementEdgeIds[iEdge][0] = elementEdgeId0;

    assert(0 <= elementId1 || elementId1 == -1);
    assert(elementId1 < NElements);
    Edge_elementIds[iEdge][1] = elementId1;

    assert(0 <= elementEdgeId1 || elementEdgeId1 == -1);
    assert(elementEdgeId1 < N_ELEMENT_EDGES);
    Edge_elementEdgeIds[iEdge][1] = elementEdgeId1;
  }

  fclose (fp);
}

void ReadElementEdgeFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementEdge;
  int count;
  
  assert(0 < NElements);
  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId);
    assert(elementId == iElement);

    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeId;
      
      fscanf (fp, "%d", 
	      &edgeId);
      assert(0 <= edgeId);
      assert(edgeId < NEdges);
      Element_edgeIds[iElement][iElementEdge] = edgeId;
    }
  }

  fclose (fp);
}

void WriteElementNeighborFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementEdge;
  
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement);

    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      fprintf (fp, "%d ", 
	       Element_neighborIds[iElement][iElementEdge]);
    }

    fprintf (fp, "\n");
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iElement;
  int iElementEdge;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeId = Element_edgeIds[iElement][iElementEdge];
      int neighborId;
      
      if (Edge_elementIds[edgeId][0] == iElement) {
	neighborId = Edge_elementIds[edgeId][1];
      } else {
	assert(Edge_elementIds[edgeId][1] == iElement);
	neighborId = Edge_elementIds[edgeId][0];
      }
      Element_neighborIds[iElement][iElementEdge] = neighborId;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_neighbor \n"
       "        element_file \n"
       "        edge_file  element_edge_file \n"
       "        element_neighbor_file_OUT \n");
    exit (1);
  }

  ReadElementFile (argv[1]);
  ReadEdgeFile (argv[2]);
  ReadElementEdgeFile (argv[3]);

  PerformCommand ();

  WriteElementNeighborFile (argv[4]);

  return 0;
}
