/* advautomesh_h_tri2d_node_on_point.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define DEFAULT_EPS 0.00001

double Eps = DEFAULT_EPS;



double Distance (double x0, double y0, 
		 double x1, double y1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) + (y1 - y0) * (y1 - y0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];
int Node_isOn[MAX_NODES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void WriteNodeListFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  int count;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOn[iNode]) {
      count++;
    }
  }
  
  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iNode = 0; iNode < NNodes; iNode++) {
    if (Node_isOn[iNode]) {
      fprintf (fp, "%d   %d\n", 
	       count,
	       iNode);
      count++;
    }
  }

  fclose (fp);
}



/************************
	 Command
*************************/

double PointX, PointY;

void PerformCommand (void)
{
  int iNode;

  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_isOn[iNode] = 0;
  }

  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    
    if (Distance (x, y, 
		  PointX, PointY) < Eps) {
      Node_isOn[iNode] = 1;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_node_on_point  node_file \n"
       "        x  y \n"
       "        node_list_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);

  PointX = atof (argv[2]);
  PointY = atof (argv[3]);

  PerformCommand ();

  WriteNodeListFile (argv[4]);

  return 0;
}


