/* advautomesh_h_tri2d_sample.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

#define AREA_COORD_EPS 0.00001



double SignedTriArea
(double x0, double y0,
 double x1, double y1,
 double x2, double y2)
{
  return ((x1 - x0) * (y2 - y0) - (y1 - y0) * (x2 - x0)) / 2;
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



int Element_containsPoint 
(int elementId, 
 double x, double y,
 double *L0_OUT, double *L1_OUT, double *L2_OUT)
{
  int nodeId0, nodeId1, nodeId2;
  double x0, y0, x1, y1, x2, y2;
  double area;
  double area0, area1, area2;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];

  x0 = Node_coord[nodeId0][0];
  y0 = Node_coord[nodeId0][1];

  x1 = Node_coord[nodeId1][0];
  y1 = Node_coord[nodeId1][1];

  x2 = Node_coord[nodeId2][0];
  y2 = Node_coord[nodeId2][1];
  
  area = SignedTriArea (x0, y0, 
			x1, y1, 
			x2, y2);
  area0 = SignedTriArea (x1, y1, 
			 x2, y2, 
			 x, y);
  area1 = SignedTriArea (x2, y2, 
			 x0, y0, 
			 x, y);
  area2 = SignedTriArea (x0, y0, 
			 x1, y1, 
			 x, y);

  *L0_OUT = area0 / area;
  *L1_OUT = area1 / area;
  *L2_OUT = area2 / area;

  if (0.0 - AREA_COORD_EPS < *L0_OUT 
      && *L0_OUT < 1.0 + AREA_COORD_EPS
      && 0.0 - AREA_COORD_EPS < *L1_OUT 
      && *L1_OUT < 1.0 + AREA_COORD_EPS
      && 0.0 - AREA_COORD_EPS < *L2_OUT 
      && *L2_OUT < 1.0 + AREA_COORD_EPS) {
    return 1;
  }
  return 0;
}

int Mesh_findElementAt 
(double x, double y,
 double *L0_OUT, double *L1_OUT, double *L2_OUT)
{
  int iElement;

  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_containsPoint (iElement, x, y,
			       L0_OUT, L1_OUT, L2_OUT)) {
      return iElement;
    }
  }
  return -1;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y;

    fscanf (fp, "%d %lf %lf", 
	    &nodeId, 
	    &x, &y);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Sample */

#define MAX_SAMPLES 1000000

int NSamples;
double Sample_coord[MAX_SAMPLES][2];
int Sample_elementId[MAX_SAMPLES];
double Sample_L0[MAX_SAMPLES];
double Sample_L1[MAX_SAMPLES];
double Sample_L2[MAX_SAMPLES];



void ReadPointFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NSamples);
  assert(0 < NSamples);
  assert(NSamples <= MAX_SAMPLES);

  for (iSample = 0; iSample < NSamples; iSample++) {
    int sampleId;
    double x, y;
    
    fscanf (fp, "%d %lf %lf", 
	    &sampleId,
	    &x, &y);
    assert(sampleId == iSample);
    Sample_coord[iSample][0] = x;
    Sample_coord[iSample][1] = y;
  }

  fclose (fp);
}

void WriteSampleFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(0 < NSamples);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NSamples);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    fprintf (fp, "%d   %22.15e %22.15e   %d   %22.15e %22.15e %22.15e\n", 
	     iSample,
	     Sample_coord[iSample][0],
	     Sample_coord[iSample][1],
	     Sample_elementId[iSample],
	     Sample_L0[iSample],
	     Sample_L1[iSample],
	     Sample_L2[iSample]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iSample;

  for (iSample = 0; iSample < NSamples; iSample++) {
    double x = Sample_coord[iSample][0];
    double y = Sample_coord[iSample][1];
    int elementId;
    double L0, L1, L2;
    
    elementId = Mesh_findElementAt (x, y,
				    &L0, &L1, &L2);
    if (elementId != -1) {
      Sample_elementId[iSample] = elementId;
      Sample_L0[iSample] = L0;
      Sample_L1[iSample] = L1;
      Sample_L2[iSample] = L2;
    } else {
      Sample_elementId[iSample] = -1;
      Sample_L0[iSample] = 0.0;
      Sample_L1[iSample] = 0.0;
      Sample_L2[iSample] = 0.0;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_sample  node_file  element_file \n"
       "        point_file \n"
       "        sample_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadPointFile (argv[3]);

  PerformCommand ();

  WriteSampleFile (argv[4]);

  return 0;
}
