/* advautomesh_h_tri2d_sample_scalar.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_scalar[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



double Element_scalar
(int elementId,
 double L0, double L1, double L2)
{
  int nodeId0, nodeId1, nodeId2;

  assert(0 <= elementId);
  assert(elementId < NElements);

  nodeId0 = Element_nodeIds[elementId][0];
  nodeId1 = Element_nodeIds[elementId][1];
  nodeId2 = Element_nodeIds[elementId][2];
  
  return Node_scalar[nodeId0] * L0
    + Node_scalar[nodeId1] * L1
    + Node_scalar[nodeId2] * L2;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeScalarFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double scalar;

    fscanf (fp, "%d %lf", 
	    &nodeId,
	    &scalar);
    assert(nodeId == iNode);
    Node_scalar[iNode] = scalar;
  }

  fclose (fp);
}



/************************
	 Other Data
*************************/

/* Sample */

#define MAX_SAMPLES 1000000

int NSamples;
double Sample_coord[MAX_SAMPLES][2];
int Sample_elementId[MAX_SAMPLES];
double Sample_L0[MAX_SAMPLES];
double Sample_L1[MAX_SAMPLES];
double Sample_L2[MAX_SAMPLES];
double Sample_scalar[MAX_SAMPLES];



void ReadSampleFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(NSamples == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NSamples);
  assert(0 < NSamples);
  assert(NSamples <= MAX_SAMPLES);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    int sampleId;
    double x, y;
    int elementId;
    double L0, L1, L2;

    fscanf (fp, "%d %lf %lf %d %lf %lf %lf", 
	    &sampleId,
	    &x, &y, 
	    &elementId, 
	    &L0, &L1, &L2);
    assert(sampleId == iSample);

    Sample_coord[iSample][0] = x;
    Sample_coord[iSample][1] = y;
    
    assert(0 <= elementId || elementId == -1);
    assert(elementId < NElements);
    Sample_elementId[iSample] = elementId;

    Sample_L0[iSample] = L0;
    Sample_L1[iSample] = L1;
    Sample_L2[iSample] = L2;
  }

  fclose (fp);
}

void WriteSampleScalarFile (const char *fileName)
{
  FILE *fp;
  int iSample;
  
  assert(0 < NSamples);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NSamples);
  
  for (iSample = 0; iSample < NSamples; iSample++) {
    fprintf (fp, "%d   %22.15e\n", 
	     iSample,
	     Sample_scalar[iSample]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iSample;

  for (iSample = 0; iSample < NSamples; iSample++) {
    int elementId = Sample_elementId[iSample];
    
    if (elementId != -1) {
      double L0 = Sample_L0[iSample];
      double L1 = Sample_L1[iSample];
      double L2 = Sample_L2[iSample];
      
      Sample_scalar[iSample] 
	= Element_scalar (elementId, L0, L1, L2);
    } else {
      Sample_scalar[iSample] = 0.0;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_sample_scalar  node_file  element_file \n"
       "        node_scalar_file  sample_file \n"
       "        sample_scalar_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeScalarFile (argv[3]);
  ReadSampleFile (argv[4]);

  PerformCommand ();

  WriteSampleScalarFile (argv[5]);

  return 0;
}
