/* advautomesh_h_tri2d_topology.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

typedef enum {
  NODE_TOPOLOGY_INTERNAL,
  NODE_TOPOLOGY_EDGE,
  NODE_TOPOLOGY_VERTEX
} NodeTopologyType;

#define MAX_NODES 1000000

int NNodes;
NodeTopologyType Node_topologyType[MAX_NODES];



/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_nodeIds[MAX_EDGES][2];
int Edge_isOnBoundary[MAX_EDGES];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate */

  fclose (fp);
}

void ReadEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(NEdges == 0);
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &NEdges);
  assert(0 < NEdges);
  assert(NEdges <= MAX_EDGES);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int elementId0, elementId1;
    int elementEdgeId0, elementEdgeId1;

    fscanf (fp, "%d %d %d %d %d", 
	    &edgeId,
	    &elementId0, &elementEdgeId0,
	    &elementId1, &elementEdgeId1);
    assert(edgeId == iEdge);
    
    if (elementId1 == -1) {
      Edge_isOnBoundary[iEdge] = 1;
    } else {
      Edge_isOnBoundary[iEdge] = 0;
    }
  }

  fclose (fp);
}

void ReadEdgeVertexFile (const char *fileName)
{
  FILE *fp;
  int nEdges;
  int iEdge;
  
  assert(0 < NNodes);
  assert(0 < NEdges);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nEdges);
  assert(nEdges == NEdges);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeId;
    int nodeId0, nodeId1;

    fscanf (fp, "%d %d %d", 
	    &edgeId,
	    &nodeId0, &nodeId1);
    assert(edgeId == iEdge);
    
    assert(0 <= nodeId0);
    assert(nodeId0 < NNodes);
    Edge_nodeIds[iEdge][0] = nodeId0;
    
    assert(0 <= nodeId1);
    assert(nodeId1 < NNodes);
    Edge_nodeIds[iEdge][1] = nodeId1;
  }

  fclose (fp);
}

void WriteNodeTopologyFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %d\n", 
	     iNode, Node_topologyType[iNode]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iNode;
  int iEdge;
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    Node_topologyType[iNode] = NODE_TOPOLOGY_INTERNAL;
  }

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    if (Edge_isOnBoundary[iEdge]) {
      int nodeId0 = Edge_nodeIds[iEdge][0];
      int nodeId1 = Edge_nodeIds[iEdge][1];
      
      Node_topologyType[nodeId0] = NODE_TOPOLOGY_EDGE;
      Node_topologyType[nodeId1] = NODE_TOPOLOGY_EDGE;
    }
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri2d_topology \n"
       "        node_file \n"
       "        edge_file  edge_vertex_file \n"
       "        node_topology_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadEdgeFile (argv[2]);
  ReadEdgeVertexFile (argv[3]);

  PerformCommand ();

  WriteNodeTopologyFile (argv[4]);

  return 0;
}
