/* tri2d_rect_node.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][2];



int Mesh_addNewNode (double x, double y)
{
  int nodeId = NNodes;

  assert(NNodes < MAX_NODES);
  NNodes++;

  Node_coord[nodeId][0] = x;
  Node_coord[nodeId][1] = y;
  
  return nodeId;
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1]); 
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (int xDivisions, int yDivisions)
{
  int ix, iy;

  for (ix = 0; ix < xDivisions; ix++) {
    for (iy = 0; iy < yDivisions; iy++) {
      double x = ix * 1.0;
      double y = iy * 1.0;

      Mesh_addNewNode (x, y);
    }
  }
}



int main (int argc, char *argv[])
{
  int xDivisions, yDivisions;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " tri2d_rect_node  xdiv  ydiv \n"
       "        node_file_OUT \n");
    exit (1);
  }

  xDivisions = atoi (argv[1]);
  yDivisions = atoi (argv[2]);
  
  PerformCommand (xDivisions, yDivisions);

  WriteNodeFile (argv[3]);

  return 0;
}
