/* advautomesh_h_tri3d_diff_view.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autogl.h>



/************************
	 Basic
*************************/

double Distance (double x0, double y0, double z0,
		 double x1, double y1, double z1) 
{
  return sqrt ((x1 - x0) * (x1 - x0) 
	       + (y1 - y0) * (y1 - y0)
	       + (z1 - z0) * (z1 - z0));
}



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];



/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 2000000

int NElementsA;
int ElementA_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];

int NElementsB;
int ElementB_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];



void Mesh_getRange
(double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT)
{
  int iNode;

  assert(0 < NNodes);

  *minRangeX_OUT = *maxRangeX_OUT = Node_coord[0][0];
  *minRangeY_OUT = *maxRangeY_OUT = Node_coord[0][1];
  *minRangeZ_OUT = *maxRangeZ_OUT = Node_coord[0][2];
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    
    if (x < *minRangeX_OUT) *minRangeX_OUT = x;
    if (y < *minRangeY_OUT) *minRangeY_OUT = y;
    if (z < *minRangeZ_OUT) *minRangeZ_OUT = z;

    if (*maxRangeX_OUT < x) *maxRangeX_OUT = x;
    if (*maxRangeY_OUT < y) *maxRangeY_OUT = y;
    if (*maxRangeZ_OUT < z) *maxRangeZ_OUT = z;
  }
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFileA (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElementsA == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElementsA);
  assert(0 < NElementsA);
  assert(NElementsA <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElementsA; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      ElementA_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadElementFileB (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElementsB == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElementsB);
  assert(0 < NElementsB);
  assert(NElementsB <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElementsB; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      ElementB_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}



/********************************
	 Visualization
********************************/

int Fill = 0;
int Edge = 0;
int NodeId = 0;
int ElementId = 0;
double Ratio = 0.9;
int SideA = 0;
int SideB = 0;

void SetView (void)
{
  double minRangeX, minRangeY, minRangeZ;
  double maxRangeX, maxRangeY, maxRangeZ;
  double centerX, centerY, centerZ;
  double size;
  
  Mesh_getRange 
    (&minRangeX, &minRangeY, &minRangeZ,
     &maxRangeX, &maxRangeY, &maxRangeZ);
  centerX = (minRangeX + maxRangeX) / 2;
  centerY = (minRangeY + maxRangeY) / 2;
  centerZ = (minRangeZ + maxRangeZ) / 2;
  size = Distance (minRangeX, minRangeY, minRangeZ,
		   maxRangeX, maxRangeY, maxRangeZ) / 2;

  AutoGL_SetViewSize (size);
  AutoGL_SetViewCenter (centerX, centerY, centerZ); 
}

void DrawElement 
(int elementId,
 int nodeId0, int nodeId1, int nodeId2) 
{
  char buf[256];

  double nodeX0 = Node_coord[nodeId0][0];
  double nodeY0 = Node_coord[nodeId0][1];
  double nodeZ0 = Node_coord[nodeId0][2];
  
  double nodeX1 = Node_coord[nodeId1][0];
  double nodeY1 = Node_coord[nodeId1][1];
  double nodeZ1 = Node_coord[nodeId1][2];
  
  double nodeX2 = Node_coord[nodeId2][0];
  double nodeY2 = Node_coord[nodeId2][1];
  double nodeZ2 = Node_coord[nodeId2][2];
  
  double centerX = (nodeX0 + nodeX1 + nodeX2) / 3;
  double centerY = (nodeY0 + nodeY1 + nodeY2) / 3;
  double centerZ = (nodeZ0 + nodeZ1 + nodeZ2) / 3;
  
  double vertexX0 = centerX + (nodeX0 - centerX) * Ratio;
  double vertexX1 = centerX + (nodeX1 - centerX) * Ratio;
  double vertexX2 = centerX + (nodeX2 - centerX) * Ratio;
  
  double vertexY0 = centerY + (nodeY0 - centerY) * Ratio;
  double vertexY1 = centerY + (nodeY1 - centerY) * Ratio;
  double vertexY2 = centerY + (nodeY2 - centerY) * Ratio;
  
  double vertexZ0 = centerZ + (nodeZ0 - centerZ) * Ratio;
  double vertexZ1 = centerZ + (nodeZ1 - centerZ) * Ratio;
  double vertexZ2 = centerZ + (nodeZ2 - centerZ) * Ratio;
  
  if (NodeId) {
    AutoGL_SetColor (1, 1, 0);
    
    sprintf(buf, "%d", nodeId0);
    AutoGL_DrawString (nodeX0, nodeY0, nodeZ0,
		       buf);
    
    sprintf(buf, "%d", nodeId1);
    AutoGL_DrawString (nodeX1, nodeY1, nodeZ1,
		       buf);
    
    sprintf(buf, "%d", nodeId2);
    AutoGL_DrawString (nodeX2, nodeY2, nodeZ2,
		       buf);
  }
  
  if (Edge) {
    AutoGL_SetColor (1, 0, 0);
    AutoGL_DrawLine (vertexX0, vertexY0, vertexZ0,
		     vertexX1, vertexY1, vertexZ1);
    AutoGL_DrawLine (vertexX1, vertexY1, vertexZ1,
		     vertexX2, vertexY2, vertexZ2);
    AutoGL_DrawLine (vertexX2, vertexY2, vertexZ2,
		     vertexX0, vertexY0, vertexZ0);
  }
  
  if (Fill) {
    AutoGL_SetColor (0, 1, 1);
    AutoGL_DrawTriangle (vertexX0, vertexY0, vertexZ0,
			 vertexX1, vertexY1, vertexZ1,
			 vertexX2, vertexY2, vertexZ2);
  }
  
  if (ElementId) {
    sprintf(buf, "%d", elementId);
    AutoGL_SetColor (0, 0, 1);
    AutoGL_DrawString (centerX, centerY, centerZ,
		       buf);
  }
}

void DrawMeshA (void) 
{
  int iElement;
  
  for (iElement = 0; iElement < NElementsA; iElement++) {
    int nodeId0 = ElementA_nodeIds[iElement][0];
    int nodeId1 = ElementA_nodeIds[iElement][1];
    int nodeId2 = ElementA_nodeIds[iElement][2];

    DrawElement (iElement,
		 nodeId0, nodeId1, nodeId2);
  }
}

void DrawMeshB (void) 
{
  int iElement;
  
  for (iElement = 0; iElement < NElementsB; iElement++) {
    int nodeId0 = ElementB_nodeIds[iElement][0];
    int nodeId1 = ElementB_nodeIds[iElement][1];
    int nodeId2 = ElementB_nodeIds[iElement][2];

    DrawElement (iElement,
		 nodeId0, nodeId1, nodeId2);
  }
}

void RedrawView (void) 
{
  if (SideA) {
    DrawMeshA ();
  }
  if (SideB) {
    DrawMeshB ();
  }
}

void UpdateModel (void) 
{
  AutoGL_DrawView ();
}

void Quit (void) 
{
  AutoGL_Terminate ();
  exit (0);
}



void AutoGL_SetUp (int argc, char *argv[]) 
{
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri3d_diff_view \n"
       "       node_file  element_file_a  element_file_b \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFileA (argv[2]);
  ReadElementFileB (argv[3]);

  SetView ();
  
  AutoGL_SetViewRedrawCallback (RedrawView);

  AutoGL_SetDefaultCallbackInMode3D (NULL);
  AutoGL_EnableDragInMode3D ();



  AutoGL_AddGroup ("model");    
  AutoGL_SetPanelInMode3D ();

  AutoGL_AddCallback (Quit, "Quit");       

  AutoGL_AddComment ();
  AutoGL_SetLabel ("Model Parameters");

  AutoGL_AddBoolean (&Fill, "Fill");
  AutoGL_AddBoolean (&Edge, "Edge");
  AutoGL_AddBoolean (&NodeId, "NodeId");
  AutoGL_AddBoolean (&ElementId, "ElementId");
  AutoGL_AddReal (&Ratio, "Ratio");
  AutoGL_AddBoolean (&SideA, "SideA");
  AutoGL_AddBoolean (&SideB, "SideB");

  AutoGL_AddCallback (UpdateModel, "UpdateModel");      
}
