/* advautomesh_h_tri3d_diff_mesh.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Basic
*************************/

void SortId3 (int id0, int id1, int id2,
	      int *minId_OUT, int *midId_OUT, int *maxId_OUT)
{
  assert(id0 != id1);
  assert(id0 != id2);
  assert(id1 != id2);

  if (id0 < id1 && id0 < id2) {
    *minId_OUT = id0;
  }
  if (id1 < id0 && id1 < id2) {
    *minId_OUT = id1;
  }
  if (id2 < id0 && id2 < id1) {
    *minId_OUT = id2;
  }

  if ((id1 < id0 && id0 < id2)
      || (id2 < id0 && id0 < id1)) {
    *midId_OUT = id0;
  }
  if ((id0 < id1 && id1 < id2)
      || (id2 < id1 && id1 < id0)) {
    *midId_OUT = id1;
  }
  if ((id0 < id2 && id2 < id1)
      || (id1 < id2 && id2 < id0)) {
    *midId_OUT = id2;
  }

  if (id1 < id0 && id2 < id0) {
    *maxId_OUT = id0;
  }
  if (id0 < id1 && id2 < id1) {
    *maxId_OUT = id1;
  }
  if (id0 < id2 && id1 < id2) {
    *maxId_OUT = id2;
  }
}



/************************
	 Mesh
*************************/

/* Element */

#define N_ELEMENT_NODES 3
#define MAX_ELEMENTS 2000000

int NElementsA;
int ElementA_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int ElementA_hasMatch[MAX_ELEMENTS];

int NElementsB;
int ElementB_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int ElementB_hasMatch[MAX_ELEMENTS];



void ReadElementFileA (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(NElementsA == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElementsA);
  assert(0 < NElementsA);
  assert(NElementsA <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElementsA; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      ElementA_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadElementFileB (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(NElementsB == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElementsB);
  assert(0 < NElementsB);
  assert(NElementsB <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElementsB; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      ElementB_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void WriteElementListFileA (const char *fileName)
{
  FILE *fp;
  int iElement;
  int count;

  assert(0 < NElementsA);

  count = 0;
  for (iElement = 0; iElement < NElementsA; iElement++) {
    if (ElementA_hasMatch[iElement]) {
      count++;
    }
  }

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iElement = 0; iElement < NElementsA; iElement++) {
    if (ElementA_hasMatch[iElement]) {
      fprintf (fp, "%d   %d\n", 
	       count, iElement);

      count++;
    }
  }
  
  fclose (fp);
}

void WriteElementListFileB (const char *fileName)
{
  FILE *fp;
  int iElement;
  int count;

  assert(0 < NElementsB);

  count = 0;
  for (iElement = 0; iElement < NElementsB; iElement++) {
    if (ElementB_hasMatch[iElement]) {
      count++;
    }
  }

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   count);
  
  count = 0;
  for (iElement = 0; iElement < NElementsB; iElement++) {
    if (ElementB_hasMatch[iElement]) {
      fprintf (fp, "%d   %d\n", 
	       count, iElement);

      count++;
    }
  }
  
  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iElement;
  int jElement;
  
  for (iElement = 0; iElement < NElementsA; iElement++) {
    int nodeId0a = ElementA_nodeIds[iElement][0];
    int nodeId1a = ElementA_nodeIds[iElement][1];
    int nodeId2a = ElementA_nodeIds[iElement][2];

    int na0, na1, na2;
    int found;

    SortId3 (nodeId0a, nodeId1a, nodeId2a,
	     &na0, &na1, &na2);

    found = 0;
    for (jElement = 0; jElement < NElementsB; jElement++) {
      int nodeId0b = ElementB_nodeIds[jElement][0];
      int nodeId1b = ElementB_nodeIds[jElement][1];
      int nodeId2b = ElementB_nodeIds[jElement][2];

      int nb0, nb1, nb2;

      SortId3 (nodeId0b, nodeId1b, nodeId2b,
	       &nb0, &nb1, &nb2);

      if (nb0 == na0 
	  && nb1 == na1
	  && nb2 == na2) {
	found = 1;
	break;
      }
    }
    
    ElementA_hasMatch[iElement] = found;
  }

  for (iElement = 0; iElement < NElementsB; iElement++) {
    int nodeId0b = ElementB_nodeIds[iElement][0];
    int nodeId1b = ElementB_nodeIds[iElement][1];
    int nodeId2b = ElementB_nodeIds[iElement][2];

    int nb0, nb1, nb2;
    int found;

    SortId3 (nodeId0b, nodeId1b, nodeId2b,
	     &nb0, &nb1, &nb2);
    
    found = 0;
    for (jElement = 0; jElement < NElementsA; jElement++) {
      int nodeId0a = ElementA_nodeIds[jElement][0];
      int nodeId1a = ElementA_nodeIds[jElement][1];
      int nodeId2a = ElementA_nodeIds[jElement][2];

      int na0, na1, na2;
      
      SortId3 (nodeId0a, nodeId1a, nodeId2a,
	       &na0, &na1, &na2);
      
      if (nb0 == na0 
	  && nb1 == na1
	  && nb2 == na2) {
	found = 1;
	break;
      }
    }
    
    ElementB_hasMatch[iElement] = found;
  }
}



int main (int argc, char *argv[])
{
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri3d_diff_mesh  \n"
       "        element_file_a  element_file_b  \n"
       "        element_list_file_a_OUT  element_list_file_b_OUT \n");
    exit (1);
  }

  ReadElementFileA (argv[1]);
  ReadElementFileB (argv[2]);

  PerformCommand ();

  WriteElementListFileA (argv[3]);
  WriteElementListFileB (argv[4]);

  return 0;
}
