/* advautomesh_h_tri3d_edge.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

#define MAX_NODES 1000000

int NNodes;



/* Element */

#define N_ELEMENT_NODES 3
#define N_ELEMENT_EDGES 3
#define MAX_ELEMENTS 2000000

const int EdgeNodeMap[N_ELEMENT_EDGES][2] = {
  { 1, 2 },
  { 2, 0 },
  { 0, 1 }
};

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_edgeIds[MAX_ELEMENTS][N_ELEMENT_EDGES];



/* Edge */

#define MAX_EDGES 4000000

int NEdges;
int Edge_nodeIds[MAX_EDGES][2];
int Edge_elementIds[MAX_EDGES][2];
int Edge_elementEdgeIds[MAX_EDGES][2];



void Edge_addElement 
(int edgeId, int elementId, int elementEdgeId)
{
  assert(0 <= edgeId);
  assert(edgeId < NEdges);
  assert(0 <= elementId);
  assert(elementId < NElements);
  assert(0 <= elementEdgeId);
  assert(elementEdgeId < N_ELEMENT_EDGES);

  assert(Edge_elementIds[edgeId][1] == -1);
  if (Edge_elementIds[edgeId][0] == -1) {
    Edge_elementIds[edgeId][0] = elementId;
    Edge_elementEdgeIds[edgeId][0] = elementEdgeId;
  } else {
    Edge_elementIds[edgeId][1] = elementId;
    Edge_elementEdgeIds[edgeId][1] = elementEdgeId;
  }
}

int Mesh_addNewEdge (int nodeId0, int nodeId1)
{
  int edgeId = NEdges;

  assert(0 <= nodeId0);
  assert(nodeId0 < NNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NNodes);

  assert(NEdges < MAX_EDGES);
  NEdges++;

  Edge_nodeIds[edgeId][0] = nodeId0;
  Edge_nodeIds[edgeId][1] = nodeId1;
  Edge_elementIds[edgeId][0] = -1;
  Edge_elementIds[edgeId][1] = -1;
  Edge_elementEdgeIds[edgeId][0] = -1;
  Edge_elementEdgeIds[edgeId][1] = -1;
  
  return edgeId;
}

int Mesh_findEdge (int nodeId0, int nodeId1)
{
  int iEdge;

  assert(0 <= nodeId0);
  assert(nodeId0 < NNodes);
  assert(0 <= nodeId1);
  assert(nodeId1 < NNodes);

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    int edgeNodeId0 = Edge_nodeIds[iEdge][0];
    int edgeNodeId1 = Edge_nodeIds[iEdge][1];

    if ((edgeNodeId0 == nodeId0 && edgeNodeId1 == nodeId1)
	|| (edgeNodeId1 == nodeId0 && edgeNodeId0 == nodeId1)) {
      return iEdge;
    }
  }

  return -1;
}

void ReadNodeFile (const char *fileName)
{
  FILE *fp;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  /* skip coordinate */

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void WriteEdgeFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NEdges);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NEdges);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    fprintf (fp, "%d   %d %d   %d %d\n", 
	     iEdge,
	     Edge_elementIds[iEdge][0],
	     Edge_elementEdgeIds[iEdge][0],
	     Edge_elementIds[iEdge][1],
	     Edge_elementEdgeIds[iEdge][1]);
  }

  fclose (fp);
}

void WriteEdgeVertexFile (const char *fileName)
{
  FILE *fp;
  int iEdge;
  
  assert(0 < NEdges);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NEdges);
  
  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    fprintf (fp, "%d   %d %d\n", 
	     iEdge,
	     Edge_nodeIds[iEdge][0],
	     Edge_nodeIds[iEdge][1]);
  }

  fclose (fp);
}

void WriteElementEdgeFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementEdge;
  
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement);

    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      fprintf (fp, "%d ", 
	       Element_edgeIds[iElement][iElementEdge]);
    }

    fprintf (fp, "\n");
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void PerformCommand (void)
{
  int iElement;
  int iElementEdge;
  int iEdge;
  
  assert(NEdges == 0);

  for (iElement = 0; iElement < NElements; iElement++) {
    for (iElementEdge = 0; 
	 iElementEdge < N_ELEMENT_EDGES; 
	 iElementEdge++) {
      int edgeNodeId0 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][0]];
      int edgeNodeId1 
	= Element_nodeIds[iElement][EdgeNodeMap[iElementEdge][1]];
      int edgeId = Mesh_findEdge (edgeNodeId0, edgeNodeId1);

      if (edgeId == -1) {
	edgeId = Mesh_addNewEdge (edgeNodeId0, edgeNodeId1);
      }
      Element_edgeIds[iElement][iElementEdge] = edgeId;
      Edge_addElement (edgeId, iElement, iElementEdge);
    }
  }

  for (iEdge = 0; iEdge < NEdges; iEdge++) {
    assert(Edge_elementIds[iEdge][0] != -1);
    assert(Edge_elementIds[iEdge][1] != -1
	   || Edge_elementIds[iEdge][1] == -1);
  }
}



int main (int argc, char *argv[])
{
  if (argc != 6) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tri3d_edge  node_file  element_file \n"
       "        edge_file_OUT  edge_vertex_file_OUT  element_edge_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);

  PerformCommand ();

  WriteEdgeFile (argv[3]);
  WriteEdgeVertexFile (argv[4]);
  WriteElementEdgeFile (argv[5]);

  return 0;
}
