/* advautomesh_export_ADVENTURE_msh.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_VolumeElement3D *Element;

void WriteMshFile 
(AutoMesh_PolyhedronType polyhedronType, 
 int nElementNodes, 
 const char *fileName)
{
  int nNodes = AutoMesh_Node3D_nNodes (Node);
  int nElements = AutoMesh_VolumeElement3D_nElements (Element);
  FILE *fp;
  int iNode;
  int iElement;
  int iElementNode;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   nElements);
  
  for (iElement = 0; iElement < nElements; iElement++) {
    for (iElementNode = 0; iElementNode < nElementNodes; iElementNode++) {
      int nodeId = AutoMesh_VolumeElement3D_nodeIdAt 
	(Element, iElement, iElementNode);
      
      fprintf (fp, "%d ", 
	       nodeId);
    }
    fprintf (fp, "\n");
  }
  
  fprintf (fp, "%d\n", 
	   nNodes);
  
  for (iNode = 0; iNode < nNodes; iNode++) {
    double x, y, z;
    
    AutoMesh_Node3D_getCoordAt (Node, iNode, 
				&x, &y, &z);
    fprintf (fp, "%22.15e %22.15e %22.15e\n", 
	     x, y, z);
  }
  
  fclose (fp);
}

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  AutoMesh_PolyhedronType polyhedronType;
  int nElementNodes;
  int flag;
  
  if (argc != 5) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_export_ADVENTURE_msh  elementType  node_file  element_file \n"
       "        msh_file_OUT  \n"
       "\n"
       "        <elementType> is any of the following keywords  \n"
       "          tetra4n, tetra10n, hexa8n, hexa20n  \n");
    exit (1);
  }
  
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();
  
  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);
  


  if (strcmp (argv[1], "tetra4n") == 0) {
    polyhedronType = AUTOMESH_POLYHEDRON_TETRA;
    nElementNodes = 4;
  } else if (strcmp (argv[1], "tetra10n") == 0) {
    polyhedronType = AUTOMESH_POLYHEDRON_TETRA;
    nElementNodes = 10;
  } else if (strcmp (argv[1], "hexa8n") == 0) {
    polyhedronType = AUTOMESH_POLYHEDRON_HEXA;
    nElementNodes = 8;
  } else if (strcmp (argv[1], "hexa20n") == 0) {
    polyhedronType = AUTOMESH_POLYHEDRON_HEXA;
    nElementNodes = 20;
  } else {
    fprintf (stderr, " unsupported element type \n");
    exit (1);
  }
  


  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  Element = AutoMesh_VolumeElement3D_New 
    (Node, polyhedronType, nElementNodes);
  assert(Element != NULL);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[3]);
  assert(flag);
  


  WriteMshFile (polyhedronType, nElementNodes, argv[4]);



  AutoMesh_VolumeElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}







