/* advautomesh_export_ADVENTURE_pch.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;

void WritePchFile 
(const char *fileName)
{
  int nNodes = AutoMesh_Node3D_nNodes (Node);
  int nElements = AutoMesh_SurfaceElement3D_nElements (Element);
  FILE *fp;
  int iNode;
  int iElement;

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   nNodes);
  
  for (iNode = 0; iNode < nNodes; iNode++) {
    double x, y, z;
    
    AutoMesh_Node3D_getCoordAt (Node, iNode, 
				&x, &y, &z);
    fprintf (fp, "%22.15e %22.15e %22.15e\n", 
	     x, y, z);
  }
  
  fprintf (fp, "%d\n", 
	   nElements);
  
  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt 
      (Element, iElement, 0);
    int nodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt 
      (Element, iElement, 1);
    int nodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt 
      (Element, iElement, 2);
      
    fprintf (fp, "%d %d %d\n", 
	     nodeId0, nodeId1, nodeId2);
  }
  
  fclose (fp);
}

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;
  
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_export_ADVENTURE_pch  node_file  element_file \n"
       "        pch_file_OUT  \n");
    exit (1);
  }
  
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();
  
  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);
  


  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);

  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);

  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Read (argv[1]);
  assert(flag);
  


  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Read (argv[2]);
  assert(flag);
  


  WritePchFile (argv[3]);



  AutoMesh_SurfaceElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}







