/* advautomesh_import_ADVENTURE_pcm.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoMesh_Node3D *Node;
static AutoMesh_SurfaceElement3D *Element;

void ReadPcmFile (const char *fileName)
{
  FILE *fp;
  int nNodes;
  int nElements;
  int nDomains;
  int iNode;
  int iElement;
  int dummy0;
  int dummy1;
  int flag;
  
  Node = AutoMesh_Node3D_New ();
  assert(Node != NULL);
  
  flag = AutoMesh_Node3D_makeCoordColumn (Node);
  assert(flag);
  
  Element = AutoMesh_SurfaceElement3D_New 
    (Node, AUTOMESH_POLYGON_TRI, 3);
  assert(Element != NULL);
  
  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (Element);
  assert(flag);



  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d %d %d", 
	  &nNodes,
	  &dummy0,
	  &nDomains);
  assert(0 < nNodes);
  assert(dummy0 == 0);
  assert(nDomains == 1);
  
  flag = AutoMesh_Node3D_allocateNodes (Node, nNodes);
  assert(flag);

  for (iNode = 0; iNode < nNodes; iNode++) {
    double x, y, z;
    
    fscanf (fp, "%lf %lf %lf", 
	    &x, &y, &z);
    AutoMesh_Node3D_setCoordAt (Node, iNode, x, y, z);
  }

  fscanf (fp, "%d %d %d", 
	  &nElements,
	  &dummy0,
	  &dummy1);
  assert(0 < nElements);
  assert(dummy0 == 0);
  assert(dummy1 == 0);

  flag = AutoMesh_SurfaceElement3D_allocateElements (Element, nElements);
  assert(flag);
  
  for (iElement = 0; iElement < nElements; iElement++) {
    int nodeId0, nodeId1, nodeId2;
    
    fscanf (fp, "%d %d %d", 
	    &nodeId0, &nodeId1, &nodeId2);
    assert(0 <= nodeId0);
    assert(nodeId0 < nNodes);
    assert(0 <= nodeId1);
    assert(nodeId1 < nNodes);
    assert(0 <= nodeId2);
    assert(nodeId2 < nNodes);

    AutoMesh_SurfaceElement3D_setNodeIdAt 
      (Element, iElement, 0, nodeId0);
    AutoMesh_SurfaceElement3D_setNodeIdAt 
      (Element, iElement, 1, nodeId1);
    AutoMesh_SurfaceElement3D_setNodeIdAt 
      (Element, iElement, 2, nodeId2);
  }

  fclose (fp);
}

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int flag;
  
  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_import_ADVENTURE_pcm  pcm_file \n"
       "        node_file_OUT  element_file_OUT  \n");
    exit (1);
  }
  
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();
  
  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);
  


  ReadPcmFile (argv[1]);



  AutoRes_TableInOut_AssociateReal1DColumn (Node->nodes_coord);
  flag = AutoRes_TableInOut_Write (argv[2]);
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (Element->elements_nodeIds);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);



  AutoMesh_SurfaceElement3D_delete (Element);
  AutoMesh_Node3D_delete (Node);

  return 0;
}







