/* advautomesh_import_hddmmrg.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>
#include <automesh.h>



static AutoRes_Table *Objects;
static AutoRes_Real1DColumn *Objects_values;

int main (int argc, char *argv[])
{
#ifdef BINARY
  int binaryFlag = 1;
  int recordIdFlag = 0;
#else
  int binaryFlag = 0;
  int recordIdFlag = 1;
#endif
  int nComponents;
  int nObjects;
  int iObject;
  int iComponent;
  FILE *fp;
  int flag;

  if (argc != 4) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_import_hddmmrg  nComponentsPerRecord \n"
       "        hddmmrg_file \n"
       "        value_file_OUT  \n");
    exit (1);
  }

  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);

  nComponents = atoi (argv[1]);



  Objects = AutoRes_Table_New ("Objects");

  Objects_values = AutoRes_Real1DColumn_New 
    (Objects, "values", nComponents);
  flag = AutoRes_Table_addReal1DColumn 
    (Objects, Objects_values);
  assert(flag);



  /* read hddmmrg output file */

  fp = fopen (argv[2], "r");
  assert(fp != NULL);

  /* header section */

  {
    char line[256];
    char *ptr;

    /* label line */
    fgets (line, 256, fp);

    /* num_items line */
    fgets (line, 256, fp);
    for (ptr = line;
	 *ptr != '=' && *ptr != 0;
	 ptr++) {
    }
    assert(*ptr == '=');
    ptr++;
    nObjects = atoi (ptr);

    /* blank line */
    fgets (line, 256, fp);
  }

  AutoRes_Table_allocateRecords (Objects, nObjects);

  /* body section */

  for (iObject = 0; iObject < nObjects; iObject++) {
    char recordIdStr[80];

    fscanf (fp, "%s", 
	      recordIdStr);

    for (iComponent = 0; iComponent < nComponents; iComponent++) {
      double value;
      
      fscanf (fp, "%lf", 
	      &value);
      AUTORES_R1D_SET(Objects_values, iObject, iComponent, value);
    }
  }

  fclose (fp);



  AutoRes_TableInOut_AssociateReal1DColumn (Objects_values);
  flag = AutoRes_TableInOut_Write (argv[3]);
  assert(flag);



  AutoRes_Table_delete (Objects);
  Objects = NULL;
  Objects_values = NULL;

  return 0;
}

