/* automesh_node_scalar3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_node_scalar3d.h"



#define COLUMN_NAME_VALUE "AutoMesh:value"



/* NodeScalar3D */

AUTORES_IMPLEMENT_HANDLE(AutoMesh_NodeScalar3D)

AutoMesh_NodeScalar3D *AutoMesh_NodeScalar3D_New 
(AutoMesh_Node3D *node)
{
  AutoMesh_NodeScalar3D *scalar;

  scalar = AutoMesh_NodeScalar3D_NewImplementation ();

  scalar->node = node;
  scalar->nodes_value = NULL;

  return scalar;
}

void AutoMesh_NodeScalar3D_delete 
(AutoMesh_NodeScalar3D *self)
{
  if (self->nodes_value != NULL) {
    AutoRes_Table_removeColumn 
      (self->node->nodes, 
       AutoRes_Table_columnIdOfColumn (self->node->nodes, 
				       self->nodes_value));
    self->nodes_value = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoMesh_NodeScalar3D_makeValueColumn 
(AutoMesh_NodeScalar3D *self)
{
  AutoRes_Table *nodes = self->node->nodes;
  int flag;

  self->nodes_value = AutoRes_RealColumn_New 
    (nodes, COLUMN_NAME_VALUE);
  assert(self->nodes_value != NULL);
  flag = AutoRes_Table_addRealColumn 
    (nodes, self->nodes_value);
  if (!flag) return 0;

  return 1;
}

void AutoMesh_NodeScalar3D_setValueAt 
(AutoMesh_NodeScalar3D *self,
 int nodeId,
 double value)
{
  AUTORES_R_SET(self->nodes_value, nodeId, value);
}

double AutoMesh_NodeScalar3D_valueAt 
(AutoMesh_NodeScalar3D *self,
 int nodeId)
{
  return AUTORES_R(self->nodes_value, nodeId);
}

void AutoMesh_NodeScalar3D_getRange 
(AutoMesh_NodeScalar3D *self,
 double *min_OUT,
 double *max_OUT)
{
  int nNodes = AutoRes_Table_nRecords (self->node->nodes);
  int iNode;
  double min, max;
  
  assert(0 < nNodes);
  
  min = max = AutoMesh_NodeScalar3D_valueAt (self, 0);

  for (iNode = 0; iNode < nNodes; iNode++) {
    double value = AutoMesh_NodeScalar3D_valueAt (self, iNode);

    min = (value < min) ? value : min;
    max = (max < value) ? value : max;
  }

  *min_OUT = min;
  *max_OUT = max;
}
