/* automesh_node_vector2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_node_vector2d.h"



#define COLUMN_NAME_VALUE "AutoMesh:value"



/* NodeVector2D */

AUTORES_IMPLEMENT_HANDLE(AutoMesh_NodeVector2D)

AutoMesh_NodeVector2D *AutoMesh_NodeVector2D_New 
(AutoMesh_Node2D *node)
{
  AutoMesh_NodeVector2D *vector;

  vector = AutoMesh_NodeVector2D_NewImplementation ();

  vector->node = node;
  vector->nodes_value = NULL;

  return vector;
}

void AutoMesh_NodeVector2D_delete 
(AutoMesh_NodeVector2D *self)
{
  if (self->nodes_value != NULL) {
    AutoRes_Table_removeColumn 
      (self->node->nodes, 
       AutoRes_Table_columnIdOfColumn (self->node->nodes, 
				       self->nodes_value));
    self->nodes_value = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoMesh_NodeVector2D_makeValueColumn 
(AutoMesh_NodeVector2D *self)
{
  AutoRes_Table *nodes = self->node->nodes;
  int flag;

  self->nodes_value = AutoRes_Real1DColumn_New 
    (nodes, COLUMN_NAME_VALUE, 2);
  assert(self->nodes_value != NULL);
  flag = AutoRes_Table_addReal1DColumn 
    (nodes, self->nodes_value);
  if (!flag) return 0;

  return 1;
}

void AutoMesh_NodeVector2D_setValueAt 
(AutoMesh_NodeVector2D *self,
 int nodeId,
 double valueX, double valueY)
{
  AUTORES_R1D_SET(self->nodes_value, nodeId, 0, valueX);
  AUTORES_R1D_SET(self->nodes_value, nodeId, 1, valueY);
}

void AutoMesh_NodeVector2D_getValueAt 
(AutoMesh_NodeVector2D *self,
 int nodeId,
 double *valueX_OUT, double *valueY_OUT)
{
  *valueX_OUT = AUTORES_R1D(self->nodes_value, nodeId, 0);
  *valueY_OUT = AUTORES_R1D(self->nodes_value, nodeId, 1);
}

void AutoMesh_NodeVector2D_getRange 
(AutoMesh_NodeVector2D *self,
 double *minValueX_OUT, double *minValueY_OUT,
 double *maxValueX_OUT, double *maxValueY_OUT)
{
  int nNodes = AutoRes_Table_nRecords (self->node->nodes);
  int iNode;
  double minX, minY;
  double maxX, maxY;
  
  assert(0 < nNodes);
  
  AutoMesh_NodeVector2D_getValueAt (self, 0,
				    &minX, &minY);
  AutoMesh_NodeVector2D_getValueAt (self, 0,
				    &maxX, &maxY);

  for (iNode = 0; iNode < nNodes; iNode++) {
    double x, y;

    AutoMesh_NodeVector2D_getValueAt (self, iNode,
				      &x, &y);

    minX = (x < minX) ? x : minX;
    minY = (y < minY) ? y : minY;

    maxX = (maxX < x) ? x : maxX;
    maxY = (maxY < y) ? y : maxY;
  }

  *minValueX_OUT = minX;
  *minValueY_OUT = minY;

  *maxValueX_OUT = maxX;
  *maxValueY_OUT = maxY;
}

double AutoMesh_NodeVector2D_maxLength 
(AutoMesh_NodeVector2D *self)
{
  double maxLength;
  int nNodes = AutoRes_Table_nRecords (self->node->nodes);
  int iNode;
  
  assert(0 < nNodes);
  
  maxLength = 0.0;
  for (iNode = 0; iNode < nNodes; iNode++) {
    double x, y;
    double length;

    AutoMesh_NodeVector2D_getValueAt (self, iNode,
				      &x, &y);
    length = AutoMesh_Length2D (x, y);

    maxLength = (maxLength < length) ? length : maxLength;
  }

  return maxLength;
}
