/* header file for automesh_shape.c */

#ifndef __AUTOMESH_SHAPE__
#define __AUTOMESH_SHAPE__



/* Triangle */
/* $B;03Q7A(B */

#define AUTOMESH_TRI_N_VERTEXS 3
#define AUTOMESH_TRI_N_EDGES 3
/* $B3FJU$N3FD:E@$KBP1~$9$k;03Q7A$ND:E@HV9f(B */
extern const int AutoMesh_Tri_EdgeVertexMap[3][2];



/* Quadrangle */
/* $B;MJU7A(B */

#define AUTOMESH_QUAD_N_VERTEXS 3
#define AUTOMESH_QUAD_N_EDGES 3
/* $B3FJU$N3FD:E@$KBP1~$9$k;MJU7A$ND:E@HV9f(B */
extern const int AutoMesh_Quad_EdgeVertexMap[4][2];



/* Polygon */
/* $BB?3Q7A(B */

/* $BB?3Q7A%?%$%W(B */
typedef enum {
  AUTOMESH_POLYGON_TRI,  /* $B;03Q7A(B */
  AUTOMESH_POLYGON_QUAD  /* $B;MJU7A(B */
} AutoMesh_PolygonType;

/* $BD:E@?t$r5a$a$k!#(B */
int AutoMesh_Polygon_NVertexs 
(AutoMesh_PolygonType type);

/* $BJU?t$r5a$a$k!#(B */
int AutoMesh_Polygon_NEdges 
(AutoMesh_PolygonType type);

/* $B3FJU$N3FD:E@$KBP1~$9$kB?3Q7A$ND:E@HV9f$r5a$a$k!#(B */
int AutoMesh_Polygon_EdgeVertex 
(AutoMesh_PolygonType type, int edgeNum, int vertexNum);



/* Tetrahedron */
/* $B;MLLBN(B */

#define AUTOMESH_TETRA_N_VERTEXS 4
#define AUTOMESH_TETRA_N_EDGES 6
#define AUTOMESH_TETRA_N_FACES 4
#define AUTOMESH_TETRA_N_FACE_VERTEXS 3
/* $B3FJU$N3FD:E@$KBP1~$9$k;MLLBN$ND:E@HV9f(B */
extern const int AutoMesh_Tetra_EdgeVertexMap[6][2];
/* $B3FLL$N3FD:E@$KBP1~$9$k;MLLBN$ND:E@HV9f(B */
extern const int AutoMesh_Tetra_FaceVertexMap[4][3];

/* $BEyCMLL$r5a$a$k!#(B */
int AutoMesh_Tetra_GetIsosurface 
(double vertexValues[4], 
 double offset,
 int *nEdges_OUT,
 int edgeNums_OUT[4], double ratios_OUT[4]);
/* vertexValues : $B;MLLBN$N3FD:E@$G$N%9%+%i!<CM(B */
/* offset : $B%9%+%i!<CM$N%*%U%;%C%HCM(B */
/* nEdges_OUT : $BEyCMLL$NJU?t!aD:E@?t(B */
/* edgeNums_OUT : $BEyCMLL$ND:E@$,B8:_$9$kJUHV9f(B */
/* ratios_OUT : $BJU>e$G$NJ,3dHfN((B */



/* Hexahedron */
/* $BO;LLBN(B */

#define AUTOMESH_HEXA_N_VERTEXS 8
#define AUTOMESH_HEXA_N_EDGES 12
#define AUTOMESH_HEXA_N_FACES 6
#define AUTOMESH_HEXA_N_FACE_VERTEXS 4
/* $B3FJU$N3FD:E@$KBP1~$9$kO;LLBN$ND:E@HV9f(B */
extern const int AutoMesh_Hexa_EdgeVertexMap[12][2];
/* $B3FLL$N3FD:E@$KBP1~$9$kO;LLBN$ND:E@HV9f(B */
extern const int AutoMesh_Hexa_FaceVertexMap[6][4];



/* Prism */
/* $B;03QCl(B */

#define AUTOMESH_PRISM_N_VERTEXS 6
#define AUTOMESH_PRISM_N_EDGES 9
#define AUTOMESH_PRISM_N_FACES 5
#define AUTOMESH_PRISM_MAX_FACE_VERTEXS 4
/* $B3FJU$N3FD:E@$KBP1~$9$kO;LLBN$ND:E@HV9f(B */
extern const int AutoMesh_Prism_EdgeVertexMap[9][2];
/* $B3FLL$4$H$ND:E@?t(B */
extern const int AutoMesh_Prism_NFaceVertexs[5];
/* $B3FLL$N3FD:E@$KBP1~$9$kO;LLBN$ND:E@HV9f(B */
extern const int AutoMesh_Prism_FaceVertexMap[5][4];



/* Polyhedron */
/* $BB?LLBN(B */

/* $BB?LLBN%?%$%W(B */
typedef enum {
  AUTOMESH_POLYHEDRON_TETRA,  /* $B;MLLBN(B */
  AUTOMESH_POLYHEDRON_HEXA,   /* $BO;LLBN(B */
  AUTOMESH_POLYHEDRON_PRISM   /* $B;03QCl(B */
} AutoMesh_PolyhedronType;

/* $BD:E@?t$r5a$a$k!#(B */
int AutoMesh_Polyhedron_NVertexs 
(AutoMesh_PolyhedronType type);

/* $BJU?t$r5a$a$k!#(B */
int AutoMesh_Polyhedron_NEdges 
(AutoMesh_PolyhedronType type);

/* $BLL?t$r5a$a$k!#(B */
int AutoMesh_Polyhedron_NFaces 
(AutoMesh_PolyhedronType type);

/* $B3FLL$ND:E@?t$N:GBgCM$r5a$a$k!#(B */
int AutoMesh_Polyhedron_MaxFaceVertexs 
(AutoMesh_PolyhedronType type);

/* $B3FJU$N3FD:E@$KBP1~$9$kB?LLBN$ND:E@HV9f$r5a$a$k!#(B */
int AutoMesh_Polyhedron_EdgeVertex 
(AutoMesh_PolyhedronType type, int edgeNum, int vertexNum);

/* $B3FLL$4$H$ND:E@?t$r5a$a$k!#(B */
int AutoMesh_Polyhedron_NFaceVertexs 
(AutoMesh_PolyhedronType type, int faceNum);

/* $B3FLL$N3FD:E@$KBP1~$9$kB?LLBN$ND:E@HV9f$r5a$a$k!#(B */
int AutoMesh_Polyhedron_FaceVertex 
(AutoMesh_PolyhedronType type, int faceNum, int vertexNum);



#endif  /* __AUTOMESH_SHAPE__ */



