/* automesh_surface_mid_node_generator2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_volume_element2d.h"
#include "automesh_volume_edge2d.h"
#include "automesh_volume_mid_node_generator2d.h"



#define COLUMN_NAME_SRC_NODE_IDS "AutoMesh:srcNodeIds"
#define COLUMN_NAME_DEST_NODE_ID "AutoMesh:destNodeId"



/* VolumeMidNodeGenerator2D */

AUTORES_IMPLEMENT_HANDLE(AutoMesh_VolumeMidNodeGenerator2D)

AutoMesh_VolumeMidNodeGenerator2D *AutoMesh_VolumeMidNodeGenerator2D_New 
(AutoMesh_VolumeEdge2D *edge)
{
  AutoMesh_VolumeMidNodeGenerator2D *generator;
  int nElementNodes;
  
  generator = AutoMesh_VolumeMidNodeGenerator2D_NewImplementation ();
  
  generator->edge = edge;

  generator->srcNode = edge->element->node;
  generator->srcNodes_destNodeId = NULL;

  generator->srcElement = edge->element;

  generator->destNode = AutoMesh_Node2D_New ();
  assert(generator->destNode != NULL);
  generator->destNodes_srcNodeIds = NULL;

  switch (generator->srcElement->polygonType) {
  case AUTOMESH_POLYGON_TRI:
    switch (generator->srcElement->nElementNodes) {
    case 3:
      nElementNodes = 6;
      break;
    case 6:
      assert(0);
      break;
    default:
      assert(0);
      break;
    }
    break;
  case AUTOMESH_POLYGON_QUAD:
    switch (generator->srcElement->nElementNodes) {
    case 4:
      nElementNodes = 8;
      break;
    case 8:
      assert(0);
      break;
    default:
      assert(0);
      break;
    }
    break;
  default:
    assert(0);
    break;
  }

  generator->destElement = AutoMesh_VolumeElement2D_New 
    (generator->destNode, 
     generator->srcElement->polygonType,
     nElementNodes);
  assert(generator->destElement != NULL);

  return generator;
}

void AutoMesh_VolumeMidNodeGenerator2D_delete 
(AutoMesh_VolumeMidNodeGenerator2D *self)
{
  if (self->srcNodes_destNodeId != NULL) {
    AutoRes_Table_removeColumn 
      (self->srcNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->srcNode->nodes, 
	self->srcNodes_destNodeId));
    self->srcNodes_destNodeId = NULL;
  }

  if (self->destNodes_srcNodeIds != NULL) {
    AutoRes_Table_removeColumn 
      (self->destNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->destNode->nodes, 
	self->destNodes_srcNodeIds));
    self->destNodes_srcNodeIds = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoMesh_VolumeMidNodeGenerator2D_do 
(AutoMesh_VolumeMidNodeGenerator2D *self,
 int nodeFlag,
 int elementFlag,
 int srcNodeIdsFlag,
 int destNodeIdFlag)
{
  int nSrcNodes = AutoMesh_Node2D_nNodes (self->srcNode);
  int nElements = AutoMesh_VolumeElement2D_nElements (self->srcElement);
  int nEdges = AutoMesh_VolumeEdge2D_nEdges (self->edge);
  int iSrcNode;
  int iEdge;
  int iElement;
  int flag;

  /* NEED TO REDEFINE */
  /* tri only */
  assert(self->srcElement->polygonType == AUTOMESH_POLYGON_TRI);



  assert(AutoMesh_Node2D_nNodes (self->destNode) == 0);
  
  flag = AutoMesh_Node2D_allocateNodes 
    (self->destNode, nSrcNodes + nEdges);
  if (!flag) return 0;

  assert(0 < AutoMesh_Node2D_nNodes (self->destNode));



  assert(AutoMesh_VolumeElement2D_nElements (self->destElement) == 0);
  
  flag = AutoMesh_VolumeElement2D_allocateElements 
    (self->destElement, nElements);
  if (!flag) return 0;

  assert(0 < AutoMesh_VolumeElement2D_nElements (self->destElement));



  if (destNodeIdFlag) {
    assert(self->srcNodes_destNodeId == NULL);

    self->srcNodes_destNodeId = AutoRes_IntegerColumn_New
      (self->srcNode->nodes, COLUMN_NAME_DEST_NODE_ID);
    assert(self->srcNodes_destNodeId != NULL);
    flag = AutoRes_Table_addIntegerColumn 
      (self->srcNode->nodes, 
       self->srcNodes_destNodeId);
    if (!flag) return 0;

    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      AUTORES_I_SET(self->srcNodes_destNodeId, iSrcNode, iSrcNode);
    }

    assert(self->srcNodes_destNodeId != NULL);
  }



  if (nodeFlag) {
    assert(0 < AutoMesh_Node2D_nNodes (self->destNode));
    assert(self->destNode->nodes_coord == NULL);
    
    flag = AutoMesh_Node2D_makeCoordColumn (self->destNode);
    if (!flag) return 0;

    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      double x, y;

      AutoMesh_Node2D_getCoordAt (self->srcNode, iSrcNode, 
				  &x, &y);
      AutoMesh_Node2D_setCoordAt (self->destNode, iSrcNode, x, y);
    }
    
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int destNodeId = iEdge + nSrcNodes;
      double x, y;
      
      AutoMesh_VolumeEdge2D_getMidPointAt (self->edge, iEdge,
					    &x, &y);
      AutoMesh_Node2D_setCoordAt (self->destNode, destNodeId, x, y);
    }
    
    assert(self->destNode->nodes_coord != NULL);
  }



  if (srcNodeIdsFlag) {
    assert(0 < AutoMesh_Node2D_nNodes (self->destNode));
    assert(self->destNodes_srcNodeIds == NULL);

    self->destNodes_srcNodeIds = AutoRes_Integer1DColumn_New
      (self->destNode->nodes, COLUMN_NAME_SRC_NODE_IDS, 2);
    assert(self->destNodes_srcNodeIds != NULL);
    flag = AutoRes_Table_addInteger1DColumn 
      (self->destNode->nodes, 
       self->destNodes_srcNodeIds);
    if (!flag) return 0;

    /* 1st node : [0] = nodeId, [1] = -1 */
    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, iSrcNode, 0, 
		      iSrcNode);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, iSrcNode, 1, 
		      -1);
    }

    /* 2nd node : [0] = nodeId0, [1] = nodeId1 */
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int destNodeId = iEdge + nSrcNodes;
      int srcNodeId0, srcNodeId1;
      
      AutoMesh_VolumeEdge2D_getVertexNodeIdsAt 
	(self->edge, iEdge,
	 &srcNodeId0, &srcNodeId1);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, destNodeId, 0, 
		      srcNodeId0);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, destNodeId, 1, 
		      srcNodeId1);
    }

    assert(self->destNodes_srcNodeIds != NULL);
  }



  if (elementFlag) {
    assert(0 < AutoMesh_VolumeElement2D_nElements (self->destElement));
    assert(self->destElement->elements_nodeIds == NULL);
    
    flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (self->destElement);
    if (!flag) return 0;
    
    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_VolumeElement2D_nodeIdAt
	(self->srcElement, iElement, 0);
      int nodeId1 = AutoMesh_VolumeElement2D_nodeIdAt
	(self->srcElement, iElement, 1);
      int nodeId2 = AutoMesh_VolumeElement2D_nodeIdAt
	(self->srcElement, iElement, 2);
      
      int edgeId0 = AutoMesh_VolumeEdge2D_edgeIdAtElement 
	(self->edge, iElement, 0);
      int edgeId1 = AutoMesh_VolumeEdge2D_edgeIdAtElement 
	(self->edge, iElement, 1);
      int edgeId2 = AutoMesh_VolumeEdge2D_edgeIdAtElement 
	(self->edge, iElement, 2);
      
      int edgeNodeId0 = edgeId0 + nSrcNodes;
      int edgeNodeId1 = edgeId1 + nSrcNodes;
      int edgeNodeId2 = edgeId2 + nSrcNodes;

      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 0, nodeId0);
      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 1, nodeId1);
      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 2, nodeId2);
      
      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 3, edgeNodeId0);
      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 4, edgeNodeId1);
      AutoMesh_VolumeElement2D_setNodeIdAt
	(self->destElement, iElement, 5, edgeNodeId2);
    }
    
    assert(self->destElement->elements_nodeIds != NULL);
  }



  return 1;
}

int AutoMesh_VolumeMidNodeGenerator2D_destNodeIdAtSrcNode
(AutoMesh_VolumeMidNodeGenerator2D *self,
 int srcNodeId)
{
  return AUTORES_I(self->srcNodes_destNodeId, srcNodeId);
}

void AutoMesh_VolumeMidNodeGenerator2D_getSrcNodeIdsAtDestNode
(AutoMesh_VolumeMidNodeGenerator2D *self,
 int destNodeId,
 int srcNodeIds_OUT[2])
{
  srcNodeIds_OUT[0] = AUTORES_I1D(self->destNodes_srcNodeIds, destNodeId, 0);
  srcNodeIds_OUT[1] = AUTORES_I1D(self->destNodes_srcNodeIds, destNodeId, 1);
}

