/* automesh_volume_mid_node_generator3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_edge3d.h"
#include "automesh_volume_mid_node_generator3d.h"



#define COLUMN_NAME_SRC_NODE_IDS "AutoMesh:srcNodeIds"
#define COLUMN_NAME_DEST_NODE_ID "AutoMesh:destNodeId"



/* VolumeMidNodeGenerator3D */

AUTORES_IMPLEMENT_HANDLE(AutoMesh_VolumeMidNodeGenerator3D)

AutoMesh_VolumeMidNodeGenerator3D *AutoMesh_VolumeMidNodeGenerator3D_New 
(AutoMesh_VolumeEdge3D *edge)
{
  AutoMesh_VolumeMidNodeGenerator3D *generator;
  int nElementNodes;
  
  generator = AutoMesh_VolumeMidNodeGenerator3D_NewImplementation ();
  
  generator->edge = edge;

  generator->srcNode = edge->element->node;
  generator->srcNodes_destNodeId = NULL;

  generator->srcElement = edge->element;

  generator->destNode = AutoMesh_Node3D_New ();
  assert(generator->destNode != NULL);
  generator->destNodes_srcNodeIds = NULL;

  switch (generator->srcElement->polyhedronType) {
  case AUTOMESH_POLYHEDRON_TETRA:
    switch (generator->srcElement->nElementNodes) {
    case 4:
      nElementNodes = 10;
      break;
    case 10:
      assert(0);
      break;
    default:
      assert(0);
      break;
    }
    break;
  case AUTOMESH_POLYHEDRON_HEXA:
    switch (generator->srcElement->nElementNodes) {
    case 8:
      nElementNodes = 20;
      break;
    case 20:
      assert(0);
      break;
    default:
      assert(0);
      break;
    }
    break;
  case AUTOMESH_POLYHEDRON_PRISM:
    switch (generator->srcElement->nElementNodes) {
    case 6:
      nElementNodes = 15;
      break;
    case 15:
      assert(0);
      break;
    default:
      assert(0);
      break;
    }
    break;
  default:
    assert(0);
    break;
  }

  generator->destElement = AutoMesh_VolumeElement3D_New 
    (generator->destNode, 
     generator->srcElement->polyhedronType,
     nElementNodes);
  assert(generator->destElement != NULL);

  return generator;
}

void AutoMesh_VolumeMidNodeGenerator3D_delete 
(AutoMesh_VolumeMidNodeGenerator3D *self)
{
  if (self->srcNodes_destNodeId != NULL) {
    AutoRes_Table_removeColumn 
      (self->srcNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->srcNode->nodes, 
	self->srcNodes_destNodeId));
    self->srcNodes_destNodeId = NULL;
  }

  if (self->destNodes_srcNodeIds != NULL) {
    AutoRes_Table_removeColumn 
      (self->destNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->destNode->nodes, 
	self->destNodes_srcNodeIds));
    self->destNodes_srcNodeIds = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoMesh_VolumeMidNodeGenerator3D_do 
(AutoMesh_VolumeMidNodeGenerator3D *self,
 int nodeFlag,
 int elementFlag,
 int srcNodeIdsFlag,
 int destNodeIdFlag)
{
  int nSrcNodes = AutoMesh_Node3D_nNodes (self->srcNode);
  int nElements = AutoMesh_VolumeElement3D_nElements (self->srcElement);
  int nEdges = AutoMesh_VolumeEdge3D_nEdges (self->edge);
  int iSrcNode;
  int iEdge;
  int iElement;
  int flag;

  /* NEED TO REDEFINE */
  /* tetra only */
  assert(self->srcElement->polyhedronType == AUTOMESH_POLYHEDRON_TETRA);



  assert(AutoMesh_Node3D_nNodes (self->destNode) == 0);
  
  flag = AutoMesh_Node3D_allocateNodes 
    (self->destNode, nSrcNodes + nEdges);
  if (!flag) return 0;

  assert(0 < AutoMesh_Node3D_nNodes (self->destNode));



  assert(AutoMesh_VolumeElement3D_nElements (self->destElement) == 0);
  
  flag = AutoMesh_VolumeElement3D_allocateElements 
    (self->destElement, nElements);
  if (!flag) return 0;

  assert(0 < AutoMesh_VolumeElement3D_nElements (self->destElement));



  if (destNodeIdFlag) {
    assert(self->srcNodes_destNodeId == NULL);

    self->srcNodes_destNodeId = AutoRes_IntegerColumn_New
      (self->srcNode->nodes, COLUMN_NAME_DEST_NODE_ID);
    assert(self->srcNodes_destNodeId != NULL);
    flag = AutoRes_Table_addIntegerColumn 
      (self->srcNode->nodes, 
       self->srcNodes_destNodeId);
    if (!flag) return 0;

    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      AUTORES_I_SET(self->srcNodes_destNodeId, iSrcNode, iSrcNode);
    }

    assert(self->srcNodes_destNodeId != NULL);
  }



  if (nodeFlag) {
    assert(0 < AutoMesh_Node3D_nNodes (self->destNode));
    assert(self->destNode->nodes_coord == NULL);
    
    flag = AutoMesh_Node3D_makeCoordColumn (self->destNode);
    if (!flag) return 0;

    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      double x, y, z;

      AutoMesh_Node3D_getCoordAt (self->srcNode, iSrcNode, 
				  &x, &y, &z);
      AutoMesh_Node3D_setCoordAt (self->destNode, iSrcNode, x, y, z);
    }
    
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int destNodeId = iEdge + nSrcNodes;
      double x, y, z;
      
      AutoMesh_VolumeEdge3D_getMidPointAt (self->edge, iEdge,
					   &x, &y, &z);
      AutoMesh_Node3D_setCoordAt (self->destNode, destNodeId, x, y, z);
    }
    
    assert(self->destNode->nodes_coord != NULL);
  }



  if (srcNodeIdsFlag) {
    assert(0 < AutoMesh_Node3D_nNodes (self->destNode));
    assert(self->destNodes_srcNodeIds == NULL);

    self->destNodes_srcNodeIds = AutoRes_Integer1DColumn_New
      (self->destNode->nodes, COLUMN_NAME_SRC_NODE_IDS, 2);
    assert(self->destNodes_srcNodeIds != NULL);
    flag = AutoRes_Table_addInteger1DColumn 
      (self->destNode->nodes, 
       self->destNodes_srcNodeIds);
    if (!flag) return 0;

    /* 1st node : [0] = nodeId, [1] = -1 */
    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, iSrcNode, 0, 
		      iSrcNode);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, iSrcNode, 1, 
		      -1);
    }

    /* 2nd node : [0] = nodeId0, [1] = nodeId1 */
    for (iEdge = 0; iEdge < nEdges; iEdge++) {
      int destNodeId = iEdge + nSrcNodes;
      int srcNodeId0, srcNodeId1;
      
      AutoMesh_VolumeEdge3D_getVertexNodeIdsAt 
	(self->edge, iEdge,
	 &srcNodeId0, &srcNodeId1);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, destNodeId, 0, 
		      srcNodeId0);
      AUTORES_I1D_SET(self->destNodes_srcNodeIds, destNodeId, 1, 
		      srcNodeId1);
    }

    assert(self->destNodes_srcNodeIds != NULL);
  }



  if (elementFlag) {
    assert(0 < AutoMesh_VolumeElement3D_nElements (self->destElement));
    assert(self->destElement->elements_nodeIds == NULL);
    
    flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (self->destElement);
    if (!flag) return 0;
    
    for (iElement = 0; iElement < nElements; iElement++) {
      int nodeId0 = AutoMesh_VolumeElement3D_nodeIdAt
	(self->srcElement, iElement, 0);
      int nodeId1 = AutoMesh_VolumeElement3D_nodeIdAt
	(self->srcElement, iElement, 1);
      int nodeId2 = AutoMesh_VolumeElement3D_nodeIdAt
	(self->srcElement, iElement, 2);
      int nodeId3 = AutoMesh_VolumeElement3D_nodeIdAt
	(self->srcElement, iElement, 3);
      
      int edgeId0 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 0);
      int edgeId1 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 1);
      int edgeId2 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 2);
      int edgeId3 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 3);
      int edgeId4 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 4);
      int edgeId5 = AutoMesh_VolumeEdge3D_edgeIdAtElement 
	(self->edge, iElement, 5);
      
      int edgeNodeId0 = edgeId0 + nSrcNodes;
      int edgeNodeId1 = edgeId1 + nSrcNodes;
      int edgeNodeId2 = edgeId2 + nSrcNodes;
      int edgeNodeId3 = edgeId3 + nSrcNodes;
      int edgeNodeId4 = edgeId4 + nSrcNodes;
      int edgeNodeId5 = edgeId5 + nSrcNodes;
      
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 0, nodeId0);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 1, nodeId1);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 2, nodeId2);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 3, nodeId3);
      
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 4, edgeNodeId0);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 5, edgeNodeId1);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 6, edgeNodeId2);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 7, edgeNodeId3);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 8, edgeNodeId4);
      AutoMesh_VolumeElement3D_setNodeIdAt
	(self->destElement, iElement, 9, edgeNodeId5);
    }
    
    assert(self->destElement->elements_nodeIds != NULL);
  }



  return 1;
}

int AutoMesh_VolumeMidNodeGenerator3D_destNodeIdAtSrcNode
(AutoMesh_VolumeMidNodeGenerator3D *self,
 int srcNodeId)
{
  return AUTORES_I(self->srcNodes_destNodeId, srcNodeId);
}

void AutoMesh_VolumeMidNodeGenerator3D_getSrcNodeIdsAtDestNode
(AutoMesh_VolumeMidNodeGenerator3D *self,
 int destNodeId,
 int srcNodeIds_OUT[2])
{
  srcNodeIds_OUT[0] = AUTORES_I1D(self->destNodes_srcNodeIds, destNodeId, 0);
  srcNodeIds_OUT[1] = AUTORES_I1D(self->destNodes_srcNodeIds, destNodeId, 1);
}




