/* automesh_volume_mid_node_remover2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_volume_element2d.h"
#include "automesh_volume_edge2d.h"
#include "automesh_volume_mid_node_remover2d.h"



#define COLUMN_NAME_SRC_NODE_ID "AutoMesh:srcNodeId"
#define COLUMN_NAME_DEST_NODE_ID "AutoMesh:destNodeId"



/* VolumeMidNodeRemover2D */

AUTORES_IMPLEMENT_HANDLE(AutoMesh_VolumeMidNodeRemover2D)

AutoMesh_VolumeMidNodeRemover2D *AutoMesh_VolumeMidNodeRemover2D_New 
(AutoMesh_VolumeElement2D *element)
{
  AutoMesh_VolumeMidNodeRemover2D *remover;
  int nElementNodes;
  
  remover = AutoMesh_VolumeMidNodeRemover2D_NewImplementation ();
  
  remover->srcNode = element->node;
  remover->srcNodes_destNodeId = NULL;

  remover->srcElement = element;

  remover->destNode = AutoMesh_Node2D_New ();
  assert(remover->destNode != NULL);
  remover->destNodes_srcNodeId = NULL;

  switch (remover->srcElement->polygonType) {
  case AUTOMESH_POLYGON_TRI:
    switch (remover->srcElement->nElementNodes) {
    case 3:
      assert(0);
      break;
    case 6:
      nElementNodes = 3;
      break;
    default:
      assert(0);
      break;
    }
    break;
  case AUTOMESH_POLYGON_QUAD:
    switch (remover->srcElement->nElementNodes) {
    case 4:
      assert(0);
      break;
    case 8:
      nElementNodes = 4;
      break;
    default:
      assert(0);
      break;
    }
    break;
  default:
    assert(0);
    break;
  }
  assert(nElementNodes == AutoMesh_Polygon_NVertexs
	 (remover->srcElement->polygonType));

  remover->destElement = AutoMesh_VolumeElement2D_New 
    (remover->destNode, 
     remover->srcElement->polygonType, 
     nElementNodes);
  assert(remover->destElement != NULL);

  return remover;
}

void AutoMesh_VolumeMidNodeRemover2D_delete 
(AutoMesh_VolumeMidNodeRemover2D *self)
{
  if (self->srcNodes_destNodeId != NULL) {
    AutoRes_Table_removeColumn 
      (self->srcNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->srcNode->nodes, 
	self->srcNodes_destNodeId));
    self->srcNodes_destNodeId = NULL;
  }

  if (self->destNodes_srcNodeId != NULL) {
    AutoRes_Table_removeColumn 
      (self->destNode->nodes, 
       AutoRes_Table_columnIdOfColumn 
       (self->destNode->nodes, 
	self->destNodes_srcNodeId));
    self->destNodes_srcNodeId = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoMesh_VolumeMidNodeRemover2D_do 
(AutoMesh_VolumeMidNodeRemover2D *self,
 int nodeFlag,
 int elementFlag,
 int srcNodeIdFlag)
{
  int nSrcNodes = AutoMesh_Node2D_nNodes (self->srcNode);
  int nElements = AutoMesh_VolumeElement2D_nElements (self->srcElement);
  int nElementVertexs = AutoMesh_Polygon_NVertexs
    (self->srcElement->polygonType);
  int destNodeId;
  int nDestNodes;
  int iSrcNode;
  int iElement;
  int iElementVertex;
  int flag;

  assert(nElementVertexs == self->destElement->nElementNodes);



  assert(AutoMesh_VolumeElement2D_nElements (self->destElement) == 0);

  flag = AutoMesh_VolumeElement2D_allocateElements 
    (self->destElement, nElements);
  if (!flag) return 0;

  assert(0 < AutoMesh_VolumeElement2D_nElements (self->destElement));



  assert(self->srcNodes_destNodeId == NULL);

  self->srcNodes_destNodeId = AutoRes_IntegerColumn_New
    (self->srcNode->nodes, COLUMN_NAME_DEST_NODE_ID);
  assert(self->srcNodes_destNodeId != NULL);
  flag = AutoRes_Table_addIntegerColumn 
    (self->srcNode->nodes, 
     self->srcNodes_destNodeId);
  if (!flag) return 0;

  for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
    AUTORES_I_SET(self->srcNodes_destNodeId, iSrcNode, -1);
  }

  for (iElement = 0; iElement < nElements; iElement++) {
    for (iElementVertex = 0; 
	 iElementVertex < nElementVertexs; 
	 iElementVertex++) {
      int srcNodeId = AutoMesh_VolumeElement2D_nodeIdAt
	(self->srcElement, iElement, iElementVertex);
      
      AUTORES_I_SET(self->srcNodes_destNodeId, srcNodeId, 0);
    }
  }

  destNodeId = 0;
  for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
    if (AUTORES_I(self->srcNodes_destNodeId, iSrcNode) == 0) {
      AUTORES_I_SET(self->srcNodes_destNodeId, iSrcNode, destNodeId);
      destNodeId++;
    }
  }
  
  assert(self->srcNodes_destNodeId != NULL);



  assert(self->srcNodes_destNodeId != NULL);
  assert(AutoMesh_Node2D_nNodes (self->destNode) == 0);

  nDestNodes = 0;
  for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
    if (AUTORES_I(self->srcNodes_destNodeId, iSrcNode) != -1) {
      nDestNodes++;
    }
  }

  flag = AutoMesh_Node2D_allocateNodes 
    (self->destNode, nDestNodes);
  if (!flag) return 0;

  assert(0 < AutoMesh_Node2D_nNodes (self->destNode));



  if (srcNodeIdFlag) {
    assert(0 < AutoMesh_Node2D_nNodes (self->destNode));
    assert(self->destNodes_srcNodeId == NULL);
    
    self->destNodes_srcNodeId = AutoRes_IntegerColumn_New
      (self->destNode->nodes, COLUMN_NAME_SRC_NODE_ID);
    assert(self->destNodes_srcNodeId != NULL);
    flag = AutoRes_Table_addIntegerColumn 
      (self->destNode->nodes, 
       self->destNodes_srcNodeId);
    if (!flag) return 0;
    
    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      int destNodeId = AUTORES_I(self->srcNodes_destNodeId, iSrcNode);
      
      if (destNodeId != -1) {
	AUTORES_I_SET(self->destNodes_srcNodeId, destNodeId, iSrcNode);
      }
    }
    
    assert(self->destNodes_srcNodeId != NULL);
  }



  if (nodeFlag) {
    assert(self->srcNodes_destNodeId != NULL);
    assert(0 < AutoMesh_Node2D_nNodes (self->destNode));
    assert(self->destNode->nodes_coord == NULL);
    
    flag = AutoMesh_Node2D_makeCoordColumn (self->destNode);
    if (!flag) return 0;
    
    for (iSrcNode = 0; iSrcNode < nSrcNodes; iSrcNode++) {
      int destNodeId = AUTORES_I(self->srcNodes_destNodeId, iSrcNode);
      
      if (destNodeId != -1) {
	double x, y;
	
	AutoMesh_Node2D_getCoordAt (self->srcNode, iSrcNode, 
				    &x, &y);
	AutoMesh_Node2D_setCoordAt (self->destNode, destNodeId, x, y);
      }
    }
    
    assert(self->destNode->nodes_coord != NULL);
  }



  if (elementFlag) {
    assert(self->srcNodes_destNodeId != NULL);
    assert(0 < AutoMesh_VolumeElement2D_nElements (self->destElement));
    assert(self->destElement->elements_nodeIds == NULL);
    
    flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (self->destElement);
    if (!flag) return 0;
    
    for (iElement = 0; iElement < nElements; iElement++) {
      for (iElementVertex = 0; 
	   iElementVertex < nElementVertexs; 
	   iElementVertex++) {
	int srcNodeId = AutoMesh_VolumeElement2D_nodeIdAt
	  (self->srcElement, iElement, iElementVertex);
	int destNodeId = AUTORES_I(self->srcNodes_destNodeId, srcNodeId);
	
	AutoMesh_VolumeElement2D_setNodeIdAt
	  (self->destElement, iElement, iElementVertex, destNodeId);
      }
    }
    
    assert(self->destElement->elements_nodeIds != NULL);
  }



  return 1;
}

int AutoMesh_VolumeMidNodeRemover2D_destNodeIdAtSrcNode
(AutoMesh_VolumeMidNodeRemover2D *self,
 int srcNodeId)
{
  return AUTORES_I(self->srcNodes_destNodeId, srcNodeId);
}

int AutoMesh_VolumeMidNodeRemover2D_srcNodeIdAtDestNode
(AutoMesh_VolumeMidNodeRemover2D *self,
 int destNodeId)
{
  return AUTORES_I(self->destNodes_srcNodeId, destNodeId);
}



