/* test_base.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{



  printf (" Test0 \n"); 



  {
    int min, mid, max;

    AutoMesh_SortId3 (1, 2, 3,
		      &min, &mid, &max);
    assert(min == 1);
    assert(mid == 2);
    assert(max == 3);

    AutoMesh_SortId3 (3, 2, 1,
		      &min, &mid, &max);
    assert(min == 1);
    assert(mid == 2);
    assert(max == 3);

    AutoMesh_SortId3 (3, 1, 2,
		      &min, &mid, &max);
    assert(min == 1);
    assert(mid == 2);
    assert(max == 3);

    AutoMesh_SortId3 (1, 3, 2,
		      &min, &mid, &max);
    assert(min == 1);
    assert(mid == 2);
    assert(max == 3);

    AutoMesh_SortId3 (2, 3, 1,
		      &min, &mid, &max);
    assert(min == 1);
    assert(mid == 2);
    assert(max == 3);
  }

  assert(!AutoMesh_IdIsEqual2 (0, 1, 
			       2, 3));
  assert(!AutoMesh_IdIsEqual2 (0, 1, 
			       0, 3));
  assert(!AutoMesh_IdIsEqual2 (0, 1, 
			       2, 1));
  assert(AutoMesh_IdIsEqual2 (0, 1, 
			      0, 1));
  assert(AutoMesh_IdIsEqual2 (0, 1, 
			      1, 0));

  assert(!AutoMesh_IdIsEqual3 (0, 1, 2,
			       3, 4, 5));
  assert(!AutoMesh_IdIsEqual3 (0, 1, 2,
			       3, 4, 2));
  assert(!AutoMesh_IdIsEqual3 (0, 1, 2,
			       0, 2, 3));
  assert(!AutoMesh_IdIsEqual3 (0, 1, 2,
			       0, 3, 2));
  assert(AutoMesh_IdIsEqual3 (0, 1, 2,
			      0, 1, 2));
  assert(AutoMesh_IdIsEqual3 (0, 1, 2,
			      0, 2, 1));
  assert(AutoMesh_IdIsEqual3 (0, 1, 2,
			      1, 2, 0));
  assert(AutoMesh_IdIsEqual3 (0, 1, 2,
			      2, 1, 0));
}

static void Test1 (void)
{
  double x, y, z;
  double x0, y0, z0;
  double x1, y1, z1;



  printf (" Test1 \n"); 



  assert(EQ(AutoMesh_Length3D (1, 0, 0), 1));
  assert(EQ(AutoMesh_Length3D (1, 2, 3), sqrt (1 * 1 + 2 * 2 + 3 * 3)));

  assert(EQ(AutoMesh_Distance3D (0, 0, 0, 
				 1, 0, 0), 1));
  assert(EQ(AutoMesh_Distance3D (-1, 0, 0, 
				 0, 0, 1), sqrt(2.0)));

  AutoMesh_GetNormalizedVector3D (10, 0, 0,
				  &x, &y, &z);
  assert(EQ(x, 1));
  assert(EQ(y, 0));
  assert(EQ(z, 0));
  
  AutoMesh_GetNormalizedVector3D (1, 2, 3,
				  &x, &y, &z);
  assert(EQ(x, 1 / sqrt (1 * 1 + 2 * 2 + 3 * 3)));
  assert(EQ(y, 2 / sqrt (1 * 1 + 2 * 2 + 3 * 3)));
  assert(EQ(z, 3 / sqrt (1 * 1 + 2 * 2 + 3 * 3)));

  assert(EQ(AutoMesh_ScalarProduct3D (0, 0, 1, 
				      1, 0, 0), 0));
  assert(EQ(AutoMesh_ScalarProduct3D (1, 2, 3, 
				      3, 2, 1), 
	    1 * 3 + 2 * 2 + 3 * 1));

  
  AutoMesh_GetVectorProduct3D (1, 2, 3,
			       4, 5, 6,
			       &x, &y, &z);
  assert(EQ(x, -3));
  assert(EQ(y, 6));
  assert(EQ(z, -3));

  AutoMesh_GetProjectedPointOnPlane3D
    (0, 0, 1, -10,
     1, 2, 3,
     &x, &y, &z);
  assert(EQ(x, 1));
  assert(EQ(y, 2));
  assert(EQ(z, 10));

  AutoMesh_GetProjectedPointOnPlane3D
    (1, 0, 0, 10,
     1, 2, 3,
     &x, &y, &z);
  assert(EQ(x, -10));
  assert(EQ(y, 2));
  assert(EQ(z, 3));

  AutoMesh_GetAnyPerpendicularDirection3D
    (1, 0, 0,
     &x0, &y0, &z0,
     &x1, &y1, &z1);

  assert(EQ(x0, 0));
  assert(EQ(y0, 0));
  assert(EQ(z0, 1));

  assert(EQ(x1, 0));
  assert(EQ(y1, -1));
  assert(EQ(z1, 0));

  AutoMesh_GetAnyPerpendicularDirection3D
    (0, 0, 1,
     &x0, &y0, &z0,
     &x1, &y1, &z1);

  assert(EQ(x0, 0));
  assert(EQ(y0, 1));
  assert(EQ(z0, 0));

  assert(EQ(x1, -1));
  assert(EQ(y1, 0));
  assert(EQ(z1, 0));


  assert(EQ(AutoMesh_SignedTetraVolume3D
	    (0, 0, 0,
	     1, 0, 0,
	     0, 2, 0,
	     0, 0, 3), 1 * 2 * 3 / 6.0));

  assert(EQ(AutoMesh_SignedTetraVolume3D
	    (0, 0, 0,
	     0, 2, 0,
	     1, 0, 0,
	     0, 0, 3), -1 * 2 * 3 / 6.0));
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();
  Test1 ();

  return 0;
}

