/* test_frame2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_frame2d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoMesh_Frame2D *frame;
  int nFrames;
  int flag;



  printf (" Test0 \n"); 



  /* frame */

  printf (" frame \n"); 

  frame = AutoMesh_Frame2D_New ();

  nFrames = 4;

  flag = AutoMesh_Frame2D_allocateFrames (frame, nFrames);
  assert(flag);

  flag = AutoMesh_Frame2D_makeEndPointsColumn (frame);
  assert(flag);

  AutoMesh_Frame2D_setEndPointsAt (frame, 0,  
				   0.0, 0.0,
				   1.0, 0.0);
  AutoMesh_Frame2D_setEndPointsAt (frame, 1,  
				   1.0, 0.0,
				   1.0, 2.0);
  AutoMesh_Frame2D_setEndPointsAt (frame, 2,  
				   1.0, 2.0,
				   0.0, 2.0);
  AutoMesh_Frame2D_setEndPointsAt (frame, 3,  
				   0.0, 2.0,
				   0.0, 0.0);

  assert(AutoMesh_Frame2D_nFrames (frame) == nFrames);

  {
    double x0, y0;
    double x1, y1;

    AutoMesh_Frame2D_getEndPointsAt (frame, 0,  
				     &x0, &y0,
				     &x1, &y1);
    assert(EQ(x0, 0.0));
    assert(EQ(y0, 0.0));

    assert(EQ(x1, 1.0));
    assert(EQ(y1, 0.0));

    AutoMesh_Frame2D_getEndPointsAt (frame, 3,  
				     &x0, &y0,
				     &x1, &y1);
    assert(EQ(x0, 0.0));
    assert(EQ(y0, 2.0));

    assert(EQ(x1, 0.0));
    assert(EQ(y1, 0.0));
  }

  {
    double minX, minY;
    double maxX, maxY;

    AutoMesh_Frame2D_getRange (frame, 
			       &minX, &minY,
			       &maxX, &maxY);
    assert(EQ(minX, 0.0));
    assert(EQ(minY, 0.0));

    assert(EQ(maxX, 1.0));
    assert(EQ(maxY, 2.0));
  }
  


  printf (" delete frame \n"); 

  AutoMesh_Frame2D_delete (frame);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}


