/* test_node2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoMesh_Node2D *node;
  int nNodes;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node2D_New ();

  nNodes = 6;

  flag = AutoMesh_Node2D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node2D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node2D_setCoordAt (node, 0,  0.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 1,  1.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 2,  2.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 3,  0.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 4,  1.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 5,  2.0, 3.0);

  assert(AutoMesh_Node2D_nNodes (node) == nNodes);

  {
    double x, y;

    AutoMesh_Node2D_getCoordAt (node, 0,  
				&x, &y);
    assert(EQ(x, 0.0));
    assert(EQ(y, 0.0));

    AutoMesh_Node2D_getCoordAt (node, 5,  
				&x, &y);
    assert(EQ(x, 2.0));
    assert(EQ(y, 3.0));
  }

  {
    double minX, minY;
    double maxX, maxY;

    AutoMesh_Node2D_getRange (node, 
			      &minX, &minY,
			      &maxX, &maxY);
    assert(EQ(minX, 0.0));
    assert(EQ(minY, 0.0));

    assert(EQ(maxX, 2.0));
    assert(EQ(maxY, 3.0));
  }

  assert(EQ(AutoMesh_Node2D_distance (node, 0, 0), 0.0));
  assert(EQ(AutoMesh_Node2D_distance (node, 0, 2), 2.0));
  assert(EQ(AutoMesh_Node2D_distance (node, 0, 5), 
	    sqrt (4.0 + 9.0)));
  


  printf (" delete node \n"); 

  AutoMesh_Node2D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}


