/* test_node_scalar2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_node_scalar2d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoMesh_Node2D *node;
  AutoMesh_NodeScalar2D *scalar;
  int nNodes;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node2D_New ();

  nNodes = 6;

  flag = AutoMesh_Node2D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node2D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node2D_setCoordAt (node, 0,  0.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 1,  1.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 2,  2.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 3,  0.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 4,  1.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 5,  2.0, 3.0);



  /* scalar */

  printf (" scalar \n"); 

  scalar = AutoMesh_NodeScalar2D_New (node);
  
  flag = AutoMesh_NodeScalar2D_makeValueColumn (scalar);
  assert(flag);
  
  AutoMesh_NodeScalar2D_setValueAt (scalar, 0,  1.0);
  AutoMesh_NodeScalar2D_setValueAt (scalar, 1,  2.0);
  AutoMesh_NodeScalar2D_setValueAt (scalar, 2,  3.0);
  AutoMesh_NodeScalar2D_setValueAt (scalar, 3,  4.0);
  AutoMesh_NodeScalar2D_setValueAt (scalar, 4,  5.0);
  AutoMesh_NodeScalar2D_setValueAt (scalar, 5,  6.0);
  
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 0), 1.0));
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 1), 2.0));
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 2), 3.0));
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 3), 4.0));
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 4), 5.0));
  assert(EQ(AutoMesh_NodeScalar2D_valueAt (scalar, 5), 6.0));
  
  {
    double min, max;

    AutoMesh_NodeScalar2D_getRange (scalar, 
				    &min, &max);
    assert(EQ(min, 1.0));
    assert(EQ(max, 6.0));
  }



  printf (" delete scalar \n"); 

  AutoMesh_NodeScalar2D_delete (scalar);

  printf (" delete node \n"); 

  AutoMesh_Node2D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

