/* test_node_vector3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_node_vector3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_NodeVector3D *vector;
  int nNodes;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* vector */

  printf (" vector \n"); 

  vector = AutoMesh_NodeVector3D_New (node);
  
  flag = AutoMesh_NodeVector3D_makeValueColumn (vector);
  assert(flag);
  
  AutoMesh_NodeVector3D_setValueAt (vector, 0,  0.0, 0.0, 0.0);
  AutoMesh_NodeVector3D_setValueAt (vector, 1,  1.0, 0.0, 0.0);
  AutoMesh_NodeVector3D_setValueAt (vector, 2,  0.0, 2.0, 0.0);
  AutoMesh_NodeVector3D_setValueAt (vector, 3,  0.0, 0.0, 3.0);
  AutoMesh_NodeVector3D_setValueAt (vector, 4,  1.0, 0.0, 3.0);
  AutoMesh_NodeVector3D_setValueAt (vector, 5,  0.0, 2.0, 3.0);
  
  {
    double x, y, z;

    AutoMesh_NodeVector3D_getValueAt (vector, 0, 
				      &x, &y, &z);
    assert(EQ(x, 0.0));
    assert(EQ(y, 0.0));
    assert(EQ(z, 0.0));
  }
  
  {
    double x, y, z;

    AutoMesh_NodeVector3D_getValueAt (vector, 5, 
				      &x, &y, &z);
    assert(EQ(x, 0.0));
    assert(EQ(y, 2.0));
    assert(EQ(z, 3.0));
  }

  {
    double minX, minY, minZ;
    double maxX, maxY, maxZ;

    AutoMesh_NodeVector3D_getRange (vector, 
				    &minX, &minY, &minZ,
				    &maxX, &maxY, &maxZ);
    assert(EQ(minX, 0.0));
    assert(EQ(minY, 0.0));
    assert(EQ(minZ, 0.0));

    assert(EQ(maxX, 1.0));
    assert(EQ(maxY, 2.0));
    assert(EQ(maxZ, 3.0));
  }

  assert(EQ(AutoMesh_NodeVector3D_maxLength (vector), 
	    sqrt (2.0 * 2.0 + 3.0 * 3.0)));



  printf (" delete vector \n"); 

  AutoMesh_NodeVector3D_delete (vector);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

