/* test_surface_boundary_edge3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_surface_boundary_edge3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int FindEdge 
(AutoMesh_SurfaceBoundaryEdge3D *edge,
 int nodeId0, int nodeId1)
{
  AutoMesh_PolygonType polygonType = edge->element->polygonType;
  int nEdges = AutoMesh_SurfaceBoundaryEdge3D_nEdges (edge);
  int iEdge;

  for (iEdge = 0; iEdge < nEdges; iEdge++) {
    int elementId;
    int elementEdgeNum;
    int elementNodeNum0, elementNodeNum1;
    int eNodeId0, eNodeId1;

    AutoMesh_SurfaceBoundaryEdge3D_getEdgeAt
      (edge, iEdge,
       &elementId, &elementEdgeNum);

    elementNodeNum0 = AutoMesh_Polygon_EdgeVertex
      (polygonType, elementEdgeNum, 0);
    elementNodeNum1 = AutoMesh_Polygon_EdgeVertex
      (polygonType, elementEdgeNum, 1);
    
    eNodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt
      (edge->element, elementId, elementNodeNum0);
    eNodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt
      (edge->element, elementId, elementNodeNum1);
    
    if (AutoMesh_IdIsEqual2 (nodeId0, nodeId1,
			     eNodeId0, eNodeId1)) {
      return iEdge;
    }
  }
  return -1;
}

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  AutoMesh_SurfaceBoundaryEdge3D *edge;
  int nNodes;
  int nElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  2.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  2.0, 3.0, 0.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 4);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_SurfaceBoundaryEdge3D_New (element);

  flag = AutoMesh_SurfaceBoundaryEdge3D_extract (edge);
  assert(flag);

  AutoMesh_SurfaceBoundaryEdge3D_purgeWorkArea (edge);

  assert(AutoMesh_SurfaceBoundaryEdge3D_nEdges (edge) == 6);
  assert(FindEdge (edge, 0, 1) != -1);
  assert(FindEdge (edge, 1, 2) != -1);
  assert(FindEdge (edge, 0, 3) != -1);
  assert(FindEdge (edge, 2, 5) != -1);
  assert(FindEdge (edge, 3, 4) != -1);
  assert(FindEdge (edge, 4, 5) != -1);



  printf (" delete edge \n"); 

  AutoMesh_SurfaceBoundaryEdge3D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

