/* test_surface_divider3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_surface_edge3d.h"
#include "automesh_surface_divider3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  AutoMesh_SurfaceEdge3D *edge;
  AutoMesh_SurfaceDivider3D *divider;
  AutoMesh_Node3D *destNode;
  AutoMesh_SurfaceElement3D *destElement;
  int nNodes;
  int nElements;
  int elementIdFlag;
  int srcNodeIdsFlag;
  int srcElementIdFlag;
  int destNodeIdFlag;
  int destElementIdsFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  2.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  2.0, 3.0, 0.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 4);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_SurfaceEdge3D_New (element);

  elementIdFlag = 0;
  flag = AutoMesh_SurfaceEdge3D_extract (edge, elementIdFlag);
  assert(flag);



  /* divider */

  printf (" divider \n"); 

  divider = AutoMesh_SurfaceDivider3D_New (edge);

  srcNodeIdsFlag = 1;
  srcElementIdFlag = 1;
  destNodeIdFlag = 1;
  destElementIdsFlag = 1;
  flag = AutoMesh_SurfaceDivider3D_do 
    (divider, 
     srcNodeIdsFlag, srcElementIdFlag,
     destNodeIdFlag, destElementIdsFlag);
  assert(flag);

  destNode = divider->destNode;
  destElement = divider->destElement;

  nDestNodes = AutoMesh_Node3D_nNodes (destNode);
  nDestElements = AutoMesh_SurfaceElement3D_nElements (destElement);

  assert(nDestNodes == 6 + 9);
  assert(nDestElements == 4 * 4);

  /* NEED TO REDEFINE */
  /* more check is required */



  printf (" delete divider \n"); 

  AutoMesh_SurfaceDivider3D_delete (divider);
  
  printf (" delete dest element \n"); 
  
  AutoMesh_SurfaceElement3D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node3D_delete (destNode);
  
  printf (" delete edge \n"); 

  AutoMesh_SurfaceEdge3D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

