/* test_surface_element3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  int nNodes;
  int nElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  2.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  2.0, 3.0, 0.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 4);

  assert(AutoMesh_SurfaceElement3D_nodeIdAt (element, 1, 0) == 1);
  assert(AutoMesh_SurfaceElement3D_nodeIdAt (element, 1, 1) == 4);
  assert(AutoMesh_SurfaceElement3D_nodeIdAt (element, 1, 2) == 3);

  {
    double minX, minY, minZ;
    double maxX, maxY, maxZ;

    AutoMesh_SurfaceElement3D_getRangeAt
      (element, 1, 
       &minX, &minY, &minZ,
       &maxX, &maxY, &maxZ);
    assert(EQ(minX, 0));
    assert(EQ(minY, 0));
    assert(EQ(minZ, 0));

    assert(EQ(maxX, 1));
    assert(EQ(maxY, 3));
    assert(EQ(maxZ, 0));
  }

  assert(EQ(AutoMesh_SurfaceElement3D_averageEdgeLengthAt (element, 1), 
	    (1 + 3 + sqrt (1 * 1 + 3 * 3)) / 3));
  
  {
    double nx, ny, nz;

    AutoMesh_SurfaceElement3D_getNormalAt
      (element, 1, 
       &nx, &ny, &nz);
    assert(EQ(nx, 0));
    assert(EQ(ny, 0));
    assert(EQ(nz, 1));
  }



  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}


