/* test_surface_extractor3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_surface_extractor3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int ElementIsFound 
(AutoMesh_SurfaceElement3D *element,
 int nodeId0, int nodeId1, int nodeId2)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int eNodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt
      (element, iElement, 0);
    int eNodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt
      (element, iElement, 1);
    int eNodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt
      (element, iElement, 2);
    
    if (nodeId0 == eNodeId0
	&& nodeId1 == eNodeId1
	&& nodeId2 == eNodeId2) {
      return 1;
    }
  }
  return 0;
}

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  AutoRes_Selection *elementSelection;
  AutoMesh_SurfaceExtractor3D *extractor;
  AutoMesh_Node3D *destNode;
  AutoMesh_SurfaceElement3D *destElement;
  int nNodes;
  int nElements;
  int nodeFlag;
  int elementFlag;
  int srcElementIdFlag;
  int destElementIdFlag;
  int srcNodeIdFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  2.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  2.0, 3.0, 0.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 4);



  /* selection */

  printf (" selection \n"); 

  elementSelection = AutoRes_Selection_New ();
  AutoRes_Selection_reserve (elementSelection, 10);
  AutoRes_Selection_addObject (elementSelection, 2);
  AutoRes_Selection_addObject (elementSelection, 3);



  /* extractor */

  printf (" extractor \n"); 

  extractor = AutoMesh_SurfaceExtractor3D_New (element, elementSelection);

  nodeFlag = 1;
  elementFlag = 1;
  srcElementIdFlag = 1;
  destElementIdFlag = 1;
  srcNodeIdFlag = 1;
  flag = AutoMesh_SurfaceExtractor3D_do 
    (extractor, 
     nodeFlag, elementFlag, 
     srcElementIdFlag, destElementIdFlag, srcNodeIdFlag);
  assert(flag);

  destNode = extractor->destNode;
  destElement = extractor->destElement;

  nDestNodes = AutoMesh_Node3D_nNodes (destNode);
  nDestElements = AutoMesh_SurfaceElement3D_nElements (destElement);

  assert(nDestNodes == 4);
  assert(nDestElements == 2);

  assert(ElementIsFound (destElement, 0, 1, 2));
  assert(ElementIsFound (destElement, 1, 3, 2));



  printf (" delete extractor \n"); 

  AutoMesh_SurfaceExtractor3D_delete (extractor);
  
  printf (" delete selection \n"); 

  AutoRes_Selection_delete (elementSelection);

  printf (" delete dest element \n"); 
  
  AutoMesh_SurfaceElement3D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node3D_delete (destNode);
  
  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

