/* test_surface_grouper3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_frame3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_surface_edge3d.h"
#include "automesh_surface_grouper3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  AutoMesh_SurfaceEdge3D *edge;
  AutoMesh_SurfaceGrouper3D *grouper;
  AutoMesh_Frame3D *frame;
  int nNodes;
  int nElements;
  int nEdges;
  int elementIdFlag;
  int groupIdFlag;
  int frameFlag;
  int frameGroupIdFlag;
  int frameVertexIdFlag;
  int iEdge;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  2.0, 0.0, 2.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  2.0, 3.0, 2.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 4);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_SurfaceEdge3D_New (element);

  elementIdFlag = 1;
  flag = AutoMesh_SurfaceEdge3D_extract (edge, elementIdFlag);
  assert(flag);

  flag = AutoMesh_SurfaceEdge3D_collectNeighbor (edge);
  assert(flag);

  nEdges = AutoRes_Table_nRecords (edge->edges);



  /* grouper */

  printf (" grouper \n"); 

  grouper = AutoMesh_SurfaceGrouper3D_New (edge);

  groupIdFlag = 1;
  frameFlag = 1;
  frameGroupIdFlag = 1;
  frameVertexIdFlag = 1;
  flag = AutoMesh_SurfaceGrouper3D_do 
    (grouper,
     30,
     groupIdFlag, frameFlag, frameGroupIdFlag, frameVertexIdFlag);
  assert(flag);

  frame = grouper->frame;

  AutoMesh_SurfaceGrouper3D_purgeWorkArea (grouper);

  assert(AutoMesh_SurfaceGrouper3D_nGroups (grouper) == 2);
  assert(AutoMesh_SurfaceGrouper3D_groupIdAtElement (grouper, 0) == 0);
  assert(AutoMesh_SurfaceGrouper3D_groupIdAtElement (grouper, 1) == 0);
  assert(AutoMesh_SurfaceGrouper3D_groupIdAtElement (grouper, 2) == 1);
  assert(AutoMesh_SurfaceGrouper3D_groupIdAtElement (grouper, 3) == 1);

  assert(AutoMesh_SurfaceGrouper3D_nFrameGroups (grouper) == 3);
  assert(AutoMesh_SurfaceGrouper3D_nFrameVertexs (grouper) == 2);

  for (iEdge = 0; iEdge < nEdges; iEdge++) {
    int nodeId0, nodeId1;

    AutoMesh_SurfaceEdge3D_getVertexNodeIdsAt (edge, iEdge,
					       &nodeId0, &nodeId1);
    
    if (AutoMesh_IdIsEqual2 (nodeId0, nodeId1,
			     1, 3)
	|| AutoMesh_IdIsEqual2 (nodeId0, nodeId1,
				2, 4)) {
      assert(!AutoMesh_SurfaceGrouper3D_frameFlagAtEdge (grouper, iEdge));
      assert(AutoMesh_SurfaceGrouper3D_frameGroupIdAtEdge 
	     (grouper, iEdge) == -1);
    } else {
      assert(AutoMesh_SurfaceGrouper3D_frameFlagAtEdge (grouper, iEdge));
      assert(0 <= AutoMesh_SurfaceGrouper3D_frameGroupIdAtEdge 
	     (grouper, iEdge));
    }
  }

  assert(AutoMesh_Frame3D_nFrames (frame) == 7);



  printf (" delete grouper \n"); 

  AutoMesh_SurfaceGrouper3D_delete (grouper);
  
  printf (" delete frame \n"); 
  
  AutoMesh_Frame3D_delete (frame);
  
  printf (" delete edge \n"); 

  AutoMesh_SurfaceEdge3D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

