/* test_surface_mid_node_remover3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_surface_edge3d.h"
#include "automesh_surface_mid_node_remover3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int ElementIsFound 
(AutoMesh_SurfaceElement3D *element,
 int nodeId0, int nodeId1, int nodeId2)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int eNodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 0);
    int eNodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 1);
    int eNodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 2);
    
    if (nodeId0 == eNodeId0
	&& nodeId1 == eNodeId1
	&& nodeId2 == eNodeId2) {
      return 1;
    }
  }
  return 0;
}

/* SurfaceElement3D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_SurfaceElement3D *element;
  AutoMesh_SurfaceMidNodeRemover3D *mnr;
  AutoMesh_Node3D *destNode;
  AutoMesh_SurfaceElement3D *destElement;
  int nNodes;
  int nElements;
  int nodeFlag;
  int elementFlag;
  int srcNodeIdFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 15;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  0.5, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  1.5, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  2.0, 0.0, 0.0);

  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 1.5, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 6,  0.5, 1.5, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 7,  1.0, 1.5, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 8,  1.5, 1.5, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 9,  2.0, 1.5, 0.0);

  AutoMesh_Node3D_setCoordAt (node, 10,  0.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 11,  0.5, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 12,  1.0, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 13,  1.5, 3.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 14,  2.0, 3.0, 0.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_SurfaceElement3D_New 
    (node, AUTOMESH_POLYGON_TRI, 6);

  nElements = 4;

  flag = AutoMesh_SurfaceElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_SurfaceElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 1, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 2, 10);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 3, 6);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 4, 5);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 0, 5, 1);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 1, 12);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 2, 10);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 3, 11);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 4, 6);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 1, 5, 7);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 1, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 2, 12);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 3, 8);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 4, 7);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 2, 5, 3);

  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 0, 4);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 1, 14);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 2, 12);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 3, 13);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 4, 8);
  AutoMesh_SurfaceElement3D_setNodeIdAt (element, 3, 5, 9);



  /* mid-node remover */

  printf (" mid-node remover \n"); 

  mnr = AutoMesh_SurfaceMidNodeRemover3D_New (element);

  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdFlag = 1;
  flag = AutoMesh_SurfaceMidNodeRemover3D_do 
    (mnr, 
     nodeFlag, elementFlag,
     srcNodeIdFlag);
  assert(flag);

  destNode = mnr->destNode;
  destElement = mnr->destElement;

  nDestNodes = AutoMesh_Node3D_nNodes (destNode);
  nDestElements = AutoMesh_SurfaceElement3D_nElements (destElement);

  assert(nDestNodes == 6);
  assert(nDestElements == 4);

  assert(ElementIsFound (destElement, 0, 1, 3));
  assert(ElementIsFound (destElement, 1, 4, 3));
  assert(ElementIsFound (destElement, 1, 2, 4));
  assert(ElementIsFound (destElement, 2, 5, 4));

  /* NEED TO REDEFINE */
  /* more check is required */



  printf (" delete mid-node remover \n"); 

  AutoMesh_SurfaceMidNodeRemover3D_delete (mnr);
  
  printf (" delete dest element \n"); 
  
  AutoMesh_SurfaceElement3D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node3D_delete (destNode);
  
  printf (" delete element \n"); 

  AutoMesh_SurfaceElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

