/* test_volume_boundary_maker3d2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_surface_element3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_boundary_face3d.h"
#include "automesh_volume_boundary_maker3d2.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int ElementIsFound 
(AutoMesh_SurfaceElement3D *element,
 int nodeId0, int nodeId1, int nodeId2)
{
  int nElements = AutoMesh_SurfaceElement3D_nElements (element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int eNodeId0 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 0);
    int eNodeId1 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 1);
    int eNodeId2 = AutoMesh_SurfaceElement3D_nodeIdAt (element, iElement, 2);
    
    if (AutoMesh_IdIsEqual3 (nodeId0, nodeId1, nodeId2,
			     eNodeId0, eNodeId1, eNodeId2)) {
      return 1;
    }
  }
  return 0;
}

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeBoundaryFace3D *face;
  AutoMesh_VolumeBoundaryMaker3D2 *bm;
  AutoMesh_Node3D *surfaceNode;
  AutoMesh_SurfaceElement3D *surfaceElement;
  int nNodes;
  int nElements;
  int nodeFlag;
  int elementFlag;
  int volumeNodeIdFlag;
  int nSurfaceNodes;
  int nSurfaceElements;
  int iNode;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoRes_Table_allocateRecords (node->nodes, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoRes_Table_allocateRecords (element->elements, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);



  /* face */

  printf (" face \n"); 

  face = AutoMesh_VolumeBoundaryFace3D_New (element);

  flag = AutoMesh_VolumeBoundaryFace3D_extract (face);
  assert(flag);

  AutoMesh_VolumeBoundaryFace3D_purgeWorkArea (face);



  /* boundary maker */

  printf (" boundary maker \n"); 

  bm = AutoMesh_VolumeBoundaryMaker3D2_New (face);

  nodeFlag = 1;
  elementFlag = 1;
  volumeNodeIdFlag = 1;
  flag = AutoMesh_VolumeBoundaryMaker3D2_do 
    (bm, nodeFlag, elementFlag, volumeNodeIdFlag);
  assert(flag);

  surfaceNode = bm->surfaceNode;
  surfaceElement = bm->surfaceElement;

  nSurfaceNodes = AutoMesh_Node3D_nNodes (surfaceNode);
  nSurfaceElements = AutoMesh_SurfaceElement3D_nElements (surfaceElement);

  assert(nSurfaceNodes == 6);
  assert(nSurfaceElements == 8);

  /* depending on renumbering scheme of boundary maker */
  /* in this case, the number of surface nodes is the same as 
     the number of volume elements. */
  assert(nSurfaceNodes == nNodes);
  for (iNode = 0; iNode < nNodes; iNode++) {
    assert(AutoMesh_VolumeBoundaryMaker3D2_volumeNodeIdAtSurfaceNode
	   (bm, iNode) == iNode);
    assert(AutoMesh_VolumeBoundaryMaker3D2_surfaceNodeIdAtVolumeNode
	   (bm, iNode) == iNode);
  }

  /* depending on renumbering scheme of boundary maker */
  assert(ElementIsFound (surfaceElement, 0, 1, 2));
  assert(ElementIsFound (surfaceElement, 3, 4, 5));
  assert(ElementIsFound (surfaceElement, 0, 1, 3));
  assert(ElementIsFound (surfaceElement, 1, 4, 3));
  assert(ElementIsFound (surfaceElement, 1, 2, 4));
  assert(ElementIsFound (surfaceElement, 2, 4, 5));
  assert(ElementIsFound (surfaceElement, 2, 0, 3));
  assert(ElementIsFound (surfaceElement, 2, 3, 5));



  printf (" delete boundary maker \n"); 

  AutoMesh_VolumeBoundaryMaker3D2_delete (bm);
  
  printf (" delete surface element \n"); 
  
  AutoMesh_SurfaceElement3D_delete (surfaceElement);
  
  printf (" delete surface node \n"); 
  
  AutoMesh_Node3D_delete (surfaceNode);
  
  printf (" delete face \n"); 

  AutoMesh_VolumeBoundaryFace3D_delete (face);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

