/* test_volume_edge2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_volume_element2d.h"
#include "automesh_volume_edge2d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int FindEdge 
(AutoMesh_VolumeEdge2D *edge,
 int nodeId0, int nodeId1)
{
  AutoMesh_PolygonType polygonType = edge->element->polygonType;
  int nEdges = AutoMesh_VolumeEdge2D_nEdges (edge);
  int iEdge;

  for (iEdge = 0; iEdge < nEdges; iEdge++) {
    int elementIds[2];
    int elementEdgeNums[2];
    int elementNodeNum0, elementNodeNum1;
    int eNodeId0, eNodeId1;
    
    AutoMesh_VolumeEdge2D_getEdgeAt 
      (edge, iEdge,
       elementIds, elementEdgeNums);

    elementNodeNum0 = AutoMesh_Polygon_EdgeVertex
      (polygonType, elementEdgeNums[0], 0);
    elementNodeNum1 = AutoMesh_Polygon_EdgeVertex
      (polygonType, elementEdgeNums[0], 1);
    
    eNodeId0 = AutoMesh_VolumeElement2D_nodeIdAt
      (edge->element, elementIds[0], elementNodeNum0);
    eNodeId1 = AutoMesh_VolumeElement2D_nodeIdAt
      (edge->element, elementIds[0], elementNodeNum1);
    
    if (AutoMesh_IdIsEqual2 (nodeId0, nodeId1,
			     eNodeId0, eNodeId1)) {
      return iEdge;
    }
  }
  return -1;
}

static void CheckEdge 
(AutoMesh_VolumeEdge2D *edge)
{
  int nElements = AutoRes_Table_nRecords (edge->element->elements);
  int iElement;
  int iElementEdge;
  
  for (iElement = 0; iElement < nElements; iElement++) {
    for (iElementEdge = 0; 
	 iElementEdge < edge->nElementEdges; 
	 iElementEdge++) {
      int edgeId = AutoMesh_VolumeEdge2D_edgeIdAtElement
	(edge, iElement, iElementEdge);
      int elementIds[2];
      int elementEdgeNums[2];
    
      AutoMesh_VolumeEdge2D_getEdgeAt 
	(edge, edgeId,
	 elementIds, elementEdgeNums);

      assert(elementIds[0] != elementIds[1]);
      
      if (elementIds[1] == -1) {
	assert(elementEdgeNums[1] == -1);
	
	assert(elementIds[0] == iElement);
	assert(elementEdgeNums[0] == iElementEdge);
      } else {
	assert(elementEdgeNums[1] != -1);
	
	if (elementIds[0] == iElement) {
	  assert(elementEdgeNums[0] == iElementEdge);
	} else {
	  assert(elementIds[1] == iElement);
	  assert(elementEdgeNums[1] == iElementEdge);
	}
      }
    }
  }
}

/* VolumeElement2D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node2D *node;
  AutoMesh_VolumeElement2D *element;
  AutoMesh_VolumeEdge2D *edge;
  int nNodes;
  int nElements;
  int elementIdsFlag;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node2D_New ();

  nNodes = 6;

  flag = AutoMesh_Node2D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node2D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node2D_setCoordAt (node, 0,  0.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 1,  1.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 2,  2.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 3,  0.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 4,  1.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 5,  2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement2D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_VolumeElement2D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 2, 4);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_VolumeEdge2D_New (element);

  elementIdsFlag = 1;
  flag = AutoMesh_VolumeEdge2D_extract (edge, elementIdsFlag);
  assert(flag);

  flag = AutoMesh_VolumeEdge2D_collectNeighbor (edge);
  assert(flag);

  AutoMesh_VolumeEdge2D_purgeWorkArea (edge);

  assert(AutoMesh_VolumeEdge2D_nEdges (edge) == 9);
  assert(FindEdge (edge, 0, 1) != -1);
  assert(FindEdge (edge, 1, 2) != -1);
  assert(FindEdge (edge, 0, 3) != -1);
  assert(FindEdge (edge, 3, 1) != -1);
  assert(FindEdge (edge, 1, 4) != -1);
  assert(FindEdge (edge, 4, 2) != -1);
  assert(FindEdge (edge, 2, 5) != -1);
  assert(FindEdge (edge, 3, 4) != -1);
  assert(FindEdge (edge, 4, 5) != -1);
  
  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 0, 1)));
  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 1, 2)));
  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 0, 3)));

  assert(!AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 3, 1)));
  assert(!AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 1, 4)));
  assert(!AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 4, 2)));

  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 2, 5)));
  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 3, 4)));
  assert(AutoMesh_VolumeEdge2D_isOnBoundaryAt 
	 (edge, FindEdge (edge, 4, 5)));

  CheckEdge (edge);



  printf (" delete edge \n"); 

  AutoMesh_VolumeEdge2D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement2D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node2D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

