/* test_volume_element3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_VolumeElement3D *element;
  int nNodes;
  int nElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);

  assert(AutoMesh_VolumeElement3D_nElements (element) == nElements);

  assert(AutoMesh_VolumeElement3D_nodeIdAt (element, 1, 0) == 1);
  assert(AutoMesh_VolumeElement3D_nodeIdAt (element, 1, 1) == 2);
  assert(AutoMesh_VolumeElement3D_nodeIdAt (element, 1, 2) == 3);
  assert(AutoMesh_VolumeElement3D_nodeIdAt (element, 1, 3) == 4);

  {
    double minX, minY, minZ;
    double maxX, maxY, maxZ;

    AutoMesh_VolumeElement3D_getRangeAt
      (element, 1, 
       &minX, &minY, &minZ,
       &maxX, &maxY, &maxZ);
    assert(EQ(minX, 0));
    assert(EQ(minY, 0));
    assert(EQ(minZ, 0));

    assert(EQ(maxX, 1));
    assert(EQ(maxY, 2));
    assert(EQ(maxZ, 3));
  }

  assert(EQ(AutoMesh_VolumeElement3D_averageEdgeLengthAt (element, 0), 
	    (1 + 2 + 3 
	     + sqrt (1 * 1 + 2 * 2)
	     + sqrt (2 * 2 + 3 * 3)
	     + sqrt (3 * 3 + 1 * 1)) / 6));
  
  {
    double lx, ly, lz;

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       0, 0, 0,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 0));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       1, 0, 0,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 1));
    assert(EQ(ly, 0));
    assert(EQ(lz, 0));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       0, 2, 0,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 0));
    assert(EQ(ly, 1));
    assert(EQ(lz, 0));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       0, 0, 3,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 1));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       0.5, 1, 0,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 0.5));
    assert(EQ(ly, 0.5));
    assert(EQ(lz, 0));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       1.0/4, 2.0/4, 3.0/4,
       &lx, &ly, &lz);
    assert(flag);
    assert(EQ(lx, 0.25));
    assert(EQ(ly, 0.25));
    assert(EQ(lz, 0.25));

    flag = AutoMesh_VolumeElement3D_containsPointAt
      (element, 0, 
       1, 1, 1,
       &lx, &ly, &lz);
    assert(!flag);
  }



  flag = AutoMesh_VolumeElement3D_makeBucket (element);
  assert(flag);

  {
    double lx, ly, lz;
    int elementId;

    elementId = AutoMesh_VolumeElement3D_findElementUsingBucket
      (element, 
       0, 0, 0,
       &lx, &ly, &lz);
    assert(elementId == 0);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 0));

    elementId = AutoMesh_VolumeElement3D_findElementUsingBucket
      (element, 
       0, 2, 3,
       &lx, &ly, &lz);
    assert(elementId == 2);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 1));

    elementId = AutoMesh_VolumeElement3D_findElementUsingBucket
      (element, 
       1.0/4, 2.0/4, 3.0/4,
       &lx, &ly, &lz);
    assert(elementId == 0);
    assert(EQ(lx, 0.25));
    assert(EQ(ly, 0.25));
    assert(EQ(lz, 0.25));

    elementId = AutoMesh_VolumeElement3D_findElementUsingBucket
      (element, 
       1, 1, 1,
       &lx, &ly, &lz);
    assert(elementId == -1);
  }



  flag = AutoMesh_VolumeElement3D_makeNeighborIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 0, 0, 1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 0, 1, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 0, 2, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 0, 3, -1);
  
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 1, 0, 2);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 1, 1, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 1, 2, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 1, 3, 0);
  
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 2, 0, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 2, 1, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 2, 2, -1);
  AutoMesh_VolumeElement3D_setNeighborIdAt (element, 2, 3, 1);

  {
    double lx, ly, lz;
    int elementId;

    elementId = AutoMesh_VolumeElement3D_findElementNear
      (element, 0, 
       0, 0, 0,
       &lx, &ly, &lz);
    assert(elementId == 0);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 0));

    elementId = AutoMesh_VolumeElement3D_findElementNear
      (element, 0, 
       1, 0, 3,
       &lx, &ly, &lz);
    assert(elementId == 1);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 1));

    elementId = AutoMesh_VolumeElement3D_findElementNear
      (element, 0, 
       0, 2, 3,
       &lx, &ly, &lz);
    assert(elementId == 2);
    assert(EQ(lx, 0));
    assert(EQ(ly, 0));
    assert(EQ(lz, 1));
  }



  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

