/* test_volume_face3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_face3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int TriFaceIsFound 
(AutoMesh_VolumeFace3D *face,
 int nodeId0, int nodeId1, int nodeId2)
{
  int nFaces = AutoMesh_VolumeFace3D_nFaces (face);
  int iFace;

  for (iFace = 0; iFace < nFaces; iFace++) {
    int elementIds[2];
    int elementFaceNums[2];
    int elementId;
    int elementFaceNum;
    int elementNodeNum0, elementNodeNum1, elementNodeNum2;
    int fNodeId0, fNodeId1, fNodeId2;
    
    AutoMesh_VolumeFace3D_getFaceAt 
      (face, iFace,
       elementIds, elementFaceNums);
    
    elementId = elementIds[0];
    elementFaceNum = elementFaceNums[0];
    
    elementNodeNum0 = AutoMesh_Tetra_FaceVertexMap[elementFaceNum][0];
    elementNodeNum1 = AutoMesh_Tetra_FaceVertexMap[elementFaceNum][1];
    elementNodeNum2 = AutoMesh_Tetra_FaceVertexMap[elementFaceNum][2];
    
    fNodeId0 = AutoMesh_VolumeElement3D_nodeIdAt 
      (face->element, elementId, elementNodeNum0);
    fNodeId1 = AutoMesh_VolumeElement3D_nodeIdAt 
      (face->element, elementId, elementNodeNum1);
    fNodeId2 = AutoMesh_VolumeElement3D_nodeIdAt 
      (face->element, elementId, elementNodeNum2);
    
    if (AutoMesh_IdIsEqual3 (nodeId0, nodeId1, nodeId2,
			     fNodeId0, fNodeId1, fNodeId2)) {
      return 1;
    }
  }
  return 0;
}

static void CheckTetraFace 
(AutoMesh_VolumeFace3D *face)
{
  int nElements = AutoRes_Table_nRecords (face->element->elements);
  int iElement;
  int iElementFace;
  
  for (iElement = 0; iElement < nElements; iElement++) {
    for (iElementFace = 0; 
	 iElementFace < face->nElementFaces; 
	 iElementFace++) {
      int faceId;
      int elementIds[2];
      int elementFaceNums[2];

      faceId = AutoMesh_VolumeFace3D_faceIdAtElement 
	(face, iElement, iElementFace);
      AutoMesh_VolumeFace3D_getFaceAt 
	(face, faceId,
	 elementIds, elementFaceNums);

      assert(elementIds[0] != elementIds[1]);
      
      if (elementIds[1] == -1) {
	assert(elementFaceNums[1] == -1);
	
	assert(elementIds[0] == iElement);
	assert(elementFaceNums[0] == iElementFace);
      } else {
	assert(elementFaceNums[1] != -1);
	
	if (elementIds[0] == iElement) {
	  assert(elementFaceNums[0] == iElementFace);
	} else {
	  assert(elementIds[1] == iElement);
	  assert(elementFaceNums[1] == iElementFace);
	}
      }
    }
  }
}

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeFace3D *face;
  int nNodes;
  int nElements;
  int elementIdsFlag;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);



  /* face */

  printf (" face \n"); 

  face = AutoMesh_VolumeFace3D_New (element);

  elementIdsFlag = 1;
  flag = AutoMesh_VolumeFace3D_extract (face, elementIdsFlag);
  assert(flag);

  flag = AutoMesh_VolumeFace3D_collectNeighbor (face);
  assert(flag);

  AutoMesh_VolumeFace3D_purgeWorkArea (face);

  assert(AutoMesh_VolumeFace3D_nFaces (face) == 10);
  assert(TriFaceIsFound (face, 0, 1, 2));
  assert(TriFaceIsFound (face, 3, 4, 5));
  assert(TriFaceIsFound (face, 0, 1, 3));
  assert(TriFaceIsFound (face, 1, 4, 3));
  assert(TriFaceIsFound (face, 1, 2, 4));
  assert(TriFaceIsFound (face, 2, 4, 5));
  assert(TriFaceIsFound (face, 2, 0, 3));
  assert(TriFaceIsFound (face, 2, 3, 5));
  assert(TriFaceIsFound (face, 1, 2, 3));
  assert(TriFaceIsFound (face, 2, 3, 4));
  
  CheckTetraFace (face);



  printf (" delete face \n"); 

  AutoMesh_VolumeFace3D_delete (face);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

