/* test_volume_isosurface3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_node_scalar3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_isosurface3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_NodeScalar3D *scalar;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeIsosurface3D *isosurface;
  int nNodes;
  int nElements;
  double levelValue;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* scalar */

  printf (" scalar \n"); 

  scalar = AutoMesh_NodeScalar3D_New (node);
  
  flag = AutoMesh_NodeScalar3D_makeValueColumn (scalar);
  assert(flag);
  
  AutoMesh_NodeScalar3D_setValueAt (scalar, 0,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 1,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 2,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 3,  3.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 4,  3.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 5,  3.0);
  


  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);



  /* isosurface */

  printf (" isosurface \n"); 

  isosurface = AutoMesh_VolumeIsosurface3D_New (element, scalar);

  flag = AutoMesh_VolumeIsosurface3D_makeColumn (isosurface);
  assert(flag);

  levelValue = 2.0;
  flag = AutoMesh_VolumeIsosurface3D_extract (isosurface, levelValue);
  assert(flag);

  assert(AutoMesh_VolumeIsosurface3D_nIsosurfaces (isosurface) == 4);

  {
    int elementId;
    int elementEdgeNum0, elementEdgeNum1, elementEdgeNum2;
    double ratio0, ratio1, ratio2;
    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;

    AutoMesh_VolumeIsosurface3D_getIsosurfaceAt 
      (isosurface, 0,
       &elementId,
       &elementEdgeNum0, &ratio0,
       &elementEdgeNum1, &ratio1,
       &elementEdgeNum2, &ratio2);
    AutoMesh_VolumeIsosurface3D_getIsosurfacePointAt 
      (isosurface, 0,
       &x0, &y0, &z0,
       &x1, &y1, &z1,
       &x2, &y2, &z2);

    assert(elementId == 0);

    assert(elementEdgeNum0 == 2);
    assert(elementEdgeNum1 == 5);
    assert(elementEdgeNum2 == 4);

    assert(EQ(ratio0, 0.5));
    assert(EQ(ratio1, 0.5));
    assert(EQ(ratio2, 0.5));

    assert(EQ(x0, 0.0));
    assert(EQ(y0, 0.0));
    assert(EQ(z0, 1.5));

    assert(EQ(x1, 0.5));
    assert(EQ(y1, 0.0));
    assert(EQ(z1, 1.5));

    assert(EQ(x2, 0.0));
    assert(EQ(y2, 1.0));
    assert(EQ(z2, 1.5));
  }

  {
    int elementId;
    int elementEdgeNum0, elementEdgeNum1, elementEdgeNum2;
    double ratio0, ratio1, ratio2;
    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;

    AutoMesh_VolumeIsosurface3D_getIsosurfaceAt 
      (isosurface, 3,
       &elementId,
       &elementEdgeNum0, &ratio0,
       &elementEdgeNum1, &ratio1,
       &elementEdgeNum2, &ratio2);
    AutoMesh_VolumeIsosurface3D_getIsosurfacePointAt 
      (isosurface, 3,
       &x0, &y0, &z0,
       &x1, &y1, &z1,
       &x2, &y2, &z2);

    assert(elementId == 2);

    assert(elementEdgeNum0 == 0);
    assert(elementEdgeNum1 == 1);
    assert(elementEdgeNum2 == 2);

    assert(EQ(ratio0, 0.5));
    assert(EQ(ratio1, 0.5));
    assert(EQ(ratio2, 0.5));

    assert(EQ(x0, 0.0));
    assert(EQ(y0, 1.0));
    assert(EQ(z0, 1.5));

    assert(EQ(x1, 0.5));
    assert(EQ(y1, 1.0));
    assert(EQ(z1, 1.5));

    assert(EQ(x2, 0.0));
    assert(EQ(y2, 2.0));
    assert(EQ(z2, 1.5));
  }



  printf (" delete isosurface \n"); 

  AutoMesh_VolumeIsosurface3D_delete (isosurface);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

