/* test_volume_mid_node_generator2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node2d.h"
#include "automesh_volume_element2d.h"
#include "automesh_volume_edge2d.h"
#include "automesh_volume_mid_node_generator2d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* VolumeElement2D, 1st order 3 node triangle */
static void Test0 (void)
{
  AutoMesh_Node2D *node;
  AutoMesh_VolumeElement2D *element;
  AutoMesh_VolumeEdge2D *edge;
  AutoMesh_VolumeMidNodeGenerator2D *mng;
  AutoMesh_Node2D *destNode;
  AutoMesh_VolumeElement2D *destElement;
  int nNodes;
  int nElements;
  int elementIdFlag;
  int nodeFlag;
  int elementFlag;
  int srcNodeIdsFlag;
  int destNodeIdFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node2D_New ();

  nNodes = 6;

  flag = AutoMesh_Node2D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node2D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node2D_setCoordAt (node, 0,  0.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 1,  1.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 2,  2.0, 0.0);
  AutoMesh_Node2D_setCoordAt (node, 3,  0.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 4,  1.0, 3.0);
  AutoMesh_Node2D_setCoordAt (node, 5,  2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement2D_New 
    (node, AUTOMESH_POLYGON_TRI, 3);

  nElements = 4;

  flag = AutoMesh_VolumeElement2D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement2D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 0, 2, 3);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 1, 2, 3);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 0, 1);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 1, 2);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 2, 2, 4);

  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 0, 2);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 1, 5);
  AutoMesh_VolumeElement2D_setNodeIdAt (element, 3, 2, 4);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_VolumeEdge2D_New (element);

  elementIdFlag = 0;
  flag = AutoMesh_VolumeEdge2D_extract (edge, elementIdFlag);
  assert(flag);



  /* mid-node generator */

  printf (" mid-node generator \n"); 

  mng = AutoMesh_VolumeMidNodeGenerator2D_New (edge);

  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdsFlag = 1;
  destNodeIdFlag = 1;
  flag = AutoMesh_VolumeMidNodeGenerator2D_do 
    (mng, 
     nodeFlag, elementFlag,
     srcNodeIdsFlag, destNodeIdFlag);
  assert(flag);

  destNode = mng->destNode;
  destElement = mng->destElement;

  nDestNodes = AutoMesh_Node2D_nNodes (destNode);
  nDestElements = AutoMesh_VolumeElement2D_nElements (destElement);

  assert(nDestNodes == 6 + 9);
  assert(nDestElements == 4);

  /* NEED TO REDEFINE */
  /* more check is required */



  printf (" delete mid-node generator \n"); 

  AutoMesh_VolumeMidNodeGenerator2D_delete (mng);
  
  printf (" delete dest element \n"); 
  
  AutoMesh_VolumeElement2D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node2D_delete (destNode);
  
  printf (" delete edge \n"); 

  AutoMesh_VolumeEdge2D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement2D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node2D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

