/* test_volume_mid_node_generator3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_edge3d.h"
#include "automesh_volume_mid_node_generator3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeEdge3D *edge;
  AutoMesh_VolumeMidNodeGenerator3D *mng;
  AutoMesh_Node3D *destNode;
  AutoMesh_VolumeElement3D *destElement;
  int nNodes;
  int nElements;
  int nodeFlag;
  int elementFlag;
  int srcNodeIdsFlag;
  int destNodeIdFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);



  /* edge */

  printf (" edge \n"); 

  edge = AutoMesh_VolumeEdge3D_New (element);

  flag = AutoMesh_VolumeEdge3D_extract (edge);
  assert(flag);



  /* mid-node generator */

  printf (" mid-node generator \n"); 

  mng = AutoMesh_VolumeMidNodeGenerator3D_New (edge);

  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdsFlag = 1;
  destNodeIdFlag = 1;
  flag = AutoMesh_VolumeMidNodeGenerator3D_do 
    (mng, 
     nodeFlag, elementFlag,
     srcNodeIdsFlag, destNodeIdFlag);
  assert(flag);

  destNode = mng->destNode;
  destElement = mng->destElement;

  nDestNodes = AutoMesh_Node3D_nNodes (destNode);
  nDestElements = AutoMesh_VolumeElement3D_nElements (destElement);

  assert(nDestNodes == 6 + 12);
  assert(nDestElements == 3);

  /* NEED TO REDEFINE */
  /* more check is required */



  printf (" delete mid-node generator \n"); 

  AutoMesh_VolumeMidNodeGenerator3D_delete (mng);
  
  printf (" delete dest element \n"); 
  
  AutoMesh_VolumeElement3D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node3D_delete (destNode);
  
  printf (" delete edge \n"); 

  AutoMesh_VolumeEdge3D_delete (edge);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

