/* test_volume_mid_node_remover3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_edge3d.h"
#include "automesh_volume_mid_node_remover3d.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static int ElementIsFound 
(AutoMesh_VolumeElement3D *element,
 int nodeId0, int nodeId1, int nodeId2, int nodeId3)
{
  int nElements = AutoMesh_VolumeElement3D_nElements (element);
  int iElement;

  for (iElement = 0; iElement < nElements; iElement++) {
    int eNodeId0 = AutoMesh_VolumeElement3D_nodeIdAt (element, iElement, 0);
    int eNodeId1 = AutoMesh_VolumeElement3D_nodeIdAt (element, iElement, 1);
    int eNodeId2 = AutoMesh_VolumeElement3D_nodeIdAt (element, iElement, 2);
    int eNodeId3 = AutoMesh_VolumeElement3D_nodeIdAt (element, iElement, 3);
    
    if (nodeId0 == eNodeId0
	&& nodeId1 == eNodeId1
	&& nodeId2 == eNodeId2
	&& nodeId3 == eNodeId3) {
      return 1;
    }
  }
  return 0;
}

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeMidNodeRemover3D *mnr;
  AutoMesh_Node3D *destNode;
  AutoMesh_VolumeElement3D *destElement;
  int nNodes;
  int nElements;
  int nodeFlag;
  int elementFlag;
  int srcNodeIdFlag;
  int nDestNodes;
  int nDestElements;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 18;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  0.5, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.5, 1.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 1.0, 0.0);

  AutoMesh_Node3D_setCoordAt (node, 6,   0.0, 0.0, 1.5);
  AutoMesh_Node3D_setCoordAt (node, 7,   0.5, 0.0, 1.5);
  AutoMesh_Node3D_setCoordAt (node, 8,   1.0, 0.0, 1.5);
  AutoMesh_Node3D_setCoordAt (node, 9,   0.5, 1.0, 1.5);
  AutoMesh_Node3D_setCoordAt (node, 10,  0.0, 2.0, 1.5);
  AutoMesh_Node3D_setCoordAt (node, 11,  0.0, 1.0, 1.5);

  AutoMesh_Node3D_setCoordAt (node, 12,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 13,  0.5, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 14,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 15,  0.5, 1.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 16,  0.0, 2.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 17,  0.0, 1.0, 3.0);



  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 10);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 12);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 4, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 5, 5);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 6, 6);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 7, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 8, 11);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 9, 7);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 12);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 14);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 4, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 5, 7);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 6, 8);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 7, 11);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 8, 13);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 9, 9);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 12);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 14);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 16);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 4, 11);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 5, 9);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 6, 10);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 7, 13);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 8, 15);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 9, 17);



  /* mid-node remover */

  printf (" mid-node remover \n"); 

  mnr = AutoMesh_VolumeMidNodeRemover3D_New (element);

  nodeFlag = 1;
  elementFlag = 1;
  srcNodeIdFlag = 1;
  flag = AutoMesh_VolumeMidNodeRemover3D_do 
    (mnr, 
     nodeFlag, elementFlag,
     srcNodeIdFlag);
  assert(flag);

  destNode = mnr->destNode;
  destElement = mnr->destElement;

  nDestNodes = AutoMesh_Node3D_nNodes (destNode);
  nDestElements = AutoMesh_VolumeElement3D_nElements (destElement);

  assert(nDestNodes == 6);
  assert(nDestElements == 3);

  assert(ElementIsFound (destElement, 0, 1, 2, 3));
  assert(ElementIsFound (destElement, 1, 2, 3, 4));
  assert(ElementIsFound (destElement, 2, 3, 4, 5));

  /* NEED TO REDEFINE */
  /* more check is required */



  printf (" delete mid-node remover \n"); 

  AutoMesh_VolumeMidNodeRemover3D_delete (mnr);
  
  printf (" delete dest element \n"); 
  
  AutoMesh_VolumeElement3D_delete (destElement);
  
  printf (" delete dest node \n"); 
  
  AutoMesh_Node3D_delete (destNode);
  
  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

