/* test_volume_section3d2.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <autores.h>

#include "automesh_base.h"
#include "automesh_shape.h"
#include "automesh_node3d.h"
#include "automesh_node_scalar3d.h"
#include "automesh_volume_element3d.h"
#include "automesh_volume_section3d2.h"
#include "automesh_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

/* VolumeElement3D, 1st order 4 node tetrahedron */
static void Test0 (void)
{
  AutoMesh_Node3D *node;
  AutoMesh_NodeScalar3D *scalar;
  AutoMesh_VolumeElement3D *element;
  AutoMesh_VolumeSection3D2 *section;
  int nNodes;
  int nElements;
  double a, b, c, d;
  double offset;
  int flag;



  printf (" Test0 \n"); 



  /* node */

  printf (" node \n"); 

  node = AutoMesh_Node3D_New ();

  nNodes = 6;

  flag = AutoMesh_Node3D_allocateNodes (node, nNodes);
  assert(flag);

  flag = AutoMesh_Node3D_makeCoordColumn (node);
  assert(flag);

  AutoMesh_Node3D_setCoordAt (node, 0,  0.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 1,  1.0, 0.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 2,  0.0, 2.0, 0.0);
  AutoMesh_Node3D_setCoordAt (node, 3,  0.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 4,  1.0, 0.0, 3.0);
  AutoMesh_Node3D_setCoordAt (node, 5,  0.0, 2.0, 3.0);



  /* scalar */

  printf (" scalar \n"); 

  scalar = AutoMesh_NodeScalar3D_New (node);
  
  flag = AutoMesh_NodeScalar3D_makeValueColumn (scalar);
  assert(flag);
  
  AutoMesh_NodeScalar3D_setValueAt (scalar, 0,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 1,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 2,  1.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 3,  3.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 4,  3.0);
  AutoMesh_NodeScalar3D_setValueAt (scalar, 5,  3.0);
  


  /* element */

  printf (" element \n"); 

  element = AutoMesh_VolumeElement3D_New 
    (node, AUTOMESH_POLYHEDRON_TETRA, 4);

  nElements = 3;

  flag = AutoMesh_VolumeElement3D_allocateElements (element, nElements);
  assert(flag);

  flag = AutoMesh_VolumeElement3D_makeNodeIdsColumn (element);
  assert(flag);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 0, 0);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 1, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 2, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 0, 3, 3);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 0, 1);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 1, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 2, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 1, 3, 4);

  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 0, 2);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 1, 3);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 2, 4);
  AutoMesh_VolumeElement3D_setNodeIdAt (element, 2, 3, 5);



  /* section */

  printf (" section \n"); 

  section = AutoMesh_VolumeSection3D2_New (element);

  flag = AutoMesh_VolumeSection3D2_makeColumn (section);
  assert(flag);

  a = 0.0;
  b = 0.0;
  c = 1.0;
  d = -1.0;
  AutoMesh_VolumeSection3D2_setEquation (section, a, b, c, d);

  flag = AutoMesh_VolumeSection3D2_makeBucket (section);
  assert(flag);

  /* offset makes, d = d - offset */
  offset = 0.5;
  flag = AutoMesh_VolumeSection3D2_extract (section, offset);
  assert(flag);

  assert(AutoMesh_VolumeSection3D2_nSections (section) == 4);

  {
    int elementId;
    int elementEdgeNum0, elementEdgeNum1, elementEdgeNum2;
    double ratio0, ratio1, ratio2;
    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;
    double value0, value1, value2;

    AutoMesh_VolumeSection3D2_getSectionAt 
      (section, 3,
       &elementId,
       &elementEdgeNum0, &ratio0,
       &elementEdgeNum1, &ratio1,
       &elementEdgeNum2, &ratio2);
    AutoMesh_VolumeSection3D2_getSectionPointAt 
      (section, 3,
       &x0, &y0, &z0,
       &x1, &y1, &z1,
       &x2, &y2, &z2);
    AutoMesh_VolumeSection3D2_getSectionScalarAt 
      (section, 3, scalar,
       &value0, &value1, &value2);

    assert(elementId == 0);

    assert(elementEdgeNum0 == 2);
    assert(elementEdgeNum1 == 5);
    assert(elementEdgeNum2 == 4);

    assert(EQ(ratio0, 0.5));
    assert(EQ(ratio1, 0.5));
    assert(EQ(ratio2, 0.5));

    assert(EQ(x0, 0.0));
    assert(EQ(y0, 0.0));
    assert(EQ(z0, 1.5));

    assert(EQ(x1, 0.5));
    assert(EQ(y1, 0.0));
    assert(EQ(z1, 1.5));

    assert(EQ(x2, 0.0));
    assert(EQ(y2, 1.0));
    assert(EQ(z2, 1.5));

    assert(EQ(value0, 2.0));
    assert(EQ(value1, 2.0));
    assert(EQ(value2, 2.0));
  }

  {
    int elementId;
    int elementEdgeNum0, elementEdgeNum1, elementEdgeNum2;
    double ratio0, ratio1, ratio2;
    double x0, y0, z0;
    double x1, y1, z1;
    double x2, y2, z2;
    double value0, value1, value2;

    AutoMesh_VolumeSection3D2_getSectionAt 
      (section, 0,
       &elementId,
       &elementEdgeNum0, &ratio0,
       &elementEdgeNum1, &ratio1,
       &elementEdgeNum2, &ratio2);
    AutoMesh_VolumeSection3D2_getSectionPointAt 
      (section, 0,
       &x0, &y0, &z0,
       &x1, &y1, &z1,
       &x2, &y2, &z2);
    AutoMesh_VolumeSection3D2_getSectionScalarAt 
      (section, 0, scalar,
       &value0, &value1, &value2);

    assert(elementId == 2);

    assert(elementEdgeNum0 == 0);
    assert(elementEdgeNum1 == 1);
    assert(elementEdgeNum2 == 2);

    assert(EQ(ratio0, 0.5));
    assert(EQ(ratio1, 0.5));
    assert(EQ(ratio2, 0.5));

    assert(EQ(x0, 0.0));
    assert(EQ(y0, 1.0));
    assert(EQ(z0, 1.5));

    assert(EQ(x1, 0.5));
    assert(EQ(y1, 1.0));
    assert(EQ(z1, 1.5));

    assert(EQ(x2, 0.0));
    assert(EQ(y2, 2.0));
    assert(EQ(z2, 1.5));

    assert(EQ(value0, 2.0));
    assert(EQ(value1, 2.0));
    assert(EQ(value2, 2.0));
  }



  printf (" delete section \n"); 

  AutoMesh_VolumeSection3D2_delete (section);

  printf (" delete element \n"); 

  AutoMesh_VolumeElement3D_delete (element);

  printf (" delete node \n"); 

  AutoMesh_Node3D_delete (node);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);
  AutoMesh_Initialize ();

  Test0 ();

  return 0;
}

