/* advautomesh_h_tetra3d_crack.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>



/************************
	 Mesh
*************************/

/* Node */

typedef enum {
  NODE_CRACK_OTHER,
  NODE_CRACK_ON_CRACK,
  NODE_CRACK_ON_CRACK_TIP
} NodeCrackType;

#define MAX_NODES 1000000

int NNodes;
double Node_coord[MAX_NODES][3];
NodeCrackType Node_crackType[MAX_NODES];
int Node_theOtherNodeId[MAX_NODES];



/* Element */

#define N_ELEMENT_NODES 4
#define MAX_ELEMENTS 1000000

int NElements;
int Element_nodeIds[MAX_ELEMENTS][N_ELEMENT_NODES];
int Element_isOnTheOtherSide[MAX_ELEMENTS];



void ReadNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(NNodes == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NNodes);
  assert(0 < NNodes);
  assert(NNodes <= MAX_NODES);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    double x, y, z;

    fscanf (fp, "%d %lf %lf %lf", 
	    &nodeId, 
	    &x, &y, &z);
    assert(nodeId == iNode);
    Node_coord[iNode][0] = x;
    Node_coord[iNode][1] = y; 
    Node_coord[iNode][2] = z; 
  }

  fclose (fp);
}

void ReadElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(NElements == 0);

  fp = fopen (fileName, "r");
  assert(fp != NULL);

  fscanf (fp, "%d", 
	  &NElements);
  assert(0 < NElements);
  assert(NElements <= MAX_ELEMENTS);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    int elementId;

    fscanf (fp, "%d", 
	    &elementId); 
    assert(elementId == iElement);

    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      int nodeId;

      fscanf (fp, "%d", 
	      &nodeId);
      assert(0 <= nodeId);
      assert(nodeId < NNodes);
      Element_nodeIds[iElement][iElementNode] = nodeId; 
    }
  }
  
  fclose (fp);
}

void ReadNodeCrackFile (const char *fileName)
{
  FILE *fp;
  int count;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &count);
  assert(count == NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    int nodeId;
    NodeCrackType crackType;

    fscanf (fp, "%d %d", 
	    &nodeId,
	    (int *)&crackType);
    assert(nodeId == iNode);
    switch (crackType) {
    case NODE_CRACK_OTHER:
    case NODE_CRACK_ON_CRACK:
    case NODE_CRACK_ON_CRACK_TIP:
      Node_crackType[iNode] = crackType;
      break;
    default:
      assert(0);
      break;
    }
  }

  fclose (fp);
}

void ReadCrackElementListFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int nItems;
  int iItem;

  assert(0 < NElements);

  for (iElement = 0; iElement < NElements; iElement++) {
    Element_isOnTheOtherSide[iElement] = 0;
  }

  fp = fopen (fileName, "r");
  assert(fp != NULL);
  
  fscanf (fp, "%d", 
	  &nItems);
  assert(0 < nItems);
  
  for (iItem = 0; iItem < nItems; iItem++) {
    int itemId;
    int elementId;
    
    fscanf (fp, "%d %d", 
	    &itemId,
	    &elementId);
    assert(itemId == iItem);
    assert(0 <= elementId);
    assert(elementId < NElements);
    Element_isOnTheOtherSide[elementId] = 1;
  }

  fclose (fp);
}

void WriteNodeFile (const char *fileName)
{
  FILE *fp;
  int iNode;

  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %22.15e %22.15e %22.15e\n", 
	     iNode,
	     Node_coord[iNode][0],
	     Node_coord[iNode][1],
	     Node_coord[iNode][2]); 
  }

  fclose (fp);
}

void WriteElementFile (const char *fileName)
{
  FILE *fp;
  int iElement;
  int iElementNode;

  assert(0 < NNodes);
  assert(0 < NElements);

  fp = fopen (fileName, "w");
  assert(fp != NULL);

  fprintf (fp, "%d\n", 
	   NElements);
  
  for (iElement = 0; iElement < NElements; iElement++) {
    fprintf (fp, "%d   ", 
	     iElement); 
    for (iElementNode = 0; 
	 iElementNode < N_ELEMENT_NODES; 
	 iElementNode++) {
      fprintf (fp, "%d ", 
	       Element_nodeIds[iElement][iElementNode]); 
    }
    fprintf (fp, "\n");
  }
  
  fclose (fp);
}

void WriteNodeCrackFile (const char *fileName)
{
  FILE *fp;
  int iNode;
  
  assert(0 < NNodes);

  fp = fopen (fileName, "w");
  assert(fp != NULL);
  
  fprintf (fp, "%d\n", 
	   NNodes);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    fprintf (fp, "%d   %d\n", 
	     iNode, 
	     Node_crackType[iNode]);
  }

  fclose (fp);
}



/************************
	 Command
*************************/

void MakeNewNodesOnTheOtherSide (void)
{
  int iNode;
  int nOldNodes = NNodes;

  for (iNode = 0; iNode < nOldNodes; iNode++) {
    Node_theOtherNodeId[iNode] = -1;
  }

  for (iNode = 0; iNode < nOldNodes; iNode++) {
    if (Node_crackType[iNode] == NODE_CRACK_ON_CRACK) {
      int newNodeId = NNodes;
      
      assert(NNodes < MAX_NODES);
      NNodes++;
      
      assert(Node_theOtherNodeId[iNode] == -1);

      Node_coord[newNodeId][0] = Node_coord[iNode][0];
      Node_coord[newNodeId][1] = Node_coord[iNode][1];
      Node_coord[newNodeId][2] = Node_coord[iNode][2];
      Node_crackType[newNodeId] = NODE_CRACK_ON_CRACK;
      Node_theOtherNodeId[newNodeId] = iNode;
      Node_theOtherNodeId[iNode] = newNodeId;
    }
  }
}

void SwitchNodeOnTheOtherSide (void)
{
  int iElement;
  int iElementNode;
  
  for (iElement = 0; iElement < NElements; iElement++) {
    if (Element_isOnTheOtherSide[iElement]) {
      for (iElementNode = 0; 
	   iElementNode < N_ELEMENT_NODES; 
	   iElementNode++) {
	int nodeId = Element_nodeIds[iElement][iElementNode];
	
	if (Node_crackType[nodeId] == NODE_CRACK_ON_CRACK) {
	  int theOtherNodeId = Node_theOtherNodeId[nodeId];
	  
	  assert(theOtherNodeId != -1);
	  Element_nodeIds[iElement][iElementNode] = theOtherNodeId;
	}
      }
    }
  }
}

void PerformCommand (void)
{
  MakeNewNodesOnTheOtherSide ();
  SwitchNodeOnTheOtherSide ();
}



int main (int argc, char *argv[])
{
  if (argc != 8) {
    fprintf 
      (stderr, 
       " Usage : \n"
       " advautomesh_h_tetra3d_crack  node_file  element_file \n"
       "        node_crack_file  crack_element_list_file \n"
       "        node_file_OUT  element_file_OUT  node_crack_file_OUT \n");
    exit (1);
  }

  ReadNodeFile (argv[1]);
  ReadElementFile (argv[2]);
  ReadNodeCrackFile (argv[3]);
  ReadCrackElementListFile (argv[4]);

  PerformCommand ();

  WriteNodeFile (argv[5]);
  WriteElementFile (argv[6]);
  WriteNodeCrackFile (argv[7]);

  return 0;
}
